// cl-translate.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

Symbol* destructure_call_tree(Cons* tree, Surrogate*& _return1, Cons*& _return2) {
  { Generalized_Symbol* test_value_000 = ((Generalized_Symbol*)(tree->value));

    if (test_value_000 == SYM_CL_TRANSLATE_STELLA_SYS_CALL_FUNCTION) {
      _return1 = NULL;
      _return2 = tree->rest->rest;
      return (((Symbol*)(tree->rest->value)));
    }
    else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_SYS_CALL_METHOD) {
      _return1 = ((Surrogate*)(tree->rest->value));
      _return2 = tree->rest->rest->rest;
      return (((Symbol*)(tree->rest->rest->value)));
    }
    else {
      _return1 = NULL;
      _return2 = tree->rest;
      return (((Symbol*)(tree->value)));
    }
  }
}

// Mapping from STELLA operators to Common-Lisp operators.
Key_Value_List* oCL_OPERATOR_TABLEo = NULL;

boolean use_cl_consesP() {
  return (oCURRENT_ENVIRONMENT_FEATURESo->membP(KWD_CL_TRANSLATE_USE_COMMON_LISP_CONSES));
}

boolean use_defconsmethodP(Method_Slot* method) {
  if ((!method->method_functionP) &&
      oCURRENT_ENVIRONMENT_FEATURESo->membP(KWD_CL_TRANSLATE_USE_COMMON_LISP_CONSES)) {
    { Surrogate* test_value_000 = method->slot_owner;

      if (test_value_000 == SGT_CL_TRANSLATE_STELLA_CONS) {
        return (TRUE);
      }
      else if (test_value_000 == SGT_CL_TRANSLATE_STELLA_STANDARD_OBJECT) {
        if (lookup_slot(((Class*)(SGT_CL_TRANSLATE_STELLA_CONS->surrogate_value)), method->slot_name)->slot_owner == SGT_CL_TRANSLATE_STELLA_CONS) {
          return (FALSE);
        }
        else {
          return (TRUE);
        }
      }
      else if (test_value_000 == SGT_CL_TRANSLATE_STELLA_OBJECT) {
        { Surrogate* test_value_001 = lookup_slot(((Class*)(SGT_CL_TRANSLATE_STELLA_CONS->surrogate_value)), method->slot_name)->slot_owner;

          if ((test_value_001 == SGT_CL_TRANSLATE_STELLA_CONS) ||
              (test_value_001 == SGT_CL_TRANSLATE_STELLA_STANDARD_OBJECT)) {
            return (FALSE);
          }
          else {
            return (TRUE);
          }
        }
      }
      else {
      }
    }
  }
  return (FALSE);
}

boolean use_cl_structsP() {
  return (oCURRENT_ENVIRONMENT_FEATURESo->membP(KWD_CL_TRANSLATE_USE_COMMON_LISP_STRUCTS));
}

boolean use_vector_structsP() {
  return (oCURRENT_ENVIRONMENT_FEATURESo->membP(KWD_CL_TRANSLATE_USE_COMMON_LISP_VECTOR_STRUCTS));
}

DEFINE_STELLA_SPECIAL(oNEEDEXPLICITRETURNpo, boolean , FALSE);

Cons* cl_translate_list_of_trees(Cons* trees) {
  { Cons* cursor = trees;

    while (!(cursor == NIL)) {
      { 
        BIND_STELLA_SPECIAL(oNEEDEXPLICITRETURNpo, boolean, oNEEDEXPLICITRETURNpo.get() ||
            (!(cursor->rest == NIL)));
        cursor->value = cl_translate_a_tree(cursor->value);
      }
      cursor = cursor->rest;
    }
    return (trees);
  }
}

Cons* cl_translate_void_sys(Cons* tree) {
  { Object* otree = cl_translate_a_tree(tree->rest->value);

    return (((Cons*)(otree)));
  }
}

Object* cl_translate_typed_sys(Cons* tree) {
  { Object* otree = cl_translate_a_tree(tree->rest->value);

    return (otree);
  }
}

Object* cl_translate_bad_sys(Cons* tree) {
  { Object* returntype = tree->rest->rest->value;

    if (returntype == SGT_CL_TRANSLATE_STELLA_VOID) {
      return (yield_illegal_statement_tree());
    }
    else {
      return (yield_illegal_expression_tree());
    }
  }
}

Object* cl_translate_a_tree(Object* tree) {
  return (help_cl_translate_a_tree(tree));
}

Object* help_cl_translate_a_tree(Object* tree) {
  if (!((boolean)(tree))) {
    return (tree);
  }
  if (safe_primary_type(tree) == SGT_CL_TRANSLATE_STELLA_CONS) {
    { Object* tree_000 = tree;
      Cons* tree = ((Cons*)(tree_000));

      if (tree == NIL) {
        return (tree);
      }
      { Generalized_Symbol* test_value_000 = ((Generalized_Symbol*)(tree->value));

        if (test_value_000 == SYM_CL_TRANSLATE_STELLA_VOID_SYS) {
          return (cl_translate_void_sys(tree));
        }
        else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_TYPED_SYS) {
          return (cl_translate_typed_sys(tree));
        }
        else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_BAD_SYS) {
          return (cl_translate_bad_sys(tree));
        }
        else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_SETQ) {
          return (cl_translate_setq_tree(tree));
        }
        else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_SYS_SLOT_VALUE) {
          return (cl_translate_fixed_slot_value_tree(tree));
        }
        else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_SYS_SLOT_VALUE_SETTER) {
          return (cl_translate_fixed_slot_value_setter_tree(tree));
        }
        else if ((test_value_000 == SYM_CL_TRANSLATE_STELLA_SYS_CALL_METHOD) ||
            (test_value_000 == SYM_CL_TRANSLATE_STELLA_SYS_CALL_FUNCTION)) {
          return (cl_translate_call_method_tree(tree));
        }
        else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_PROGN) {
          tree->value = intern_common_lisp_symbol("PROGN");
          tree->rest = cl_translate_list_of_trees(tree->rest);
          return (tree);
        }
        else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_WITH_PROCESS_LOCK) {
          tree->second_setter(cl_translate_a_tree(tree->rest->value));
          tree->rest->rest = cl_translate_list_of_trees(tree->rest->rest);
          return (tree);
        }
        else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_LOOP) {
          { 
            BIND_STELLA_SPECIAL(oNEEDEXPLICITRETURNpo, boolean, TRUE);
            tree->value = intern_common_lisp_symbol("LOOP");
            tree->rest = maybe_wrap_with_continue_label(cl_translate_list_of_trees(tree->rest));
            return (tree);
          }
        }
        else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_SYS_FOREACH) {
          { 
            BIND_STELLA_SPECIAL(oNEEDEXPLICITRETURNpo, boolean, TRUE);
            return (cl_translate_foreach_tree(tree));
          }
        }
        else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_LET) {
          return (cl_translate_let_tree(tree, FALSE));
        }
        else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_STARTUP_TIME_PROGN) {
          { 
            BIND_STELLA_SPECIAL(oNEEDEXPLICITRETURNpo, boolean, TRUE);
            return (cl_translate_startup_time_progn_tree(tree));
          }
        }
        else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_SPECIAL) {
          return (cl_translate_special_tree(tree));
        }
        else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_SYS_UNWIND_PROTECT) {
          return (cl_translate_unwind_protect_tree(tree));
        }
        else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_SYS_HANDLER_CASE) {
          return (cl_translate_handler_case_tree(tree));
        }
        else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_SYS_HANDLE_EXCEPTION) {
          return (cl_translate_handle_exception_tree(tree));
        }
        else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_SYS_SIGNAL) {
          return (cl_translate_signal_tree(tree));
        }
        else if ((test_value_000 == SYM_CL_TRANSLATE_STELLA_CAST) ||
            (test_value_000 == SYM_CL_TRANSLATE_STELLA_SAFE_CAST)) {
          return (cl_translate_cast_tree(tree));
        }
        else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_RETURN) {
          return (cl_translate_return_tree(tree));
        }
        else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_BREAK) {
          tree->first_setter(intern_common_lisp_symbol("RETURN"));
          return (tree);
        }
        else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_CONTINUE) {
          return (listO(3, intern_common_lisp_symbol("GO"), KWD_CL_TRANSLATE_CONTINUE, NIL));
        }
        else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_WHILE) {
          { 
            BIND_STELLA_SPECIAL(oNEEDEXPLICITRETURNpo, boolean, TRUE);
            return (cl_translate_while_tree(tree));
          }
        }
        else if ((test_value_000 == SYM_CL_TRANSLATE_STELLA_IF) ||
            ((test_value_000 == SYM_CL_TRANSLATE_STELLA_CHOOSE) ||
             ((test_value_000 == SYM_CL_TRANSLATE_STELLA_WHEN) ||
              (test_value_000 == SYM_CL_TRANSLATE_STELLA_UNLESS)))) {
          return (cl_translate_conditional_tree(tree));
        }
        else if ((test_value_000 == SYM_CL_TRANSLATE_STELLA_AND) ||
            ((test_value_000 == SYM_CL_TRANSLATE_STELLA_OR) ||
             (test_value_000 == SYM_CL_TRANSLATE_STELLA_NOT))) {
          return (cl_translate_boolean_tree(tree));
        }
        else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_COND) {
          return (cl_translate_cond_tree(tree));
        }
        else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_CASE) {
          return (cl_translate_case_tree(tree));
        }
        else if ((test_value_000 == SYM_CL_TRANSLATE_STELLA_EQp) ||
            ((test_value_000 == SYM_CL_TRANSLATE_STELLA_g) ||
             ((test_value_000 == SYM_CL_TRANSLATE_STELLA_ge) ||
              ((test_value_000 == SYM_CL_TRANSLATE_STELLA_l) ||
               (test_value_000 == SYM_CL_TRANSLATE_STELLA_le))))) {
          return (cl_translate_operator_tree(tree));
        }
        else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_MAKE) {
          return (cl_translate_make_tree(tree));
        }
        else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_SYS_NEW) {
          return (cl_translate_new_tree(tree));
        }
        else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_MV_SETQ) {
          tree->first_setter(intern_common_lisp_symbol("MULTIPLE-VALUE-SETQ"));
          tree->third_setter(cl_translate_a_tree(tree->rest->rest->value));
          return (tree);
        }
        else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_QUOTE) {
          tree->first_setter(intern_common_lisp_symbol("QUOTE"));
          return (tree);
        }
        else if ((test_value_000 == SYM_CL_TRANSLATE_STELLA_SYS_CALL_FUNCTION_CODE) ||
            (test_value_000 == SYM_CL_TRANSLATE_STELLA_SYS_CALL_METHOD_CODE)) {
          return (cl_translate_call_function_code_tree(tree));
        }
        else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_SYS_INLINE_CALL) {
          return (cl_translate_inline_call_tree(tree));
        }
        else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_THE_CODE) {
          return (cl_translate_the_code_tree(tree));
        }
        else if ((test_value_000 == SYM_CL_TRANSLATE_STELLA_PRINT_STREAM) ||
            (test_value_000 == SYM_CL_TRANSLATE_STELLA_PRINT_NATIVE_STREAM)) {
          return (cl_translate_print_stream_tree(tree));
        }
        else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_VERBATIM) {
          return (cl_translate_verbatim_tree(tree));
        }
        else if ((test_value_000 == SYM_CL_TRANSLATE_STELLA_INLINE) ||
            (test_value_000 == SYM_CL_TRANSLATE_STELLA_IGNORE)) {
          return (cl_translate_inline_tree(tree));
        }
        else {
          { Output_String_Stream* stream_000 = new_output_string_stream();

            *(stream_000->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
            throw *new_stella_exception(stream_000->the_string_reader());
          }
        }
      }
    }
  }
  else {
    return (tree->cl_translate_atomic_tree());
  }
}

Object* Object::cl_translate_atomic_tree() {
  { Object* tree = this;

    return (tree);
  }
}

Object* Float_Wrapper::cl_translate_atomic_tree() {
  { Float_Wrapper* tree = this;

    { char* float_string = stringify(tree);

      if (string_memberP(float_string, 'e')) {
        float_string = string_substitute(float_string, 'd', 'e');
      }
      else if (string_memberP(float_string, 'E')) {
        float_string = string_substitute(float_string, 'd', 'E');
      }
      else {
        float_string = string_concatenate(float_string, "d0", 0);
      }
      return (new_verbatim_string_wrapper(float_string));
    }
  }
}

Object* Symbol::cl_translate_atomic_tree() {
  { Symbol* tree = this;

    if (((boolean)(tree->lookup_global_variable()))) {
      return (cl_translate_global_symbol(tree));
    }
    else {
      return (cl_translate_local_symbol(tree));
    }
  }
}

Object* Keyword::cl_translate_atomic_tree() {
  { Keyword* tree = this;

    if (use_hardcoded_symbolsP()) {
      { Integer_Wrapper* offset = wrap_integer(tree->keywordify()->symbol_id);

        return (listO(3, SYM_CL_TRANSLATE_STELLA_GET_KWD, offset, NIL));
      }
    }
    else {
      return (cl_translate_global_symbol(yield_symbol_constant_name(tree)));
    }
  }
}

Object* Surrogate::cl_translate_atomic_tree() {
  { Surrogate* tree = this;

    if (use_hardcoded_symbolsP()) {
      { Integer_Wrapper* offset = wrap_integer(tree->symbol_id);

        return (listO(3, SYM_CL_TRANSLATE_STELLA_GET_SGT, offset, NIL));
      }
    }
    else {
      return (cl_translate_global_symbol(yield_symbol_constant_name(tree)));
    }
  }
}

Object* cl_translate_global_symbol(Symbol* symbol) {
  { Module* symbolmodule = ((Module*)(symbol->home_context));

    if (symbolmodule == oCOMMON_LISP_MODULEo) {
      return (symbol);
    }
    if (!((boolean)(symbolmodule))) {
      symbolmodule = oMODULEo.get();
    }
    return (yield_global_lisp_symbol(symbolmodule, symbol->symbol_name));
  }
}

Object* yield_global_lisp_symbol(Module* symbolmodule, char* symbolname) {
  { char* lispsymbolpackage = (((!((boolean)(symbolmodule))) ||
        ((symbolmodule == oMODULEo.get()) ||
         string_eqlP(symbolmodule->lisp_package(), oMODULEo.get()->lisp_package()))) ? ((char*)(NULL)) : symbolmodule->lisp_package());
    char* lispsymbolname = ((((String_Wrapper*)(dynamic_slot_value(symbolmodule->dynamic_slots, SYM_CL_TRANSLATE_STELLA_MODULE_LISP_PACKAGE, NULL_STRING_WRAPPER)))->wrapper_value != NULL) ? symbolname : compute_full_name(symbolname, symbolmodule));

    if (symbolmodule->case_sensitiveP) {
      lispsymbolname = string_concatenate("|", string_concatenate(lispsymbolname, "|", 0), 0);
    }
    if (lispsymbolpackage == NULL) {
      return (new_verbatim_string_wrapper(lispsymbolname));
    }
    else {
      return (new_verbatim_string_wrapper(string_concatenate(lispsymbolpackage, string_concatenate("::", lispsymbolname, 0), 0)));
    }
  }
}

Object* cl_translate_local_symbol(Symbol* symbol) {
  { Module* symbolmodule = ((Module*)(symbol->home_context));

    if ((!((boolean)(symbolmodule))) ||
        (symbolmodule == oMODULEo.get())) {
      return (symbol);
    }
    else {
      return (new_verbatim_string_wrapper(symbol->symbol_name));
    }
  }
}

Cons* cl_translate_setq_tree(Cons* tree) {
  tree->first_setter(intern_common_lisp_symbol("SETQ"));
  tree->second_setter(cl_translate_a_tree(tree->rest->value));
  tree->third_setter(cl_translate_a_tree(tree->rest->rest->value));
  return (tree);
}

Object* yield_struct_slot_accessor_name(Surrogate* classtype, Symbol* slotname) {
  return (yield_global_lisp_symbol(((Module*)(classtype->home_context)), string_concatenate("%", string_concatenate(classtype->symbol_name, string_concatenate(".", slotname->symbol_name, 0), 0), 0)));
}

Object* yield_clos_slot_accessor_name(Symbol* slotname) {
  return (yield_global_lisp_symbol(((Module*)(slotname->home_context)), string_concatenate("%", slotname->symbol_name, 0)));
}

Cons* yield_slot_value_reader_tree(Symbol* slotname, Object* objectref, Surrogate* objecttype) {
  if ((objecttype == SGT_CL_TRANSLATE_STELLA_CONS) &&
      oCURRENT_ENVIRONMENT_FEATURESo->membP(KWD_CL_TRANSLATE_USE_COMMON_LISP_CONSES)) {
    if (slotname == SYM_CL_TRANSLATE_STELLA_VALUE) {
      return (cons(cl_translate_global_symbol(SYM_CL_TRANSLATE_STELLA_rrVALUE), cons(objectref, NIL)));
    }
    else if (slotname == SYM_CL_TRANSLATE_STELLA_REST) {
      return (cons(cl_translate_global_symbol(SYM_CL_TRANSLATE_STELLA_rrREST), cons(objectref, NIL)));
    }
    else {
    }
  }
  { Class* renamed_Class = ((Class*)(objecttype->surrogate_value));

    if ((!oCURRENT_ENVIRONMENT_FEATURESo->membP(KWD_CL_TRANSLATE_USE_COMMON_LISP_STRUCTS)) ||
        (exception_classP(renamed_Class) ||
         cl_always_translate_to_clos_classP(renamed_Class))) {
      return (cons(yield_clos_slot_accessor_name(slotname), cons(objectref, NIL)));
    }
    else if (oCURRENT_ENVIRONMENT_FEATURESo->membP(KWD_CL_TRANSLATE_USE_COMMON_LISP_VECTOR_STRUCTS)) {
      return (cons(cl_translate_global_symbol(SYM_CL_TRANSLATE_STELLA_CLSYS_SVAL), cons(wrap_integer(cl_struct_slot_offset(slotname, objecttype)), cons(cl_translate_global_symbol(intern_symbol_in_module(objecttype->symbol_name, ((Module*)(objecttype->home_context)), TRUE)), cons(cl_translate_global_symbol(slotname), cons(objectref, NIL))))));
    }
    else {
      return (cons(yield_struct_slot_accessor_name(objecttype, slotname), cons(objectref, NIL)));
    }
  }
}

Cons* yield_slot_value_writer_tree(Symbol* slotname, Object* objectref, Surrogate* objecttype, Object* valueref) {
  return (listO(3, intern_common_lisp_symbol("SETF"), yield_slot_value_reader_tree(slotname, objectref, objecttype), cons(valueref, NIL)));
}

Symbol* true_slot_name(Symbol* slotname, Surrogate* type) {
  return (canonical_slot(((Storage_Slot*)(lookup_slot(((Class*)(type->surrogate_value)), slotname))))->slot_name);
}

Cons* cl_translate_fixed_slot_value_tree(Cons* tree) {
  { Surrogate* objtype = ((Surrogate*)(tree->rest->value));
    Symbol* slotname = true_slot_name(((Symbol*)(tree->rest->rest->value)), objtype);
    Object* objref = cl_translate_a_tree(tree->fourth());
    Cons* otree = yield_slot_value_reader_tree(slotname, objref, objtype);

    return (otree);
  }
}

Cons* cl_translate_fixed_slot_value_setter_tree(Cons* tree) {
  { Surrogate* objtype = ((Surrogate*)(tree->rest->value));
    Symbol* slotname = true_slot_name(((Symbol*)(tree->rest->rest->value)), objtype);
    Object* objref = cl_translate_a_tree(tree->fourth());
    Object* valueref = cl_translate_a_tree(tree->fifth());
    Cons* otree = yield_slot_value_writer_tree(slotname, objref, objtype, valueref);

    return (otree);
  }
}

Object* cl_translate_call_method_tree(Cons* tree) {
  { Symbol* renamed_Operator = NULL;
    Surrogate* owner = NULL;
    Cons* arguments = NULL;

    renamed_Operator = destructure_call_tree(tree, owner, arguments);
    if ((renamed_Operator == SYM_CL_TRANSLATE_STELLA_NTH) ||
        (renamed_Operator == SYM_CL_TRANSLATE_STELLA_NTH_SETTER)) {
      return (cl_translate_nth_tree(tree));
    }
    else if ((renamed_Operator == SYM_CL_TRANSLATE_STELLA_AREF) ||
        (renamed_Operator == SYM_CL_TRANSLATE_STELLA_AREF_SETTER)) {
      return (cl_translate_aref_tree(tree));
    }
    else if ((renamed_Operator == SYM_CL_TRANSLATE_STELLA_NULLp) ||
        (renamed_Operator == SYM_CL_TRANSLATE_STELLA_DEFINEDp)) {
      return (cl_translate_plain_boolean_test(tree, FALSE));
    }
    else {
    }
    if ((owner == SGT_CL_TRANSLATE_STELLA_ARGUMENT_LIST) ||
        (owner == SGT_CL_TRANSLATE_STELLA_ARGUMENT_LIST_ITERATOR)) {
      return (cl_translate_argument_list_tree(tree));
    }
    if (((boolean)(oCL_OPERATOR_TABLEo->lookup(renamed_Operator)))) {
      return (cl_translate_operator_tree(tree));
    }
    return (cl_translate_normal_call_method_tree(renamed_Operator, owner, arguments));
  }
}

Symbol* method_call_type_for_vector_structs(Symbol* slotname, Surrogate* owner, boolean functionP) {
  if (functionP ||
      ((!((boolean)(owner))) ||
       ((slotname == SYM_CL_TRANSLATE_STELLA_NULLp) ||
        ((slotname == SYM_CL_TRANSLATE_STELLA_DEFINEDp) ||
         (symbol_common_lispP(slotname) ||
          ((!oCURRENT_ENVIRONMENT_FEATURESo->membP(KWD_CL_TRANSLATE_USE_COMMON_LISP_STRUCTS)) ||
           (!oCURRENT_ENVIRONMENT_FEATURESo->membP(KWD_CL_TRANSLATE_USE_COMMON_LISP_VECTOR_STRUCTS)))))))) {
    return (SYM_CL_TRANSLATE_STELLA_NORMAL_CALL);
  }
  if (((!subtype_ofP(owner, SGT_CL_TRANSLATE_STELLA_OBJECT)) &&
      (!((Class*)(owner->surrogate_value))->mixinP)) ||
      ((owner == SGT_CL_TRANSLATE_STELLA_CONS) &&
       oCURRENT_ENVIRONMENT_FEATURESo->membP(KWD_CL_TRANSLATE_USE_COMMON_LISP_CONSES))) {
    return (SYM_CL_TRANSLATE_STELLA_NON_OBJECT_METHOD);
  }
  else {
    if (slotname == SYM_CL_TRANSLATE_STELLA_PRINT_OBJECT) {
      return (SYM_CL_TRANSLATE_STELLA_PRINT_OBJECT);
    }
    else {
      return (SYM_CL_TRANSLATE_STELLA_OBJECT_METHOD);
    }
  }
}

Cons* cl_translate_normal_call_method_tree(Symbol* renamed_Operator, Surrogate* owner, Cons* arguments) {
  { Object* translatedoperator = cl_translate_global_symbol(yield_renamed_name_if_native(renamed_Operator, KWD_CL_TRANSLATE_COMMON_LISP, KWD_CL_TRANSLATE_FUNCTION));

    { Symbol* test_value_000 = method_call_type_for_vector_structs(renamed_Operator, owner, !((boolean)(owner)));

      if ((test_value_000 == SYM_CL_TRANSLATE_STELLA_OBJECT_METHOD) ||
          (test_value_000 == SYM_CL_TRANSLATE_STELLA_PRINT_OBJECT)) {
        return (cons(cl_translate_global_symbol(SYM_CL_TRANSLATE_STELLA_CLSYS_CALL), cons(translatedoperator, cl_translate_list_of_trees(arguments)->concatenate(NIL, 0))));
      }
      else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_NON_OBJECT_METHOD) {
        return (cons(translatedoperator, cons(cl_translate_a_tree(arguments->value), cons(SYM_CL_TRANSLATE_STELLA_NULL, cl_translate_list_of_trees(arguments->rest)->concatenate(NIL, 0)))));
      }
      else {
        return (cons(translatedoperator, cl_translate_list_of_trees(arguments)->concatenate(NIL, 0)));
      }
    }
  }
}

Cons* cl_translate_nth_tree(Cons* tree) {
  { Symbol* renamed_Operator = NULL;
    Surrogate* owner = NULL;
    Cons* arguments = NULL;

    renamed_Operator = destructure_call_tree(tree, owner, arguments);
    { boolean setterP = renamed_Operator == SYM_CL_TRANSLATE_STELLA_NTH_SETTER;
      Object* collection = arguments->value;
      Object* position = arguments->last();
      Object* value = (setterP ? arguments->rest->value : ((Object*)(NULL)));

      if ((owner == SGT_CL_TRANSLATE_STELLA_STRING) ||
          ((owner == SGT_CL_TRANSLATE_STELLA_MUTABLE_STRING) ||
           (owner == SGT_CL_TRANSLATE_STELLA_NATIVE_VECTOR))) {
        if ((owner == SGT_CL_TRANSLATE_STELLA_NATIVE_VECTOR) ||
            method_call_inlining_enabledP()) {
          renamed_Operator = intern_common_lisp_symbol("AREF");
          tree = cons(renamed_Operator, cons(cl_translate_a_tree(collection), cons(cl_translate_a_tree(position), NIL)));
          if (setterP) {
            tree = listO(3, intern_common_lisp_symbol("SETF"), tree, cons(cl_translate_a_tree(value), NIL));
          }
          return (tree);
        }
      }
      else {
      }
      return (cl_translate_normal_call_method_tree(renamed_Operator, owner, arguments));
    }
  }
}

Cons* cl_translate_aref_tree(Cons* tree) {
  { Symbol* renamed_Operator = NULL;
    Surrogate* owner = NULL;
    Cons* arguments = NULL;

    renamed_Operator = destructure_call_tree(tree, owner, arguments);
    if (owner == SGT_CL_TRANSLATE_STELLA_ARRAY) {
      { boolean setterP = renamed_Operator == SYM_CL_TRANSLATE_STELLA_AREF_SETTER;
        Object* array = arguments->value;
        Standard_Object* arraytype = walked_expression_type(array);
        Cons* indices = (setterP ? arguments->rest->rest : arguments->rest);
        Object* value = (setterP ? arguments->rest->value : ((Object*)(NULL)));

        renamed_Operator = intern_common_lisp_symbol("AREF");
        array = cl_translate_a_tree(array);
        indices = cl_translate_list_of_trees(indices);
        if (generate_cl_type_declarationsP()) {
          if (consP(array)) {
            array = cl_yield_typed_expression_tree(((Cons*)(array)), arraytype);
          }
          { Cons_Iterator* it = indices->allocate_iterator();

            while (it->nextP()) {
              if (consP(it->value)) {
                it->value_setter(cl_yield_typed_expression_tree(((Cons*)(it->value)), SGT_CL_TRANSLATE_STELLA_INTEGER));
              }
            }
          }
        }
        tree = cons(renamed_Operator, cons(array, indices->concatenate(NIL, 0)));
        if (setterP) {
          tree = listO(3, intern_common_lisp_symbol("SETF"), tree, cons(cl_translate_a_tree(value), NIL));
        }
        return (tree);
      }
    }
    else {
      return (cl_translate_normal_call_method_tree(renamed_Operator, owner, arguments));
    }
  }
}

Object* cl_translate_argument_list_tree(Cons* tree) {
  { Symbol* renamed_Operator = NULL;
    Surrogate* owner = NULL;
    Cons* arguments = NULL;

    renamed_Operator = destructure_call_tree(tree, owner, arguments);
    owner = owner;
    if (renamed_Operator == SYM_CL_TRANSLATE_STELLA_ALLOCATE_ITERATOR) {
      return (cl_translate_a_tree(arguments->value));
    }
    else if (renamed_Operator == SYM_CL_TRANSLATE_STELLA_LENGTH) {
      return (listO(3, intern_common_lisp_symbol("LENGTH"), cl_translate_a_tree(arguments->value), NIL));
    }
    else if (renamed_Operator == SYM_CL_TRANSLATE_STELLA_NEXTp) {
      return (cl_translate_a_tree(arguments->value));
    }
    else if (renamed_Operator == SYM_CL_TRANSLATE_STELLA_ARGUMENT) {
      return (listO(3, intern_common_lisp_symbol("POP"), cl_translate_a_tree(arguments->value), NIL));
    }
    else {
      { 
        BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
        signal_translation_error();
        if (!(suppress_warningsP())) {
          print_error_context(">> ERROR: ", STANDARD_ERROR);
          *(STANDARD_ERROR->native_stream) << endl << " " << "Cannot translate call to " << "`" << de_uglify_parse_tree(renamed_Operator) << "'" << " on an `ARGUMENT-LIST'" << "." << endl;
        }
      }
      return (cons(renamed_Operator, cons(cl_translate_a_tree(arguments->value), NIL)));
    }
  }
}

boolean duplicate_cursor_variableP(Symbol* variable, Cons* iteratorbindings) {
  { boolean foundP_000 = FALSE;

    { Cons* ib = NULL;
      Cons* iter_000 = ((Cons*)(iteratorbindings));

      while (!(iter_000 == NIL)) {
        ib = ((Cons*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (ib->value == variable) {
          foundP_000 = TRUE;
          break;
        }
      }
    }
    { boolean value_000 = foundP_000;

      return (value_000);
    }
  }
}

Cons* cl_translate_foreach_tree(Cons* tree) {
  { Cons* iteratorbindings = ((Cons*)(tree->rest->value));
    Cons* cursorbindings = ((Cons*)(tree->rest->rest->value));
    Object* continuationtest = cl_translate_boolean_test(tree->fourth(), FALSE);
    Cons* body = cl_translate_list_of_trees(tree->nth_rest(4));
    Cons* declarations = NIL;
    Cons* nextassignments = NIL;
    Cons* typedeclarations = NIL;
    Object* iteratorconstructor = NULL;

    if (generate_cl_type_declarationsP()) {
      typedeclarations = cl_yield_let_variable_type_declaration_trees(iteratorbindings);
    }
    { Cons* binding = NULL;
      Cons* iter_000 = iteratorbindings;
      Cons* collect_000 = NULL;

      while (!(iter_000 == NIL)) {
        binding = ((Cons*)(iter_000->value));
        iter_000 = iter_000->rest;
        iteratorconstructor = cl_translate_a_tree(binding->rest->rest->value);
        binding->third_setter(NULL);
        if (!((boolean)(collect_000))) {
          {
            collect_000 = cons(cons(cl_translate_local_symbol(((Symbol*)(binding->value))), cons(iteratorconstructor, NIL)), NIL);
            if (declarations == NIL) {
              declarations = collect_000;
            }
            else {
              add_cons_to_end_of_cons_list(declarations, collect_000);
            }
          }
        }
        else {
          {
            collect_000->rest = cons(cons(cl_translate_local_symbol(((Symbol*)(binding->value))), cons(iteratorconstructor, NIL)), NIL);
            collect_000 = collect_000->rest;
          }
        }
      }
    }
    if (generate_cl_type_declarationsP()) {
      typedeclarations = typedeclarations->concatenate(cl_yield_let_variable_type_declaration_trees(cursorbindings), 0);
    }
    { Cons* binding = NULL;
      Cons* iter_001 = cursorbindings;

      while (!(iter_001 == NIL)) {
        binding = ((Cons*)(iter_001->value));
        iter_001 = iter_001->rest;
        nextassignments = cons(cl_translate_a_tree(binding->rest->rest->value), nextassignments);
        binding->third_setter(NULL);
        if (!(duplicate_cursor_variableP(((Symbol*)(binding->value)), iteratorbindings))) {
          declarations = cons(cons(cl_translate_local_symbol(((Symbol*)(binding->value))), cons(cl_translate_a_tree(type_to_walked_null_value_tree(type_spec_to_base_type(((Standard_Object*)(binding->rest->value))))), NIL)), declarations);
        }
      }
    }
    return (listO(3, intern_common_lisp_symbol("LET*"), declarations, cl_yield_declare_tree(typedeclarations)->concatenate(cons(listO(4, intern_common_lisp_symbol("LOOP"), SYM_CL_TRANSLATE_STELLA_WHILE, continuationtest, cons(SYM_CL_TRANSLATE_STELLA_DO, nextassignments->reverse()->concatenate(maybe_wrap_with_continue_label(body)->concatenate(NIL, 0), 0))), NIL), 0)));
  }
}

Cons* maybe_wrap_with_continue_label(Cons* translatedloopbody) {
  if (search_cons_treeP(translatedloopbody, KWD_CL_TRANSLATE_CONTINUE)) {
    return (cons(cons(intern_common_lisp_symbol("TAGBODY"), translatedloopbody->concatenate(cons(KWD_CL_TRANSLATE_CONTINUE, NIL), 0)), NIL));
  }
  else {
    return (translatedloopbody);
  }
}

Cons* cl_translate_let_tree(Cons* tree, boolean specialP) {
  { Cons* odeclarations = NIL;
    Cons* typedeclarations = (generate_cl_type_declarationsP() ? cl_yield_let_variable_type_declaration_trees(((Cons*)(tree->rest->value))) : NIL);

    { Cons* d = NULL;
      Cons* iter_000 = ((Cons*)(tree->rest->value));

      while (!(iter_000 == NIL)) {
        d = ((Cons*)(iter_000->value));
        iter_000 = iter_000->rest;
        odeclarations = cons(cons((specialP ? cl_translate_global_symbol(((Symbol*)(d->value))) : cl_translate_local_symbol(((Symbol*)(d->value)))), cons(cl_translate_a_tree(d->rest->rest->value), NIL)), odeclarations);
      }
    }
    tree->first_setter(intern_common_lisp_symbol("LET*"));
    tree->second_setter(odeclarations->reverse());
    tree->rest->rest = cl_yield_declare_tree(typedeclarations)->concatenate(cl_translate_list_of_trees(tree->rest->rest), 0);
    return (tree);
  }
}

Cons* cl_translate_special_tree(Cons* tree) {
  { Cons* translatedtree = cl_translate_let_tree(tree, TRUE);
    Cons* specialvars = NIL;

    { Cons* decl = NULL;
      Cons* iter_000 = ((Cons*)(tree->rest->value));
      Cons* collect_000 = NULL;

      while (!(iter_000 == NIL)) {
        decl = ((Cons*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (!((boolean)(collect_000))) {
          {
            collect_000 = cons(decl->value, NIL);
            if (specialvars == NIL) {
              specialvars = collect_000;
            }
            else {
              add_cons_to_end_of_cons_list(specialvars, collect_000);
            }
          }
        }
        else {
          {
            collect_000->rest = cons(decl->value, NIL);
            collect_000 = collect_000->rest;
          }
        }
      }
    }
    translatedtree->rest->rest = cons(listO(3, intern_common_lisp_symbol("DECLARE"), cons(intern_common_lisp_symbol("SPECIAL"), specialvars->concatenate(NIL, 0)), NIL), translatedtree->rest->rest);
    return (translatedtree);
  }
}

Cons* cl_translate_startup_time_progn_tree(Cons* tree) {
  tree->value = intern_common_lisp_symbol("PROGN");
  tree->rest = cl_translate_list_of_trees(tree->rest);
  return (listO(4, intern_common_lisp_symbol("EVAL-WHEN"), listO(3, intern_common_lisp_symbol("LOAD"), intern_common_lisp_symbol("EVAL"), NIL), tree, NIL));
}

Cons* cl_translate_unwind_protect_tree(Cons* tree) {
  tree->value = intern_common_lisp_symbol("UNWIND-PROTECT");
  tree->rest = cl_translate_list_of_trees(tree->rest);
  return (tree);
}

Cons* cl_translate_signal_tree(Cons* tree) {
  tree->value = intern_common_lisp_symbol("ERROR");
  tree->second_setter(cl_translate_a_tree(tree->rest->value));
  return (tree);
}

Cons* cl_translate_handler_case_tree(Cons* tree) {
  tree->value = intern_common_lisp_symbol("HANDLER-CASE");
  tree->rest = cl_translate_list_of_trees(tree->rest);
  return (tree);
}

Cons* cl_translate_handle_exception_tree(Cons* tree) {
  { Symbol* var = ((Symbol*)(((Cons*)(tree->rest->value))->value));
    Standard_Object* type = ((Standard_Object*)(((Cons*)(tree->rest->value))->rest->value));

    tree->value = cl_translate_class_name(type_to_class(type_spec_to_base_type(type)));
    if (((boolean)(var))) {
      tree->second_setter(cons(cl_translate_local_symbol(var), NIL));
    }
    else {
      tree->second_setter(NIL);
    }
    tree->rest->rest = cl_translate_list_of_trees(tree->rest->rest);
    return (tree);
  }
}

Object* cast(Object* value, Surrogate* type) {
  // Perform a run-time type check, and then return `value'.
  if ((oSAFETYo.get() >= 2) &&
      ((oSTARTUP_TIME_PHASEo == 999) &&
       oUNFINALIZED_CLASSESo->emptyP())) {
    if (((boolean)(value)) &&
        (!isaP(value, type))) {
      { Output_String_Stream* stream_000 = new_output_string_stream();

        *(stream_000->native_stream) << "Run-time casting violation.  The value: " << endl << "`" << value << "'" << endl << " is not of type " << "`" << type << "'" << ".";
        throw *new_stella_exception(stream_000->the_string_reader());
      }
    }
  }
  return (value);
}

Object* cl_translate_cast_tree(Cons* tree) {
  { Object* expression = tree->rest->value;
    Surrogate* type = type_spec_to_base_type(((Standard_Object*)(tree->rest->rest->value)));

    tree->second_setter(cl_translate_a_tree(expression));
    tree->third_setter(cl_translate_a_tree(type));
    if (subtype_ofP(type, SGT_CL_TRANSLATE_STELLA_FLOAT)) {
      if (isaP(expression, SGT_CL_TRANSLATE_STELLA_INTEGER_WRAPPER)) {
        return (wrap_float(((double)(((Integer_Wrapper*)(expression))->wrapper_value)))->cl_translate_atomic_tree());
      }
      else {
        return (listO(3, intern_common_lisp_symbol("FLOAT"), tree->rest->value, cons(new_verbatim_string_wrapper("0.0d0"), NIL)));
      }
    }
    if (subtype_ofP(type, SGT_CL_TRANSLATE_STELLA_INTEGER)) {
      if (isaP(expression, SGT_CL_TRANSLATE_STELLA_FLOAT_WRAPPER)) {
        return (wrap_integer(((int)(((Float_Wrapper*)(expression))->wrapper_value))));
      }
      else {
        return (listO(3, intern_common_lisp_symbol("TRUNCATE"), tree->rest->value, NIL));
      }
    }
    if ((oSAFETYo.get() < 2) ||
        ((tree->value == SYM_CL_TRANSLATE_STELLA_SAFE_CAST) ||
         (!((boolean)(lookup_slot(type_to_class(type), SYM_CL_TRANSLATE_STELLA_PRIMARY_TYPE)))))) {
      expression = tree->rest->value;
      tree->second_setter(NIL);
      return (expression);
    }
    tree->first_setter(cl_translate_global_symbol(SYM_CL_TRANSLATE_STELLA_CAST));
    return (tree);
  }
}

Object* cl_translate_return_tree(Cons* tree) {
  { Cons* returnvalues = cl_translate_list_of_trees(tree->rest);

    if (!(returnvalues->rest == NIL)) {
      returnvalues = cons(cons(intern_common_lisp_symbol("VALUES"), returnvalues->concatenate(NIL, 0)), NIL);
    }
    return ((((!oNEEDEXPLICITRETURNpo.get()) &&
        preserve_tail_merge_optimizabilityP()) ? returnvalues->value : listO(3, intern_common_lisp_symbol("RETURN-FROM"), cl_translate_global_symbol(yield_renamed_name_if_native(((Method_Slot*)(oCURRENTTRANSLATIONUNITo.get()->the_object))->slot_name, KWD_CL_TRANSLATE_COMMON_LISP, KWD_CL_TRANSLATE_FUNCTION)), returnvalues->concatenate(NIL, 0))));
  }
}

Object* cl_translate_boolean_test(Object* tree, boolean invertP) {
  if (safe_primary_type(tree) == SGT_CL_TRANSLATE_STELLA_CONS) {
    { Object* tree_000 = tree;
      Cons* tree = ((Cons*)(tree_000));

      { Generalized_Symbol* test_value_000 = ((Generalized_Symbol*)(tree->value));

        if (test_value_000 == SYM_CL_TRANSLATE_STELLA_NOT) {
          { Object* postree = tree->rest->value;

            return (cl_translate_boolean_test(postree, !invertP));
          }
        }
        else if ((test_value_000 == SYM_CL_TRANSLATE_STELLA_AND) ||
            (test_value_000 == SYM_CL_TRANSLATE_STELLA_OR)) {
          { Cons* cursor = tree->rest;

            while (!(cursor == NIL)) {
              cursor->value = cl_translate_boolean_test(cursor->value, FALSE);
              cursor = cursor->rest;
            }
          }
          if (tree->value == SYM_CL_TRANSLATE_STELLA_AND) {
            tree->first_setter(intern_common_lisp_symbol("AND"));
          }
          else {
            tree->first_setter(intern_common_lisp_symbol("OR"));
          }
          if (invertP) {
            tree = listO(3, intern_common_lisp_symbol("NOT"), tree, NIL);
          }
          return (tree);
        }
        else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_TYPED_SYS) {
          return (cl_translate_boolean_test(tree->rest->value, invertP));
        }
        else {
        }
      }
    }
  }
  else {
  }
  return (cl_translate_plain_boolean_test(tree, invertP));
}

Object* cl_translate_plain_boolean_test(Object* tree, boolean invertP) {
  if (safe_primary_type(tree) == SGT_CL_TRANSLATE_STELLA_CONS) {
    { Object* tree_000 = tree;
      Cons* tree = ((Cons*)(tree_000));

      { Symbol* renamed_Operator = NULL;
        Surrogate* owner = NULL;
        Cons* arguments = NULL;

        renamed_Operator = destructure_call_tree(tree, owner, arguments);
        if ((renamed_Operator == SYM_CL_TRANSLATE_STELLA_l) ||
            ((renamed_Operator == SYM_CL_TRANSLATE_STELLA_le) ||
             ((renamed_Operator == SYM_CL_TRANSLATE_STELLA_e) ||
              ((renamed_Operator == SYM_CL_TRANSLATE_STELLA_ge) ||
               (renamed_Operator == SYM_CL_TRANSLATE_STELLA_g))))) {
          if (optimize_boolean_testsP()) {
            if (invertP) {
              if (renamed_Operator == SYM_CL_TRANSLATE_STELLA_l) {
                renamed_Operator = SYM_CL_TRANSLATE_STELLA_ge;
              }
              else if (renamed_Operator == SYM_CL_TRANSLATE_STELLA_le) {
                renamed_Operator = SYM_CL_TRANSLATE_STELLA_g;
              }
              else if (renamed_Operator == SYM_CL_TRANSLATE_STELLA_e) {
                renamed_Operator = SYM_CL_TRANSLATE_STELLA_xe;
              }
              else if (renamed_Operator == SYM_CL_TRANSLATE_STELLA_ge) {
                renamed_Operator = SYM_CL_TRANSLATE_STELLA_l;
              }
              else if (renamed_Operator == SYM_CL_TRANSLATE_STELLA_g) {
                renamed_Operator = SYM_CL_TRANSLATE_STELLA_le;
              }
              else {
                { Output_String_Stream* stream_000 = new_output_string_stream();

                  *(stream_000->native_stream) << "`" << renamed_Operator << "'" << " is not a valid case option";
                  throw *new_stella_exception(stream_000->the_string_reader());
                }
              }
              invertP = FALSE;
            }
          }
          tree = cons(oCL_OPERATOR_TABLEo->lookup(renamed_Operator), cl_translate_list_of_trees(arguments)->concatenate(NIL, 0));
          if (invertP) {
            return (listO(3, intern_common_lisp_symbol("NOT"), tree, NIL));
          }
          else {
            return (tree);
          }
        }
        else if (renamed_Operator == SYM_CL_TRANSLATE_STELLA_EQp) {
          if (optimize_boolean_testsP()) {
            { Standard_Object* argtype1 = walked_expression_type(arguments->value);
              Standard_Object* argtype2 = walked_expression_type(arguments->rest->value);

              if (sub_type_spec_ofP(argtype1, SGT_CL_TRANSLATE_STELLA_LITERAL) &&
                  sub_type_spec_ofP(argtype2, SGT_CL_TRANSLATE_STELLA_LITERAL)) {
                if ((argtype1 == SGT_CL_TRANSLATE_STELLA_STRING) ||
                    sub_type_spec_ofP(argtype1, SGT_CL_TRANSLATE_STELLA_BOOLEAN)) {
                  renamed_Operator = intern_common_lisp_symbol("EQ");
                }
                else if (sub_type_spec_ofP(argtype1, SGT_CL_TRANSLATE_STELLA_NUMBER)) {
                  renamed_Operator = intern_common_lisp_symbol("=");
                }
                else {
                  renamed_Operator = intern_common_lisp_symbol("EQL");
                }
              }
              else {
                renamed_Operator = intern_common_lisp_symbol("EQ");
              }
              tree->first_setter(renamed_Operator);
            }
          }
          tree->rest = cl_translate_list_of_trees(tree->rest);
          if (invertP) {
            return (listO(3, intern_common_lisp_symbol("NOT"), tree, NIL));
          }
          else {
            return (tree);
          }
        }
        else if ((renamed_Operator == SYM_CL_TRANSLATE_STELLA_NULLp) ||
            (renamed_Operator == SYM_CL_TRANSLATE_STELLA_DEFINEDp)) {
          if (method_call_inlining_enabledP()) {
            if ((owner == SGT_CL_TRANSLATE_STELLA_OBJECT) ||
                ((owner == SGT_CL_TRANSLATE_STELLA_SECOND_CLASS_OBJECT) ||
                 ((owner == SGT_CL_TRANSLATE_STELLA_NATIVE_VECTOR) ||
                  ((owner == SGT_CL_TRANSLATE_STELLA_STRING) ||
                   ((owner == SGT_CL_TRANSLATE_STELLA_CODE) ||
                    (owner == SGT_CL_TRANSLATE_STELLA_FLOAT)))))) {
              tree = listO(3, intern_common_lisp_symbol("EQ"), cl_translate_a_tree(arguments->value), cons(cl_translate_a_tree(type_to_walked_null_value_tree(owner)), NIL));
              if (owner == SGT_CL_TRANSLATE_STELLA_FLOAT) {
                tree->first_setter(intern_common_lisp_symbol("="));
              }
              else {
              }
              if ((invertP &&
                  (renamed_Operator == SYM_CL_TRANSLATE_STELLA_NULLp)) ||
                  ((!invertP) &&
                   (renamed_Operator == SYM_CL_TRANSLATE_STELLA_DEFINEDp))) {
                return (listO(3, intern_common_lisp_symbol("NOT"), tree, NIL));
              }
              else {
                return (tree);
              }
            }
            else {
            }
          }
          tree = cons(cl_translate_global_symbol(renamed_Operator), cons(cl_translate_a_tree(arguments->value), NIL));
          if (invertP) {
            return (listO(3, intern_common_lisp_symbol("NOT"), tree, NIL));
          }
          else {
            return (tree);
          }
        }
        else {
        }
      }
    }
  }
  else {
  }
  tree = cl_translate_a_tree(tree);
  if (invertP) {
    return (listO(3, intern_common_lisp_symbol("NOT"), tree, NIL));
  }
  else {
    return (tree);
  }
}

Cons* cl_translate_while_tree(Cons* tree) {
  { Object* test = cl_translate_boolean_test(tree->rest->value, FALSE);
    Cons* body = cl_translate_list_of_trees(tree->rest->rest);

    tree->rest->rest = NIL;
    return (((body == NIL) ? listO(4, intern_common_lisp_symbol("LOOP"), SYM_CL_TRANSLATE_STELLA_WHILE, test, NIL) : listO(4, intern_common_lisp_symbol("LOOP"), SYM_CL_TRANSLATE_STELLA_WHILE, test, cons(SYM_CL_TRANSLATE_STELLA_DO, maybe_wrap_with_continue_label(body)->concatenate(NIL, 0)))));
  }
}

Cons* cl_translate_conditional_tree(Cons* tree) {
  { Object* renamed_Operator = tree->value;
    Object* test = tree->rest->value;
    Cons* body = tree->rest->rest;

    tree->rest->rest = NIL;
    { Generalized_Symbol* test_value_000 = ((Generalized_Symbol*)(renamed_Operator));

      if ((test_value_000 == SYM_CL_TRANSLATE_STELLA_IF) ||
          (test_value_000 == SYM_CL_TRANSLATE_STELLA_CHOOSE)) {
        return (listO(3, intern_common_lisp_symbol("IF"), cl_translate_boolean_test(test, FALSE), cons(cl_translate_a_tree(body->value), cons(cl_translate_a_tree(body->rest->value), NIL))));
      }
      else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_WHEN) {
        return (listO(3, intern_common_lisp_symbol("WHEN"), cl_translate_boolean_test(test, FALSE), cl_translate_list_of_trees(body)->concatenate(NIL, 0)));
      }
      else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_UNLESS) {
        return (listO(3, intern_common_lisp_symbol("WHEN"), cl_translate_boolean_test(test, TRUE), cl_translate_list_of_trees(body)->concatenate(NIL, 0)));
      }
      else {
        { Output_String_Stream* stream_000 = new_output_string_stream();

          *(stream_000->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
          throw *new_stella_exception(stream_000->the_string_reader());
        }
      }
    }
  }
}

Cons* cl_translate_cond_tree(Cons* tree) {
  { Cons* condition = NULL;
    Cons* iter_000 = ((Cons*)(tree->rest));

    while (!(iter_000 == NIL)) {
      condition = ((Cons*)(iter_000->value));
      iter_000 = iter_000->rest;
      if (condition->value == SYM_CL_TRANSLATE_STELLA_OTHERWISE) {
        condition->value = intern_common_lisp_symbol("T");
      }
      else {
        condition->value = cl_translate_boolean_test(condition->value, FALSE);
      }
      condition->rest = cl_translate_list_of_trees(condition->rest);
    }
  }
  tree->value = intern_common_lisp_symbol("COND");
  return (tree);
}

Cons* cl_translate_case_tree(Cons* tree) {
  { boolean otherwiseP = FALSE;

    tree->second_setter(cl_translate_a_tree(tree->rest->value));
    { Cons* cond = NULL;
      Cons* iter_000 = ((Cons*)(tree->rest->rest));

      while (!(iter_000 == NIL)) {
        cond = ((Cons*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (cond->value == SYM_CL_TRANSLATE_STELLA_OTHERWISE) {
          otherwiseP = TRUE;
          cond->first_setter(intern_common_lisp_symbol("OTHERWISE"));
        }
        else if (consP(cond->value)) {
          cond->first_setter(cl_translate_list_of_trees(((Cons*)(cond->value))));
        }
        else {
          cond->first_setter(cl_translate_a_tree(cond->value));
        }
        cond->rest = cl_translate_list_of_trees(cond->rest);
      }
    }
    tree->first_setter((otherwiseP ? intern_common_lisp_symbol("CASE") : intern_common_lisp_symbol("ECASE")));
    return (tree);
  }
}

Cons* cl_translate_boolean_tree(Cons* tree) {
  return (((Cons*)(cl_translate_boolean_test(tree, FALSE))));
}

Object* cl_translate_class_name(Class* renamed_Class) {
  if (oCURRENT_ENVIRONMENT_FEATURESo->membP(KWD_CL_TRANSLATE_USE_COMMON_LISP_CONSES)) {
    if (renamed_Class->class_type == SGT_CL_TRANSLATE_STELLA_CONS) {
      return (intern_common_lisp_symbol("CONS"));
    }
    else {
    }
  }
  if (((String_Wrapper*)(dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CL_TRANSLATE_STELLA_CLASS_CL_NATIVE_TYPE, NULL_STRING_WRAPPER)))->wrapper_value != NULL) {
    { char* nativetype = ((String_Wrapper*)(dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CL_TRANSLATE_STELLA_CLASS_CL_NATIVE_TYPE, NULL_STRING_WRAPPER)))->wrapper_value;

      if (!(string_position(nativetype, ':', 0) != NULL_INTEGER)) {
        nativetype = string_concatenate("CL:", nativetype, 0);
      }
      return (new_verbatim_string_wrapper(nativetype));
    }
  }
  else {
    return (cl_translate_global_symbol(class_symbol(renamed_Class)));
  }
}

Object* yield_struct_constructor_name(Class* renamed_Class) {
  return (yield_global_lisp_symbol(((Module*)(renamed_Class->class_type->home_context)), string_concatenate("%MAKE-", renamed_Class->class_type->symbol_name, 0)));
}

Cons* cl_translate_make_tree(Cons* tree) {
  { Class* renamed_Class = ((Class*)(typify(tree->rest->value)->surrogate_value));
    Object* classsymbol = cl_translate_class_name(renamed_Class);
    Cons* otree = NULL;

    if ((classsymbol == intern_common_lisp_symbol("CONS")) &&
        oCURRENT_ENVIRONMENT_FEATURESo->membP(KWD_CL_TRANSLATE_USE_COMMON_LISP_CONSES)) {
      return (listO(4, intern_common_lisp_symbol("CONS"), intern_common_lisp_symbol("NIL"), intern_common_lisp_symbol("NIL"), NIL));
    }
    if (exception_classP(renamed_Class)) {
      otree = listO(5, intern_common_lisp_symbol("MAKE-CONDITION"), listO(3, intern_common_lisp_symbol("QUOTE"), classsymbol, NIL), SYM_CL_TRANSLATE_STELLA_oCONDITION_MESSAGE_KEYWORDo, cl_translate_a_tree(tree->rest->rest->value), NIL);
    }
    else if ((!oCURRENT_ENVIRONMENT_FEATURESo->membP(KWD_CL_TRANSLATE_USE_COMMON_LISP_STRUCTS)) ||
        cl_always_translate_to_clos_classP(renamed_Class)) {
      otree = listO(3, intern_common_lisp_symbol("MAKE-INSTANCE"), listO(3, intern_common_lisp_symbol("QUOTE"), classsymbol, NIL), NIL);
    }
    else if (oCURRENT_ENVIRONMENT_FEATURESo->membP(KWD_CL_TRANSLATE_USE_COMMON_LISP_VECTOR_STRUCTS)) {
      otree = cons(cl_translate_global_symbol(SYM_CL_TRANSLATE_STELLA_CLSYS_MAKE), cons(classsymbol, cons(wrap_integer(cl_struct_slots(renamed_Class)->length() + 1), NIL)));
    }
    else {
      otree = cons(yield_struct_constructor_name(renamed_Class), NIL);
    }
    return (otree);
  }
}

Cons* cl_translate_new_tree(Cons* tree) {
  if (array_type_specifierP(((Standard_Object*)(tree->rest->value)))) {
    return (cl_translate_new_array_tree(tree));
  }
  { Object* typespec = tree->rest->value;
    Object* constructorname = cl_translate_global_symbol(yield_constructor_name(((typeP(typespec) ? ((Surrogate*)(typespec)) : ((Parametric_Type_Specifier*)(typespec))->specifier_base_type))->get_stella_class(TRUE)));
    Cons* requiredargrefs = cl_translate_list_of_trees(tree->rest->rest);

    return (cons(constructorname, requiredargrefs->concatenate(NIL, 0)));
  }
}

Cons* cl_translate_new_array_tree(Cons* tree) {
  { Object* initialelement = tree->rest->rest->value;
    Cons* dimensions = cl_translate_list_of_trees(tree->rest->rest->rest);

    initialelement = NULL;
    return (listO(3, intern_common_lisp_symbol("MAKE-ARRAY"), ((dimensions->length() == 1) ? dimensions->value : cons(intern_common_lisp_symbol("LIST"), dimensions->concatenate(NIL, 0))), ((((boolean)(initialelement)) ? listO(3, KWD_CL_TRANSLATE_INITIAL_ELEMENT, cl_translate_a_tree(initialelement), NIL) : NIL))->concatenate(NIL, 0)));
  }
}

Cons* cl_translate_operator_tree(Cons* tree) {
  { Symbol* renamed_Operator = NULL;
    Surrogate* owner = NULL;
    Cons* arguments = NULL;

    renamed_Operator = destructure_call_tree(tree, owner, arguments);
    owner = owner;
    if (renamed_Operator == SYM_CL_TRANSLATE_STELLA_i) {
      if ((arguments->length() == 2) &&
          arguments->memberP(ONE_WRAPPER)) {
        if (eqlP(arguments->value, ONE_WRAPPER)) {
          arguments = arguments->rest;
        }
        else {
          arguments->rest = NIL;
        }
        renamed_Operator = intern_common_lisp_symbol("1+");
      }
    }
    else if (renamed_Operator == SYM_CL_TRANSLATE_STELLA__) {
      if ((arguments->length() == 2) &&
          eqlP(arguments->rest->value, ONE_WRAPPER)) {
        arguments->rest = NIL;
        renamed_Operator = intern_common_lisp_symbol("1-");
      }
    }
    else if (renamed_Operator == SYM_CL_TRANSLATE_STELLA_EQp) {
      return (((Cons*)(cl_translate_plain_boolean_test(tree, FALSE))));
    }
    else {
    }
    if ((renamed_Operator == SYM_CL_TRANSLATE_STELLA_i) ||
        ((renamed_Operator == SYM_CL_TRANSLATE_STELLA__) ||
         ((renamed_Operator == SYM_CL_TRANSLATE_STELLA_o) ||
          (renamed_Operator == SYM_CL_TRANSLATE_STELLA_s)))) {
      renamed_Operator = ((Symbol*)(oCL_OPERATOR_TABLEo->lookup(renamed_Operator)));
    }
    else {
    }
    return (cons(cl_translate_global_symbol(renamed_Operator), cl_translate_list_of_trees(arguments)->concatenate(NIL, 0)));
  }
}

Cons* yield_format_arguments(Cons* tree) {
  { char* formatstring = "";
    Cons* formatarguments = NIL;

    tree->rest->rest = tree->rest->rest->reverse();
    { Object* arg = NULL;
      Cons* iter_000 = tree->rest->rest;

      while (!(iter_000 == NIL)) {
        arg = iter_000->value;
        iter_000 = iter_000->rest;
        if (stringP(arg)) {
          formatstring = string_concatenate(((String_Wrapper*)(arg))->wrapper_value, formatstring, 0);
        }
        else if (arg == EOL) {
          formatstring = string_concatenate("~%", formatstring, 0);
        }
        else {
          formatstring = string_concatenate("~A", formatstring, 0);
          formatarguments = cons(cl_translate_a_tree(arg), formatarguments);
        }
      }
    }
    formatarguments = cons(wrap_string(formatstring), formatarguments);
    return (formatarguments);
  }
}

Cons* yield_print_tree(Cons* tree) {
  { Object* stream = tree->rest->value;
    Cons* arguments = tree->rest->rest;
    Surrogate* argumenttype = NULL;
    Symbol* printoperator = NULL;
    char* compoundedstring = NULL;
    Cons* printforms = NIL;

    stream = cl_translate_a_tree(stream);
    { Object* arg = NULL;
      Cons* iter_000 = arguments;

      while (!(iter_000 == NIL)) {
        arg = iter_000->value;
        iter_000 = iter_000->rest;
        if (arg == EOL) {
          printforms = cons(arg, printforms);
          compoundedstring = NULL;
        }
        else {
          argumenttype = type_spec_to_base_type(walked_expression_type(arg));
          if (argumenttype == SGT_CL_TRANSLATE_STELLA_STRING) {
            printoperator = intern_common_lisp_symbol("WRITE-STRING");
          }
          else {
            if (subtype_ofP(argumenttype, SGT_CL_TRANSLATE_STELLA_OBJECT)) {
              printoperator = SYM_CL_TRANSLATE_STELLA_PRINT_OBJECT;
            }
            else {
              printoperator = intern_common_lisp_symbol("PRINC");
            }
          }
          if (subtype_of_stringP(safe_primary_type(arg))) {
            { Object* arg_000 = arg;
              String_Wrapper* arg = ((String_Wrapper*)(arg_000));

              if (compoundedstring != NULL) {
                compoundedstring = string_concatenate(compoundedstring, arg->wrapper_value, 0);
                printforms->first_setter(wrap_string(compoundedstring));
                continue;
              }
              compoundedstring = arg->wrapper_value;
            }
          }
          else {
            compoundedstring = NULL;
          }
          if (method_call_type_for_vector_structs(printoperator, argumenttype, !(printoperator == SYM_CL_TRANSLATE_STELLA_PRINT_OBJECT)) == SYM_CL_TRANSLATE_STELLA_PRINT_OBJECT) {
            printforms = cons(cl_translate_a_tree(arg), printforms);
          }
          else {
            printforms = cons(cl_translate_a_tree(arg), printforms);
          }
        }
      }
    }
    printforms = printforms->reverse();
    return (cons(cl_translate_global_symbol(SYM_CL_TRANSLATE_STELLA_rrPRINT_STREAM), cons(stream, printforms->concatenate(NIL, 0))));
  }
}

Cons* cl_translate_print_stream_tree(Cons* tree) {
  { Object* stream = tree->rest->value;

    if (stream == KWD_CL_TRANSLATE_WARN) {
      return (cons(intern_common_lisp_symbol("WARN"), yield_format_arguments(tree)->concatenate(NIL, 0)));
    }
    else if (stream == KWD_CL_TRANSLATE_ERROR) {
      return (cons(intern_common_lisp_symbol("ERROR"), yield_format_arguments(tree)->concatenate(NIL, 0)));
    }
    else if (stream == KWD_CL_TRANSLATE_CONTINUABLE_ERROR) {
      return (listO(3, intern_common_lisp_symbol("CERROR"), wrap_string("Continue anyway? "), yield_format_arguments(tree)->concatenate(NIL, 0)));
    }
    else {
      return (yield_print_tree(tree));
    }
  }
}

Cons* cl_translate_call_function_code_tree(Cons* tree) {
  { boolean methodcodeP = tree->value == SYM_CL_TRANSLATE_STELLA_SYS_CALL_METHOD_CODE;
    Cons* otree = tree->rest;

    otree->rest = cl_translate_list_of_trees(otree->rest);
    otree->first_setter(((methodcodeP &&
        oCURRENT_ENVIRONMENT_FEATURESo->membP(KWD_CL_TRANSLATE_USE_COMMON_LISP_VECTOR_STRUCTS)) ? SYM_CL_TRANSLATE_STELLA_CLSYS_METHOD_CODE_CALL : intern_common_lisp_symbol("FUNCALL")));
    return (otree);
  }
}

Object* cl_translate_inline_call_tree(Cons* tree) {
  { Standard_Object* dummy1;

    { Cons* calltree = ((Cons*)(tree->rest->value));
      Method_Slot* method = ((Method_Slot*)(slot_from_expression_tree(calltree)));
      Cons* args = ((calltree->value == SYM_CL_TRANSLATE_STELLA_SYS_CALL_FUNCTION) ? calltree->nth_rest(2) : calltree->nth_rest(3));
      Cons* translatedargs = cl_translate_list_of_trees(args);
      Cons* usedcomplexargs = NIL;
      Object* argument = NULL;
      Object* body = lookup_verbatim_tree(((Cons*)(walk_verbatim_tree(((Cons*)(inlinable_method_body(method))), dummy1))), NULL);
      List* parameters = method->method_parameter_names_reader();
      List* parametertypes = method->method_parameter_type_specifiers_reader();
      int parindex = 0;
      Cons* otree = NIL;

      if (safe_primary_type(body) == SGT_CL_TRANSLATE_STELLA_CONS) {
        { Object* body_000 = body;
          Cons* body = ((Cons*)(body_000));

          { boolean alwaysP_000 = TRUE;

            { Object* elt = NULL;
              Cons* iter_000 = body;

              while (!(iter_000 == NIL)) {
                elt = iter_000->value;
                iter_000 = iter_000->rest;
                if (!(!consP(elt))) {
                  alwaysP_000 = FALSE;
                  break;
                }
              }
            }
            if (alwaysP_000) {
              otree = cons(body->value, NIL);
              { Object* barg = NULL;
                Cons* iter_001 = body->rest;

                while (!(iter_001 == NIL)) {
                  barg = iter_001->value;
                  iter_001 = iter_001->rest;
                  if (parameters->memberP(((Symbol*)(barg)))) {
                    parindex = parameters->position(barg, 0);
                    argument = translatedargs->nth(parindex);
                    if (usedcomplexargs->memberP(argument)) {
                      otree = NIL;
                      break;
                    }
                    if (consP(argument)) {
                      usedcomplexargs = cons(argument, usedcomplexargs);
                      argument = cl_yield_typed_expression_tree(((Cons*)(argument)), ((Standard_Object*)(parametertypes->nth(parindex))));
                    }
                    otree = cons(argument, otree);
                  }
                  else {
                    otree = cons(barg, otree);
                  }
                }
              }
              if (!(otree == NIL)) {
                return (cl_yield_typed_expression_tree(otree->reverse(), method->type()));
              }
            }
          }
        }
      }
      else {
      }
      { Symbol* par = NULL;
        Object* arg = NULL;
        Cons* iter_002 = parameters->the_cons_list;
        Cons* iter_003 = translatedargs;
        Cons* collect_000 = NULL;

        while ((!(iter_002 == NIL)) &&
            (!(iter_003 == NIL))) {
          par = ((Symbol*)(iter_002->value));
          iter_002 = iter_002->rest;
          arg = iter_003->value;
          iter_003 = iter_003->rest;
          if (!((boolean)(collect_000))) {
            {
              collect_000 = cons(cons(par, cons(arg, NIL)), NIL);
              if (otree == NIL) {
                otree = collect_000;
              }
              else {
                add_cons_to_end_of_cons_list(otree, collect_000);
              }
            }
          }
          else {
            {
              collect_000->rest = cons(cons(par, cons(arg, NIL)), NIL);
              collect_000 = collect_000->rest;
            }
          }
        }
      }
      return (listO(3, intern_common_lisp_symbol("LET"), otree, cl_yield_declare_tree(cl_yield_method_parameter_type_declaration_trees(method))->concatenate(cons(cl_yield_typed_expression_tree(((Cons*)(copy_cons_tree(body))), method->type()), NIL), 0)));
    }
  }
}

Cons* cl_translate_the_code_tree(Cons* tree) {
  { Symbol* name = ((tree->rest->value == KWD_CL_TRANSLATE_FUNCTION) ? ((Symbol*)(tree->rest->rest->value)) : ((Symbol*)(tree->fourth())));

    return (listO(3, intern_common_lisp_symbol("FUNCTION"), cl_translate_global_symbol(yield_renamed_name_if_native(name, KWD_CL_TRANSLATE_COMMON_LISP, KWD_CL_TRANSLATE_FUNCTION)), NIL));
  }
}

Object* cl_translate_verbatim_tree(Cons* tree) {
  { Cons* cursor = tree->rest;
    Object* verbatimcode = NULL;

    while (!(cursor == NIL)) {
      if (cursor->value == KWD_CL_TRANSLATE_COMMON_LISP) {
        verbatimcode = cursor->rest->value;
        cursor->rest->value = NULL;
        break;
      }
      cursor = cursor->rest->rest;
    }
    return (verbatimcode);
  }
}

Cons* cl_translate_inline_tree(Cons* tree) {
  { Object* renamed_Operator = tree->value;
    Cons* body = cl_translate_list_of_trees(tree->rest);

    { Generalized_Symbol* test_value_000 = ((Generalized_Symbol*)(renamed_Operator));

      if (test_value_000 == SYM_CL_TRANSLATE_STELLA_INLINE) {
        renamed_Operator = intern_common_lisp_symbol("INLINE");
      }
      else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_IGNORE) {
        renamed_Operator = intern_common_lisp_symbol("IGNORE");
      }
      else {
        { Output_String_Stream* stream_000 = new_output_string_stream();

          *(stream_000->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
          throw *new_stella_exception(stream_000->the_string_reader());
        }
      }
    }
    return (listO(3, intern_common_lisp_symbol("DECLARE"), cons(renamed_Operator, body->concatenate(NIL, 0)), NIL));
  }
}

boolean generate_cl_type_declarationsP() {
  return (method_call_inlining_enabledP());
}

Object* lookup_cl_type_from_stella_type(Standard_Object* type) {
  if ((!((boolean)(type))) ||
      anchored_type_specifierP(type)) {
    return (NULL);
  }
  { Surrogate* basetype = type_spec_to_base_type(type);

    if (basetype == SGT_CL_TRANSLATE_STELLA_INTEGER) {
      return (intern_common_lisp_symbol("FIXNUM"));
    }
    else if (basetype == SGT_CL_TRANSLATE_STELLA_SHORT_INTEGER) {
      return (intern_common_lisp_symbol("FIXNUM"));
    }
    else if (basetype == SGT_CL_TRANSLATE_STELLA_FLOAT) {
      return (intern_common_lisp_symbol("DOUBLE-FLOAT"));
    }
    else if (basetype == SGT_CL_TRANSLATE_STELLA_STRING) {
      return (intern_common_lisp_symbol("SIMPLE-STRING"));
    }
    else if (basetype == SGT_CL_TRANSLATE_STELLA_MUTABLE_STRING) {
      return (intern_common_lisp_symbol("SIMPLE-STRING"));
    }
    else if (basetype == SGT_CL_TRANSLATE_STELLA_NATIVE_VECTOR) {
      return (intern_common_lisp_symbol("SIMPLE-VECTOR"));
    }
    else if (basetype == SGT_CL_TRANSLATE_STELLA_ARRAY) {
      if (array_type_specifierP(type)) {
        switch (array_type_rank(((Parametric_Type_Specifier*)(type)))) {
          case 1: 
            return (intern_common_lisp_symbol("SIMPLE-VECTOR"));
          default:
            { Cons* dimensions = NIL;

              { Integer_Wrapper* dim = NULL;
                Cons* iter_000 = array_type_dimensions(((Parametric_Type_Specifier*)(type)))->the_cons_list;
                Cons* collect_000 = NULL;

                while (!(iter_000 == NIL)) {
                  dim = ((Integer_Wrapper*)(iter_000->value));
                  iter_000 = iter_000->rest;
                  if (!((boolean)(collect_000))) {
                    {
                      collect_000 = cons(((!((boolean)(dim))) ? ((Object*)(intern_common_lisp_symbol("*"))) : ((Object*)(dim))), NIL);
                      if (dimensions == NIL) {
                        dimensions = collect_000;
                      }
                      else {
                        add_cons_to_end_of_cons_list(dimensions, collect_000);
                      }
                    }
                  }
                  else {
                    {
                      collect_000->rest = cons(((!((boolean)(dim))) ? ((Object*)(intern_common_lisp_symbol("*"))) : ((Object*)(dim))), NIL);
                      collect_000 = collect_000->rest;
                    }
                  }
                }
              }
              return (listO(4, intern_common_lisp_symbol("ARRAY"), intern_common_lisp_symbol("T"), dimensions, NIL));
            }
          break;
        }
      }
    }
    else {
    }
    return (NULL);
  }
}

Cons* cl_yield_typed_expression_tree(Cons* tree, Standard_Object* type) {
  { Object* cltype = lookup_cl_type_from_stella_type(type);

    if (((boolean)(cltype))) {
      if (safe_primary_type(tree) == SGT_CL_TRANSLATE_STELLA_CONS) {
        { Cons* tree_000 = tree;
          Cons* tree = tree_000;

          if ((tree->value == intern_common_lisp_symbol("THE")) &&
              eqlP(tree->rest->value, cltype)) {
            return (tree);
          }
        }
      }
      else {
      }
      return (listO(3, intern_common_lisp_symbol("THE"), cltype, cons(tree, NIL)));
    }
    else {
      return (tree);
    }
  }
}

Cons* cl_yield_method_parameter_type_declaration_trees(Method_Slot* method) {
  { Cons* types = NIL;
    Symbol* cltype = NULL;

    { Standard_Object* tspec = NULL;
      Symbol* pname = NULL;
      Cons* iter_000 = method->method_parameter_type_specifiers_reader()->the_cons_list;
      Cons* iter_001 = method->method_parameter_names_reader()->the_cons_list;

      while ((!(iter_000 == NIL)) &&
          (!(iter_001 == NIL))) {
        tspec = ((Standard_Object*)(iter_000->value));
        iter_000 = iter_000->rest;
        pname = ((Symbol*)(iter_001->value));
        iter_001 = iter_001->rest;
        cltype = ((Symbol*)(lookup_cl_type_from_stella_type(compute_relative_type_spec(tspec, method->slot_owner))));
        if (((boolean)(cltype))) {
          types = cons(listO(3, intern_common_lisp_symbol("TYPE"), cltype, cons(cl_translate_local_symbol(pname), NIL)), types);
        }
      }
    }
    return (types->reverse());
  }
}

Cons* cl_yield_let_variable_type_declaration_trees(Cons* bindingstree) {
  { Cons* types = NIL;
    Symbol* cltype = NULL;

    { Cons* binding = NULL;
      Cons* iter_000 = bindingstree;

      while (!(iter_000 == NIL)) {
        binding = ((Cons*)(iter_000->value));
        iter_000 = iter_000->rest;
        { Object* type = binding->rest->value;

          if (((boolean)(type))) {
            cltype = ((Symbol*)(lookup_cl_type_from_stella_type(((Standard_Object*)(binding->rest->value)))));
          }
          if (((boolean)(cltype))) {
            types = cons(listO(3, intern_common_lisp_symbol("TYPE"), cltype, cons(cl_translate_local_symbol(((Symbol*)(binding->value))), NIL)), types);
          }
        }
      }
    }
    return (types->reverse());
  }
}

Cons* cl_yield_global_variable_type_declaration_tree(Global_Variable* global) {
  { Object* cltype = lookup_cl_type_from_stella_type(global_variable_type_spec(global));

    if (((boolean)(cltype))) {
      return (listO(3, intern_common_lisp_symbol("TYPE"), cltype, cons(cl_translate_global_symbol(global->variable_name), NIL)));
    }
    else {
      return (NULL);
    }
  }
}

Cons* cl_yield_declare_tree(Cons* declarations) {
  if (declarations == NIL) {
    return (NIL);
  }
  { Cons* otherdeclarations = declarations->rest;

    { Cons* decl = NULL;
      Cons* iter_000 = declarations;

      while (!(iter_000 == NIL)) {
        decl = ((Cons*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (((boolean)(decl)) &&
            (decl->value == intern_common_lisp_symbol("TYPE"))) {
          { Cons* otherdecl = NULL;
            Cons* iter_001 = otherdeclarations;

            while (!(iter_001 == NIL)) {
              otherdecl = ((Cons*)(iter_001->value));
              iter_001 = iter_001->rest;
              if (((boolean)(otherdecl)) &&
                  ((otherdecl->value == intern_common_lisp_symbol("TYPE")) &&
                   eqlP(otherdecl->rest->value, decl->rest->value))) {
                decl->concatenate(otherdecl->rest->rest, 0);
                otherdeclarations->substitute(NULL, otherdecl);
              }
            }
          }
        }
        otherdeclarations = otherdeclarations->rest;
      }
    }
    return (cons(cons(intern_common_lisp_symbol("DECLARE"), declarations->remove(NULL)->concatenate(NIL, 0)), NIL));
  }
}

Object* cl_translate_unit(Translation_Unit* unit) {
  { Symbol* test_value_000 = unit->category;

    if (test_value_000 == SYM_CL_TRANSLATE_STELLA_GLOBAL_VARIABLE) {
      return (cl_translate_define_global_variable_unit(unit));
    }
    else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_CLASS) {
      return (cl_translate_define_native_class_unit(unit));
    }
    else if ((test_value_000 == SYM_CL_TRANSLATE_STELLA_METHOD) ||
        (test_value_000 == SYM_CL_TRANSLATE_STELLA_MACRO)) {
      return (cl_translate_define_method_unit(unit));
    }
    else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_PRINT_METHOD) {
      return (cl_translate_defprint_unit(unit));
    }
    else if ((test_value_000 == SYM_CL_TRANSLATE_STELLA_STARTUP_TIME_PROGN) ||
        (test_value_000 == SYM_CL_TRANSLATE_STELLA_VERBATIM)) {
      return (cl_translate_a_tree(unit->the_object));
    }
    else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_TYPE) {
      unit->category = NULL;
      return (NIL);
    }
    else {
      { Output_String_Stream* stream_000 = new_output_string_stream();

        *(stream_000->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
        throw *new_stella_exception(stream_000->the_string_reader());
      }
    }
  }
}

Cons* cl_translate_method_parameters(Method_Slot* method) {
  { boolean functionP = method->method_functionP;
    Cons* otree = NIL;
    Object* oparameter = NULL;
    Object* firstparametertype = NULL;

    { Symbol* parameter = NULL;
      Standard_Object* ptype = NULL;
      int i = NULL_INTEGER;
      Cons* iter_000 = method->method_parameter_names_reader()->the_cons_list;
      Cons* iter_001 = method->method_parameter_type_specifiers_reader()->the_cons_list;
      int iter_002 = 1;

      while ((!(iter_000 == NIL)) &&
          (!(iter_001 == NIL))) {
        parameter = ((Symbol*)(iter_000->value));
        iter_000 = iter_000->rest;
        ptype = ((Standard_Object*)(iter_001->value));
        iter_001 = iter_001->rest;
        i = iter_002;
        iter_002 = iter_002 + 1;
        oparameter = cl_translate_local_symbol(parameter);
        if ((!functionP) &&
            (i == 1)) {
          firstparametertype = cl_translate_class_name(type_to_class(type_spec_to_base_type(ptype)));
          { Symbol* test_value_000 = method_call_type_for_vector_structs(method->slot_name, method->slot_owner, functionP);

            if (test_value_000 == SYM_CL_TRANSLATE_STELLA_OBJECT_METHOD) {
              otree = cons(listO(3, SYM_CL_TRANSLATE_STELLA_CLSYS_SELF, firstparametertype, NIL), otree);
              otree = cons(oparameter, otree);
            }
            else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_NON_OBJECT_METHOD) {
              otree = cons(cons(oparameter, cons(firstparametertype, NIL)), otree);
              otree = cons(SYM_CL_TRANSLATE_STELLA_CLSYS_DUMMY, otree);
            }
            else {
              otree = cons(cons(oparameter, cons(firstparametertype, NIL)), otree);
            }
          }
        }
        else {
          otree = cons(oparameter, otree);
        }
      }
    }
    if (((Boolean_Wrapper*)(dynamic_slot_value(method->dynamic_slots, SYM_CL_TRANSLATE_STELLA_METHOD_VARIABLE_ARGUMENTSp, FALSE_WRAPPER)))->wrapper_value &&
        (!pass_variable_arguments_as_listP(method))) {
      otree->rest = cons(intern_common_lisp_symbol("&REST"), otree->rest->concatenate(NIL, 0));
    }
    return (otree->reverse());
  }
}

Cons* cl_translate_define_method_unit(Translation_Unit* unit) {
  { Method_Slot* method = ((Method_Slot*)(unit->the_object));
    Symbol* methodname = method->slot_name;
    char* documentation = ((String_Wrapper*)(dynamic_slot_value(method->dynamic_slots, SYM_CL_TRANSLATE_STELLA_DOCUMENTATION, NULL_STRING_WRAPPER)))->wrapper_value;
    boolean functionP = method->method_functionP;
    Cons* bodytree = ((Cons*)(unit->code_register));
    Cons* declarations = NIL;
    Cons* otree = NIL;

    if ((methodname == SYM_CL_TRANSLATE_STELLA_PRINT_OBJECT) &&
        ((method->slot_owner == SGT_CL_TRANSLATE_STELLA_CONS) &&
         oCURRENT_ENVIRONMENT_FEATURESo->membP(KWD_CL_TRANSLATE_USE_COMMON_LISP_CONSES))) {
      return (NULL);
    }
    { Symbol* test_value_000 = method_call_type_for_vector_structs(methodname, method->slot_owner, functionP);

      if (test_value_000 == SYM_CL_TRANSLATE_STELLA_PRINT_OBJECT) {
        bodytree = cons(listO(4, SYM_CL_TRANSLATE_STELLA_SETQ, SYM_CL_TRANSLATE_STELLA_SELF, cl_translate_global_symbol(SYM_CL_TRANSLATE_STELLA_oCLSYS_SELFo), NIL), bodytree);
      }
      else if (test_value_000 == SYM_CL_TRANSLATE_STELLA_NON_OBJECT_METHOD) {
        declarations = cons(listO(3, intern_common_lisp_symbol("IGNORE"), SYM_CL_TRANSLATE_STELLA_CLSYS_DUMMY, NIL), NIL);
      }
      else {
      }
    }
    if (generate_cl_type_declarationsP()) {
      declarations = declarations->concatenate(cl_yield_method_parameter_type_declaration_trees(method), 0);
    }
    otree = cons((functionP ? intern_common_lisp_symbol("DEFUN") : ((use_defconsmethodP(method) ? SYM_CL_TRANSLATE_STELLA_rrDEFCONSMETHOD : intern_common_lisp_symbol("DEFMETHOD")))), cons(cl_translate_global_symbol(yield_renamed_name_if_native(methodname, KWD_CL_TRANSLATE_COMMON_LISP, KWD_CL_TRANSLATE_FUNCTION)), cons(cl_translate_method_parameters(method), (((documentation != NULL) ? cons(wrap_string(documentation), NIL) : NIL))->concatenate(cl_yield_declare_tree(declarations)->concatenate(cl_translate_list_of_trees(bodytree)->concatenate((((voidP(method->type()) &&
        (!preserve_tail_merge_optimizabilityP())) ? cons(KWD_CL_TRANSLATE_VOID, NIL) : NIL))->concatenate(NIL, 0), 0), 0), 0))));
    if (((Boolean_Wrapper*)(dynamic_slot_value(method->dynamic_slots, SYM_CL_TRANSLATE_STELLA_SLOT_AUXILIARYp, FALSE_WRAPPER)))->wrapper_value) {
      method->free();
    }
    return (otree);
  }
}

Cons* yield_clos_slot_tree(Storage_Slot* slot) {
  { Symbol* slotname = true_slot_name(slot->slot_name, slot->slot_owner);
    Keyword* allocation = slot->allocation();
    Cons* allocationform = NIL;
    char* documentation = ((String_Wrapper*)(dynamic_slot_value(slot->dynamic_slots, SYM_CL_TRANSLATE_STELLA_DOCUMENTATION, NULL_STRING_WRAPPER)))->wrapper_value;
    Cons* documentationform = NIL;

    if (allocation == KWD_CL_TRANSLATE_EMBEDDED) {
      allocation = KWD_CL_TRANSLATE_INSTANCE;
    }
    else {
    }
    if (((boolean)(allocation))) {
      allocationform = listO(3, KWD_CL_TRANSLATE_ALLOCATION, allocation, NIL);
    }
    if (documentation != NULL) {
      documentationform = listO(3, KWD_CL_TRANSLATE_DOCUMENTATION, wrap_string(documentation), NIL);
    }
    return (cons(cl_translate_global_symbol(slotname), documentationform->concatenate(allocationform->concatenate(listO(3, KWD_CL_TRANSLATE_ACCESSOR, yield_clos_slot_accessor_name(slotname), NIL), 0), 0)));
  }
}

Cons* yield_clos_class_tree(Class* renamed_Class) {
  { Cons* directsupers = NIL;
    Cons* closslotdefs = NIL;
    char* documentation = ((String_Wrapper*)(dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CL_TRANSLATE_STELLA_DOCUMENTATION, NULL_STRING_WRAPPER)))->wrapper_value;

    { Surrogate* super = NULL;
      Cons* iter_000 = class_native_supers(renamed_Class)->the_cons_list;
      Cons* collect_000 = NULL;

      while (!(iter_000 == NIL)) {
        super = ((Surrogate*)(iter_000->value));
        iter_000 = iter_000->rest;
        if ((!((Class*)(super->surrogate_value))->mixinP) ||
            (!translate_to_single_inheritance_languageP())) {
          if (!((boolean)(collect_000))) {
            {
              collect_000 = cons(cl_translate_class_name(type_to_class(super)), NIL);
              if (directsupers == NIL) {
                directsupers = collect_000;
              }
              else {
                add_cons_to_end_of_cons_list(directsupers, collect_000);
              }
            }
          }
          else {
            {
              collect_000->rest = cons(cl_translate_class_name(type_to_class(super)), NIL);
              collect_000 = collect_000->rest;
            }
          }
        }
      }
    }
    if (directsupers == NIL) {
      directsupers = cons(cl_translate_global_symbol(SYM_CL_TRANSLATE_STELLA_CLSYS_ROOT_OBJECT), directsupers);
    }
    { Slot* slot = NULL;
      Iterator* iter_001 = renamed_Class->class_slots();
      Cons* collect_001 = NULL;

      while (iter_001->nextP()) {
        slot = ((Slot*)(iter_001->value));
        if (storage_slotP(slot) &&
            native_slotP(((Storage_Slot*)(slot)))) {
          if (!(native_slot_home(slot, renamed_Class) == renamed_Class)) {
            if (((boolean)(slot->slot_base_type)) &&
                (!((boolean)(((Class*)(slot->slot_base_type->surrogate_value)))))) {
              *(STANDARD_WARNING->native_stream) << "Warning: " << "Can't generate a native slot named " << "`" << slot->slot_name << "'" << " for the class " << "`" << class_name(renamed_Class) << "'" << endl << " because the slot's :type " << "`" << slot->slot_base_type << "'" << " is undefined." << endl;
            }
            continue;
          }
          if (!((boolean)(collect_001))) {
            {
              collect_001 = cons(yield_clos_slot_tree(((Storage_Slot*)(slot))), NIL);
              if (closslotdefs == NIL) {
                closslotdefs = collect_001;
              }
              else {
                add_cons_to_end_of_cons_list(closslotdefs, collect_001);
              }
            }
          }
          else {
            {
              collect_001->rest = cons(yield_clos_slot_tree(((Storage_Slot*)(slot))), NIL);
              collect_001 = collect_001->rest;
            }
          }
        }
      }
    }
    return (listO(3, intern_common_lisp_symbol("DEFCLASS"), cl_translate_global_symbol(class_symbol(renamed_Class)), cons(directsupers, cons(closslotdefs, (((documentation != NULL) ? cons(listO(3, KWD_CL_TRANSLATE_DOCUMENTATION, wrap_string(documentation), NIL), NIL) : NIL))->concatenate(NIL, 0)))));
  }
}

List* compute_cl_struct_slots(Class* renamed_Class) {
  { Surrogate* value_000 = NULL;

    { Surrogate* super = NULL;
      Cons* iter_000 = class_native_supers(renamed_Class)->the_cons_list;

      while (!(iter_000 == NIL)) {
        super = ((Surrogate*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (!((Class*)(super->surrogate_value))->mixinP) {
          value_000 = super;
          break;
        }
      }
    }
    { Surrogate* realsuper = value_000;
      List* result = (((boolean)(realsuper)) ? compute_cl_struct_slots(((Class*)(realsuper->surrogate_value))) : ((List*)(NULL)));

      if (!(((boolean)(result)))) {
        result = new_list();
      }
      { Surrogate* super = NULL;
        Cons* iter_001 = class_native_supers(renamed_Class)->the_cons_list;

        while (!(iter_001 == NIL)) {
          super = ((Surrogate*)(iter_001->value));
          iter_001 = iter_001->rest;
          if (!(super == realsuper)) {
            { Slot* slot = NULL;
              Iterator* iter_002 = ((Class*)(super->surrogate_value))->class_slots();
              Cons* collect_000 = NULL;

              while (iter_002->nextP()) {
                slot = ((Slot*)(iter_002->value));
                if (storage_slotP(slot) &&
                    native_slotP(((Storage_Slot*)(slot)))) {
                  if (!((boolean)(collect_000))) {
                    {
                      collect_000 = cons(slot, NIL);
                      if (result->the_cons_list == NIL) {
                        result->the_cons_list = collect_000;
                      }
                      else {
                        add_cons_to_end_of_cons_list(result->the_cons_list, collect_000);
                      }
                    }
                  }
                  else {
                    {
                      collect_000->rest = cons(slot, NIL);
                      collect_000 = collect_000->rest;
                    }
                  }
                }
              }
            }
          }
        }
      }
      { Slot* slot = NULL;
        Cons* iter_003 = renamed_Class->class_local_slots->the_cons_list;
        Cons* collect_001 = NULL;

        while (!(iter_003 == NIL)) {
          slot = ((Slot*)(iter_003->value));
          iter_003 = iter_003->rest;
          if (storage_slotP(slot) &&
              native_slotP(((Storage_Slot*)(slot)))) {
            if (!((boolean)(collect_001))) {
              {
                collect_001 = cons(slot, NIL);
                if (result->the_cons_list == NIL) {
                  result->the_cons_list = collect_001;
                }
                else {
                  add_cons_to_end_of_cons_list(result->the_cons_list, collect_001);
                }
              }
            }
            else {
              {
                collect_001->rest = cons(slot, NIL);
                collect_001 = collect_001->rest;
              }
            }
          }
        }
      }
      return (result);
    }
  }
}

List* cl_struct_slots(Class* renamed_Class) {
  { List* slots = renamed_Class->class_cl_struct_slots_reader();

    if (defined_listP(slots)) {
      return (slots);
    }
    else {
      return (((List*)(set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CL_TRANSLATE_STELLA_CLASS_CL_STRUCT_SLOTS, compute_cl_struct_slots(renamed_Class), NULL))));
    }
  }
}

int cl_struct_slot_offset(Symbol* slotname, Surrogate* classtype) {
  { List* slots = cl_struct_slots(((Class*)(classtype->surrogate_value)));
    int offset = 1;

    { Storage_Slot* slot = NULL;
      Cons* iter_000 = slots->the_cons_list;

      while (!(iter_000 == NIL)) {
        slot = ((Storage_Slot*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (slot->slot_name == slotname) {
          return (offset);
        }
        else {
          offset = offset + 1;
        }
      }
    }
    { 
      BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
      signal_translation_error();
      if (!(suppress_warningsP())) {
        print_error_context(">> ERROR: ", STANDARD_ERROR);
        *(STANDARD_ERROR->native_stream) << endl << " " << "Can't find slot " << "`" << de_uglify_parse_tree(slotname) << "'" << " on CL-struct class " << "`" << de_uglify_parse_tree(classtype) << "'" << "." << endl;
      }
    }
    return (NULL_INTEGER);
  }
}

Cons* yield_struct_slot_trees(Class* renamed_Class) {
  { Cons* structslotdefs = NIL;

    { Storage_Slot* slot = NULL;
      Cons* iter_000 = cl_struct_slots(renamed_Class)->the_cons_list;
      Cons* collect_000 = NULL;

      while (!(iter_000 == NIL)) {
        slot = ((Storage_Slot*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (!(native_slot_home(slot, renamed_Class) == renamed_Class)) {
          if (((boolean)(slot->slot_base_type)) &&
              (!((boolean)(((Class*)(slot->slot_base_type->surrogate_value)))))) {
            *(STANDARD_WARNING->native_stream) << "Warning: " << "Can't generate a native slot named " << "`" << slot->slot_name << "'" << " for the class " << "`" << class_name(renamed_Class) << "'" << endl << " because the slot's :type " << "`" << slot->slot_base_type << "'" << " is undefined." << endl;
          }
          continue;
        }
        if (!((boolean)(collect_000))) {
          {
            collect_000 = cons(cons(slot->slot_name, cons(SYM_CL_TRANSLATE_STELLA_NULL, NIL)), NIL);
            if (structslotdefs == NIL) {
              structslotdefs = collect_000;
            }
            else {
              add_cons_to_end_of_cons_list(structslotdefs, collect_000);
            }
          }
        }
        else {
          {
            collect_000->rest = cons(cons(slot->slot_name, cons(SYM_CL_TRANSLATE_STELLA_NULL, NIL)), NIL);
            collect_000 = collect_000->rest;
          }
        }
      }
    }
    return (structslotdefs);
  }
}

Cons* yield_struct_tree(Class* renamed_Class) {
  { Surrogate* value_000 = NULL;

    { Surrogate* super = NULL;
      Cons* iter_000 = class_native_supers(renamed_Class)->the_cons_list;

      while (!(iter_000 == NIL)) {
        super = ((Surrogate*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (!((Class*)(super->surrogate_value))->mixinP) {
          value_000 = super;
          break;
        }
      }
    }
    { Surrogate* super = value_000;
      Object* translatedsuper = NULL;
      char* documentation = ((String_Wrapper*)(dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CL_TRANSLATE_STELLA_DOCUMENTATION, NULL_STRING_WRAPPER)))->wrapper_value;

      if (((boolean)(super))) {
        translatedsuper = cl_translate_global_symbol(intern_symbol_in_module(super->symbol_name, ((Module*)(super->home_context)), TRUE));
      }
      else {
        translatedsuper = cl_translate_global_symbol(SYM_CL_TRANSLATE_STELLA_CLSYS_ROOT_STRUCT);
      }
      return (listO(3, intern_common_lisp_symbol("DEFSTRUCT"), cons(cl_translate_global_symbol(class_symbol(renamed_Class)), cons(listO(3, KWD_CL_TRANSLATE_CONC_NAME, yield_struct_slot_accessor_name(renamed_Class->class_type, SYM_CL_TRANSLATE_STELLA_), NIL), ((((boolean)(translatedsuper)) ? cons(listO(3, KWD_CL_TRANSLATE_INCLUDE, translatedsuper, NIL), NIL) : NIL))->concatenate(listO(4, listO(3, KWD_CL_TRANSLATE_CONSTRUCTOR, yield_struct_constructor_name(renamed_Class), cons(NIL, NIL)), listO(3, KWD_CL_TRANSLATE_COPIER, NIL, NIL), listO(3, KWD_CL_TRANSLATE_PREDICATE, NIL, NIL), NIL), 0))), (((documentation != NULL) ? cons(wrap_string(documentation), NIL) : NIL))->concatenate(yield_struct_slot_trees(renamed_Class)->concatenate(NIL, 0), 0)));
    }
  }
}

Object* yield_vector_struct_name(Class* renamed_Class) {
  return (yield_global_lisp_symbol(((Module*)(renamed_Class->class_type->home_context)), string_concatenate("%", class_name(renamed_Class), 0)));
}

Cons* yield_vector_struct_tree(Class* renamed_Class) {
  { Surrogate* value_000 = NULL;

    { Surrogate* super = NULL;
      Cons* iter_000 = class_native_supers(renamed_Class)->the_cons_list;

      while (!(iter_000 == NIL)) {
        super = ((Surrogate*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (!((Class*)(super->surrogate_value))->mixinP) {
          value_000 = super;
          break;
        }
      }
    }
    { Surrogate* super = value_000;
      char* documentation = ((String_Wrapper*)(dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CL_TRANSLATE_STELLA_DOCUMENTATION, NULL_STRING_WRAPPER)))->wrapper_value;

      return (listO(3, intern_common_lisp_symbol("DEFCLASS"), cl_translate_global_symbol(class_symbol(renamed_Class)), cons((((boolean)(super)) ? cons(cl_translate_global_symbol(intern_symbol_in_module(super->symbol_name, ((Module*)(super->home_context)), TRUE)), NIL) : NIL), cons(NIL, (((documentation != NULL) ? cons(listO(3, KWD_CL_TRANSLATE_DOCUMENTATION, wrap_string(documentation), NIL), NIL) : NIL))->concatenate(NIL, 0)))));
    }
  }
}

Cons* yield_condition_tree(Class* renamed_Class) {
  { Cons* defn = yield_clos_class_tree(renamed_Class);

    defn->first_setter(intern_common_lisp_symbol("DEFINE-CONDITION"));
    if (((boolean)(defn->rest->rest->rest->rest))) {
      defn->rest->rest->rest->rest = NIL;
    }
    return (defn);
  }
}

boolean cl_always_translate_to_clos_classP(Class* renamed_Class) {
  { boolean test_value_000 = FALSE;

    test_value_000 = !subtype_ofP(renamed_Class->class_type, SGT_CL_TRANSLATE_STELLA_OBJECT);
    if (test_value_000) {
      test_value_000 = create_native_classP(renamed_Class);
      if (test_value_000) {
        { boolean foundP_000 = FALSE;

          { Class* super = NULL;
            Cons* iter_000 = renamed_Class->class_all_super_classes;

            while (!(iter_000 == NIL)) {
              super = ((Class*)(iter_000->value));
              iter_000 = iter_000->rest;
              if (!create_native_classP(super)) {
                foundP_000 = TRUE;
                break;
              }
            }
          }
          test_value_000 = foundP_000;
        }
      }
    }
    { boolean value_000 = test_value_000;

      return (value_000);
    }
  }
}

Cons* cl_translate_define_native_class_unit(Translation_Unit* unit) {
  { Class* renamed_Class = ((Class*)(unit->the_object));

    if (exception_classP(renamed_Class)) {
      return (yield_condition_tree(renamed_Class));
    }
    else if ((!oCURRENT_ENVIRONMENT_FEATURESo->membP(KWD_CL_TRANSLATE_USE_COMMON_LISP_STRUCTS)) ||
        cl_always_translate_to_clos_classP(renamed_Class)) {
      return (yield_clos_class_tree(renamed_Class));
    }
    else if (oCURRENT_ENVIRONMENT_FEATURESo->membP(KWD_CL_TRANSLATE_USE_COMMON_LISP_VECTOR_STRUCTS)) {
      return (yield_vector_struct_tree(renamed_Class));
    }
    else {
      return (yield_struct_tree(renamed_Class));
    }
  }
}

Cons* cl_translate_define_global_variable_unit(Translation_Unit* unit) {
  { Global_Variable* global = ((Global_Variable*)(unit->the_object));
    Object* name = cl_translate_global_symbol(global->variable_name);
    char* documentation = global->documentation;
    boolean unboundspecialP = global->variable_specialP &&
        (unit->code_register == KWD_CL_TRANSLATE_UNBOUND_SPECIAL_VARIABLE);
    Object* initialvaluetree = cl_translate_a_tree(unit->code_register);
    Cons* typedeclarationtree = (generate_cl_type_declarationsP() ? cl_yield_global_variable_type_declaration_tree(global) : ((Cons*)(NULL)));
    Symbol* renamed_Operator = intern_common_lisp_symbol("DEFVAR");
    Cons* otree = NIL;

    if (global->variable_constantP &&
        FALSE) {
      renamed_Operator = intern_common_lisp_symbol("DEFCONSTANT");
    }
    otree = (unboundspecialP ? listO(3, intern_common_lisp_symbol("DEFVAR"), name, NIL) : cons(renamed_Operator, cons(name, cons(initialvaluetree, (((documentation != NULL) ? cons(wrap_string(documentation), NIL) : NIL))->concatenate(NIL, 0)))));
    if (((boolean)(typedeclarationtree))) {
      otree = listO(3, intern_common_lisp_symbol("PROGN"), otree, cons(listO(3, intern_common_lisp_symbol("DECLAIM"), typedeclarationtree, NIL), NIL));
    }
    return (otree);
  }
}

Cons* cl_translate_defprint_unit(Translation_Unit* unit) {
  { Class* renamed_Class = ((Class*)(unit->the_object));
    Object* body = unit->code_register;
    Cons* extratrees = NIL;

    if (method_call_type_for_vector_structs(SYM_CL_TRANSLATE_STELLA_PRINT_OBJECT, renamed_Class->class_type, FALSE) == SYM_CL_TRANSLATE_STELLA_PRINT_OBJECT) {
      extratrees = cons(listO(4, SYM_CL_TRANSLATE_STELLA_SETQ, SYM_CL_TRANSLATE_STELLA_SELF, cl_translate_global_symbol(SYM_CL_TRANSLATE_STELLA_oCLSYS_SELFo), NIL), NIL);
    }
    else {
    }
    return (listO(4, intern_common_lisp_symbol("DEFMETHOD"), intern_common_lisp_symbol("PRINT-OBJECT"), listO(3, listO(3, SYM_CL_TRANSLATE_STELLA_SELF, class_symbol(renamed_Class), NIL), SYM_CL_TRANSLATE_STELLA_STREAM, NIL), extratrees->concatenate(cons(cl_translate_a_tree(body), NIL), 0)));
  }
}

void help_startup_cl_translate1() {
  {
    SYM_CL_TRANSLATE_STELLA_SYS_CALL_FUNCTION = ((Symbol*)(intern_rigid_symbol_wrt_module("SYS-CALL-FUNCTION", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_SYS_CALL_METHOD = ((Symbol*)(intern_rigid_symbol_wrt_module("SYS-CALL-METHOD", NULL, 0)));
    SGT_CL_TRANSLATE_STELLA_KEY_VALUE_LIST = ((Surrogate*)(intern_rigid_symbol_wrt_module("KEY-VALUE-LIST", NULL, 1)));
    SYM_CL_TRANSLATE_STELLA_i = ((Symbol*)(intern_rigid_symbol_wrt_module("+", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA__ = ((Symbol*)(intern_rigid_symbol_wrt_module("-", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_o = ((Symbol*)(intern_rigid_symbol_wrt_module("*", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_s = ((Symbol*)(intern_rigid_symbol_wrt_module("/", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_EQp = ((Symbol*)(intern_rigid_symbol_wrt_module("EQ?", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_l = ((Symbol*)(intern_rigid_symbol_wrt_module("<", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_le = ((Symbol*)(intern_rigid_symbol_wrt_module("<=", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_ge = ((Symbol*)(intern_rigid_symbol_wrt_module(">=", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_g = ((Symbol*)(intern_rigid_symbol_wrt_module(">", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_e = ((Symbol*)(intern_rigid_symbol_wrt_module("=", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_xe = ((Symbol*)(intern_rigid_symbol_wrt_module("!=", NULL, 0)));
    KWD_CL_TRANSLATE_USE_COMMON_LISP_CONSES = ((Keyword*)(intern_rigid_symbol_wrt_module("USE-COMMON-LISP-CONSES", NULL, 2)));
    SGT_CL_TRANSLATE_STELLA_CONS = ((Surrogate*)(intern_rigid_symbol_wrt_module("CONS", NULL, 1)));
    SGT_CL_TRANSLATE_STELLA_STANDARD_OBJECT = ((Surrogate*)(intern_rigid_symbol_wrt_module("STANDARD-OBJECT", NULL, 1)));
    SGT_CL_TRANSLATE_STELLA_OBJECT = ((Surrogate*)(intern_rigid_symbol_wrt_module("OBJECT", NULL, 1)));
    KWD_CL_TRANSLATE_USE_COMMON_LISP_STRUCTS = ((Keyword*)(intern_rigid_symbol_wrt_module("USE-COMMON-LISP-STRUCTS", NULL, 2)));
    KWD_CL_TRANSLATE_USE_COMMON_LISP_VECTOR_STRUCTS = ((Keyword*)(intern_rigid_symbol_wrt_module("USE-COMMON-LISP-VECTOR-STRUCTS", NULL, 2)));
    SGT_CL_TRANSLATE_STELLA_VOID = ((Surrogate*)(intern_rigid_symbol_wrt_module("VOID", NULL, 1)));
    SYM_CL_TRANSLATE_STELLA_VOID_SYS = ((Symbol*)(intern_rigid_symbol_wrt_module("VOID-SYS", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_TYPED_SYS = ((Symbol*)(intern_rigid_symbol_wrt_module("TYPED-SYS", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_BAD_SYS = ((Symbol*)(intern_rigid_symbol_wrt_module("BAD-SYS", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_SETQ = ((Symbol*)(intern_rigid_symbol_wrt_module("SETQ", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_SYS_SLOT_VALUE = ((Symbol*)(intern_rigid_symbol_wrt_module("SYS-SLOT-VALUE", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_SYS_SLOT_VALUE_SETTER = ((Symbol*)(intern_rigid_symbol_wrt_module("SYS-SLOT-VALUE-SETTER", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_PROGN = ((Symbol*)(intern_rigid_symbol_wrt_module("PROGN", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_WITH_PROCESS_LOCK = ((Symbol*)(intern_rigid_symbol_wrt_module("WITH-PROCESS-LOCK", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_LOOP = ((Symbol*)(intern_rigid_symbol_wrt_module("LOOP", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_SYS_FOREACH = ((Symbol*)(intern_rigid_symbol_wrt_module("SYS-FOREACH", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_LET = ((Symbol*)(intern_rigid_symbol_wrt_module("LET", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_STARTUP_TIME_PROGN = ((Symbol*)(intern_rigid_symbol_wrt_module("STARTUP-TIME-PROGN", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_SPECIAL = ((Symbol*)(intern_rigid_symbol_wrt_module("SPECIAL", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_SYS_UNWIND_PROTECT = ((Symbol*)(intern_rigid_symbol_wrt_module("SYS-UNWIND-PROTECT", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_SYS_HANDLER_CASE = ((Symbol*)(intern_rigid_symbol_wrt_module("SYS-HANDLER-CASE", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_SYS_HANDLE_EXCEPTION = ((Symbol*)(intern_rigid_symbol_wrt_module("SYS-HANDLE-EXCEPTION", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_SYS_SIGNAL = ((Symbol*)(intern_rigid_symbol_wrt_module("SYS-SIGNAL", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_CAST = ((Symbol*)(intern_rigid_symbol_wrt_module("CAST", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_SAFE_CAST = ((Symbol*)(intern_rigid_symbol_wrt_module("SAFE-CAST", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_RETURN = ((Symbol*)(intern_rigid_symbol_wrt_module("RETURN", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_BREAK = ((Symbol*)(intern_rigid_symbol_wrt_module("BREAK", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_CONTINUE = ((Symbol*)(intern_rigid_symbol_wrt_module("CONTINUE", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_WHILE = ((Symbol*)(intern_rigid_symbol_wrt_module("WHILE", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_IF = ((Symbol*)(intern_rigid_symbol_wrt_module("IF", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_CHOOSE = ((Symbol*)(intern_rigid_symbol_wrt_module("CHOOSE", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_WHEN = ((Symbol*)(intern_rigid_symbol_wrt_module("WHEN", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_UNLESS = ((Symbol*)(intern_rigid_symbol_wrt_module("UNLESS", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_AND = ((Symbol*)(intern_rigid_symbol_wrt_module("AND", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_OR = ((Symbol*)(intern_rigid_symbol_wrt_module("OR", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_NOT = ((Symbol*)(intern_rigid_symbol_wrt_module("NOT", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_COND = ((Symbol*)(intern_rigid_symbol_wrt_module("COND", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_CASE = ((Symbol*)(intern_rigid_symbol_wrt_module("CASE", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_MAKE = ((Symbol*)(intern_rigid_symbol_wrt_module("MAKE", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_SYS_NEW = ((Symbol*)(intern_rigid_symbol_wrt_module("SYS-NEW", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_MV_SETQ = ((Symbol*)(intern_rigid_symbol_wrt_module("MV-SETQ", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_QUOTE = ((Symbol*)(intern_rigid_symbol_wrt_module("QUOTE", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_SYS_CALL_FUNCTION_CODE = ((Symbol*)(intern_rigid_symbol_wrt_module("SYS-CALL-FUNCTION-CODE", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_SYS_CALL_METHOD_CODE = ((Symbol*)(intern_rigid_symbol_wrt_module("SYS-CALL-METHOD-CODE", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_SYS_INLINE_CALL = ((Symbol*)(intern_rigid_symbol_wrt_module("SYS-INLINE-CALL", NULL, 0)));
  }
}

void help_startup_cl_translate2() {
  {
    SYM_CL_TRANSLATE_STELLA_THE_CODE = ((Symbol*)(intern_rigid_symbol_wrt_module("THE-CODE", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_PRINT_STREAM = ((Symbol*)(intern_rigid_symbol_wrt_module("PRINT-STREAM", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_PRINT_NATIVE_STREAM = ((Symbol*)(intern_rigid_symbol_wrt_module("PRINT-NATIVE-STREAM", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_VERBATIM = ((Symbol*)(intern_rigid_symbol_wrt_module("VERBATIM", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_INLINE = ((Symbol*)(intern_rigid_symbol_wrt_module("INLINE", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_IGNORE = ((Symbol*)(intern_rigid_symbol_wrt_module("IGNORE", NULL, 0)));
    KWD_CL_TRANSLATE_CONTINUE = ((Keyword*)(intern_rigid_symbol_wrt_module("CONTINUE", NULL, 2)));
    SYM_CL_TRANSLATE_STELLA_GET_KWD = ((Symbol*)(intern_rigid_symbol_wrt_module("GET-KWD", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_GET_SGT = ((Symbol*)(intern_rigid_symbol_wrt_module("GET-SGT", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_MODULE_LISP_PACKAGE = ((Symbol*)(intern_rigid_symbol_wrt_module("MODULE-LISP-PACKAGE", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_VALUE = ((Symbol*)(intern_rigid_symbol_wrt_module("VALUE", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_REST = ((Symbol*)(intern_rigid_symbol_wrt_module("REST", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_rrVALUE = ((Symbol*)(intern_rigid_symbol_wrt_module("%%VALUE", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_rrREST = ((Symbol*)(intern_rigid_symbol_wrt_module("%%REST", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_CLSYS_SVAL = ((Symbol*)(intern_rigid_symbol_wrt_module("CLSYS-SVAL", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_NTH = ((Symbol*)(intern_rigid_symbol_wrt_module("NTH", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_NTH_SETTER = ((Symbol*)(intern_rigid_symbol_wrt_module("NTH-SETTER", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_AREF = ((Symbol*)(intern_rigid_symbol_wrt_module("AREF", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_AREF_SETTER = ((Symbol*)(intern_rigid_symbol_wrt_module("AREF-SETTER", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_NULLp = ((Symbol*)(intern_rigid_symbol_wrt_module("NULL?", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_DEFINEDp = ((Symbol*)(intern_rigid_symbol_wrt_module("DEFINED?", NULL, 0)));
    SGT_CL_TRANSLATE_STELLA_ARGUMENT_LIST = ((Surrogate*)(intern_rigid_symbol_wrt_module("ARGUMENT-LIST", NULL, 1)));
    SGT_CL_TRANSLATE_STELLA_ARGUMENT_LIST_ITERATOR = ((Surrogate*)(intern_rigid_symbol_wrt_module("ARGUMENT-LIST-ITERATOR", NULL, 1)));
    SYM_CL_TRANSLATE_STELLA_NORMAL_CALL = ((Symbol*)(intern_rigid_symbol_wrt_module("NORMAL-CALL", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_NON_OBJECT_METHOD = ((Symbol*)(intern_rigid_symbol_wrt_module("NON-OBJECT-METHOD", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_PRINT_OBJECT = ((Symbol*)(intern_rigid_symbol_wrt_module("PRINT-OBJECT", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_OBJECT_METHOD = ((Symbol*)(intern_rigid_symbol_wrt_module("OBJECT-METHOD", NULL, 0)));
    KWD_CL_TRANSLATE_COMMON_LISP = ((Keyword*)(intern_rigid_symbol_wrt_module("COMMON-LISP", NULL, 2)));
    KWD_CL_TRANSLATE_FUNCTION = ((Keyword*)(intern_rigid_symbol_wrt_module("FUNCTION", NULL, 2)));
    SYM_CL_TRANSLATE_STELLA_CLSYS_CALL = ((Symbol*)(intern_rigid_symbol_wrt_module("CLSYS-CALL", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_NULL = ((Symbol*)(intern_rigid_symbol_wrt_module("NULL", NULL, 0)));
    SGT_CL_TRANSLATE_STELLA_STRING = ((Surrogate*)(intern_rigid_symbol_wrt_module("STRING", NULL, 1)));
    SGT_CL_TRANSLATE_STELLA_MUTABLE_STRING = ((Surrogate*)(intern_rigid_symbol_wrt_module("MUTABLE-STRING", NULL, 1)));
    SGT_CL_TRANSLATE_STELLA_NATIVE_VECTOR = ((Surrogate*)(intern_rigid_symbol_wrt_module("NATIVE-VECTOR", NULL, 1)));
    SGT_CL_TRANSLATE_STELLA_ARRAY = ((Surrogate*)(intern_rigid_symbol_wrt_module("ARRAY", NULL, 1)));
    SGT_CL_TRANSLATE_STELLA_INTEGER = ((Surrogate*)(intern_rigid_symbol_wrt_module("INTEGER", NULL, 1)));
    SYM_CL_TRANSLATE_STELLA_ALLOCATE_ITERATOR = ((Symbol*)(intern_rigid_symbol_wrt_module("ALLOCATE-ITERATOR", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_LENGTH = ((Symbol*)(intern_rigid_symbol_wrt_module("LENGTH", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_NEXTp = ((Symbol*)(intern_rigid_symbol_wrt_module("NEXT?", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_ARGUMENT = ((Symbol*)(intern_rigid_symbol_wrt_module("ARGUMENT", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_DO = ((Symbol*)(intern_rigid_symbol_wrt_module("DO", NULL, 0)));
    SGT_CL_TRANSLATE_STELLA_FLOAT = ((Surrogate*)(intern_rigid_symbol_wrt_module("FLOAT", NULL, 1)));
    SGT_CL_TRANSLATE_STELLA_INTEGER_WRAPPER = ((Surrogate*)(intern_rigid_symbol_wrt_module("INTEGER-WRAPPER", NULL, 1)));
    SGT_CL_TRANSLATE_STELLA_FLOAT_WRAPPER = ((Surrogate*)(intern_rigid_symbol_wrt_module("FLOAT-WRAPPER", NULL, 1)));
    SYM_CL_TRANSLATE_STELLA_PRIMARY_TYPE = ((Symbol*)(intern_rigid_symbol_wrt_module("PRIMARY-TYPE", NULL, 0)));
    SGT_CL_TRANSLATE_STELLA_LITERAL = ((Surrogate*)(intern_rigid_symbol_wrt_module("LITERAL", NULL, 1)));
    SGT_CL_TRANSLATE_STELLA_BOOLEAN = ((Surrogate*)(intern_rigid_symbol_wrt_module("BOOLEAN", NULL, 1)));
    SGT_CL_TRANSLATE_STELLA_NUMBER = ((Surrogate*)(intern_rigid_symbol_wrt_module("NUMBER", NULL, 1)));
    SGT_CL_TRANSLATE_STELLA_SECOND_CLASS_OBJECT = ((Surrogate*)(intern_rigid_symbol_wrt_module("SECOND-CLASS-OBJECT", NULL, 1)));
    SGT_CL_TRANSLATE_STELLA_CODE = ((Surrogate*)(intern_rigid_symbol_wrt_module("CODE", NULL, 1)));
    SYM_CL_TRANSLATE_STELLA_OTHERWISE = ((Symbol*)(intern_rigid_symbol_wrt_module("OTHERWISE", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_CLASS_CL_NATIVE_TYPE = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-CL-NATIVE-TYPE", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_oCONDITION_MESSAGE_KEYWORDo = ((Symbol*)(intern_rigid_symbol_wrt_module("*CONDITION-MESSAGE-KEYWORD*", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_CLSYS_MAKE = ((Symbol*)(intern_rigid_symbol_wrt_module("CLSYS-MAKE", NULL, 0)));
    KWD_CL_TRANSLATE_INITIAL_ELEMENT = ((Keyword*)(intern_rigid_symbol_wrt_module("INITIAL-ELEMENT", NULL, 2)));
    SYM_CL_TRANSLATE_STELLA_rrPRINT_STREAM = ((Symbol*)(intern_rigid_symbol_wrt_module("%%PRINT-STREAM", NULL, 0)));
    KWD_CL_TRANSLATE_WARN = ((Keyword*)(intern_rigid_symbol_wrt_module("WARN", NULL, 2)));
    KWD_CL_TRANSLATE_ERROR = ((Keyword*)(intern_rigid_symbol_wrt_module("ERROR", NULL, 2)));
    KWD_CL_TRANSLATE_CONTINUABLE_ERROR = ((Keyword*)(intern_rigid_symbol_wrt_module("CONTINUABLE-ERROR", NULL, 2)));
    SYM_CL_TRANSLATE_STELLA_CLSYS_METHOD_CODE_CALL = ((Symbol*)(intern_rigid_symbol_wrt_module("CLSYS-METHOD-CODE-CALL", NULL, 0)));
  }
}

void help_startup_cl_translate3() {
  {
    SGT_CL_TRANSLATE_STELLA_SHORT_INTEGER = ((Surrogate*)(intern_rigid_symbol_wrt_module("SHORT-INTEGER", NULL, 1)));
    SYM_CL_TRANSLATE_STELLA_GLOBAL_VARIABLE = ((Symbol*)(intern_rigid_symbol_wrt_module("GLOBAL-VARIABLE", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_CLASS = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_METHOD = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_MACRO = ((Symbol*)(intern_rigid_symbol_wrt_module("MACRO", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_PRINT_METHOD = ((Symbol*)(intern_rigid_symbol_wrt_module("PRINT-METHOD", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_TYPE = ((Symbol*)(intern_rigid_symbol_wrt_module("TYPE", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_CLSYS_SELF = ((Symbol*)(intern_rigid_symbol_wrt_module("CLSYS-SELF", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_CLSYS_DUMMY = ((Symbol*)(intern_rigid_symbol_wrt_module("CLSYS-DUMMY", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_METHOD_VARIABLE_ARGUMENTSp = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-VARIABLE-ARGUMENTS?", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_DOCUMENTATION = ((Symbol*)(intern_rigid_symbol_wrt_module("DOCUMENTATION", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_SELF = ((Symbol*)(intern_rigid_symbol_wrt_module("SELF", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_oCLSYS_SELFo = ((Symbol*)(intern_rigid_symbol_wrt_module("*CLSYS-SELF*", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_rrDEFCONSMETHOD = ((Symbol*)(intern_rigid_symbol_wrt_module("%%DEFCONSMETHOD", NULL, 0)));
    KWD_CL_TRANSLATE_VOID = ((Keyword*)(intern_rigid_symbol_wrt_module("VOID", NULL, 2)));
    SYM_CL_TRANSLATE_STELLA_SLOT_AUXILIARYp = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-AUXILIARY?", NULL, 0)));
    KWD_CL_TRANSLATE_EMBEDDED = ((Keyword*)(intern_rigid_symbol_wrt_module("EMBEDDED", NULL, 2)));
    KWD_CL_TRANSLATE_INSTANCE = ((Keyword*)(intern_rigid_symbol_wrt_module("INSTANCE", NULL, 2)));
    KWD_CL_TRANSLATE_ALLOCATION = ((Keyword*)(intern_rigid_symbol_wrt_module("ALLOCATION", NULL, 2)));
    KWD_CL_TRANSLATE_DOCUMENTATION = ((Keyword*)(intern_rigid_symbol_wrt_module("DOCUMENTATION", NULL, 2)));
    KWD_CL_TRANSLATE_ACCESSOR = ((Keyword*)(intern_rigid_symbol_wrt_module("ACCESSOR", NULL, 2)));
    SYM_CL_TRANSLATE_STELLA_CLSYS_ROOT_OBJECT = ((Symbol*)(intern_rigid_symbol_wrt_module("CLSYS-ROOT-OBJECT", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_CLASS_CL_STRUCT_SLOTS = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-CL-STRUCT-SLOTS", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_CLSYS_ROOT_STRUCT = ((Symbol*)(intern_rigid_symbol_wrt_module("CLSYS-ROOT-STRUCT", NULL, 0)));
    KWD_CL_TRANSLATE_CONC_NAME = ((Keyword*)(intern_rigid_symbol_wrt_module("CONC-NAME", NULL, 2)));
    KWD_CL_TRANSLATE_CONSTRUCTOR = ((Keyword*)(intern_rigid_symbol_wrt_module("CONSTRUCTOR", NULL, 2)));
    KWD_CL_TRANSLATE_COPIER = ((Keyword*)(intern_rigid_symbol_wrt_module("COPIER", NULL, 2)));
    KWD_CL_TRANSLATE_PREDICATE = ((Keyword*)(intern_rigid_symbol_wrt_module("PREDICATE", NULL, 2)));
    SYM_CL_TRANSLATE_STELLA_ = ((Symbol*)(intern_rigid_symbol_wrt_module("", NULL, 0)));
    KWD_CL_TRANSLATE_INCLUDE = ((Keyword*)(intern_rigid_symbol_wrt_module("INCLUDE", NULL, 2)));
    KWD_CL_TRANSLATE_UNBOUND_SPECIAL_VARIABLE = ((Keyword*)(intern_rigid_symbol_wrt_module("UNBOUND-SPECIAL-VARIABLE", NULL, 2)));
    SYM_CL_TRANSLATE_STELLA_STREAM = ((Symbol*)(intern_rigid_symbol_wrt_module("STREAM", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_STARTUP_CL_TRANSLATE = ((Symbol*)(intern_rigid_symbol_wrt_module("STARTUP-CL-TRANSLATE", NULL, 0)));
    SYM_CL_TRANSLATE_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-STARTUP-CLASSNAME", NULL, 0)));
  }
}

void help_startup_cl_translate4() {
  {
    define_function_object("DESTRUCTURE-CALL-TREE", "(DEFUN (DESTRUCTURE-CALL-TREE SYMBOL TYPE CONS) ((TREE CONS)))", ((cpp_function_code)(&destructure_call_tree)), NULL);
    define_function_object("USE-CL-CONSES?", "(DEFUN (USE-CL-CONSES? BOOLEAN) () :GLOBALLY-INLINE? TRUE (RETURN (MEMB? *CURRENT-ENVIRONMENT-FEATURES* :USE-COMMON-LISP-CONSES)))", ((cpp_function_code)(&use_cl_consesP)), NULL);
    define_function_object("USE-DEFCONSMETHOD?", "(DEFUN (USE-DEFCONSMETHOD? BOOLEAN) ((METHOD METHOD-SLOT)))", ((cpp_function_code)(&use_defconsmethodP)), NULL);
    define_function_object("USE-CL-STRUCTS?", "(DEFUN (USE-CL-STRUCTS? BOOLEAN) () :GLOBALLY-INLINE? TRUE (RETURN (MEMB? *CURRENT-ENVIRONMENT-FEATURES* :USE-COMMON-LISP-STRUCTS)))", ((cpp_function_code)(&use_cl_structsP)), NULL);
    define_function_object("USE-VECTOR-STRUCTS?", "(DEFUN (USE-VECTOR-STRUCTS? BOOLEAN) () :GLOBALLY-INLINE? TRUE (RETURN (MEMB? *CURRENT-ENVIRONMENT-FEATURES* :USE-COMMON-LISP-VECTOR-STRUCTS)))", ((cpp_function_code)(&use_vector_structsP)), NULL);
    define_function_object("CL-TRANSLATE-LIST-OF-TREES", "(DEFUN (CL-TRANSLATE-LIST-OF-TREES CONS) ((TREES CONS)))", ((cpp_function_code)(&cl_translate_list_of_trees)), NULL);
    define_function_object("CL-TRANSLATE-VOID-SYS", "(DEFUN (CL-TRANSLATE-VOID-SYS CONS) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_void_sys)), NULL);
    define_function_object("CL-TRANSLATE-TYPED-SYS", "(DEFUN (CL-TRANSLATE-TYPED-SYS OBJECT) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_typed_sys)), NULL);
    define_function_object("CL-TRANSLATE-BAD-SYS", "(DEFUN (CL-TRANSLATE-BAD-SYS OBJECT) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_bad_sys)), NULL);
    define_function_object("CL-TRANSLATE-A-TREE", "(DEFUN (CL-TRANSLATE-A-TREE OBJECT) ((TREE OBJECT)))", ((cpp_function_code)(&cl_translate_a_tree)), NULL);
    define_function_object("HELP-CL-TRANSLATE-A-TREE", "(DEFUN (HELP-CL-TRANSLATE-A-TREE OBJECT) ((TREE OBJECT)))", ((cpp_function_code)(&help_cl_translate_a_tree)), NULL);
    define_method_object("(DEFMETHOD (CL-TRANSLATE-ATOMIC-TREE OBJECT) ((TREE OBJECT)))", ((cpp_method_code)(&Object::cl_translate_atomic_tree)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (CL-TRANSLATE-ATOMIC-TREE OBJECT) ((TREE FLOAT-WRAPPER)))", ((cpp_method_code)(&Float_Wrapper::cl_translate_atomic_tree)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (CL-TRANSLATE-ATOMIC-TREE OBJECT) ((TREE SYMBOL)))", ((cpp_method_code)(&Symbol::cl_translate_atomic_tree)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (CL-TRANSLATE-ATOMIC-TREE OBJECT) ((TREE KEYWORD)))", ((cpp_method_code)(&Keyword::cl_translate_atomic_tree)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (CL-TRANSLATE-ATOMIC-TREE OBJECT) ((TREE SURROGATE)))", ((cpp_method_code)(&Surrogate::cl_translate_atomic_tree)), ((cpp_method_code)(NULL)));
    define_function_object("CL-TRANSLATE-GLOBAL-SYMBOL", "(DEFUN (CL-TRANSLATE-GLOBAL-SYMBOL OBJECT) ((SYMBOL SYMBOL)))", ((cpp_function_code)(&cl_translate_global_symbol)), NULL);
    define_function_object("YIELD-GLOBAL-LISP-SYMBOL", "(DEFUN (YIELD-GLOBAL-LISP-SYMBOL OBJECT) ((SYMBOLMODULE MODULE) (SYMBOLNAME STRING)))", ((cpp_function_code)(&yield_global_lisp_symbol)), NULL);
    define_function_object("CL-TRANSLATE-LOCAL-SYMBOL", "(DEFUN (CL-TRANSLATE-LOCAL-SYMBOL OBJECT) ((SYMBOL SYMBOL)))", ((cpp_function_code)(&cl_translate_local_symbol)), NULL);
    define_function_object("CL-TRANSLATE-SETQ-TREE", "(DEFUN (CL-TRANSLATE-SETQ-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_setq_tree)), NULL);
    define_function_object("YIELD-STRUCT-SLOT-ACCESSOR-NAME", "(DEFUN (YIELD-STRUCT-SLOT-ACCESSOR-NAME OBJECT) ((CLASSTYPE TYPE) (SLOTNAME SYMBOL)))", ((cpp_function_code)(&yield_struct_slot_accessor_name)), NULL);
    define_function_object("YIELD-CLOS-SLOT-ACCESSOR-NAME", "(DEFUN (YIELD-CLOS-SLOT-ACCESSOR-NAME OBJECT) ((SLOTNAME SYMBOL)))", ((cpp_function_code)(&yield_clos_slot_accessor_name)), NULL);
    define_function_object("YIELD-SLOT-VALUE-READER-TREE", "(DEFUN (YIELD-SLOT-VALUE-READER-TREE CONS) ((SLOTNAME SYMBOL) (OBJECTREF OBJECT) (OBJECTTYPE TYPE)))", ((cpp_function_code)(&yield_slot_value_reader_tree)), NULL);
    define_function_object("YIELD-SLOT-VALUE-WRITER-TREE", "(DEFUN (YIELD-SLOT-VALUE-WRITER-TREE CONS) ((SLOTNAME SYMBOL) (OBJECTREF OBJECT) (OBJECTTYPE TYPE) (VALUEREF OBJECT)))", ((cpp_function_code)(&yield_slot_value_writer_tree)), NULL);
    define_function_object("TRUE-SLOT-NAME", "(DEFUN (TRUE-SLOT-NAME SYMBOL) ((SLOTNAME SYMBOL) (TYPE TYPE)))", ((cpp_function_code)(&true_slot_name)), NULL);
    define_function_object("CL-TRANSLATE-FIXED-SLOT-VALUE-TREE", "(DEFUN (CL-TRANSLATE-FIXED-SLOT-VALUE-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_fixed_slot_value_tree)), NULL);
    define_function_object("CL-TRANSLATE-FIXED-SLOT-VALUE-SETTER-TREE", "(DEFUN (CL-TRANSLATE-FIXED-SLOT-VALUE-SETTER-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_fixed_slot_value_setter_tree)), NULL);
    define_function_object("CL-TRANSLATE-CALL-METHOD-TREE", "(DEFUN (CL-TRANSLATE-CALL-METHOD-TREE OBJECT) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_call_method_tree)), NULL);
    define_function_object("METHOD-CALL-TYPE-FOR-VECTOR-STRUCTS", "(DEFUN (METHOD-CALL-TYPE-FOR-VECTOR-STRUCTS SYMBOL) ((SLOTNAME SYMBOL) (OWNER TYPE) (FUNCTION? BOOLEAN)))", ((cpp_function_code)(&method_call_type_for_vector_structs)), NULL);
    define_function_object("CL-TRANSLATE-NORMAL-CALL-METHOD-TREE", "(DEFUN (CL-TRANSLATE-NORMAL-CALL-METHOD-TREE CONS) ((OPERATOR SYMBOL) (OWNER TYPE) (ARGUMENTS CONS)))", ((cpp_function_code)(&cl_translate_normal_call_method_tree)), NULL);
    define_function_object("CL-TRANSLATE-NTH-TREE", "(DEFUN (CL-TRANSLATE-NTH-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_nth_tree)), NULL);
    define_function_object("CL-TRANSLATE-AREF-TREE", "(DEFUN (CL-TRANSLATE-AREF-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_aref_tree)), NULL);
    define_function_object("CL-TRANSLATE-ARGUMENT-LIST-TREE", "(DEFUN (CL-TRANSLATE-ARGUMENT-LIST-TREE OBJECT) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_argument_list_tree)), NULL);
    define_function_object("DUPLICATE-CURSOR-VARIABLE?", "(DEFUN (DUPLICATE-CURSOR-VARIABLE? BOOLEAN) ((VARIABLE SYMBOL) (ITERATORBINDINGS CONS)))", ((cpp_function_code)(&duplicate_cursor_variableP)), NULL);
    define_function_object("CL-TRANSLATE-FOREACH-TREE", "(DEFUN (CL-TRANSLATE-FOREACH-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_foreach_tree)), NULL);
    define_function_object("MAYBE-WRAP-WITH-CONTINUE-LABEL", "(DEFUN (MAYBE-WRAP-WITH-CONTINUE-LABEL CONS) ((TRANSLATEDLOOPBODY CONS)))", ((cpp_function_code)(&maybe_wrap_with_continue_label)), NULL);
    define_function_object("CL-TRANSLATE-LET-TREE", "(DEFUN (CL-TRANSLATE-LET-TREE CONS) ((TREE CONS) (SPECIAL? BOOLEAN)))", ((cpp_function_code)(&cl_translate_let_tree)), NULL);
    define_function_object("CL-TRANSLATE-SPECIAL-TREE", "(DEFUN (CL-TRANSLATE-SPECIAL-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_special_tree)), NULL);
    define_function_object("CL-TRANSLATE-STARTUP-TIME-PROGN-TREE", "(DEFUN (CL-TRANSLATE-STARTUP-TIME-PROGN-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_startup_time_progn_tree)), NULL);
    define_function_object("CL-TRANSLATE-UNWIND-PROTECT-TREE", "(DEFUN (CL-TRANSLATE-UNWIND-PROTECT-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_unwind_protect_tree)), NULL);
    define_function_object("CL-TRANSLATE-SIGNAL-TREE", "(DEFUN (CL-TRANSLATE-SIGNAL-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_signal_tree)), NULL);
    define_function_object("CL-TRANSLATE-HANDLER-CASE-TREE", "(DEFUN (CL-TRANSLATE-HANDLER-CASE-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_handler_case_tree)), NULL);
    define_function_object("CL-TRANSLATE-HANDLE-EXCEPTION-TREE", "(DEFUN (CL-TRANSLATE-HANDLE-EXCEPTION-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_handle_exception_tree)), NULL);
    define_function_object("CAST", "(DEFUN (CAST OBJECT) ((VALUE OBJECT) (TYPE TYPE)) :DOCUMENTATION \"Perform a run-time type check, and then return `value'.\")", ((cpp_function_code)(&cast)), NULL);
    define_function_object("CL-TRANSLATE-CAST-TREE", "(DEFUN (CL-TRANSLATE-CAST-TREE OBJECT) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_cast_tree)), NULL);
    define_function_object("CL-TRANSLATE-RETURN-TREE", "(DEFUN (CL-TRANSLATE-RETURN-TREE OBJECT) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_return_tree)), NULL);
    define_function_object("CL-TRANSLATE-BOOLEAN-TEST", "(DEFUN (CL-TRANSLATE-BOOLEAN-TEST OBJECT) ((TREE OBJECT) (INVERT? BOOLEAN)))", ((cpp_function_code)(&cl_translate_boolean_test)), NULL);
    define_function_object("CL-TRANSLATE-PLAIN-BOOLEAN-TEST", "(DEFUN (CL-TRANSLATE-PLAIN-BOOLEAN-TEST OBJECT) ((TREE OBJECT) (INVERT? BOOLEAN)))", ((cpp_function_code)(&cl_translate_plain_boolean_test)), NULL);
    define_function_object("CL-TRANSLATE-WHILE-TREE", "(DEFUN (CL-TRANSLATE-WHILE-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_while_tree)), NULL);
    define_function_object("CL-TRANSLATE-CONDITIONAL-TREE", "(DEFUN (CL-TRANSLATE-CONDITIONAL-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_conditional_tree)), NULL);
    define_function_object("CL-TRANSLATE-COND-TREE", "(DEFUN (CL-TRANSLATE-COND-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_cond_tree)), NULL);
    define_function_object("CL-TRANSLATE-CASE-TREE", "(DEFUN (CL-TRANSLATE-CASE-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_case_tree)), NULL);
    define_function_object("CL-TRANSLATE-BOOLEAN-TREE", "(DEFUN (CL-TRANSLATE-BOOLEAN-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_boolean_tree)), NULL);
    define_function_object("CL-TRANSLATE-CLASS-NAME", "(DEFUN (CL-TRANSLATE-CLASS-NAME OBJECT) ((CLASS CLASS)))", ((cpp_function_code)(&cl_translate_class_name)), NULL);
    define_function_object("YIELD-STRUCT-CONSTRUCTOR-NAME", "(DEFUN (YIELD-STRUCT-CONSTRUCTOR-NAME OBJECT) ((CLASS CLASS)))", ((cpp_function_code)(&yield_struct_constructor_name)), NULL);
    define_function_object("CL-TRANSLATE-MAKE-TREE", "(DEFUN (CL-TRANSLATE-MAKE-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_make_tree)), NULL);
    define_function_object("CL-TRANSLATE-NEW-TREE", "(DEFUN (CL-TRANSLATE-NEW-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_new_tree)), NULL);
    define_function_object("CL-TRANSLATE-NEW-ARRAY-TREE", "(DEFUN (CL-TRANSLATE-NEW-ARRAY-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_new_array_tree)), NULL);
    define_function_object("CL-TRANSLATE-OPERATOR-TREE", "(DEFUN (CL-TRANSLATE-OPERATOR-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_operator_tree)), NULL);
    define_function_object("YIELD-FORMAT-ARGUMENTS", "(DEFUN (YIELD-FORMAT-ARGUMENTS CONS) ((TREE CONS)))", ((cpp_function_code)(&yield_format_arguments)), NULL);
  }
}

void startup_cl_translate() {
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (current_startup_time_phaseP(2)) {
      help_startup_cl_translate1();
      help_startup_cl_translate2();
      help_startup_cl_translate3();
    }
    if (current_startup_time_phaseP(6)) {
      finalize_classes();
    }
    if (current_startup_time_phaseP(7)) {
      help_startup_cl_translate4();
      define_function_object("YIELD-PRINT-TREE", "(DEFUN (YIELD-PRINT-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&yield_print_tree)), NULL);
      define_function_object("CL-TRANSLATE-PRINT-STREAM-TREE", "(DEFUN (CL-TRANSLATE-PRINT-STREAM-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_print_stream_tree)), NULL);
      define_function_object("CL-TRANSLATE-CALL-FUNCTION-CODE-TREE", "(DEFUN (CL-TRANSLATE-CALL-FUNCTION-CODE-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_call_function_code_tree)), NULL);
      define_function_object("CL-TRANSLATE-INLINE-CALL-TREE", "(DEFUN (CL-TRANSLATE-INLINE-CALL-TREE OBJECT) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_inline_call_tree)), NULL);
      define_function_object("CL-TRANSLATE-THE-CODE-TREE", "(DEFUN (CL-TRANSLATE-THE-CODE-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_the_code_tree)), NULL);
      define_function_object("CL-TRANSLATE-VERBATIM-TREE", "(DEFUN (CL-TRANSLATE-VERBATIM-TREE OBJECT) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_verbatim_tree)), NULL);
      define_function_object("CL-TRANSLATE-INLINE-TREE", "(DEFUN (CL-TRANSLATE-INLINE-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&cl_translate_inline_tree)), NULL);
      define_function_object("GENERATE-CL-TYPE-DECLARATIONS?", "(DEFUN (GENERATE-CL-TYPE-DECLARATIONS? BOOLEAN) ())", ((cpp_function_code)(&generate_cl_type_declarationsP)), NULL);
      define_function_object("LOOKUP-CL-TYPE-FROM-STELLA-TYPE", "(DEFUN (LOOKUP-CL-TYPE-FROM-STELLA-TYPE OBJECT) ((TYPE TYPE-SPEC)))", ((cpp_function_code)(&lookup_cl_type_from_stella_type)), NULL);
      define_function_object("CL-YIELD-TYPED-EXPRESSION-TREE", "(DEFUN (CL-YIELD-TYPED-EXPRESSION-TREE CONS) ((TREE CONS) (TYPE TYPE-SPEC)))", ((cpp_function_code)(&cl_yield_typed_expression_tree)), NULL);
      define_function_object("CL-YIELD-METHOD-PARAMETER-TYPE-DECLARATION-TREES", "(DEFUN (CL-YIELD-METHOD-PARAMETER-TYPE-DECLARATION-TREES CONS) ((METHOD METHOD-SLOT)))", ((cpp_function_code)(&cl_yield_method_parameter_type_declaration_trees)), NULL);
      define_function_object("CL-YIELD-LET-VARIABLE-TYPE-DECLARATION-TREES", "(DEFUN (CL-YIELD-LET-VARIABLE-TYPE-DECLARATION-TREES CONS) ((BINDINGSTREE (CONS OF CONS))))", ((cpp_function_code)(&cl_yield_let_variable_type_declaration_trees)), NULL);
      define_function_object("CL-YIELD-GLOBAL-VARIABLE-TYPE-DECLARATION-TREE", "(DEFUN (CL-YIELD-GLOBAL-VARIABLE-TYPE-DECLARATION-TREE CONS) ((GLOBAL GLOBAL-VARIABLE)))", ((cpp_function_code)(&cl_yield_global_variable_type_declaration_tree)), NULL);
      define_function_object("CL-YIELD-DECLARE-TREE", "(DEFUN (CL-YIELD-DECLARE-TREE CONS) ((DECLARATIONS (CONS OF CONS))))", ((cpp_function_code)(&cl_yield_declare_tree)), NULL);
      define_function_object("CL-TRANSLATE-UNIT", "(DEFUN (CL-TRANSLATE-UNIT OBJECT) ((UNIT TRANSLATION-UNIT)))", ((cpp_function_code)(&cl_translate_unit)), NULL);
      define_function_object("CL-TRANSLATE-METHOD-PARAMETERS", "(DEFUN (CL-TRANSLATE-METHOD-PARAMETERS CONS) ((METHOD METHOD-SLOT)))", ((cpp_function_code)(&cl_translate_method_parameters)), NULL);
      define_function_object("CL-TRANSLATE-DEFINE-METHOD-UNIT", "(DEFUN (CL-TRANSLATE-DEFINE-METHOD-UNIT CONS) ((UNIT TRANSLATION-UNIT)))", ((cpp_function_code)(&cl_translate_define_method_unit)), NULL);
      define_function_object("YIELD-CLOS-SLOT-TREE", "(DEFUN (YIELD-CLOS-SLOT-TREE CONS) ((SLOT STORAGE-SLOT)))", ((cpp_function_code)(&yield_clos_slot_tree)), NULL);
      define_function_object("YIELD-CLOS-CLASS-TREE", "(DEFUN (YIELD-CLOS-CLASS-TREE CONS) ((CLASS CLASS)))", ((cpp_function_code)(&yield_clos_class_tree)), NULL);
      define_function_object("COMPUTE-CL-STRUCT-SLOTS", "(DEFUN (COMPUTE-CL-STRUCT-SLOTS (LIST OF STORAGE-SLOT)) ((CLASS CLASS)))", ((cpp_function_code)(&compute_cl_struct_slots)), NULL);
      define_function_object("CL-STRUCT-SLOTS", "(DEFUN (CL-STRUCT-SLOTS (LIST OF STORAGE-SLOT)) ((CLASS CLASS)))", ((cpp_function_code)(&cl_struct_slots)), NULL);
      define_function_object("CL-STRUCT-SLOT-OFFSET", "(DEFUN (CL-STRUCT-SLOT-OFFSET INTEGER) ((SLOTNAME SYMBOL) (CLASSTYPE TYPE)))", ((cpp_function_code)(&cl_struct_slot_offset)), NULL);
      define_function_object("YIELD-STRUCT-SLOT-TREES", "(DEFUN (YIELD-STRUCT-SLOT-TREES CONS) ((CLASS CLASS)))", ((cpp_function_code)(&yield_struct_slot_trees)), NULL);
      define_function_object("YIELD-STRUCT-TREE", "(DEFUN (YIELD-STRUCT-TREE CONS) ((CLASS CLASS)))", ((cpp_function_code)(&yield_struct_tree)), NULL);
      define_function_object("YIELD-VECTOR-STRUCT-NAME", "(DEFUN (YIELD-VECTOR-STRUCT-NAME OBJECT) ((CLASS CLASS)))", ((cpp_function_code)(&yield_vector_struct_name)), NULL);
      define_function_object("YIELD-VECTOR-STRUCT-TREE", "(DEFUN (YIELD-VECTOR-STRUCT-TREE CONS) ((CLASS CLASS)))", ((cpp_function_code)(&yield_vector_struct_tree)), NULL);
      define_function_object("YIELD-CONDITION-TREE", "(DEFUN (YIELD-CONDITION-TREE CONS) ((CLASS CLASS)))", ((cpp_function_code)(&yield_condition_tree)), NULL);
      define_function_object("CL-ALWAYS-TRANSLATE-TO-CLOS-CLASS?", "(DEFUN (CL-ALWAYS-TRANSLATE-TO-CLOS-CLASS? BOOLEAN) ((CLASS CLASS)))", ((cpp_function_code)(&cl_always_translate_to_clos_classP)), NULL);
      define_function_object("CL-TRANSLATE-DEFINE-NATIVE-CLASS-UNIT", "(DEFUN (CL-TRANSLATE-DEFINE-NATIVE-CLASS-UNIT CONS) ((UNIT TRANSLATION-UNIT)))", ((cpp_function_code)(&cl_translate_define_native_class_unit)), NULL);
      define_function_object("CL-TRANSLATE-DEFINE-GLOBAL-VARIABLE-UNIT", "(DEFUN (CL-TRANSLATE-DEFINE-GLOBAL-VARIABLE-UNIT CONS) ((UNIT TRANSLATION-UNIT)))", ((cpp_function_code)(&cl_translate_define_global_variable_unit)), NULL);
      define_function_object("CL-TRANSLATE-DEFPRINT-UNIT", "(DEFUN (CL-TRANSLATE-DEFPRINT-UNIT CONS) ((UNIT TRANSLATION-UNIT)))", ((cpp_function_code)(&cl_translate_defprint_unit)), NULL);
      define_function_object("STARTUP-CL-TRANSLATE", "(DEFUN STARTUP-CL-TRANSLATE () :PUBLIC? TRUE)", ((cpp_function_code)(&startup_cl_translate)), NULL);
      { Method_Slot* function = lookup_function(SYM_CL_TRANSLATE_STELLA_STARTUP_CL_TRANSLATE);

        set_dynamic_slot_value(function->dynamic_slots, SYM_CL_TRANSLATE_STELLA_METHOD_STARTUP_CLASSNAME, wrap_string("Startup-Cl-Translate"), NULL_STRING_WRAPPER);
      }
    }
    if (current_startup_time_phaseP(8)) {
      finalize_slots();
      cleanup_unfinalized_classes();
    }
    if (current_startup_time_phaseP(9)) {
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *CL-OPERATOR-TABLE* KEY-VALUE-LIST NULL :DOCUMENTATION \"Mapping from STELLA operators to Common-Lisp operators.\")");
      oCL_OPERATOR_TABLEo = ((Key_Value_List*)(dictionary(SGT_CL_TRANSLATE_STELLA_KEY_VALUE_LIST, 22, SYM_CL_TRANSLATE_STELLA_i, intern_common_lisp_symbol("+"), SYM_CL_TRANSLATE_STELLA__, intern_common_lisp_symbol("-"), SYM_CL_TRANSLATE_STELLA_o, intern_common_lisp_symbol("*"), SYM_CL_TRANSLATE_STELLA_s, intern_common_lisp_symbol("/"), SYM_CL_TRANSLATE_STELLA_EQp, intern_common_lisp_symbol("EQL"), SYM_CL_TRANSLATE_STELLA_l, intern_common_lisp_symbol("<"), SYM_CL_TRANSLATE_STELLA_le, intern_common_lisp_symbol("<="), SYM_CL_TRANSLATE_STELLA_ge, intern_common_lisp_symbol(">="), SYM_CL_TRANSLATE_STELLA_g, intern_common_lisp_symbol(">"), SYM_CL_TRANSLATE_STELLA_e, intern_common_lisp_symbol("="), SYM_CL_TRANSLATE_STELLA_xe, intern_common_lisp_symbol("/="))));
      define_stella_global_variable_from_stringified_source("(DEFSPECIAL *NEEDEXPLICITRETURN?* BOOLEAN FALSE)");
    }
  }
}

Symbol* SYM_CL_TRANSLATE_STELLA_SYS_CALL_FUNCTION = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_SYS_CALL_METHOD = NULL;

Surrogate* SGT_CL_TRANSLATE_STELLA_KEY_VALUE_LIST = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_i = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA__ = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_o = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_s = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_EQp = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_l = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_le = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_ge = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_g = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_e = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_xe = NULL;

Keyword* KWD_CL_TRANSLATE_USE_COMMON_LISP_CONSES = NULL;

Surrogate* SGT_CL_TRANSLATE_STELLA_CONS = NULL;

Surrogate* SGT_CL_TRANSLATE_STELLA_STANDARD_OBJECT = NULL;

Surrogate* SGT_CL_TRANSLATE_STELLA_OBJECT = NULL;

Keyword* KWD_CL_TRANSLATE_USE_COMMON_LISP_STRUCTS = NULL;

Keyword* KWD_CL_TRANSLATE_USE_COMMON_LISP_VECTOR_STRUCTS = NULL;

Surrogate* SGT_CL_TRANSLATE_STELLA_VOID = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_VOID_SYS = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_TYPED_SYS = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_BAD_SYS = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_SETQ = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_SYS_SLOT_VALUE = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_SYS_SLOT_VALUE_SETTER = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_PROGN = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_WITH_PROCESS_LOCK = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_LOOP = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_SYS_FOREACH = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_LET = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_STARTUP_TIME_PROGN = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_SPECIAL = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_SYS_UNWIND_PROTECT = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_SYS_HANDLER_CASE = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_SYS_HANDLE_EXCEPTION = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_SYS_SIGNAL = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_CAST = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_SAFE_CAST = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_RETURN = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_BREAK = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_CONTINUE = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_WHILE = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_IF = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_CHOOSE = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_WHEN = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_UNLESS = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_AND = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_OR = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_NOT = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_COND = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_CASE = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_MAKE = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_SYS_NEW = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_MV_SETQ = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_QUOTE = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_SYS_CALL_FUNCTION_CODE = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_SYS_CALL_METHOD_CODE = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_SYS_INLINE_CALL = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_THE_CODE = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_PRINT_STREAM = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_PRINT_NATIVE_STREAM = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_VERBATIM = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_INLINE = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_IGNORE = NULL;

Keyword* KWD_CL_TRANSLATE_CONTINUE = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_GET_KWD = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_GET_SGT = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_MODULE_LISP_PACKAGE = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_VALUE = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_REST = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_rrVALUE = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_rrREST = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_CLSYS_SVAL = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_NTH = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_NTH_SETTER = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_AREF = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_AREF_SETTER = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_NULLp = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_DEFINEDp = NULL;

Surrogate* SGT_CL_TRANSLATE_STELLA_ARGUMENT_LIST = NULL;

Surrogate* SGT_CL_TRANSLATE_STELLA_ARGUMENT_LIST_ITERATOR = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_NORMAL_CALL = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_NON_OBJECT_METHOD = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_PRINT_OBJECT = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_OBJECT_METHOD = NULL;

Keyword* KWD_CL_TRANSLATE_COMMON_LISP = NULL;

Keyword* KWD_CL_TRANSLATE_FUNCTION = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_CLSYS_CALL = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_NULL = NULL;

Surrogate* SGT_CL_TRANSLATE_STELLA_STRING = NULL;

Surrogate* SGT_CL_TRANSLATE_STELLA_MUTABLE_STRING = NULL;

Surrogate* SGT_CL_TRANSLATE_STELLA_NATIVE_VECTOR = NULL;

Surrogate* SGT_CL_TRANSLATE_STELLA_ARRAY = NULL;

Surrogate* SGT_CL_TRANSLATE_STELLA_INTEGER = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_ALLOCATE_ITERATOR = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_LENGTH = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_NEXTp = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_ARGUMENT = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_DO = NULL;

Surrogate* SGT_CL_TRANSLATE_STELLA_FLOAT = NULL;

Surrogate* SGT_CL_TRANSLATE_STELLA_INTEGER_WRAPPER = NULL;

Surrogate* SGT_CL_TRANSLATE_STELLA_FLOAT_WRAPPER = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_PRIMARY_TYPE = NULL;

Surrogate* SGT_CL_TRANSLATE_STELLA_LITERAL = NULL;

Surrogate* SGT_CL_TRANSLATE_STELLA_BOOLEAN = NULL;

Surrogate* SGT_CL_TRANSLATE_STELLA_NUMBER = NULL;

Surrogate* SGT_CL_TRANSLATE_STELLA_SECOND_CLASS_OBJECT = NULL;

Surrogate* SGT_CL_TRANSLATE_STELLA_CODE = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_OTHERWISE = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_CLASS_CL_NATIVE_TYPE = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_oCONDITION_MESSAGE_KEYWORDo = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_CLSYS_MAKE = NULL;

Keyword* KWD_CL_TRANSLATE_INITIAL_ELEMENT = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_rrPRINT_STREAM = NULL;

Keyword* KWD_CL_TRANSLATE_WARN = NULL;

Keyword* KWD_CL_TRANSLATE_ERROR = NULL;

Keyword* KWD_CL_TRANSLATE_CONTINUABLE_ERROR = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_CLSYS_METHOD_CODE_CALL = NULL;

Surrogate* SGT_CL_TRANSLATE_STELLA_SHORT_INTEGER = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_GLOBAL_VARIABLE = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_CLASS = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_METHOD = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_MACRO = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_PRINT_METHOD = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_TYPE = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_CLSYS_SELF = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_CLSYS_DUMMY = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_METHOD_VARIABLE_ARGUMENTSp = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_DOCUMENTATION = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_SELF = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_oCLSYS_SELFo = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_rrDEFCONSMETHOD = NULL;

Keyword* KWD_CL_TRANSLATE_VOID = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_SLOT_AUXILIARYp = NULL;

Keyword* KWD_CL_TRANSLATE_EMBEDDED = NULL;

Keyword* KWD_CL_TRANSLATE_INSTANCE = NULL;

Keyword* KWD_CL_TRANSLATE_ALLOCATION = NULL;

Keyword* KWD_CL_TRANSLATE_DOCUMENTATION = NULL;

Keyword* KWD_CL_TRANSLATE_ACCESSOR = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_CLSYS_ROOT_OBJECT = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_CLASS_CL_STRUCT_SLOTS = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_CLSYS_ROOT_STRUCT = NULL;

Keyword* KWD_CL_TRANSLATE_CONC_NAME = NULL;

Keyword* KWD_CL_TRANSLATE_CONSTRUCTOR = NULL;

Keyword* KWD_CL_TRANSLATE_COPIER = NULL;

Keyword* KWD_CL_TRANSLATE_PREDICATE = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_ = NULL;

Keyword* KWD_CL_TRANSLATE_INCLUDE = NULL;

Keyword* KWD_CL_TRANSLATE_UNBOUND_SPECIAL_VARIABLE = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_STREAM = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_STARTUP_CL_TRANSLATE = NULL;

Symbol* SYM_CL_TRANSLATE_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella
