// classes.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

char* class_name(Class* renamed_Class) {
  return (renamed_Class->class_type->symbol_name);
}

Symbol* class_symbol(Class* renamed_Class) {
  return (intern_symbol_in_module(renamed_Class->class_type->symbol_name, ((Module*)(renamed_Class->class_type->home_context)), TRUE));
}

Class* Object::primary_class() {
  { Object* self = this;

    if (((boolean)(self->primary_type()))) {
      return (((Class*)(self->primary_type()->surrogate_value)));
    }
    return (NULL);
  }
}

Class* string_lookup_class(char* name) {
  // Return a class with name `name'.  Scan all
  // visible surrogates looking for one that has a class defined for it.
  { Class* renamed_Class = NULL;
    Surrogate* surrogate = NULL;

    { Module* module = NULL;
      Iterator* iter_000 = visible_modules(oMODULEo.get());

      while (iter_000->nextP()) {
        module = ((Module*)(iter_000->value));
        { boolean test_value_000 = FALSE;

          { 
            surrogate = ((Surrogate*)(lookup_rigid_symbol_locally(name, module, SURROGATE_SYM)));
            test_value_000 = ((boolean)(surrogate));
          }
          if (test_value_000) {
            { 
              renamed_Class = ((Class*)(surrogate->surrogate_value));
              test_value_000 = ((boolean)(renamed_Class));
            }
            if (test_value_000) {
              test_value_000 = isaP(renamed_Class, SGT_CLASSES_STELLA_CLASS);
            }
          }
          if (test_value_000) {
            return (renamed_Class);
          }
        }
      }
    }
    return (NULL);
  }
}

Class* Symbol::lookup_class() {
  // Return a class with name `name'.  Scan all
  // visible surrogates looking for one that has a class defined for it.
  { Symbol* name = this;

    { 
      BIND_STELLA_SPECIAL(oMODULEo, Module*, ((Module*)(name->home_context)));
      return (string_lookup_class(name->symbol_name));
    }
  }
}

Class* type_to_class(Surrogate* type) {
  return (((Class*)(type->surrogate_value)));
}

Class* Surrogate::get_stella_class(boolean errorP) {
  // Return a class with name `class-name'.  If none exists, break
  // if `error?', else return `null'.
  { Surrogate* class_name = this;

    { Object* renamed_Class = class_name->surrogate_value;

      if (((boolean)(renamed_Class)) &&
          stella_classP(renamed_Class)) {
        return (((Class*)(renamed_Class)));
      }
      if (errorP) {
        cerr << "Class " << "`" << class_name << "'" << " does not exist.";
      }
      return (NULL);
    }
  }
}

Class* string_get_stella_class(char* class_name, boolean errorP) {
  // Return a class with name `class-name'.  If none exists, break
  // if `error?', else return `null'.
  { Surrogate* type = lookup_surrogate(class_name);

    if (((boolean)(type))) {
      return (type->get_stella_class(errorP));
    }
    if (errorP) {
      cerr << "Class " << "`" << class_name << "'" << " does not exist.";
    }
    return (NULL);
  }
}

Class* Symbol::get_stella_class(boolean errorP) {
  // Return a class with name `class-name'.  If non exists, break
  // if `error?', else return `null'.
  { Symbol* class_name = this;

    return (string_get_stella_class(class_name->symbol_name, errorP));
  }
}

// If set, warn about each redefinition.
DEFINE_STELLA_SPECIAL(oWARNIFREDEFINEpo, boolean , TRUE);

boolean bind_to_surrogateP(Object* self, char* name, boolean clipoldvalueP, boolean askforpermissionP, Object*& _return1, Surrogate*& _return2) {
  { Surrogate* oldsurrogate = lookup_surrogate(name);
    Object* oldvalue = (((boolean)(oldsurrogate)) ? oldsurrogate->surrogate_value : ((Object*)(NULL)));
    Surrogate* surrogate = shadow_surrogate(name);
    Module* oldmodule = NULL;

    if (!((boolean)(oldvalue))) {
      surrogate->surrogate_value = self;
      _return1 = NULL;
      _return2 = surrogate;
      return (TRUE);
    }
    if (eqlP(oldvalue, self)) {
      _return1 = NULL;
      _return2 = surrogate;
      return (TRUE);
    }
    oldmodule = ((Module*)(oldsurrogate->home_context));
    if (!(oldmodule == oMODULEo.get())) {
      surrogate->surrogate_value = self;
      if (askforpermissionP) {
        *(STANDARD_WARNING->native_stream) << "Warning: " << "Shadowing the " << "`" << self->primary_type()->symbol_name << "'" << " named " << "`" << name << "'" << endl;
        cout << "CAUTION: Automatic shadowing can be dangerous, because forward " << endl << "   references to a shadowed object may be bound to the now-shadowed " << endl << "   object.  Suggestion: Explicitly shadow the name using" << endl << "   DEFMODULE's `:shadow' option." << endl;
        if (yes_or_noP("Do it anyway? ")) {
          surrogate->surrogate_value = self;
          _return1 = oldvalue;
          _return2 = surrogate;
          return (TRUE);
        }
      }
      _return1 = NULL;
      _return2 = surrogate;
      return (TRUE);
    }
    else if (clipoldvalueP) {
      if (oWARNIFREDEFINEpo.get()) {
        cout << "Redefining the " << "`" << self->primary_type()->symbol_name << "'" << " named " << "`" << name << "'" << endl;
      }
      surrogate->surrogate_value = self;
      if (!(self->primary_type() == oldvalue->primary_type())) {
        oldvalue = NULL;
      }
      _return1 = oldvalue;
      _return2 = surrogate;
      return (TRUE);
    }
    else {
      if (askforpermissionP) {
        *(STANDARD_WARNING->native_stream) << "Warning: " << "Can't define the " << "`" << self->primary_type()->symbol_name << "'" << " named " << "`" << name << "'" << " in module " << "`" << oMODULEo.get() << "'" << endl << "   because that term is already bound to " << "`" << oldvalue << "'" << endl << endl;
        if (yes_or_noP("Do it anyway? ")) {
          surrogate->surrogate_value = self;
          _return1 = oldvalue;
          _return2 = surrogate;
          return (TRUE);
        }
      }
      _return1 = NULL;
      _return2 = surrogate;
      return (FALSE);
    }
  }
}

void Class::unbind_from_surrogate() {
  { Class* self = this;

    { Surrogate* surrogate = self->class_type;

      if (((boolean)(surrogate))) {
        surrogate->surrogate_value = NULL;
        self->class_type = NULL;
      }
    }
  }
}

Symbol* shadow_symbol(char* name) {
  return (((Symbol*)(intern_rigid_symbol_locally(name, oMODULEo.get(), SYMBOL_SYM))));
}

Surrogate* shadow_surrogate(char* name) {
  shadow_symbol(name);
  return (((Surrogate*)(intern_rigid_symbol_locally(name, oMODULEo.get(), SURROGATE_SYM))));
}

boolean shadowed_symbolP(Generalized_Symbol* symbol) {
  // Return `true' if `symbol' is shadowed in its home module.
  { char* symbolname = symbol->symbol_name;
    Module* module = ((Module*)(symbol->home_context));

    { boolean test_value_000 = FALSE;

      test_value_000 = ((boolean)(module));
      if (test_value_000) {
        { boolean foundP_000 = FALSE;

          { Surrogate* shadow = NULL;
            Cons* iter_000 = module->shadowed_surrogates_reader()->the_cons_list;

            while (!(iter_000 == NIL)) {
              shadow = ((Surrogate*)(iter_000->value));
              iter_000 = iter_000->rest;
              if (string_eqlP(shadow->symbol_name, symbolname)) {
                foundP_000 = TRUE;
                break;
              }
            }
          }
          test_value_000 = foundP_000;
        }
      }
      { boolean value_000 = test_value_000;

        return (value_000);
      }
    }
  }
}

cpp_function_code get_constructor(Class* renamed_Class, boolean warnP) {
  { cpp_function_code constructor = renamed_Class->class_constructor_code;

    if (constructor != NULL) {
      return (constructor);
    }
    if (warnP) {
      if (renamed_Class->abstractP) {
        *(STANDARD_WARNING->native_stream) << "Warning: " << "The abstract class " << "`" << class_symbol(renamed_Class) << "'" << " cannot have a constructor." << endl;
        return (NULL);
      }
      *(STANDARD_WARNING->native_stream) << "Warning: " << "The " << "`" << ((renamed_Class->privateP() ? (char*)"private " : (char*)"")) << "'" << "class " << "`" << class_symbol(renamed_Class) << "'" << " does not have a callable constructor." << endl;
    }
    return (NULL);
  }
}

Object* create_object(Surrogate* type, int initial_value_pairs, ...) {
  // Funcallable version of the `new' operator.
  // Return an instance of the class named by `type'.  If `initial-value-pairs'
  // is supplied, it has to be a key/value list similar to what's accepted by `new'
  // and the named slots will be initialized with the supplied values.  Similar to
  // `new', all required arguments for `type' must be included.  Since all the
  // slot initialization, etc. is handled dynamically at run time, `create-object'
  // is much slower than `new'; therefore, it should only be used if `type' cannot
  // be known at translation time.
  { Class* renamed_Class = type->get_stella_class(TRUE);
    cpp_function_code constructorcode = get_constructor(renamed_Class, TRUE);

    { Property_List* choose_value_000 = NULL;

      if (initial_value_pairs == 0) {
        choose_value_000 = NULL;
      }
      else {
        { Property_List* self_000 = new_property_list();

          { Cons* arglist_000 = NIL;

            { Object* arg_000 = NULL;
              va_list iter_000;
              int iter_000_count = initial_value_pairs;
              Cons* collect_000 = NULL;

              va_start(iter_000, initial_value_pairs);
              while ((iter_000_count--) > 0) {
                arg_000 = va_arg(iter_000, Object*);
                if (!((boolean)(collect_000))) {
                  {
                    collect_000 = cons(arg_000, NIL);
                    if (arglist_000 == NIL) {
                      arglist_000 = collect_000;
                    }
                    else {
                      add_cons_to_end_of_cons_list(arglist_000, collect_000);
                    }
                  }
                }
                else {
                  {
                    collect_000->rest = cons(arg_000, NIL);
                    collect_000 = collect_000->rest;
                  }
                }
              }
              va_end(iter_000);
            }
            self_000->the_plist = arglist_000;
          }
          choose_value_000 = self_000;
        }
      }
      { Property_List* initialvalues = choose_value_000;
        List* requiredslots = renamed_Class->class_required_slot_names_reader();
        Cons* requiredslotvalues = NIL;
        Object* slotvalue = NULL;
        Slot* slot = NULL;
        Object* object = NULL;

        if (!(constructorcode != NULL)) {
          { Output_String_Stream* stream_000 = new_output_string_stream();

            *(stream_000->native_stream) << "create-object: no funcallable constructor available for " << "`" << type << "'";
            throw *new_stella_exception(stream_000->the_string_reader());
          }
        }
        if (requiredslots->non_emptyP()) {
          if (!((boolean)(initialvalues))) {
            { Output_String_Stream* stream_001 = new_output_string_stream();

              *(stream_001->native_stream) << "create-object: missing initial values for " << "`" << type << "'" << "'s required slots";
              throw *new_stella_exception(stream_001->the_string_reader());
            }
          }
          { Symbol* reqslotname = NULL;
            Cons* iter_001 = requiredslots->the_cons_list;
            Cons* collect_001 = NULL;

            while (!(iter_001 == NIL)) {
              reqslotname = ((Symbol*)(iter_001->value));
              iter_001 = iter_001->rest;
              slot = lookup_slot(renamed_Class, reqslotname);
              if (!(subtype_ofP(slot->type(), SGT_CLASSES_STELLA_OBJECT))) {
                { Output_String_Stream* stream_002 = new_output_string_stream();

                  *(stream_002->native_stream) << "create-object: can't yet initialize required slot " << "`" << type << "'" << "." << "`" << reqslotname << "'" << ", since its type is not a subtype of OBJECT";
                  throw *new_stella_exception(stream_002->the_string_reader());
                }
              }
              slotvalue = initialvalues->lookup(reqslotname->keywordify());
              initialvalues->remove_at(reqslotname->keywordify());
              if (!((boolean)(slotvalue))) {
                { Output_String_Stream* stream_003 = new_output_string_stream();

                  *(stream_003->native_stream) << "create-object: missing initial value for required slot " << "`" << type << "'" << "." << "`" << reqslotname << "'";
                  throw *new_stella_exception(stream_003->the_string_reader());
                }
              }
              if (!((boolean)(collect_001))) {
                {
                  collect_001 = cons(slotvalue, NIL);
                  if (requiredslotvalues == NIL) {
                    requiredslotvalues = collect_001;
                  }
                  else {
                    add_cons_to_end_of_cons_list(requiredslotvalues, collect_001);
                  }
                }
              }
              else {
                {
                  collect_001->rest = cons(slotvalue, NIL);
                  collect_001 = collect_001->rest;
                }
              }
            }
          }
        }
        object = apply(constructorcode, requiredslotvalues);
        if (((boolean)(initialvalues))) {
          { Object* slotname = NULL;
            Object* value = NULL;
            Cons* iter_002 = initialvalues->the_plist;

            while (!(iter_002 == NIL)) {
              slotname = iter_002->value;
              value = iter_002->rest->value;
              iter_002 = iter_002->rest->rest;
              if (subtype_of_keywordP(safe_primary_type(slotname))) {
                { Object* slotname_000 = slotname;
                  Keyword* slotname = ((Keyword*)(slotname_000));

                  slot = lookup_slot(renamed_Class, intern_derived_symbol(type, slotname->symbol_name));
                  if (!storage_slotP(slot)) {
                    { Output_String_Stream* stream_004 = new_output_string_stream();

                      *(stream_004->native_stream) << "create-object: slot " << "`" << slotname << "'" << " does not exist on class " << "`" << type << "'";
                      throw *new_stella_exception(stream_004->the_string_reader());
                    }
                  }
                  put_slot_value(((Standard_Object*)(object)), ((Storage_Slot*)(slot)), value);
                }
              }
              else {
                { Output_String_Stream* stream_005 = new_output_string_stream();

                  *(stream_005->native_stream) << "create-object: illegal initialization option: " << "`" << slotname << "'";
                  throw *new_stella_exception(stream_005->the_string_reader());
                }
              }
            }
          }
        }
        return (object);
      }
    }
  }
}

void Object::free() {
  // Default method.  Deallocate storage for `self'.
  { Object* self = this;

    unmake(self);
  }
}

void Active_Object::free() {
  // Remove all pointers between `self' and other objects,
  // and then deallocate the storage for self.
  { Active_Object* self = this;

    unmake(self);
  }
}

Boolean_Wrapper* coerce_to_boolean(Object* object) {
  { Surrogate* test_value_000 = safe_primary_type(object);

    if (subtype_of_symbolP(test_value_000)) {
      { Object* object_000 = object;
        Symbol* object = ((Symbol*)(object_000));

        if (object == SYM_CLASSES_STELLA_TRUE) {
          return (TRUE_WRAPPER);
        }
        else if (object == SYM_CLASSES_STELLA_FALSE) {
          return (FALSE_WRAPPER);
        }
        else {
        }
      }
    }
    else if (subtype_of_booleanP(test_value_000)) {
      { Object* object_001 = object;
        Boolean_Wrapper* object = ((Boolean_Wrapper*)(object_001));

        return (object);
      }
    }
    else {
    }
  }
  cout << "Don't know how to coerce " << "`" << object << "'" << " of type " << "`" << ((((boolean)(object)) ? object->primary_type() : SGT_CLASSES_STELLA_UNKNOWN)) << "'" << endl << "   into a boolean." << endl;
  return (NULL);
}

Class* define_stella_class(Surrogate* name, List* supers, List* slots, Keyword_Key_Value_List* options) {
  // Return a Stella class with name `name'.
  // Caution:  If the class already exists, the Stella class object gets
  // redefined, but the native C++ class is not redefined.
  { Class* renamed_Class = new_class();
    List* parameterslots = ((List*)(options->lookup(KWD_CLASSES_PARAMETERS)));
    Object* oldclass = NULL;
    boolean successP = FALSE;

    if ((!(name->home_context == oMODULEo.get())) &&
        (!visible_fromP(name->home_context, oMODULEo.get()))) {
      *(STANDARD_WARNING->native_stream) << "Warning: " << "Can't define a class named " << "`" << name << "'" << " because the module " << endl << "   " << "`" << name->home_context->context_name() << "'" << " is not visible from the current module " << "`" << oMODULEo.get()->context_name() << "'" << "." << endl << endl;
      return (NULL);
    }
    { boolean value_000 = FALSE;
      Object* value_001 = NULL;
      Surrogate* value_002 = NULL;

      value_000 = bind_to_surrogateP(renamed_Class, name->symbol_name, TRUE, TRUE, value_001, value_002);
      {
        successP = value_000;
        oldclass = value_001;
        name = value_002;
      }
    }
    if (!successP) {
      return (NULL);
    }
    renamed_Class->class_type = name;
    if (((boolean)(parameterslots))) {
      { Slot* s = NULL;
        Cons* iter_000 = parameterslots->the_cons_list;

        while (!(iter_000 == NIL)) {
          s = ((Slot*)(iter_000->value));
          iter_000 = iter_000->rest;
          s->abstractP = TRUE;
          slots->insert_last(s);
        }
      }
    }
    initialize_slot_and_method_cache(renamed_Class);
    { Surrogate* s = NULL;
      Cons* iter_001 = supers->the_cons_list;
      List* into_000 = renamed_Class->class_direct_supers;
      Cons* collect_000 = NULL;

      while (!(iter_001 == NIL)) {
        s = ((Surrogate*)(iter_001->value));
        iter_001 = iter_001->rest;
        if ((!stella_classP(s->surrogate_value)) &&
            ((!(((Module*)(s->home_context)) == oMODULEo.get())) &&
             visible_fromP(((Module*)(s->home_context)), oMODULEo.get()))) {
          if (oDEBUGLEVELo.get() >= 3) {
            cout << "Automatically shadowing bogus super " << "`" << s << "'" << " of class " << "`" << name << "'" << endl;
          }
          s = shadow_surrogate(s->symbol_name);
        }
        if (!((boolean)(collect_000))) {
          {
            collect_000 = cons(s, NIL);
            if (into_000->the_cons_list == NIL) {
              into_000->the_cons_list = collect_000;
            }
            else {
              add_cons_to_end_of_cons_list(into_000->the_cons_list, collect_000);
            }
          }
        }
        else {
          {
            collect_000->rest = cons(s, NIL);
            collect_000 = collect_000->rest;
          }
        }
      }
    }
    incorporate_class_options(renamed_Class, options);
    { boolean test_value_000 = FALSE;

      test_value_000 = !renamed_Class->primitiveP();
      if (test_value_000) {
        { boolean foundP_000 = FALSE;

          { Slot* slot = NULL;
            Cons* iter_002 = slots->the_cons_list;

            while (!(iter_002 == NIL)) {
              slot = ((Slot*)(iter_002->value));
              iter_002 = iter_002->rest;
              if (slot->primitiveP()) {
                foundP_000 = TRUE;
                break;
              }
            }
          }
          test_value_000 = foundP_000;
        }
      }
      if (test_value_000) {
        *(STANDARD_WARNING->native_stream) << "Warning: " << "Defined class " << "`" << renamed_Class << "'" << " illegally specifies primitive slots." << endl;
      }
    }
    renamed_Class->class_local_slots = slots;
    if (((boolean)(oldclass)) &&
        isaP(oldclass, SGT_CLASSES_STELLA_CLASS)) {
      undefine_old_class(((Class*)(oldclass)), renamed_Class);
    }
    if (!((boolean)(renamed_Class->class_taxonomy_node))) {
      renamed_Class->class_taxonomy_node = create_taxonomy_node(oCLASS_TAXONOMY_GRAPHo, NULL, renamed_Class, renamed_Class->class_direct_supers->emptyP());
    }
    { Slot* slot = NULL;
      Cons* iter_003 = slots->the_cons_list;

      while (!(iter_003 == NIL)) {
        slot = ((Slot*)(iter_003->value));
        iter_003 = iter_003->rest;
        register_slot_name(slot);
      }
    }
    remember_unfinalized_class(renamed_Class, TRUE);
    return (renamed_Class);
  }
}

Slot* define_stella_slot(Symbol* name, Surrogate* owner, Surrogate* basetype, Cons* typespecifier, Keyword_Key_Value_List* options) {
  { Storage_Slot* slot = new_storage_slot();

    slot->slot_name = name;
    slot->slot_owner = owner;
    slot->slot_base_type = basetype;
    if (!(typespecifier == NIL)) {
      set_dynamic_slot_value(slot->dynamic_slots, SYM_CLASSES_STELLA_SLOT_TYPE_SPECIFIER, ((Compound_Type_Specifier*)(typespecifier->yield_type_specifier())), NULL);
    }
    { Keyword* key = NULL;
      Object* value = NULL;
      Kv_List_Iterator* iter_000 = ((Kv_List_Iterator*)(options->allocate_iterator()));

      while (iter_000->nextP()) {
        key = ((Keyword*)(iter_000->key));
        value = iter_000->value;
        if (key == KWD_CLASSES_PUBLICp) {
          slot->slot_publicP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
        }
        else if (key == KWD_CLASSES_REQUIREDp) {
          slot->slot_requiredP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
        }
        else if (key == KWD_CLASSES_COMPONENTp) {
          slot->slot_componentP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
        }
        else if (key == KWD_CLASSES_READ_ONLYp) {
          slot->slot_read_onlyP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
        }
        else if (key == KWD_CLASSES_ACTIVEp) {
          set_dynamic_slot_value(slot->dynamic_slots, SYM_CLASSES_STELLA_STORED_ACTIVEp, ((Boolean_Wrapper*)(value)), NULL);
        }
        else if (key == KWD_CLASSES_CONTEXT_SENSITIVEp) {
          slot->slot_context_sensitiveP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
        }
        else if (key == KWD_CLASSES_HARDWIREDp) {
          slot->slot_hardwiredP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
        }
        else if (key == KWD_CLASSES_ABSTRACTp) {
          slot->abstractP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
        }
        else if (key == KWD_CLASSES_ALLOCATION) {
          if (value == KWD_CLASSES_CLASS) {
            *(STANDARD_WARNING->native_stream) << "Warning: " << "In definition of slot " << "`" << slot << "'" << ": ':class' allocation is unsupported right now." << endl << " Maybe use ':hardwired? TRUE' for read-only slots." << endl;
          }
          else {
            set_dynamic_slot_value(slot->dynamic_slots, SYM_CLASSES_STELLA_SLOT_ALLOCATION, ((Keyword*)(value)), NULL);
          }
        }
        else if (key == KWD_CLASSES_INITIALLY) {
          set_dynamic_slot_value(slot->dynamic_slots, SYM_CLASSES_STELLA_SLOT_INITIAL_VALUE, value, NULL);
        }
        else if (key == KWD_CLASSES_DEFAULT) {
          set_dynamic_slot_value(slot->dynamic_slots, SYM_CLASSES_STELLA_SLOT_DEFAULT_EXPRESSION, value, NULL);
        }
        else if (key == KWD_CLASSES_READER) {
          set_dynamic_slot_value(slot->dynamic_slots, SYM_CLASSES_STELLA_SLOT_READER, ((Symbol*)(value)), NULL);
        }
        else if (key == KWD_CLASSES_WRITER) {
          set_dynamic_slot_value(slot->dynamic_slots, SYM_CLASSES_STELLA_SLOT_WRITER, ((Symbol*)(value)), NULL);
        }
        else if (key == KWD_CLASSES_INVERSE) {
          set_dynamic_slot_value(slot->dynamic_slots, SYM_CLASSES_STELLA_SLOT_INVERSE, ((Symbol*)(value)), NULL);
        }
        else if (key == KWD_CLASSES_RENAMES) {
          set_dynamic_slot_value(slot->dynamic_slots, SYM_CLASSES_STELLA_SLOT_RENAMES, ((Symbol*)(value)), NULL);
        }
        else if (key == KWD_CLASSES_DOCUMENTATION) {
          set_dynamic_slot_value(slot->dynamic_slots, SYM_CLASSES_STELLA_DOCUMENTATION, wrap_string(((String_Wrapper*)(value))->wrapper_value), NULL_STRING_WRAPPER);
        }
        else if (key == KWD_CLASSES_PROPERTIES) {
          set_dynamic_slot_value(slot->dynamic_slots, SYM_CLASSES_STELLA_PROPERTIES, ((List*)(value)), NULL);
        }
        else if (key == KWD_CLASSES_META_ATTRIBUTES) {
          set_dynamic_slot_value(slot->dynamic_slots, SYM_CLASSES_STELLA_META_ATTRIBUTES, ((Key_Value_List*)(value)), NULL);
        }
        else if (key == KWD_CLASSES_OPTION_KEYWORD) {
          set_dynamic_slot_value(slot->dynamic_slots, SYM_CLASSES_STELLA_SLOT_OPTION_KEYWORD, ((Keyword*)(value)), NULL);
        }
        else if (key == KWD_CLASSES_OPTION_HANDLER) {
          set_dynamic_slot_value(slot->dynamic_slots, SYM_CLASSES_STELLA_SLOT_OPTION_HANDLER, ((Symbol*)(value)), NULL);
        }
        else {
          if (!(run_option_handlerP(slot, key, value))) {
            { 
              BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
              signal_translation_warning();
              if (!(suppress_warningsP())) {
                print_error_context(">> WARNING: ", STANDARD_WARNING);
                *(STANDARD_WARNING->native_stream) << endl << " " << "Skipping invalid slot option " << "`" << de_uglify_parse_tree(key) << "'" << endl << "in the definition of slot " << "`" << intern_symbol_in_module(owner->symbol_name, ((Module*)(owner->home_context)), TRUE) << "'" << "." << "`" << de_uglify_parse_tree(name) << "'" << "." << endl;
              }
            }
          }
        }
      }
    }
    if ((!((boolean)(slot->slot_allocation_reader()))) &&
        (basetype == SGT_CLASSES_STELLA_BOOLEAN)) {
      set_dynamic_slot_value(slot->dynamic_slots, SYM_CLASSES_STELLA_SLOT_ALLOCATION, KWD_CLASSES_BIT, NULL);
    }
    return (slot);
  }
}

void incorporate_class_options(Class* renamed_Class, Keyword_Key_Value_List* options) {
  { Keyword* key = NULL;
    Object* value = NULL;
    Kv_List_Iterator* iter_000 = ((Kv_List_Iterator*)(options->allocate_iterator()));

    while (iter_000->nextP()) {
      key = ((Keyword*)(iter_000->key));
      value = iter_000->value;
      if (key == KWD_CLASSES_DOCUMENTATION) {
        set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_DOCUMENTATION, wrap_string(((String_Wrapper*)(value))->wrapper_value), NULL_STRING_WRAPPER);
      }
      else if (key == KWD_CLASSES_CL_NATIVE_TYPE) {
        set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_CLASS_CL_NATIVE_TYPE, wrap_string(((String_Wrapper*)(value))->wrapper_value), NULL_STRING_WRAPPER);
      }
      else if (key == KWD_CLASSES_CPP_NATIVE_TYPE) {
        set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_CLASS_CPP_NATIVE_TYPE, wrap_string(((String_Wrapper*)(value))->wrapper_value), NULL_STRING_WRAPPER);
      }
      else if (key == KWD_CLASSES_IDL_NATIVE_TYPE) {
        set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_CLASS_IDL_NATIVE_TYPE, wrap_string(((String_Wrapper*)(value))->wrapper_value), NULL_STRING_WRAPPER);
      }
      else if (key == KWD_CLASSES_JAVA_NATIVE_TYPE) {
        set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_CLASS_JAVA_NATIVE_TYPE, wrap_string(((String_Wrapper*)(value))->wrapper_value), NULL_STRING_WRAPPER);
      }
      else if (key == KWD_CLASSES_PUBLICp) {
        renamed_Class->class_publicP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
      }
      else if (key == KWD_CLASSES_ABSTRACTp) {
        renamed_Class->abstractP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
      }
      else if (key == KWD_CLASSES_ACTIVEp) {
        set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_STORED_ACTIVEp, ((Boolean_Wrapper*)(value)), NULL);
      }
      else if (key == KWD_CLASSES_CL_STRUCTp) {
        renamed_Class->cl_structP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
      }
      else if (key == KWD_CLASSES_MIXINp) {
        renamed_Class->mixinP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
        if (renamed_Class->mixinP) {
          renamed_Class->abstractP = TRUE;
        }
      }
      else if (key == KWD_CLASSES_RECYCLE_METHOD) {
        set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_CLASS_RECYCLE_METHOD, ((Keyword*)(value)), NULL);
      }
      else if (key == KWD_CLASSES_EXTENSION) {
        set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_CLASS_EXTENSION_NAME, ((Symbol*)(value)), NULL);
      }
      else if (key == KWD_CLASSES_CREATOR) {
        set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_CLASS_CREATOR, ((Symbol*)(value)), NULL);
      }
      else if (key == KWD_CLASSES_INITIALIZER) {
        set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_CLASS_INITIALIZER, ((Symbol*)(value)), NULL);
      }
      else if (key == KWD_CLASSES_TERMINATOR) {
        set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_CLASS_TERMINATOR, ((Symbol*)(value)), NULL);
      }
      else if (key == KWD_CLASSES_DESTRUCTOR) {
        set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_CLASS_DESTRUCTOR, ((Symbol*)(value)), NULL);
      }
      else if (key == KWD_CLASSES_INITIAL_VALUE) {
        set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_CLASS_INITIAL_VALUE, value, NULL);
      }
      else if (key == KWD_CLASSES_PRINT_FORM) {
        set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_PRINT_FORM, value, NULL);
      }
      else if (key == KWD_CLASSES_EQUALITY_TEST) {
      }
      else if (key == KWD_CLASSES_KEY) {
        set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_CLASS_KEY, new_list(), NULL);
        { Object* slotname = NULL;
          Cons* iter_001 = ((Cons*)(value));
          List* into_000 = renamed_Class->class_key_reader();
          Cons* collect_000 = NULL;

          while (!(iter_001 == NIL)) {
            slotname = iter_001->value;
            iter_001 = iter_001->rest;
            if (!((boolean)(collect_000))) {
              {
                collect_000 = cons(slotname, NIL);
                if (into_000->the_cons_list == NIL) {
                  into_000->the_cons_list = collect_000;
                }
                else {
                  add_cons_to_end_of_cons_list(into_000->the_cons_list, collect_000);
                }
              }
            }
            else {
              {
                collect_000->rest = cons(slotname, NIL);
                collect_000 = collect_000->rest;
              }
            }
          }
        }
      }
      else if (key == KWD_CLASSES_PARAMETERS) {
        set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_CLASS_PARAMETERS, new_list(), NULL);
        { Slot* slot = NULL;
          Cons* iter_002 = ((List*)(value))->the_cons_list;
          List* into_001 = renamed_Class->class_parameters_reader();
          Cons* collect_001 = NULL;

          while (!(iter_002 == NIL)) {
            slot = ((Slot*)(iter_002->value));
            iter_002 = iter_002->rest;
            if (!((boolean)(collect_001))) {
              {
                collect_001 = cons(slot->slot_name, NIL);
                if (into_001->the_cons_list == NIL) {
                  into_001->the_cons_list = collect_001;
                }
                else {
                  add_cons_to_end_of_cons_list(into_001->the_cons_list, collect_001);
                }
              }
            }
            else {
              {
                collect_001->rest = cons(slot->slot_name, NIL);
                collect_001 = collect_001->rest;
              }
            }
          }
        }
      }
      else if (key == KWD_CLASSES_SYNONYMS) {
        set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_CLASS_SYNONYMS, new_list(), NULL);
        { Surrogate* type = NULL;
          Cons* iter_003 = ((Cons*)(value));
          List* into_002 = renamed_Class->class_synonyms_reader();
          Cons* collect_002 = NULL;

          while (!(iter_003 == NIL)) {
            type = ((Surrogate*)(iter_003->value));
            iter_003 = iter_003->rest;
            if (!((boolean)(collect_002))) {
              {
                collect_002 = cons(type, NIL);
                if (into_002->the_cons_list == NIL) {
                  into_002->the_cons_list = collect_002;
                }
                else {
                  add_cons_to_end_of_cons_list(into_002->the_cons_list, collect_002);
                }
              }
            }
            else {
              {
                collect_002->rest = cons(type, NIL);
                collect_002 = collect_002->rest;
              }
            }
          }
        }
      }
      else if (key == KWD_CLASSES_PROPERTIES) {
        set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_PROPERTIES, ((List*)(value)), NULL);
      }
      else if (key == KWD_CLASSES_META_ATTRIBUTES) {
        set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_META_ATTRIBUTES, ((Key_Value_List*)(value)), NULL);
      }
      else if (key == KWD_CLASSES_CHILDREN) {
        renamed_Class->class_direct_subs = ((List*)(value));
        oCLASSES_WITH_UNRESOLVED_CHILDREN_REFERENCESo->push(renamed_Class);
      }
      else {
        if (!(run_option_handlerP(renamed_Class, key, value))) {
          { 
            BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
            signal_translation_warning();
            if (!(suppress_warningsP())) {
              print_error_context(">> WARNING: ", STANDARD_WARNING);
              *(STANDARD_WARNING->native_stream) << endl << " " << "Skipping invalid class option " << "`" << de_uglify_parse_tree(key) << "'" << endl << "in the definition of class " << "`" << class_symbol(renamed_Class) << "'" << "." << endl;
            }
          }
        }
      }
    }
  }
}

boolean inline_methodP(Slot* slot) {
  return ((!slot->slot_externalP) &&
      (isaP(slot, SGT_CLASSES_STELLA_METHOD_SLOT) &&
       (!((Method_Slot*)(slot))->method_functionP)));
}

void transfer_external_slot_to_new_class(Slot* externalslot, Class* newclass) {
  { Slot* newclassslot = lookup_local_slot(newclass, externalslot->slot_name);

    if (!((boolean)(newclassslot))) {
      newclass->class_local_slots->insert(externalslot);
    }
    else if (inline_methodP(newclassslot) ||
        ((!(newclassslot->primary_type() == externalslot->primary_type())) ||
         FALSE)) {
    }
    else {
      newclass->class_local_slots->substitute(externalslot, newclassslot);
      newclassslot->free();
    }
  }
}

void transfer_demons_from_oldclass(Class* oldclass, Class* newclass) {
  { Slot* newslot = NULL;

    { Slot* oldslot = NULL;
      Cons* iter_000 = oldclass->class_local_slots->the_cons_list;

      while (!(iter_000 == NIL)) {
        oldslot = ((Slot*)(iter_000->value));
        iter_000 = iter_000->rest;
        { 
          newslot = lookup_local_slot(newclass, oldslot->slot_name);
          if (((boolean)(newslot))) {
            if (subtype_of_storage_slotP(safe_primary_type(oldslot))) {
              { Slot* oldslot_000 = oldslot;
                Storage_Slot* oldslot = ((Storage_Slot*)(oldslot_000));

                if (subtype_of_storage_slotP(safe_primary_type(newslot))) {
                  { Slot* newslot_000 = newslot;
                    Storage_Slot* newslot = ((Storage_Slot*)(newslot_000));

                    set_dynamic_slot_value(newslot->dynamic_slots, SYM_CLASSES_STELLA_SLOT_GUARD_DEMONS, oldslot->slot_guard_demons_reader(), NULL);
                    set_dynamic_slot_value(oldslot->dynamic_slots, SYM_CLASSES_STELLA_SLOT_GUARD_DEMONS, NULL, NULL);
                    set_dynamic_slot_value(newslot->dynamic_slots, SYM_CLASSES_STELLA_SLOT_DEMONS, oldslot->slot_demons_reader(), NULL);
                    set_dynamic_slot_value(oldslot->dynamic_slots, SYM_CLASSES_STELLA_SLOT_DEMONS, NULL, NULL);
                  }
                }
                else {
                }
              }
            }
            else {
            }
          }
        }
      }
    }
  }
  set_dynamic_slot_value(newclass->dynamic_slots, SYM_CLASSES_STELLA_CLASS_CONSTRUCTOR_DEMONS, oldclass->class_constructor_demons_reader(), NULL);
  set_dynamic_slot_value(oldclass->dynamic_slots, SYM_CLASSES_STELLA_CLASS_CONSTRUCTOR_DEMONS, NULL, NULL);
  set_dynamic_slot_value(newclass->dynamic_slots, SYM_CLASSES_STELLA_CLASS_GUARD_CONSTRUCTOR_DEMONS, oldclass->class_guard_constructor_demons_reader(), NULL);
  set_dynamic_slot_value(oldclass->dynamic_slots, SYM_CLASSES_STELLA_CLASS_GUARD_CONSTRUCTOR_DEMONS, NULL, NULL);
  set_dynamic_slot_value(newclass->dynamic_slots, SYM_CLASSES_STELLA_CLASS_DESTRUCTOR_DEMONS, oldclass->class_destructor_demons_reader(), NULL);
  set_dynamic_slot_value(oldclass->dynamic_slots, SYM_CLASSES_STELLA_CLASS_DESTRUCTOR_DEMONS, NULL, NULL);
  set_dynamic_slot_value(newclass->dynamic_slots, SYM_CLASSES_STELLA_CLASS_GUARD_DESTRUCTOR_DEMONS, oldclass->class_guard_destructor_demons_reader(), NULL);
  set_dynamic_slot_value(oldclass->dynamic_slots, SYM_CLASSES_STELLA_CLASS_GUARD_DESTRUCTOR_DEMONS, NULL, NULL);
}

void undefine_old_class(Class* oldclass, Class* newclass) {
  { Cons* newsubs = copy_cons_list(newclass->class_direct_subs->the_cons_list);

    unfinalize_class_and_subclasses(oldclass);
    forget_unfinalized_class(oldclass);
    { Surrogate* oldsub = NULL;
      Cons* iter_000 = oldclass->class_direct_subs->the_cons_list;
      Cons* collect_000 = NULL;

      while (!(iter_000 == NIL)) {
        oldsub = ((Surrogate*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (!((boolean)(collect_000))) {
          {
            collect_000 = cons(oldsub, NIL);
            if (newsubs == NIL) {
              newsubs = collect_000;
            }
            else {
              add_cons_to_end_of_cons_list(newsubs, collect_000);
            }
          }
        }
        else {
          {
            collect_000->rest = cons(oldsub, NIL);
            collect_000 = collect_000->rest;
          }
        }
      }
    }
    newclass->class_direct_subs->clear();
    newclass->class_direct_subs->the_cons_list = newsubs->remove_duplicates();
    oldclass->class_direct_subs->clear();
  }
  { Taxonomy_Node* taxonomynode = oldclass->class_taxonomy_node;

    { Surrogate* super = NULL;
      Cons* iter_001 = oldclass->class_direct_supers->the_cons_list;

      while (!(iter_001 == NIL)) {
        super = ((Surrogate*)(iter_001->value));
        iter_001 = iter_001->rest;
        if (((boolean)(((Class*)(super->surrogate_value)))) &&
            (!newclass->class_direct_supers->memberP(super))) {
          unlink_taxonomy_nodes(oCLASS_TAXONOMY_GRAPHo, ((Class*)(super->surrogate_value))->class_taxonomy_node, taxonomynode);
        }
      }
    }
    newclass->class_taxonomy_node = taxonomynode;
    oldclass->class_taxonomy_node = NULL;
  }
  { Slot* oldslot = NULL;
    Cons* iter_002 = oldclass->class_local_slots->the_cons_list;

    while (!(iter_002 == NIL)) {
      oldslot = ((Slot*)(iter_002->value));
      iter_002 = iter_002->rest;
      if (oldslot->slot_externalP) {
        transfer_external_slot_to_new_class(oldslot, newclass);
      }
    }
  }
  newclass->class_constructor_code = oldclass->class_constructor_code;
  transfer_demons_from_oldclass(oldclass, newclass);
  run_hooks(oREDEFINE_RELATION_HOOKSo, list(2, oldclass, newclass));
  oldclass->free();
}

void Class::destroy_class() {
  // Destroy the Stella class `self'.  
  // Unfinalize its subclasses (if it has any).
  { Class* self = this;

    if (self->deletedP()) {
      return;
    }
    if (self->class_type->home_context == oSTELLA_MODULEo) {
      *(STANDARD_WARNING->native_stream) << "Warning: " << "Can't delete STELLA class " << "`" << class_name(self) << "'" << "." << endl;
      return;
    }
    unfinalize_class_and_subclasses(self);
    self->unbind_from_surrogate();
    if (((boolean)(self->class_taxonomy_node))) {
      remove_taxonomy_node(oCLASS_TAXONOMY_GRAPHo, self->class_taxonomy_node);
    }
    self->deletedP_setter(TRUE);
    self->free();
  }
}

void Surrogate::destroy_class() {
  { Surrogate* self = this;

    { Class* renamed_Class = ((Class*)(self->surrogate_value));

      if (!((boolean)(renamed_Class))) {
        cout << "Can't destroy non-existent class " << "`" << self << "'" << "." << endl;
      }
      else {
        renamed_Class->destroy_class();
      }
    }
  }
}

void destroy_class_and_subclasses(Class* self) {
  // Destroy the Stella class `self' and all its subclasses.
  { Surrogate* subtype = NULL;
    Cons* iter_000 = self->class_direct_subs->the_cons_list;

    while (!(iter_000 == NIL)) {
      subtype = ((Surrogate*)(iter_000->value));
      iter_000 = iter_000->rest;
      if (((boolean)(((Class*)(subtype->surrogate_value))))) {
        destroy_class_and_subclasses(((Class*)(subtype->surrogate_value)));
      }
    }
  }
  self->destroy_class();
}

boolean Relation::primitiveP() {
  // Return `true' if `self' is not a defined relation.
  { Relation* self = this;

    return (TRUE);
  }
}

void insert_primitive_super(Surrogate* newsuper, List* supers) {
  { boolean foundP_000 = FALSE;

    { Surrogate* sup = NULL;
      Cons* iter_000 = supers->the_cons_list;

      while (!(iter_000 == NIL)) {
        sup = ((Surrogate*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (subtype_ofP(newsuper, sup)) {
          foundP_000 = TRUE;
          break;
        }
      }
    }
    if (!(foundP_000)) {
      { Surrogate* subsumedsuper = NULL;

        for (;;) {
          { 
            { Surrogate* value_000 = NULL;

              { Surrogate* sup = NULL;
                Cons* iter_001 = supers->the_cons_list;

                while (!(iter_001 == NIL)) {
                  sup = ((Surrogate*)(iter_001->value));
                  iter_001 = iter_001->rest;
                  if (subtype_ofP(sup, newsuper)) {
                    value_000 = sup;
                    break;
                  }
                }
              }
              subsumedsuper = value_000;
            }
            if (!(((boolean)(subsumedsuper)))) {
              break;
            }
          }
          supers = ((List*)(supers->remove(subsumedsuper)));
        }
        supers->insert(newsuper);
      }
    }
  }
}

void collect_most_specific_primitive_supers(Class* renamed_Class, List* supers) {
  { Surrogate* super = NULL;
    Cons* iter_000 = renamed_Class->class_direct_supers->the_cons_list;

    while (!(iter_000 == NIL)) {
      super = ((Surrogate*)(iter_000->value));
      iter_000 = iter_000->rest;
      if (type_to_class(super)->primitiveP()) {
        insert_primitive_super(super, supers);
      }
      else {
        collect_most_specific_primitive_supers(type_to_class(super), supers);
      }
    }
  }
}

List* most_specific_primitive_supers(Class* renamed_Class) {
  { List* supers = new_list();

    collect_most_specific_primitive_supers(renamed_Class, supers);
    return (supers->reverse());
  }
}

List* class_native_supers(Class* renamed_Class) {
  { boolean alwaysP_000 = TRUE;

    { Surrogate* super = NULL;
      Cons* iter_000 = renamed_Class->class_direct_supers->the_cons_list;

      while (!(iter_000 == NIL)) {
        super = ((Surrogate*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (!type_to_class(super)->primitiveP()) {
          alwaysP_000 = FALSE;
          break;
        }
      }
    }
    if (alwaysP_000) {
      return (renamed_Class->class_direct_supers);
    }
    else {
      return (most_specific_primitive_supers(renamed_Class));
    }
  }
}

// List of classes whose class or slot inheritance is
// currently unfinalized.
List* oUNFINALIZED_CLASSESo = NULL;

// Set to `true' by `remember-unfinalized-class'; set
// to `false' by `cleanup-unfinalized-classes'.  Minimizes the time that
// `finalize-classes' spends searching for classes to finalize.
boolean oNEWLY_UNFINALIZED_CLASSESpo = FALSE;

// List of classes defined with a :children option
// which still have some of their children references unresolved.
List* oCLASSES_WITH_UNRESOLVED_CHILDREN_REFERENCESo = NULL;

void remember_unfinalized_class(Class* renamed_Class, boolean forceP) {
  if (forceP ||
      (renamed_Class->class_finalizedP &&
       renamed_Class->class_slots_finalizedP)) {
    oUNFINALIZED_CLASSESo->push(renamed_Class);
    oNEWLY_UNFINALIZED_CLASSESpo = TRUE;
  }
}

void forget_unfinalized_class(Class* renamed_Class) {
  if (!(renamed_Class->class_finalizedP &&
      renamed_Class->class_slots_finalizedP)) {
    oUNFINALIZED_CLASSESo->remove(renamed_Class);
  }
}

void finalize_classes() {
  // Finalize all currently unfinalized classes.
  if (!oNEWLY_UNFINALIZED_CLASSESpo) {
    return;
  }
  { boolean hierarchymighthavechangedP = FALSE;

    oUNFINALIZED_CLASSESo->remove_deleted_members();
    resolve_children_references();
    { Class* renamed_Class = NULL;
      Cons* iter_000 = oUNFINALIZED_CLASSESo->the_cons_list;

      while (!(iter_000 == NIL)) {
        renamed_Class = ((Class*)(iter_000->value));
        iter_000 = iter_000->rest;
        if ((!renamed_Class->class_finalizedP) &&
            (!coerce_wrapped_boolean_to_boolean(renamed_Class->badP_reader()))) {
          finalize_class(renamed_Class);
          hierarchymighthavechangedP = TRUE;
        }
      }
    }
    if (hierarchymighthavechangedP) {
      finalize_taxonomy_graph(oCLASS_TAXONOMY_GRAPHo);
    }
  }
}

void resolve_children_references() {
  if (oCLASSES_WITH_UNRESOLVED_CHILDREN_REFERENCESo->emptyP() ||
      oUNFINALIZED_CLASSESo->emptyP()) {
    return;
  }
  { Cons* unresolvableclasses = NIL;

    oCLASSES_WITH_UNRESOLVED_CHILDREN_REFERENCESo->remove_deleted_members();
    { Class* renamed_Class = NULL;
      Cons* iter_000 = oCLASSES_WITH_UNRESOLVED_CHILDREN_REFERENCESo->the_cons_list;

      while (!(iter_000 == NIL)) {
        renamed_Class = ((Class*)(iter_000->value));
        iter_000 = iter_000->rest;
        add_direct_subs_back_links(renamed_Class);
        { boolean foundP_000 = FALSE;

          { Surrogate* sub = NULL;
            Cons* iter_001 = renamed_Class->class_direct_subs->the_cons_list;

            while (!(iter_001 == NIL)) {
              sub = ((Surrogate*)(iter_001->value));
              iter_001 = iter_001->rest;
              if (!((boolean)(type_to_class(sub)))) {
                foundP_000 = TRUE;
                break;
              }
            }
          }
          if (foundP_000) {
            unresolvableclasses = cons(renamed_Class, unresolvableclasses);
          }
        }
      }
    }
    oCLASSES_WITH_UNRESOLVED_CHILDREN_REFERENCESo->clear();
    oCLASSES_WITH_UNRESOLVED_CHILDREN_REFERENCESo->the_cons_list = unresolvableclasses;
  }
}

void finalize_slots() {
  // Finalize all currently unfinalized slots.
  if (!oNEWLY_UNFINALIZED_CLASSESpo) {
    return;
  }
  { Class* renamed_Class = NULL;
    Cons* iter_000 = oUNFINALIZED_CLASSESo->the_cons_list;

    while (!(iter_000 == NIL)) {
      renamed_Class = ((Class*)(iter_000->value));
      iter_000 = iter_000->rest;
      if (renamed_Class->class_finalizedP &&
          ((!renamed_Class->class_slots_finalizedP) &&
           (!coerce_wrapped_boolean_to_boolean(renamed_Class->badP_reader())))) {
        finalize_class_slots(renamed_Class);
      }
    }
  }
}

void cleanup_unfinalized_classes() {
  // Remove all finalized classes from `*UNFINALIZED-CLASSES*',
  // and set `*NEWLY-UNFINALIZED-CLASSES?*' to `false'.
  { Cons* unfinalizedclasses = NIL;

    { Class* renamed_Class = NULL;
      Cons* iter_000 = oUNFINALIZED_CLASSESo->the_cons_list;
      Cons* collect_000 = NULL;

      while (!(iter_000 == NIL)) {
        renamed_Class = ((Class*)(iter_000->value));
        iter_000 = iter_000->rest;
        if ((!renamed_Class->class_finalizedP) ||
            (!renamed_Class->class_slots_finalizedP)) {
          if (!((boolean)(collect_000))) {
            {
              collect_000 = cons(renamed_Class, NIL);
              if (unfinalizedclasses == NIL) {
                unfinalizedclasses = collect_000;
              }
              else {
                add_cons_to_end_of_cons_list(unfinalizedclasses, collect_000);
              }
            }
          }
          else {
            {
              collect_000->rest = cons(renamed_Class, NIL);
              collect_000 = collect_000->rest;
            }
          }
        }
      }
    }
    oUNFINALIZED_CLASSESo->clear();
    oUNFINALIZED_CLASSESo->the_cons_list = unfinalizedclasses;
    oNEWLY_UNFINALIZED_CLASSESpo = FALSE;
  }
}

void finalize_classes_and_slots() {
  // Finalize all currently unfinalized classes and slots.
  finalize_classes();
  finalize_slots();
  cleanup_unfinalized_classes();
}

void activate_class(Class* renamed_Class) {
  { Class* activeobjectclass = type_to_class(SGT_CLASSES_STELLA_ACTIVE_OBJECT);
    List* supers = renamed_Class->class_direct_supers;

    if (((boolean)(activeobjectclass)) &&
        subclass_ofP(renamed_Class, activeobjectclass)) {
      return;
    }
    if (supers->length() == 0) {
      supers->insert(SGT_CLASSES_STELLA_ACTIVE_OBJECT);
    }
    else if ((supers->length() == 1) &&
        ((((Surrogate*)(supers->first())) == SGT_CLASSES_STELLA_OBJECT) ||
         (((Surrogate*)(supers->first())) == SGT_CLASSES_STELLA_STANDARD_OBJECT))) {
      remove_direct_supers_back_links(renamed_Class);
      supers->clear();
      supers->insert(SGT_CLASSES_STELLA_ACTIVE_OBJECT);
    }
    else {
      set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_STORED_ACTIVEp, FALSE_WRAPPER, NULL);
      if (((boolean)(((Symbol*)(dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_CLASS_EXTENSION_NAME, NULL)))))) {
        set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_CLASS_EXTENSION_NAME, NULL, NULL);
      }
      *(STANDARD_WARNING->native_stream) << "Warning: " << "Cannot convert " << "`" << renamed_Class << "'" << " into an ACTIVE-OBJECT." << endl << "You have to modify its superclasses by hand." << endl;
      return;
    }
    add_direct_supers_back_links(renamed_Class);
    inherit_superclasses(renamed_Class);
  }
}

void add_primary_type(Class* renamed_Class) {
  { Surrogate* classtype = renamed_Class->class_type;
    Cons* slottree = NULL;

    { boolean test_value_000 = FALSE;

      test_value_000 = !renamed_Class->abstractP;
      if (test_value_000) {
        test_value_000 = subtype_ofP(classtype, SGT_CLASSES_STELLA_OBJECT);
        if (test_value_000) {
          { boolean foundP_000 = FALSE;

            { Slot* slot = NULL;
              Cons* iter_000 = renamed_Class->class_local_slots->the_cons_list;

              while (!(iter_000 == NIL)) {
                slot = ((Slot*)(iter_000->value));
                iter_000 = iter_000->rest;
                if (slot->slot_name == SYM_CLASSES_STELLA_PRIMARY_TYPE) {
                  foundP_000 = TRUE;
                  break;
                }
              }
            }
            test_value_000 = foundP_000;
          }
          test_value_000 = !test_value_000;
        }
      }
      if (test_value_000) {
        slottree = listO(7, cons(listO(3, SYM_CLASSES_STELLA_SELF, intern_symbol_in_module(classtype->symbol_name, ((Module*)(classtype->home_context)), TRUE), NIL), NIL), KWD_CLASSES_TYPE, SYM_CLASSES_STELLA_TYPE, KWD_CLASSES_AUXILIARYp, SYM_CLASSES_STELLA_TRUE, listO(3, SYM_CLASSES_STELLA_RETURN, classtype, NIL), NIL);
        renamed_Class->class_local_slots->push(define_inline_method(SYM_CLASSES_STELLA_PRIMARY_TYPE, slottree));
      }
    }
  }
}

void finalize_one_class(Class* renamed_Class) {
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, renamed_Class->home_module());
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (((boolean)(((Symbol*)(dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_CLASS_EXTENSION_NAME, NULL)))))) {
      set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_STORED_ACTIVEp, TRUE_WRAPPER, NULL);
    }
    add_direct_supers_back_links(renamed_Class);
    inherit_superclasses(renamed_Class);
    if (((boolean)(((Boolean_Wrapper*)(dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_STORED_ACTIVEp, NULL))))) &&
        coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_STORED_ACTIVEp, NULL))))) {
      activate_class(renamed_Class);
    }
    add_primary_type(renamed_Class);
    { Surrogate* alias = NULL;
      Cons* iter_000 = renamed_Class->class_synonyms_reader()->the_cons_list;

      while (!(iter_000 == NIL)) {
        alias = ((Surrogate*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (((boolean)(((Class*)(alias->surrogate_value)))) &&
            ((!(((Class*)(alias->surrogate_value)) == renamed_Class)) &&
             (!string_eqlP(class_name(((Class*)(alias->surrogate_value))), class_name(renamed_Class))))) {
          *(STANDARD_WARNING->native_stream) << "Warning: " << "Alias " << "`" << intern_symbol_in_module(alias->symbol_name, ((Module*)(alias->home_context)), TRUE) << "'" << " can't point to " << "`" << class_symbol(renamed_Class) << "'" << " because it already points to " << endl << "the class " << "`" << class_symbol(((Class*)(alias->surrogate_value))) << "'" << "." << endl;
        }
        else {
          alias->surrogate_value = renamed_Class;
        }
      }
    }
    if (((boolean)(((Symbol*)(dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_CLASS_EXTENSION_NAME, NULL))))) &&
        (!((boolean)(((Class_Extension*)(dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_CLASS_EXTENSION, NULL))))))) {
      set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_CLASS_EXTENSION, new_class_extension(), NULL);
    }
    run_hooks(oFINALIZE_RELATION_HOOKSo, renamed_Class);
    renamed_Class->class_finalizedP = TRUE;
  }
}

boolean finalize_class(Class* renamed_Class) {
  return (help_finalize_class(renamed_Class, NULL));
}

boolean help_finalize_class(Class* renamed_Class, Surrogate* finalizedparent) {
  if (renamed_Class->class_finalizedP) {
    return (TRUE);
  }
  set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_BADp, TRUE_WRAPPER, NULL);
  { Surrogate* super = NULL;
    Cons* iter_000 = renamed_Class->class_direct_supers->the_cons_list;

    while (!(iter_000 == NIL)) {
      super = ((Surrogate*)(iter_000->value));
      iter_000 = iter_000->rest;
      if (!(super == finalizedparent)) {
        { Class* superclass = type_to_class(super);

          if (((boolean)(superclass))) {
            if (!(help_finalize_class(superclass, NULL))) {
              return (FALSE);
            }
          }
          else {
            return (FALSE);
          }
        }
      }
    }
  }
  if (renamed_Class->class_finalizedP) {
    set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_BADp, NULL, NULL);
    return (TRUE);
  }
  finalize_one_class(renamed_Class);
  { Surrogate* sub = NULL;
    Cons* iter_001 = renamed_Class->class_direct_subs->the_cons_list;

    while (!(iter_001 == NIL)) {
      sub = ((Surrogate*)(iter_001->value));
      iter_001 = iter_001->rest;
      { Class* subclass = type_to_class(sub);

        if (((boolean)(subclass))) {
          help_finalize_class(subclass, renamed_Class->class_type);
        }
      }
    }
  }
  set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_BADp, NULL, NULL);
  return (TRUE);
}

void unfinalize_class_and_subclasses(Class* renamed_Class) {
  if (!renamed_Class->class_finalizedP) {
    return;
  }
  remove_direct_supers_back_links(renamed_Class);
  { List* subs = renamed_Class->class_direct_subs->copy();

    { Surrogate* subtype = NULL;
      Cons* iter_000 = subs->the_cons_list;

      while (!(iter_000 == NIL)) {
        subtype = ((Surrogate*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (((boolean)(((Class*)(subtype->surrogate_value))))) {
          unfinalize_class_and_subclasses(((Class*)(subtype->surrogate_value)));
        }
      }
    }
    subs->free();
  }
  { Surrogate* alias = NULL;
    Cons* iter_001 = renamed_Class->class_synonyms_reader()->the_cons_list;

    while (!(iter_001 == NIL)) {
      alias = ((Surrogate*)(iter_001->value));
      iter_001 = iter_001->rest;
      if (((Class*)(alias->surrogate_value)) == renamed_Class) {
        alias->surrogate_value = NULL;
      }
    }
  }
  uninherit_superclasses(renamed_Class);
  unfinalize_class_slots(renamed_Class);
  remember_unfinalized_class(renamed_Class, FALSE);
  renamed_Class->class_finalizedP = FALSE;
}

Slot* attach_slot_to_owner(Slot* newslot) {
  { Symbol* name = newslot->slot_name;
    Class* ownerclass = ((Class*)(newslot->slot_owner->surrogate_value));
    Slot* oldslot = lookup_local_slot(ownerclass, name);

    newslot->slot_externalP = TRUE;
    register_slot_name(newslot);
    if (!((boolean)(oldslot))) {
      ownerclass->class_local_slots->insert(newslot);
      unfinalize_class_slots(ownerclass);
      return (newslot);
    }
    else if (!local_slotP(oldslot, ownerclass)) {
      ownerclass->class_local_slots->insert(newslot);
      unfinalize_class_slots(ownerclass);
      return (newslot);
    }
    else if (!(newslot->primary_type() == oldslot->primary_type())) {
      *(STANDARD_WARNING->native_stream) << "Warning: " << "Can't define a " << "`" << newslot->primary_type() << "'" << " named " << "`" << name << "'" << " on the class " << "`" << class_name(ownerclass) << "'" << endl << "because it already has a " << "`" << oldslot->primary_type() << "'" << " with the same name." << endl;
      return (NULL);
    }
    else {
      ownerclass->class_local_slots->substitute(newslot, oldslot);
      run_hooks(oREDEFINE_RELATION_HOOKSo, list(2, oldslot, newslot));
      oldslot->free();
      help_unfinalize_class_slots(ownerclass);
      return (newslot);
    }
  }
}

void compute_slot_direct_equivalent(Slot* self) {
  { Class* renamed_Class = ((Class*)(self->slot_owner->surrogate_value));
    Symbol* slotname = self->slot_name;
    Symbol* slotrenames = ((Symbol*)(dynamic_slot_value(self->dynamic_slots, SYM_CLASSES_STELLA_SLOT_RENAMES, NULL)));
    Slot* renamesslot = NULL;

    if (((boolean)(slotrenames))) {
      { Slot* value_000 = NULL;

        { Slot* s = NULL;
          Iterator* iter_000 = renamed_Class->class_slots();

          while (iter_000->nextP()) {
            s = ((Slot*)(iter_000->value));
            if (s->slot_name == slotrenames) {
              value_000 = s;
              break;
            }
          }
        }
        renamesslot = value_000;
      }
      if (((boolean)(renamesslot))) {
        self->slot_direct_equivalent = renamesslot;
        renamesslot->slot_renamedP = TRUE;
      }
      else {
        *(STANDARD_WARNING->native_stream) << "Warning: " << "Slot " << "`" << slotname << "'" << " renames a non-existent self: " << "`" << slotrenames << "'" << "." << endl << "Cancelling the renames option." << endl;
        set_dynamic_slot_value(self->dynamic_slots, SYM_CLASSES_STELLA_SLOT_RENAMES, NULL, NULL);
      }
    }
    else {
      { Slot* value_001 = NULL;

        { Slot* s = NULL;
          Iterator* iter_001 = renamed_Class->class_slots();

          while (iter_001->nextP()) {
            s = ((Slot*)(iter_001->value));
            if ((!(s == self)) &&
                (s->slot_name == slotname)) {
              value_001 = s;
              break;
            }
          }
        }
        self->slot_direct_equivalent = value_001;
      }
    }
  }
}

void check_conformance_of_slot_signature(Slot* self) {
  { Slot* equivalentslot = self->slot_direct_equivalent;

    if (((boolean)(equivalentslot)) &&
        (!self->conforming_signaturesP(equivalentslot))) {
      *(STANDARD_WARNING->native_stream) << "Warning: " << "The signature of slot " << "`" << self << "'" << " does not conform to the" << endl << "   signature of the inherited slot " << "`" << equivalentslot << "'" << endl;
    }
  }
}

Surrogate* collection_to_active_collection(Surrogate* type) {
  if ((type == SGT_CLASSES_STELLA_SET) ||
      (type == SGT_CLASSES_STELLA_ACTIVE_SET)) {
    return (SGT_CLASSES_STELLA_ACTIVE_SET);
  }
  else if ((type == SGT_CLASSES_STELLA_LIST) ||
      (type == SGT_CLASSES_STELLA_ACTIVE_LIST)) {
    return (SGT_CLASSES_STELLA_ACTIVE_LIST);
  }
  else {
    return (NULL);
  }
}

void activate_slot(Storage_Slot* self) {
  if (self->collection_valuedP()) {
    { Surrogate* activecollection = collection_to_active_collection(self->slot_base_type);

      if (!((boolean)(activecollection))) {
        *(STANDARD_WARNING->native_stream) << "Warning: " << "No active collection defined for " << "`" << self->slot_base_type << "'" << "." << endl << "   Cannot activate slot " << "`" << self << "'" << "." << endl;
        set_dynamic_slot_value(self->dynamic_slots, SYM_CLASSES_STELLA_STORED_ACTIVEp, FALSE_WRAPPER, NULL);
        return;
      }
      self->slot_base_type = collection_to_active_collection(self->slot_base_type);
    }
  }
  set_dynamic_slot_value(self->dynamic_slots, SYM_CLASSES_STELLA_STORED_ACTIVEp, TRUE_WRAPPER, NULL);
}

void Slot::finalize_slot_type_computations() {
  { Slot* self = this;

  }
}

void Storage_Slot::finalize_slot_type_computations() {
  { Storage_Slot* self = this;

    if (((boolean)(((Compound_Type_Specifier*)(dynamic_slot_value(self->dynamic_slots, SYM_CLASSES_STELLA_SLOT_TYPE_SPECIFIER, NULL)))))) {
      self->slot_base_type = validate_type_specifier(((Compound_Type_Specifier*)(dynamic_slot_value(self->dynamic_slots, SYM_CLASSES_STELLA_SLOT_TYPE_SPECIFIER, NULL))), ((Class*)(self->slot_owner->surrogate_value)), FALSE);
    }
    if (!((boolean)(self->slot_base_type))) {
      self->slot_base_type = self->type();
    }
    if (!((boolean)(self->slot_base_type))) {
      if (!self->abstractP) {
        *(STANDARD_WARNING->native_stream) << "Warning: " << "Missing type specification for the slot " << "`" << self << "'" << endl;
      }
      self->slot_base_type = SGT_CLASSES_STELLA_UNKNOWN;
      return;
    }
    if (self->activeP()) {
      activate_slot(self);
    }
  }
}

boolean multi_valued_slot_with_duplicatesP(Slot* self) {
  return (subtype_ofP(self->type(), SGT_CLASSES_STELLA_COLLECTION) &&
      (!subtype_ofP(self->type(), SGT_CLASSES_STELLA_SET_MIXIN)));
}

void compute_slot_inverses(Slot* self) {
  if (!((boolean)(((Symbol*)(dynamic_slot_value(self->dynamic_slots, SYM_CLASSES_STELLA_SLOT_INVERSE, NULL)))))) {
    return;
  }
  if (multi_valued_slot_with_duplicatesP(self)) {
    *(STANDARD_WARNING->native_stream) << "Warning: " << "Can't define an inverse on slot " << "`" << self << "'" << " because it allows duplicate values." << endl;
    set_dynamic_slot_value(self->dynamic_slots, SYM_CLASSES_STELLA_SLOT_INVERSE, NULL, NULL);
    return;
  }
  { Class* inverseclass = ((Class*)(self->type()->surrogate_value));
    Slot* inverseslot = lookup_slot(inverseclass, ((Symbol*)(dynamic_slot_value(self->dynamic_slots, SYM_CLASSES_STELLA_SLOT_INVERSE, NULL))));

    if (((boolean)(inverseslot))) {
      if (multi_valued_slot_with_duplicatesP(inverseslot)) {
        *(STANDARD_WARNING->native_stream) << "Warning: " << "Can't define an inverse on slot " << "`" << inverseslot << "'" << " because it allows duplicate values." << endl;
        set_dynamic_slot_value(self->dynamic_slots, SYM_CLASSES_STELLA_SLOT_INVERSE, NULL, NULL);
        set_dynamic_slot_value(inverseslot->dynamic_slots, SYM_CLASSES_STELLA_SLOT_INVERSE, NULL, NULL);
        return;
      }
      if (!inverseslot->activeP()) {
        set_dynamic_slot_value(inverseslot->dynamic_slots, SYM_CLASSES_STELLA_SLOT_INVERSE, self->slot_name, NULL);
        inverseslot->finalize_slot_type_computations();
      }
      set_dynamic_slot_value(inverseslot->dynamic_slots, SYM_CLASSES_STELLA_INVERSE, self, NULL);
      attach_inverse_slot_demon(self);
      set_dynamic_slot_value(self->dynamic_slots, SYM_CLASSES_STELLA_INVERSE, inverseslot, NULL);
      attach_inverse_slot_demon(inverseslot);
      return;
    }
    if (!((boolean)(inverseclass))) {
      *(STANDARD_WARNING->native_stream) << "Warning: " << "Can't finalize inverse slot computation for slot " << "`" << self << "'" << endl << " because the class " << "`" << self->type() << "'" << " is not defined." << endl;
    }
    else {
      *(STANDARD_WARNING->native_stream) << "Warning: " << "Can't finalize inverse slot computation for slot " << "`" << self << "'" << endl << " because the inverse slot " << "`" << ((Symbol*)(dynamic_slot_value(self->dynamic_slots, SYM_CLASSES_STELLA_SLOT_INVERSE, NULL))) << "'" << " does not exist." << endl;
    }
  }
}

void Slot::help_finalize_local_slot() {
  { Slot* self = this;

  }
}

void Storage_Slot::help_finalize_local_slot() {
  { Storage_Slot* self = this;

    { Class* owner = ((Class*)(self->slot_owner->surrogate_value));

      if ((self->allocation() == KWD_CLASSES_DYNAMIC) &&
          ((!subtype_ofP(owner->class_type, SGT_CLASSES_STELLA_DYNAMIC_SLOTS_MIXIN)) &&
           (!owner->abstractP))) {
        *(STANDARD_WARNING->native_stream) << "Warning: " << "Slot " << "`" << self->slot_name << "'" << " on the class " << "`" << class_symbol(owner) << "'" << endl << "   can't have :dynamic slot allocation because the class doesn't" << endl << "   inherit the class `DYNAMIC-SLOTS-MIXIN'.  Resetting the" << endl << "   allocation to ':instance'." << endl;
        set_dynamic_slot_value(self->dynamic_slots, SYM_CLASSES_STELLA_SLOT_ALLOCATION, KWD_CLASSES_INSTANCE, NULL);
      }
      if (((boolean)(((Symbol*)(dynamic_slot_value(self->dynamic_slots, SYM_CLASSES_STELLA_SLOT_RENAMES, NULL)))))) {
        { Storage_Slot* renamesslot = ((Storage_Slot*)(lookup_slot(owner, ((Symbol*)(dynamic_slot_value(self->dynamic_slots, SYM_CLASSES_STELLA_SLOT_RENAMES, NULL))))));

          if (!(self->primary_type() == renamesslot->primary_type())) {
            *(STANDARD_WARNING->native_stream) << "Warning: " << "Slot " << "`" << self->slot_name << "'" << " renames a slot of a different kind" << endl << "   (e.g., a storage slot renaming a method slot, or vice-versa)." << endl << "  Cancelling the renames option." << endl;
            set_dynamic_slot_value(self->dynamic_slots, SYM_CLASSES_STELLA_SLOT_RENAMES, NULL, NULL);
          }
          else if (!(self->allocation() == renamesslot->allocation())) {
            *(STANDARD_WARNING->native_stream) << "Warning: " << "Slot " << "`" << self->slot_name << "'" << " renames a slot with a different" << endl << "   allocation.  Changing its allocation to " << "`" << renamesslot->allocation() << "'" << "." << endl;
            set_dynamic_slot_value(self->dynamic_slots, SYM_CLASSES_STELLA_SLOT_ALLOCATION, renamesslot->allocation(), NULL);
          }
        }
      }
      if (self->slot_context_sensitiveP &&
          ((!subtype_ofP(self->slot_owner, SGT_CLASSES_STELLA_CONTEXT_SENSITIVE_OBJECT)) &&
           (!subtype_ofP(owner->class_type, SGT_CLASSES_STELLA_CONTEXT_SENSITIVE_MIXIN)))) {
        { Output_String_Stream* stream_000 = new_output_string_stream();

          *(stream_000->native_stream) << "Class " << "`" << self->slot_owner->symbol_name << "'" << " must inherit either the " << "class CONTEXT-SENSITIVE-OBJECT" << endl << "   or the class CONTEXT-SENSITIVE-MIXIN because it contains the" << endl << "   context sensitive slot " << "`" << self->slot_name << "'" << " " << endl;
          throw *new_stella_exception(stream_000->the_string_reader());
        }
      }
    }
  }
}

void finalize_local_slot(Slot* self) {
  compute_slot_direct_equivalent(self);
  self->finalize_slot_type_computations();
  check_conformance_of_slot_signature(self);
  compute_slot_inverses(self);
  self->help_finalize_local_slot();
  run_hooks(oFINALIZE_RELATION_HOOKSo, self);
}

void Slot::unfinalize_local_slot() {
  { Slot* self = this;

  }
}

void Storage_Slot::unfinalize_local_slot() {
  { Storage_Slot* self = this;

  }
}

void finalize_class_slots(Class* renamed_Class) {
  set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_BADp, TRUE_WRAPPER, NULL);
  { Surrogate* super = NULL;
    Cons* iter_000 = renamed_Class->class_direct_supers->the_cons_list;

    while (!(iter_000 == NIL)) {
      super = ((Surrogate*)(iter_000->value));
      iter_000 = iter_000->rest;
      if (!((Class*)(super->surrogate_value))->class_slots_finalizedP) {
        finalize_class_slots(((Class*)(super->surrogate_value)));
      }
    }
  }
  compute_required_slot_names(renamed_Class);
  { Surrogate* classtype = renamed_Class->class_type;

    { Slot* slot = NULL;
      Cons* iter_001 = renamed_Class->class_local_slots->the_cons_list;

      while (!(iter_001 == NIL)) {
        slot = ((Slot*)(iter_001->value));
        iter_001 = iter_001->rest;
        register_slot_name(slot);
        slot->slot_owner = classtype;
      }
    }
  }
  { Slot* slot = NULL;
    Cons* iter_002 = renamed_Class->class_local_slots->the_cons_list;

    while (!(iter_002 == NIL)) {
      slot = ((Slot*)(iter_002->value));
      iter_002 = iter_002->rest;
      finalize_local_slot(slot);
    }
  }
  renamed_Class->class_slots_finalizedP = TRUE;
  set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_BADp, NULL, NULL);
}

void unfinalize_class_slots(Class* renamed_Class) {
  if (!renamed_Class->class_slots_finalizedP) {
    return;
  }
  { Surrogate* subtype = NULL;
    Cons* iter_000 = renamed_Class->class_direct_subs->the_cons_list;

    while (!(iter_000 == NIL)) {
      subtype = ((Surrogate*)(iter_000->value));
      iter_000 = iter_000->rest;
      if (((boolean)(((Class*)(subtype->surrogate_value))))) {
        unfinalize_class_slots(((Class*)(subtype->surrogate_value)));
      }
    }
  }
  help_unfinalize_class_slots(renamed_Class);
}

void help_unfinalize_class_slots(Class* renamed_Class) {
  clear_slot_and_method_cache(renamed_Class);
  if (!renamed_Class->class_slots_finalizedP) {
    return;
  }
  free_required_slot_names(renamed_Class);
  { Slot* slot = NULL;
    Cons* iter_000 = renamed_Class->local_slots()->the_cons_list;

    while (!(iter_000 == NIL)) {
      slot = ((Slot*)(iter_000->value));
      iter_000 = iter_000->rest;
      slot->unfinalize_local_slot();
    }
  }
  remember_unfinalized_class(renamed_Class, FALSE);
  renamed_Class->class_slots_finalizedP = FALSE;
}

boolean Class::multiple_parentsP() {
  // Return `true' if `class' has more than one direct superclass.
  { Class* renamed_Class = this;

    return (!(renamed_Class->class_direct_supers->rest() == NIL));
  }
}

void add_direct_supers_back_links(Class* renamed_Class) {
  { List* directsupers = renamed_Class->class_direct_supers;
    Surrogate* classtype = renamed_Class->class_type;

    directsupers->remove_duplicates();
    { Surrogate* supertype = NULL;
      Cons* iter_000 = directsupers->the_cons_list;

      while (!(iter_000 == NIL)) {
        supertype = ((Surrogate*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (((boolean)(((Class*)(supertype->surrogate_value))))) {
          { Class* superclass = ((Class*)(supertype->surrogate_value));

            superclass->class_direct_subs->insert_new(classtype);
            link_taxonomy_nodes(oCLASS_TAXONOMY_GRAPHo, ((Class*)(supertype->surrogate_value))->class_taxonomy_node, renamed_Class->class_taxonomy_node);
          }
        }
      }
    }
  }
}

void add_direct_subs_back_links(Class* renamed_Class) {
  { List* directsubs = renamed_Class->class_direct_subs;
    Surrogate* classtype = renamed_Class->class_type;

    directsubs->remove_duplicates();
    { Surrogate* subtype = NULL;
      Cons* iter_000 = directsubs->the_cons_list;

      while (!(iter_000 == NIL)) {
        subtype = ((Surrogate*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (((boolean)(((Class*)(subtype->surrogate_value))))) {
          { Class* subclass = ((Class*)(subtype->surrogate_value));

            if (subclass->class_finalizedP) {
              if (!(subclass->class_all_super_classes->memberP(renamed_Class))) {
                subclass->class_direct_supers->insert(classtype);
                unfinalize_class_and_subclasses(renamed_Class);
              }
            }
            else {
              subclass->class_direct_supers->insert_new(classtype);
            }
          }
        }
      }
    }
  }
}

void remove_direct_supers_back_links(Class* renamed_Class) {
  { Surrogate* classtype = renamed_Class->class_type;

    { Surrogate* supertype = NULL;
      Cons* iter_000 = renamed_Class->class_direct_supers->the_cons_list;

      while (!(iter_000 == NIL)) {
        supertype = ((Surrogate*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (((boolean)(((Class*)(supertype->surrogate_value))))) {
          ((Class*)(supertype->surrogate_value))->class_direct_subs->remove(classtype);
        }
      }
    }
  }
}

void collect_direct_super_classes(Class* renamed_Class, List* parents) {
  { List* directsupertypes = renamed_Class->class_direct_supers;
    List* nondirectparents = new_list();

    { Surrogate* supertype = NULL;
      Cons* iter_000 = directsupertypes->the_cons_list;
      Cons* collect_000 = NULL;

      while (!(iter_000 == NIL)) {
        supertype = ((Surrogate*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (((boolean)(((Class*)(supertype->surrogate_value))))) {
          if (!((boolean)(collect_000))) {
            {
              collect_000 = cons(((Class*)(supertype->surrogate_value)), NIL);
              if (parents->the_cons_list == NIL) {
                parents->the_cons_list = collect_000;
              }
              else {
                add_cons_to_end_of_cons_list(parents->the_cons_list, collect_000);
              }
            }
          }
          else {
            {
              collect_000->rest = cons(((Class*)(supertype->surrogate_value)), NIL);
              collect_000 = collect_000->rest;
            }
          }
        }
      }
    }
    { Class* superclass = NULL;
      Cons* iter_001 = parents->the_cons_list;

      while (!(iter_001 == NIL)) {
        superclass = ((Class*)(iter_001->value));
        iter_001 = iter_001->rest;
        { Class* othersuperclass = NULL;
          Cons* iter_002 = parents->the_cons_list;

          while (!(iter_002 == NIL)) {
            othersuperclass = ((Class*)(iter_002->value));
            iter_002 = iter_002->rest;
            if (!(othersuperclass == superclass)) {
              if (superclass->class_all_super_classes->memberP(othersuperclass)) {
                nondirectparents->insert(othersuperclass);
              }
            }
          }
        }
      }
    }
    { Class* p = NULL;
      Cons* iter_003 = nondirectparents->the_cons_list;

      while (!(iter_003 == NIL)) {
        p = ((Class*)(iter_003->value));
        iter_003 = iter_003->rest;
        parents->remove(p);
      }
    }
    nondirectparents->free();
  }
}

void inherit_superclasses(Class* renamed_Class) {
  { List* parentclasses = new_list();

    collect_direct_super_classes(renamed_Class, parentclasses);
    if (!renamed_Class->multiple_parentsP()) {
      { Class* onlyparent = ((Class*)(parentclasses->first()));

        if (!((boolean)(onlyparent))) {
          return;
        }
        renamed_Class->class_all_super_classes = cons(onlyparent, onlyparent->class_all_super_classes);
        return;
      }
    }
    parentclasses = parentclasses->reverse();
    { Cons* allsuperclasses = NIL;
      Cons* sublist = NULL;

      { Class* parent = NULL;
        Cons* iter_000 = parentclasses->the_cons_list;

        while (!(iter_000 == NIL)) {
          parent = ((Class*)(iter_000->value));
          iter_000 = iter_000->rest;
          sublist = NIL;
          { Class* ancestor = NULL;
            Cons* iter_001 = parent->class_all_super_classes;
            Cons* collect_000 = NULL;

            while (!(iter_001 == NIL)) {
              ancestor = ((Class*)(iter_001->value));
              iter_001 = iter_001->rest;
              if (!allsuperclasses->memberP(ancestor)) {
                if (!((boolean)(collect_000))) {
                  {
                    collect_000 = cons(ancestor, NIL);
                    if (sublist == NIL) {
                      sublist = collect_000;
                    }
                    else {
                      add_cons_to_end_of_cons_list(sublist, collect_000);
                    }
                  }
                }
                else {
                  {
                    collect_000->rest = cons(ancestor, NIL);
                    collect_000 = collect_000->rest;
                  }
                }
              }
            }
          }
          sublist = cons(parent, sublist);
          allsuperclasses = sublist->concatenate(allsuperclasses, 0);
        }
      }
      renamed_Class->class_all_super_classes = allsuperclasses;
    }
  }
}

void uninherit_superclasses(Class* renamed_Class) {
  renamed_Class->class_all_super_classes = NIL;
}

Class* two_argument_least_common_superclass(Class* class1, Class* class2) {
  // Return the most specific class that is a superclass of
  // both `class1' and `class2'.  If there is more than one, arbitrarily pick one.
  // If there is none, return `null'.
  if (subclass_ofP(class1, class2)) {
    return (class2);
  }
  if (subclass_ofP(class2, class1)) {
    return (class1);
  }
  { Class* c = NULL;
    Cons* iter_000 = class1->class_all_super_classes;

    while (!(iter_000 == NIL)) {
      c = ((Class*)(iter_000->value));
      iter_000 = iter_000->rest;
      c->class_markedP = FALSE;
    }
  }
  { Class* c = NULL;
    Cons* iter_001 = class2->class_all_super_classes;

    while (!(iter_001 == NIL)) {
      c = ((Class*)(iter_001->value));
      iter_001 = iter_001->rest;
      c->class_markedP = TRUE;
    }
  }
  { Class* c = NULL;
    Cons* iter_002 = class1->class_all_super_classes;

    while (!(iter_002 == NIL)) {
      c = ((Class*)(iter_002->value));
      iter_002 = iter_002->rest;
      if (c->class_markedP) {
        return (c);
      }
    }
  }
  return (NULL);
}

Standard_Object* two_argument_least_common_supertype(Standard_Object* type1, Standard_Object* type2) {
  // Return the most specific type that is a supertype of
  // both `type1' and `type2'.  If there is more than one, arbitrarily pick one.
  // If there is none, return @VOID.  If one or both types are parametric,
  // also try to generalize parameter types if necessary.
  { boolean dummy1;

    if (type1 == type2) {
      return (type1);
    }
    { Surrogate* basetype1 = type_spec_to_base_type(type1);
      Surrogate* basetype2 = type_spec_to_base_type(type2);
      Class* superclass = two_argument_least_common_superclass(((Class*)(basetype1->surrogate_value)), ((Class*)(basetype2->surrogate_value)));

      if (!((boolean)(superclass))) {
        return (SGT_CLASSES_STELLA_VOID);
      }
      if ((type1 == basetype1) &&
          (type2 == basetype2)) {
        return (superclass->class_type);
      }
      { Surrogate* supertype = superclass->class_type;
        Standard_Object* supertypespec = base_type_to_type_spec(supertype);
        List* parameters = superclass->parameters();
        List* parametertypes = NULL;
        Standard_Object* sptype = NULL;
        Standard_Object* ptype1 = NULL;
        Standard_Object* ptype2 = NULL;
        boolean parameterexistsP = FALSE;
        Parametric_Type_Specifier* result = NULL;

        if (parameters->emptyP()) {
          return (supertype);
        }
        parametertypes = new_list();
        { Symbol* parameter = NULL;
          Cons* iter_000 = parameters->the_cons_list;
          Cons* collect_000 = NULL;

          while (!(iter_000 == NIL)) {
            parameter = ((Symbol*)(iter_000->value));
            iter_000 = iter_000->rest;
            ptype1 = extract_parameter_type(type1, parameter, parameterexistsP);
            if ((!parameterexistsP) ||
                unknown_typeP(((Surrogate*)(ptype1)))) {
              return (supertype);
            }
            ptype2 = extract_parameter_type(type2, parameter, parameterexistsP);
            if ((!parameterexistsP) ||
                unknown_typeP(((Surrogate*)(ptype2)))) {
              return (supertype);
            }
            sptype = two_argument_least_common_supertype(ptype1, ptype2);
            if ((!(supertype == supertypespec)) &&
                (!sub_type_spec_ofP(sptype, extract_parameter_type(supertype, parameter, dummy1)))) {
              return (supertype);
            }
            if (!((boolean)(collect_000))) {
              {
                collect_000 = cons(sptype, NIL);
                if (parametertypes->the_cons_list == NIL) {
                  parametertypes->the_cons_list = collect_000;
                }
                else {
                  add_cons_to_end_of_cons_list(parametertypes->the_cons_list, collect_000);
                }
              }
            }
            else {
              {
                collect_000->rest = cons(sptype, NIL);
                collect_000 = collect_000->rest;
              }
            }
          }
        }
        { Parametric_Type_Specifier* self_001 = new_parametric_type_specifier();

          self_001->specifier_base_type = supertype;
          self_001->specifier_parameter_types = parametertypes;
          result = self_001;
        }
        if (array_type_specifierP(type1) &&
            array_type_specifierP(type2)) {
          if (!equal_cons_treesP(array_type_dimensions(((Parametric_Type_Specifier*)(type1)))->the_cons_list, array_type_dimensions(((Parametric_Type_Specifier*)(type2)))->the_cons_list)) {
            return (supertype);
          }
          result->specifier_dimensions = ((Parametric_Type_Specifier*)(type1))->specifier_dimensions;
        }
        return (result);
      }
    }
  }
}

void inherit_slots(Class* renamed_Class) {
  { Cons* slots = copy_cons_list(renamed_Class->class_local_slots->the_cons_list);

    if (renamed_Class->class_direct_supers->emptyP()) {
    }
    else if (renamed_Class->multiple_parentsP()) {
      { Surrogate* supertype = NULL;
        Cons* iter_000 = renamed_Class->class_direct_supers->the_cons_list;

        while (!(iter_000 == NIL)) {
          supertype = ((Surrogate*)(iter_000->value));
          iter_000 = iter_000->rest;
          slots = slots->concatenate(copy_cons_list(((Class*)(supertype->surrogate_value))->class_all_slots), 0);
        }
      }
      slots = slots->reverse();
      slots->remove_duplicates();
      slots = slots->reverse();
    }
    else {
      { Cons* parentslots = ((Class*)(((Surrogate*)(renamed_Class->class_direct_supers->first()))->surrogate_value))->class_all_slots;

        slots = slots->concatenate(parentslots, 0);
      }
    }
    renamed_Class->class_all_slots = slots;
  }
}

boolean local_slotP(Slot* slot, Class* renamed_Class) {
  return (slot->slot_owner->surrogate_value == renamed_Class);
}

void uninherit_slots(Class* renamed_Class) {
  { Cons* slotscursor = renamed_Class->class_all_slots;

    if (((boolean)(slotscursor))) {
      if (!(renamed_Class->multiple_parentsP())) {
        { Slot* slot = NULL;
          Cons* iter_000 = renamed_Class->class_all_slots;

          while (!(iter_000 == NIL)) {
            slot = ((Slot*)(iter_000->value));
            iter_000 = iter_000->rest;
            if ((!(slotscursor->rest == NIL)) &&
                local_slotP(((Slot*)(slotscursor->rest->value)), renamed_Class)) {
              slotscursor = slotscursor->rest;
            }
            else {
              break;
            }
          }
        }
        slotscursor->rest = NIL;
      }
      renamed_Class->class_all_slots = NULL;
    }
  }
}

int oSYMBOL_SLOT_OFFSET_COUNTERo = 0;

int oSLOT_CACHE_SIZEo = 20;

void initialize_slot_and_method_cache(Class* renamed_Class) {
  renamed_Class->class_slot_and_method_cache = new_vector(oSLOT_CACHE_SIZEo + 1);
}

void register_slot_name(Slot* slot) {
  // Register the name symbol of `slot' as a slot name, and
  // initialize its `symbol-slot-offset' so that it can be used by `lookup-slot'.
  warn_about_slot_shadowed_by_a_functionP(slot);
  slot->slot_name->symbol_slot_offset = 0;
}

void unregister_slot_name(Slot* slot) {
  // Unregister the name symbol of `slot' as a slot name,
  // so that it can no longer be used by `lookup-slot'.
  slot->slot_name->symbol_slot_offset = NULL_INTEGER;
}

boolean registered_slot_nameP(Symbol* slot_name) {
  // Return `true' if `slot-name' is the name of some registered;
  // slot in the system.
  return (slot_name->symbol_slot_offset != NULL_INTEGER);
}

boolean warn_about_slot_shadowed_by_a_functionP(Slot* slot) {
  { Symbol* name = slot->slot_name;

    if (!(name->symbol_slot_offset != NULL_INTEGER)) {
      if (((boolean)(lookup_macro(name)))) {
        { 
          BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
          signal_translation_warning();
          if (!(suppress_warningsP())) {
            print_error_context(">> WARNING: ", STANDARD_WARNING);
            *(STANDARD_WARNING->native_stream) << endl << " " << "Slot " << "`" << de_uglify_parse_tree(slot) << "'" << " is shadowed by the macro " << "`" << de_uglify_parse_tree(name) << "'" << "." << endl;
          }
        }
        return (TRUE);
      }
      if (((boolean)(lookup_function(name)))) {
        { 
          BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
          signal_translation_warning();
          if (!(suppress_warningsP())) {
            print_error_context(">> WARNING: ", STANDARD_WARNING);
            *(STANDARD_WARNING->native_stream) << endl << " " << "Slot " << "`" << de_uglify_parse_tree(slot) << "'" << " is shadowed by the function " << "`" << de_uglify_parse_tree(name) << "'" << "." << endl;
          }
        }
        return (TRUE);
      }
    }
    return (FALSE);
  }
}

boolean warn_about_function_shadowing_slotsP(Method_Slot* function) {
  { Symbol* name = function->slot_name;

    if (name->symbol_slot_offset != NULL_INTEGER) {
      { Cons* slots = NIL;

        { Module* module = NULL;
          Iterator* iter_000 = all_modules();

          while (iter_000->nextP()) {
            module = ((Module*)(iter_000->value));
            { Slot* slot = NULL;
              Iterator* iter_001 = all_slots(module, TRUE);
              Cons* collect_000 = NULL;

              while (iter_001->nextP()) {
                slot = ((Slot*)(iter_001->value));
                if (slot->slot_name == name) {
                  if (!((boolean)(collect_000))) {
                    {
                      collect_000 = cons(slot, NIL);
                      if (slots == NIL) {
                        slots = collect_000;
                      }
                      else {
                        add_cons_to_end_of_cons_list(slots, collect_000);
                      }
                    }
                  }
                  else {
                    {
                      collect_000->rest = cons(slot, NIL);
                      collect_000 = collect_000->rest;
                    }
                  }
                }
              }
            }
          }
        }
        { 
          BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
          signal_translation_warning();
          if (!(suppress_warningsP())) {
            print_error_context(">> WARNING: ", STANDARD_WARNING);
            *(STANDARD_WARNING->native_stream) << endl << " " << "Function " << "`" << de_uglify_parse_tree(name) << "'" << " shadows the following methods/slots:" << endl << "    " << "`" << de_uglify_parse_tree(slots) << "'" << "." << endl;
          }
        }
        return (TRUE);
      }
    }
    return (FALSE);
  }
}

Slot* lookup_slot(Class* renamed_Class, Symbol* slot_name) {
  // Return a slot owned by the class `class' with name `slot-name'.
  // Multiply inherited slots are disambiguated by a left-to-right class
  // precedence order for classes with multiple parents (similar to CLOS).
  if (!((boolean)(renamed_Class))) {
    return (NULL);
  }
  { int offset = slot_name->symbol_slot_offset;
    Slot* cachedslot = NULL;

    if (offset != NULL_INTEGER) {
      cachedslot = ((Slot*)((renamed_Class->class_slot_and_method_cache->the_array)[(slot_name->symbol_slot_offset)]));
      if (((boolean)(cachedslot)) &&
          (cachedslot->slot_name == slot_name)) {
        return (cachedslot);
      }
    }
    cachedslot = NULL;
    { Slot* slot = NULL;
      Iterator* iter_000 = renamed_Class->class_slots();

      while (iter_000->nextP()) {
        slot = ((Slot*)(iter_000->value));
        if (slot->slot_name == slot_name) {
          cachedslot = slot;
          break;
        }
      }
    }
    if (!((boolean)(cachedslot))) {
      return (NULL);
    }
    { int newoffset = ((oSYMBOL_SLOT_OFFSET_COUNTERo + 1) % oSLOT_CACHE_SIZEo);

      oSYMBOL_SLOT_OFFSET_COUNTERo = newoffset;
      slot_name->symbol_slot_offset = newoffset;
      renamed_Class->class_slot_and_method_cache->insert_at(newoffset, cachedslot);
      renamed_Class->class_slot_and_method_cache->insert_at(oSLOT_CACHE_SIZEo, cachedslot);
      return (cachedslot);
    }
  }
}

Slot* safe_lookup_slot(Class* renamed_Class, Symbol* slot_name) {
  // Alias for `lookup-slot'.  Kept for backwards compatibility.
  return (lookup_slot(renamed_Class, slot_name));
}

Slot* lookup_visible_slot(Class* renamed_Class, Object* slot_name) {
  { Slot* slot = NULL;
    char* slotnamestring = NULL;
    Module* module = oMODULEo.get();

    { Surrogate* test_value_000 = safe_primary_type(slot_name);

      if (subtype_of_symbolP(test_value_000)) {
        { Object* slot_name_000 = slot_name;
          Symbol* slot_name = ((Symbol*)(slot_name_000));

          slot = lookup_slot(renamed_Class, slot_name);
          if (((boolean)(slot))) {
            return (slot);
          }
          slotnamestring = slot_name->symbol_name;
          module = ((Module*)(slot_name->home_context));
        }
      }
      else if (subtype_of_surrogateP(test_value_000)) {
        { Object* slot_name_001 = slot_name;
          Surrogate* slot_name = ((Surrogate*)(slot_name_001));

          slotnamestring = slot_name->symbol_name;
          module = ((Module*)(slot_name->home_context));
        }
      }
      else if (subtype_of_keywordP(test_value_000)) {
        { Object* slot_name_002 = slot_name;
          Keyword* slot_name = ((Keyword*)(slot_name_002));

          slotnamestring = slot_name->symbol_name;
        }
      }
      else if (subtype_of_stringP(test_value_000)) {
        { Object* slot_name_003 = slot_name;
          String_Wrapper* slot_name = ((String_Wrapper*)(slot_name_003));

          slotnamestring = slot_name->wrapper_value;
        }
      }
      else {
        { Output_String_Stream* stream_000 = new_output_string_stream();

          *(stream_000->native_stream) << "Illegal slot name: " << "`" << slot_name << "'";
          throw *new_stella_exception(stream_000->the_string_reader());
        }
      }
    }
    { Symbol* symbol = NULL;
      Cons* iter_000 = lookup_visible_symbols_in_module(slotnamestring, module, TRUE);

      while (!(iter_000 == NIL)) {
        symbol = ((Symbol*)(iter_000->value));
        iter_000 = iter_000->rest;
        slot = lookup_slot(renamed_Class, symbol);
        if (((boolean)(slot))) {
          return (slot);
        }
      }
    }
    return (NULL);
  }
}

Slot* lookup_local_slot(Class* renamed_Class, Symbol* slot_name) {
  // Lookup a local slot with `slot-name' on `class'.
  { Slot* slot = NULL;
    Cons* iter_000 = renamed_Class->class_local_slots->the_cons_list;

    while (!(iter_000 == NIL)) {
      slot = ((Slot*)(iter_000->value));
      iter_000 = iter_000->rest;
      if (slot->slot_name == slot_name) {
        return (slot);
      }
    }
  }
  return (NULL);
}

void clear_slot_and_method_cache(Class* renamed_Class) {
  if (((boolean)(((Slot*)((renamed_Class->class_slot_and_method_cache->the_array)[oSLOT_CACHE_SIZEo]))))) {
    renamed_Class->class_slot_and_method_cache->clear();
  }
}

void resize_slot_caches(int size) {
  // Reset all slot caches to have size `size'.
  oSLOT_CACHE_SIZEo = size;
  { Class* c = NULL;
    Iterator* iter_000 = all_classes(NULL, FALSE);

    while (iter_000->nextP()) {
      c = ((Class*)(iter_000->value));
      c->class_slot_and_method_cache = new_vector(size);
      { Slot* s = NULL;
        Cons* iter_001 = c->local_slots()->the_cons_list;

        while (!(iter_001 == NIL)) {
          s = ((Slot*)(iter_001->value));
          iter_001 = iter_001->rest;
          s->slot_name->symbol_slot_offset = 0;
        }
      }
    }
  }
}

Slot* get_slot(Standard_Object* self, Symbol* slot_name) {
  // Return the slot named `slot-name' on the class
  // representing the type of `self'.
  return (lookup_slot(self->primary_class(), slot_name));
}

boolean conforming_type_specP(Standard_Object* sub_type_spec, Standard_Object* super_type_spec) {
  return ((!((boolean)(sub_type_spec))) ||
      ((sub_type_spec == SGT_CLASSES_STELLA_UNKNOWN) ||
       ((typeP(sub_type_spec) &&
      (!((boolean)(((Class*)(((Surrogate*)(sub_type_spec))->surrogate_value)))))) ||
        (anchored_type_specifierP(sub_type_spec) ||
         ((!((boolean)(super_type_spec))) ||
          ((super_type_spec == SGT_CLASSES_STELLA_UNKNOWN) ||
           ((typeP(super_type_spec) &&
      (!((boolean)(((Class*)(((Surrogate*)(super_type_spec))->surrogate_value)))))) ||
            (anchored_type_specifierP(super_type_spec) ||
             sub_type_spec_ofP(sub_type_spec, super_type_spec)))))))));
}

boolean Slot::conforming_signaturesP(Slot* superslot) {
  { Slot* self = this;

    superslot = superslot;
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "conforming-signatures?: Not defined on " << "`" << self << "'";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
}

boolean Storage_Slot::conforming_signaturesP(Slot* superslot) {
  { Storage_Slot* self = this;

    return (conforming_type_specP(self->slot_base_type, superslot->type()) &&
        (conforming_type_specP(((Compound_Type_Specifier*)(dynamic_slot_value(self->dynamic_slots, SYM_CLASSES_STELLA_SLOT_TYPE_SPECIFIER, NULL))), ((Compound_Type_Specifier*)(dynamic_slot_value(superslot->dynamic_slots, SYM_CLASSES_STELLA_SLOT_TYPE_SPECIFIER, NULL)))) &&
         (superslot->method_argument_count() == 1)));
  }
}

boolean Method_Slot::conforming_signaturesP(Slot* superslot) {
  { Method_Slot* self = this;

    if (conforming_type_specP(self->slot_base_type, superslot->type()) &&
        (conforming_type_specP(((Compound_Type_Specifier*)(dynamic_slot_value(self->dynamic_slots, SYM_CLASSES_STELLA_SLOT_TYPE_SPECIFIER, NULL))), ((Compound_Type_Specifier*)(dynamic_slot_value(superslot->dynamic_slots, SYM_CLASSES_STELLA_SLOT_TYPE_SPECIFIER, NULL)))) &&
         (self->method_argument_count() == superslot->method_argument_count()))) {
      { Surrogate* test_value_000 = safe_primary_type(superslot);

        if (subtype_of_storage_slotP(test_value_000)) {
          { Slot* superslot_000 = superslot;
            Storage_Slot* superslot = ((Storage_Slot*)(superslot_000));

            return (self->method_return_type_specifiers_reader()->length() == 1);
          }
        }
        else if (subtype_of_method_slotP(test_value_000)) {
          { Slot* superslot_001 = superslot;
            Method_Slot* superslot = ((Method_Slot*)(superslot_001));

            { boolean test_value_001 = FALSE;

              { boolean alwaysP_000 = TRUE;

                { Standard_Object* ts1 = NULL;
                  Standard_Object* ts2 = NULL;
                  Cons* iter_000 = self->method_parameter_type_specifiers_reader()->rest();
                  Cons* iter_001 = superslot->method_parameter_type_specifiers_reader()->rest();

                  while ((!(iter_000 == NIL)) &&
                      (!(iter_001 == NIL))) {
                    ts1 = ((Standard_Object*)(iter_000->value));
                    iter_000 = iter_000->rest;
                    ts2 = ((Standard_Object*)(iter_001->value));
                    iter_001 = iter_001->rest;
                    if (!conforming_type_specP(ts2, ts1)) {
                      alwaysP_000 = FALSE;
                      break;
                    }
                  }
                }
                test_value_001 = alwaysP_000;
              }
              if (test_value_001) {
                { boolean alwaysP_001 = TRUE;

                  { Standard_Object* ts1 = NULL;
                    Standard_Object* ts2 = NULL;
                    Cons* iter_002 = self->method_return_type_specifiers_reader()->the_cons_list;
                    Cons* iter_003 = superslot->method_return_type_specifiers_reader()->the_cons_list;

                    while ((!(iter_002 == NIL)) &&
                        (!(iter_003 == NIL))) {
                      ts1 = ((Standard_Object*)(iter_002->value));
                      iter_002 = iter_002->rest;
                      ts2 = ((Standard_Object*)(iter_003->value));
                      iter_003 = iter_003->rest;
                      if (!conforming_type_specP(ts1, ts2)) {
                        alwaysP_001 = FALSE;
                        break;
                      }
                    }
                  }
                  test_value_001 = alwaysP_001;
                }
              }
              { boolean value_000 = test_value_001;

                return (value_000);
              }
            }
          }
        }
        else {
        }
      }
    }
    return (FALSE);
  }
}

boolean Table::conforming_signaturesP(Slot* superslot) {
  { Table* self = this;

    if (conforming_type_specP(self->slot_base_type, superslot->type()) &&
        (conforming_type_specP(((Compound_Type_Specifier*)(dynamic_slot_value(self->dynamic_slots, SYM_CLASSES_STELLA_SLOT_TYPE_SPECIFIER, NULL))), ((Compound_Type_Specifier*)(dynamic_slot_value(superslot->dynamic_slots, SYM_CLASSES_STELLA_SLOT_TYPE_SPECIFIER, NULL)))) &&
         (self->method_argument_count() == superslot->method_argument_count()))) {
      if (subtype_ofP(safe_primary_type(superslot), SGT_CLASSES_STELLA_TABLE)) {
        { Slot* superslot_000 = superslot;
          Table* superslot = ((Table*)(superslot_000));

          { boolean alwaysP_000 = TRUE;

            { Standard_Object* ts1 = NULL;
              Standard_Object* ts2 = NULL;
              Cons* iter_000 = self->tuple_domains->rest();
              Cons* iter_001 = superslot->tuple_domains->rest();

              while ((!(iter_000 == NIL)) &&
                  (!(iter_001 == NIL))) {
                ts1 = ((Standard_Object*)(iter_000->value));
                iter_000 = iter_000->rest;
                ts2 = ((Standard_Object*)(iter_001->value));
                iter_001 = iter_001->rest;
                if (!conforming_type_specP(ts2, ts1)) {
                  alwaysP_000 = FALSE;
                  break;
                }
              }
            }
            { boolean value_000 = alwaysP_000;

              return (value_000);
            }
          }
        }
      }
      else {
      }
    }
    return (FALSE);
  }
}

boolean identical_signaturesP(Method_Slot* method1, Method_Slot* method2) {
  { boolean test_value_000 = FALSE;

    { boolean alwaysP_000 = TRUE;

      { Standard_Object* ts1 = NULL;
        Standard_Object* ts2 = NULL;
        Cons* iter_000 = method1->method_parameter_type_specifiers_reader()->rest();
        Cons* iter_001 = method2->method_parameter_type_specifiers_reader()->rest();

        while ((!(iter_000 == NIL)) &&
            (!(iter_001 == NIL))) {
          ts1 = ((Standard_Object*)(iter_000->value));
          iter_000 = iter_000->rest;
          ts2 = ((Standard_Object*)(iter_001->value));
          iter_001 = iter_001->rest;
          if (!(sub_type_spec_ofP(ts1 = compute_relative_type_spec(ts1, method1->slot_owner), ts2 = compute_relative_type_spec(ts2, method2->slot_owner)) &&
              sub_type_spec_ofP(ts2, ts1))) {
            alwaysP_000 = FALSE;
            break;
          }
        }
      }
      test_value_000 = alwaysP_000;
    }
    if (test_value_000) {
      { boolean alwaysP_001 = TRUE;

        { Standard_Object* ts1 = NULL;
          Standard_Object* ts2 = NULL;
          Cons* iter_002 = method1->method_return_type_specifiers_reader()->the_cons_list;
          Cons* iter_003 = method2->method_return_type_specifiers_reader()->the_cons_list;

          while ((!(iter_002 == NIL)) &&
              (!(iter_003 == NIL))) {
            ts1 = ((Standard_Object*)(iter_002->value));
            iter_002 = iter_002->rest;
            ts2 = ((Standard_Object*)(iter_003->value));
            iter_003 = iter_003->rest;
            if (!(sub_type_spec_ofP(ts1 = compute_relative_type_spec(ts1, method1->slot_owner), ts2 = compute_relative_type_spec(ts2, method2->slot_owner)) &&
                sub_type_spec_ofP(ts2, ts1))) {
              alwaysP_001 = FALSE;
              break;
            }
          }
        }
        test_value_000 = alwaysP_001;
      }
    }
    { boolean value_000 = test_value_000;

      return (value_000);
    }
  }
}

void compute_required_slot_names(Class* renamed_Class) {
  { List* slotnames = new_list();

    { Slot* slot = NULL;
      Iterator* iter_000 = renamed_Class->class_slots();

      while (iter_000->nextP()) {
        slot = ((Slot*)(iter_000->value));
        if (storage_slotP(slot) &&
            (((Storage_Slot*)(slot))->slot_requiredP &&
             (!slotnames->memberP(slot->slot_name)))) {
          slotnames->push(slot->slot_name);
        }
      }
    }
    if (slotnames->emptyP()) {
      slotnames->free();
    }
    else {
      set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_CLASS_REQUIRED_SLOT_NAMES, slotnames->reverse(), NULL);
    }
  }
}

void free_required_slot_names(Class* renamed_Class) {
  if (renamed_Class->class_required_slot_names_reader()->non_emptyP()) {
    renamed_Class->class_required_slot_names_reader()->free();
    set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CLASSES_STELLA_CLASS_REQUIRED_SLOT_NAMES, NULL, NULL);
  }
}

boolean Storage_Slot::dynamic_storageP() {
  { Storage_Slot* self = this;

    { Keyword* allocation = self->allocation();

      return ((allocation == KWD_CLASSES_DYNAMIC) ||
          (allocation == KWD_CLASSES_BIT));
    }
  }
}

boolean Relation::privateP() {
  // Return `true' if `self' is not public.
  { Relation* self = this;

    return (!self->publicP());
  }
}

boolean Slot::publicP() {
  // True if `self' or one it its ancestors is marked public.
  { Slot* self = this;

    return (self->slot_publicP ||
        (((boolean)(self->slot_direct_equivalent)) &&
         self->slot_direct_equivalent->publicP()));
  }
}

Iterator* Class::public_slots() {
  // Return an iterator over public slots of `self'.
  { Class* self = this;

    { Cons* publicslots = ((Cons*)(NIL));
      All_Purpose_Iterator* iterator = new_all_purpose_iterator();

      { Slot* slot = NULL;
        Iterator* iter_000 = self->class_slots();

        while (iter_000->nextP()) {
          slot = ((Slot*)(iter_000->value));
          { boolean test_value_000 = FALSE;

            test_value_000 = slot->slot_publicP;
            if (test_value_000) {
              { boolean foundP_000 = FALSE;

                { Slot* ps = NULL;
                  Cons* iter_001 = publicslots;

                  while (!(iter_001 == NIL)) {
                    ps = ((Slot*)(iter_001->value));
                    iter_001 = iter_001->rest;
                    if (ps->slot_name == slot->slot_name) {
                      foundP_000 = TRUE;
                      break;
                    }
                  }
                }
                test_value_000 = foundP_000;
              }
              test_value_000 = !test_value_000;
            }
            if (test_value_000) {
              publicslots = ((Cons*)(cons(slot, publicslots)));
            }
          }
        }
      }
      iterator->iterator_next_code = ((cpp_function_code)(&iterator_cons_list_nextP));
      iterator->iterator_cons_list = publicslots;
      iterator->first_iterationP = TRUE;
      return (iterator);
    }
  }
}

Iterator* Object::public_slots() {
  // Return an iterator over public slots of `self'.
  { Object* self = this;

    return (self->primary_class()->public_slots());
  }
}

All_Class_Slots_Iterator* new_all_class_slots_iterator() {
  { All_Class_Slots_Iterator* self = NULL;

    self = new All_Class_Slots_Iterator();
    self->first_iterationP = TRUE;
    self->value = NULL;
    self->iterator_object = NULL;
    self->iterator_cons_list = NULL;
    return (self);
  }
}

Surrogate* All_Class_Slots_Iterator::primary_type() {
  { All_Class_Slots_Iterator* self = this;

    return (SGT_CLASSES_STELLA_ALL_CLASS_SLOTS_ITERATOR);
  }
}

Object* access_all_class_slots_iterator_slot_value(All_Class_Slots_Iterator* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_CLASSES_STELLA_ITERATOR_CONS_LIST) {
    if (setvalueP) {
      self->iterator_cons_list = ((Cons*)(value));
    }
    else {
      value = self->iterator_cons_list;
    }
  }
  else if (slotname == SYM_CLASSES_STELLA_ITERATOR_OBJECT) {
    if (setvalueP) {
      self->iterator_object = value;
    }
    else {
      value = self->iterator_object;
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

Iterator* Class::all_class_slots() {
  { Class* self = this;

    { All_Class_Slots_Iterator* iterator = new_all_class_slots_iterator();

      iterator->iterator_cons_list = self->class_local_slots->the_cons_list;
      iterator->iterator_object = self->class_all_super_classes;
      iterator->first_iterationP = TRUE;
      return (iterator);
    }
  }
}

boolean All_Class_Slots_Iterator::nextP() {
  { All_Class_Slots_Iterator* self = this;

    { Cons* localslots = self->iterator_cons_list;

      if (localslots == NIL) {
        { Cons* classes = ((Cons*)(self->iterator_object));

          while ((!(classes == NIL)) &&
              (localslots == NIL)) {
            localslots = ((Class*)(classes->value))->class_local_slots->the_cons_list;
            classes = classes->rest;
          }
          if (classes == NIL) {
            if (localslots == NIL) {
              self->value = NULL;
              self->iterator_object = NULL;
              return (FALSE);
            }
          }
          self->iterator_object = classes;
          self->value = ((Slot*)(localslots->value));
          self->iterator_cons_list = localslots->rest;
          return (TRUE);
        }
      }
      else {
        self->value = ((Slot*)(localslots->value));
        self->iterator_cons_list = localslots->rest;
        return (TRUE);
      }
    }
  }
}

Iterator* Class::class_slots() {
  { Class* self = this;

    return (self->all_class_slots());
  }
}

List* Class::local_slots() {
  { Class* self = this;

    return (self->class_local_slots);
  }
}

Iterator* Class::direct_super_classes() {
  // Returns an iterator that generates all direct
  // super classes of `self'.
  { Class* self = this;

    return (new_types_to_classes_iterator(self->class_direct_supers->the_cons_list));
  }
}

Iterator* Class::super_classes() {
  // Returns an iterator that generates all super classes
  // of `self'.  Non-reflexive.
  { Class* self = this;

    { List_Iterator* iterator = new_list_iterator();

      iterator->list_iterator_cursor = self->class_all_super_classes;
      return (iterator);
    }
  }
}

boolean Slot::activeP() {
  { Slot* slot = this;

    return ((((boolean)(((Boolean_Wrapper*)(dynamic_slot_value(slot->dynamic_slots, SYM_CLASSES_STELLA_STORED_ACTIVEp, NULL))))) ? coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(dynamic_slot_value(slot->dynamic_slots, SYM_CLASSES_STELLA_STORED_ACTIVEp, NULL)))) : (((boolean)(((Class*)(SGT_CLASSES_STELLA_COLLECTION->surrogate_value)))) &&
        (((boolean)(((Class*)(slot->slot_base_type->surrogate_value)))) &&
         (((boolean)(((Class*)(slot->slot_owner->surrogate_value)))) &&
          (slot->primitiveP() &&
           (((boolean)(((Symbol*)(dynamic_slot_value(slot->dynamic_slots, SYM_CLASSES_STELLA_SLOT_INVERSE, NULL))))) ||
            (((boolean)(slot->slot_direct_equivalent)) &&
             slot->slot_direct_equivalent->activeP()))))))));
  }
}

Object* Storage_Slot::initial_value() {
  // Return an initial value for `self', or `null'.  The
  // initial value can be defined by the slot itself, inherited from an
  // equivalent slot, or inherit from the :initial-value option for the 
  // class representing the type of `self'.
  { Storage_Slot* self = this;

    { Storage_Slot* slot = self;
      Object* value = dynamic_slot_value(slot->dynamic_slots, SYM_CLASSES_STELLA_SLOT_INITIAL_VALUE, NULL);

      for (;;) {
        if (((boolean)(value))) {
          return (value);
        }
        slot = ((Storage_Slot*)(slot->slot_direct_equivalent));
        if (((boolean)(slot))) {
          value = dynamic_slot_value(slot->dynamic_slots, SYM_CLASSES_STELLA_SLOT_INITIAL_VALUE, NULL);
        }
        else {
          break;
        }
      }
      return (dynamic_slot_value(((Class*)(self->type()->surrogate_value))->dynamic_slots, SYM_CLASSES_STELLA_CLASS_INITIAL_VALUE, NULL));
    }
  }
}

Object* Slot::system_default_value() {
  // Return a default value expression, or if `self'
  // has dynamic storage, an initial value expression.
  { Slot* self = this;

    return (dynamic_slot_value(((Class*)(self->type()->surrogate_value))->dynamic_slots, SYM_CLASSES_STELLA_CLASS_INITIAL_VALUE, NULL));
  }
}

Object* Storage_Slot::system_default_value() {
  // Return a default value expression, or if `self'
  // has dynamic storage, an initial value expression.
  { Storage_Slot* self = this;

    { Object* value = self->default_form();

      if (((boolean)(value))) {
        return (value);
      }
      else if (self->dynamic_storageP() &&
          ((boolean)(((Class*)(self->type()->surrogate_value))))) {
        return (dynamic_slot_value(((Class*)(self->type()->surrogate_value))->dynamic_slots, SYM_CLASSES_STELLA_CLASS_INITIAL_VALUE, NULL));
      }
      else {
        return (NULL);
      }
    }
  }
}

Object* Storage_Slot::initially() {
  // Defines the value of a slot before it has been assigned
  // a value.
  { Storage_Slot* self = this;

    { Object* value = dynamic_slot_value(self->dynamic_slots, SYM_CLASSES_STELLA_SLOT_INITIAL_VALUE, NULL);

      if (((boolean)(value))) {
        return (value);
      }
      else if (((boolean)(self->slot_direct_equivalent))) {
        return (((Storage_Slot*)(self->slot_direct_equivalent))->initially());
      }
      else if (self->slot_base_type == SGT_CLASSES_STELLA_BOOLEAN) {
        return (SYM_CLASSES_STELLA_FALSE);
      }
      else {
        return (NULL);
      }
    }
  }
}

void repair_slots() {
  { Class* top = ((Class*)(SGT_CLASSES_STELLA_OBJECT->surrogate_value));

    unfinalize_class_slots(top);
    finalize_classes_and_slots();
  }
}

void disconnect_classes() {
  { Surrogate* surrogate = NULL;
    Iterator* iter_000 = all_surrogates(oMODULEo.get(), FALSE);

    while (iter_000->nextP()) {
      surrogate = ((Surrogate*)(iter_000->value));
      if (((boolean)(surrogate->surrogate_value)) &&
          stella_classP(surrogate->surrogate_value)) {
        surrogate->surrogate_value = NULL;
      }
    }
  }
}

boolean filter_unbound_surrogateP(Surrogate* self, All_Purpose_Iterator* iterator) {
  iterator = iterator;
  return (!((boolean)(self->surrogate_value)));
}

Iterator* unbound_surrogates(Module* module, boolean localP) {
  // Iterate over all unbound surrogates visible from `module'.
  // Look at all modules if `module' is `null'.  If `local?', only consider
  // surrogates interned in `module'.
  { All_Purpose_Iterator* iterator = new_all_purpose_iterator();

    iterator->iterator_nested_iterator = all_surrogates(module, localP);
    iterator->iterator_next_code = ((cpp_function_code)(&filtered_nested_iterator_nextP));
    iterator->iterator_filter_code = ((cpp_function_code)(&filter_unbound_surrogateP));
    return (iterator);
  }
}

char* name_to_string(Object* name) {
  // Return the string represented by `name'.  Return `null'
  // if `name' is undefined or does not represent a string.
  if (((boolean)(name))) {
    { Surrogate* test_value_000 = safe_primary_type(name);

      if (subtype_ofP(test_value_000, SGT_CLASSES_STELLA_GENERALIZED_SYMBOL)) {
        { Object* name_000 = name;
          Generalized_Symbol* name = ((Generalized_Symbol*)(name_000));

          return (name->symbol_name);
        }
      }
      else if (subtype_of_stringP(test_value_000)) {
        { Object* name_001 = name;
          String_Wrapper* name = ((String_Wrapper*)(name_001));

          return (name->wrapper_value);
        }
      }
      else {
      }
    }
  }
  return (NULL);
}

void print_unbound_surrogates(Cons* args) {
  // Print all unbound surrogates visible from the module named by the first
  // argument (a symbol or string).  Look at all modules if no module name or
  // `null' was supplied.  If the second argument is `true', only consider
  // surrogates interned in the specified module.
  { Cons* arglist = args;
    char* name = name_to_string(arglist->value);
    Module* module = ((name != NULL) ? get_stella_module(name, TRUE) : ((Module*)(NULL)));
    boolean localP = FALSE;

    if (arglist->rest->value == SYM_CLASSES_STELLA_TRUE) {
      localP = TRUE;
    }
    { Surrogate* surrogate = NULL;
      Iterator* iter_000 = unbound_surrogates(module, localP);

      while (iter_000->nextP()) {
        surrogate = ((Surrogate*)(iter_000->value));
        cout << surrogate << endl;
      }
    }
  }
}

void print_unbound_surrogates_evaluator_wrapper(Cons* arguments) {
  print_unbound_surrogates(arguments);
}

Generalized_Symbol* coerce_to_symbol(Object* name) {
  // Return the (generalized) symbol represented by `name'.
  // Return `null' if `name' is undefined or does not represent a string.
  if (((boolean)(name))) {
    { Surrogate* test_value_000 = safe_primary_type(name);

      if (subtype_ofP(test_value_000, SGT_CLASSES_STELLA_GENERALIZED_SYMBOL)) {
        { Object* name_000 = name;
          Generalized_Symbol* name = ((Generalized_Symbol*)(name_000));

          return (name);
        }
      }
      else if (subtype_of_stringP(test_value_000)) {
        { Object* name_001 = name;
          String_Wrapper* name = ((String_Wrapper*)(name_001));

          return (lookup_symbol(name->wrapper_value));
        }
      }
      else {
      }
    }
  }
  return (NULL);
}

void print_undefined_super_classes(Object* renamed_Class) {
  // Print all undefined or bad (indirect) super classes of `class'.
  { Surrogate* type = typify(coerce_to_symbol(renamed_Class));
    List* badones = new_list();

    if (!((boolean)(type))) {
      cout << "Illegal class name: " << renamed_Class << endl;
      return;
    }
    collect_bad_super_classes(type, badones);
    if (badones->memberP(type)) {
      if (!((boolean)(type->surrogate_value))) {
        cout << type << " is itself undefined." << endl;
      }
      else {
        cout << type << " itself points to non-class " << type->surrogate_value << endl;
      }
      return;
    }
    { boolean foundP_000 = FALSE;

      { Surrogate* super = NULL;
        Cons* iter_000 = badones->the_cons_list;

        while (!(iter_000 == NIL)) {
          super = ((Surrogate*)(iter_000->value));
          iter_000 = iter_000->rest;
          if (!((boolean)(super->surrogate_value))) {
            foundP_000 = TRUE;
            break;
          }
        }
      }
      if (foundP_000) {
        cout << "The following (indirect) super(s) of " << type << " are undefined:" << endl;
        { Surrogate* super = NULL;
          Cons* iter_001 = badones->the_cons_list;

          while (!(iter_001 == NIL)) {
            super = ((Surrogate*)(iter_001->value));
            iter_001 = iter_001->rest;
            if (!((boolean)(super->surrogate_value))) {
              cout << "    " << super << endl;
            }
          }
        }
      }
    }
    { boolean foundP_001 = FALSE;

      { Surrogate* super = NULL;
        Cons* iter_002 = badones->the_cons_list;

        while (!(iter_002 == NIL)) {
          super = ((Surrogate*)(iter_002->value));
          iter_002 = iter_002->rest;
          if (((boolean)(super->surrogate_value))) {
            foundP_001 = TRUE;
            break;
          }
        }
      }
      if (foundP_001) {
        cout << "The following (indirect) super(s) of " << type << " point to non-classes:" << endl;
        { Surrogate* super = NULL;
          Cons* iter_003 = badones->the_cons_list;

          while (!(iter_003 == NIL)) {
            super = ((Surrogate*)(iter_003->value));
            iter_003 = iter_003->rest;
            if (((boolean)(super->surrogate_value))) {
              cout << "    " << super << " points to " << super->surrogate_value << endl;
            }
          }
        }
      }
    }
  }
}

List* collect_bad_super_classes(Surrogate* type, List* badones) {
  { Object* value = type->surrogate_value;

    if (stella_classP(value)) {
      { Surrogate* super = NULL;
        Cons* iter_000 = ((Class*)(value))->class_direct_supers->the_cons_list;

        while (!(iter_000 == NIL)) {
          super = ((Surrogate*)(iter_000->value));
          iter_000 = iter_000->rest;
          collect_bad_super_classes(super, badones);
        }
      }
    }
    else {
      badones->insert_new(type);
    }
    return (badones);
  }
}

boolean root_classP(Class* renamed_Class) {
  return (renamed_Class->class_direct_supers->emptyP() &&
      (!renamed_Class->mixinP));
}

Taxonomy_Graph* oCLASS_TAXONOMY_GRAPHo = NULL;

void create_class_taxonomy() {
  { Taxonomy_Graph* graph = oCLASS_TAXONOMY_GRAPHo;

    if (!((boolean)(graph))) {
      graph = new_taxonomy_graph();
    }
    else {
      initialize_taxonomy_graph(graph);
    }
    { Class* renamed_Class = NULL;
      Iterator* iter_000 = all_classes(NULL, FALSE);

      while (iter_000->nextP()) {
        renamed_Class = ((Class*)(iter_000->value));
        renamed_Class->class_taxonomy_node = create_taxonomy_node(graph, renamed_Class->class_taxonomy_node, renamed_Class, renamed_Class->class_direct_supers->emptyP());
      }
    }
    { Class* renamed_Class = NULL;
      Iterator* iter_001 = all_classes(NULL, FALSE);

      while (iter_001->nextP()) {
        renamed_Class = ((Class*)(iter_001->value));
        { Surrogate* sub = NULL;
          Cons* iter_002 = renamed_Class->class_direct_subs->the_cons_list;

          while (!(iter_002 == NIL)) {
            sub = ((Surrogate*)(iter_002->value));
            iter_002 = iter_002->rest;
            if (((boolean)(((Class*)(sub->surrogate_value))))) {
              link_taxonomy_nodes(graph, renamed_Class->class_taxonomy_node, ((Class*)(sub->surrogate_value))->class_taxonomy_node);
            }
          }
        }
      }
    }
    finalize_taxonomy_graph(graph);
    oCLASS_TAXONOMY_GRAPHo = graph;
  }
}

boolean subclass_ofP(Class* subclass, Class* superclass) {
  // Return `true' if `subClass' is a subclass of `superClass'.
  if (subclass == superclass) {
    return (TRUE);
  }
  { Taxonomy_Node* subnode = subclass->class_taxonomy_node;
    Taxonomy_Node* supernode = superclass->class_taxonomy_node;
    int sublabel = NULL_INTEGER;

    if ((!((boolean)(subnode))) ||
        ((!((boolean)(supernode))) ||
         ((sublabel = subnode->label) == NULL_INTEGER))) {
      return (subclass->class_all_super_classes->membP(superclass));
    }
    { Interval* interval = NULL;
      Cons* iter_000 = supernode->intervals;

      while (!(iter_000 == NIL)) {
        interval = ((Interval*)(iter_000->value));
        iter_000 = iter_000->rest;
        if ((sublabel >= interval->lower_bound) &&
            (sublabel <= interval->upper_bound)) {
          return (TRUE);
        }
      }
    }
    return (FALSE);
  }
}

boolean fast_subclass_ofP(Class* subclass, Class* superclass) {
  { int sublabel = subclass->class_taxonomy_node->label;

    { Interval* interval = NULL;
      Cons* iter_000 = superclass->class_taxonomy_node->intervals;

      while (!(iter_000 == NIL)) {
        interval = ((Interval*)(iter_000->value));
        iter_000 = iter_000->rest;
        if ((sublabel >= interval->lower_bound) &&
            (sublabel <= interval->upper_bound)) {
          return (TRUE);
        }
      }
    }
    return (FALSE);
  }
}

boolean subtype_ofP(Surrogate* sub_type, Surrogate* super_type) {
  // Return `true' iff the class named `sub-type' is a subclass 
  // of the class named `super-type'.
  // inline method
  { Class* subclass = type_to_class(sub_type);
    Class* superclass = type_to_class(super_type);

    if (!((boolean)(subclass))) {
      { 
        BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
        signal_translation_error();
        if (!(suppress_warningsP())) {
          print_error_context(">> ERROR: ", STANDARD_ERROR);
          *(STANDARD_ERROR->native_stream) << endl << " " << "Reference to non-existent class: " << "`" << de_uglify_parse_tree(sub_type) << "'" << endl << "." << endl;
        }
      }
      return (FALSE);
    }
    if (!((boolean)(superclass))) {
      { 
        BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
        signal_translation_error();
        if (!(suppress_warningsP())) {
          print_error_context(">> ERROR: ", STANDARD_ERROR);
          *(STANDARD_ERROR->native_stream) << endl << " " << "Reference to non-existent class: " << "`" << de_uglify_parse_tree(super_type) << "'" << endl << "." << endl;
        }
      }
      return (FALSE);
    }
    return (subclass_ofP(subclass, superclass));
  }
}

boolean isaP(Object* object, Surrogate* type) {
  // Return `true' iff `object' is an instance of the class named `type'.
  return (subclass_ofP(((Class*)(object->primary_type()->surrogate_value)), ((Class*)(type->surrogate_value))));
}

boolean taxonomy_subclass_ofP(Class* subclass, Class* superclass) {
  return (subclass_ofP(subclass, superclass));
}

boolean taxonomy_isaP(Object* object, Surrogate* type) {
  return (isaP(object, type));
}

Surrogate* intern_slotref(char* classname, char* slotname) {
  return (intern_surrogate(string_concatenate(classname, string_concatenate(".", slotname, 0), 0)));
}

Surrogate* lookup_slotref(Surrogate* self, Symbol* slotname) {
  self = ((Surrogate*)(real_type_specifier(self)));
  { Class* renamed_Class = ((Class*)(self->surrogate_value));
    Slot* slot = NULL;
    Surrogate* slotref = NULL;

    slot = lookup_function(slotname);
    if (!((boolean)(slot))) {
      slot = lookup_slot(renamed_Class, slotname);
    }
    if (!((boolean)(slot))) {
      if (((boolean)(renamed_Class))) {
        return (NULL);
      }
    }
    else {
      self = slot->slot_owner;
      if (!((boolean)(self))) {
        self = SGT_CLASSES_STELLA_ANY;
      }
      slotref = slot->slot_slotref;
    }
    if (!((boolean)(slotref))) {
      { 
        BIND_STELLA_SPECIAL(oMODULEo, Module*, (((boolean)(slot)) ? slot->home_module() : ((Module*)(((Symbol*)(slotname->permanentify()))->home_context))));
        BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
        slotref = intern_slotref(self->symbol_name, slotname->symbol_name);
      }
      slotref->surrogate_value = slot;
      if (((boolean)(slot))) {
        slot->slot_slotref = slotref;
      }
    }
    return (slotref);
  }
}

boolean slotrefP(Surrogate* self) {
  return (string_position(self->symbol_name, '.', 0) != NULL_INTEGER);
}

Slot* lookup_slot_from_slotref(Surrogate* slotref) {
  { Object* value = slotref->surrogate_value;

    if (((boolean)(value))) {
      if (isaP(value, SGT_CLASSES_STELLA_SLOT)) {
        return (((Slot*)(value)));
      }
      *(STANDARD_WARNING->native_stream) << "Warning: " << "lookup-slot-from-slotref: " << "`" << slotref << "'" << " already points to non-slot " << "`" << value << "'" << endl;
      return (NULL);
    }
  }
  { char* slotrefname = slotref->symbol_name;
    Module* slotrefmodule = ((Module*)(slotref->home_context));
    int dotposition = string_position(slotrefname, '.', 0);
    Surrogate* type = NULL;
    Symbol* slotname = NULL;
    Slot* slot = NULL;

    if (dotposition != NULL_INTEGER) {
      type = lookup_surrogate_in_module(string_subsequence(slotrefname, 0, dotposition), slotrefmodule, FALSE);
    }
    else {
      type = SGT_CLASSES_STELLA_ANY;
      dotposition = -1;
    }
    if (((boolean)(type))) {
      type = ((Surrogate*)(real_type_specifier(type)));
    }
    if (((boolean)(type)) &&
        stella_classP(type->surrogate_value)) {
      slotname = lookup_symbol_in_module(string_subsequence(slotrefname, dotposition + 1, NULL_INTEGER), slotrefmodule, FALSE);
      if (((boolean)(slotname))) {
        slot = lookup_function(slotname);
        if (!((boolean)(slot))) {
          slot = lookup_slot(((Class*)(type->surrogate_value)), slotname);
        }
        if (((boolean)(slot))) {
          if (!((boolean)(slot->slot_slotref))) {
            lookup_slotref(((!((boolean)(slot->slot_owner))) ? SGT_CLASSES_STELLA_ANY : slot->slot_owner), slot->slot_name);
          }
          slotref->surrogate_value = slot;
        }
      }
    }
    return (slot);
  }
}

Surrogate* slotref_type(Surrogate* slotref) {
  return (((Slot*)(slotref->surrogate_value))->type());
}

void help_startup_classes1() {
  {
    SGT_CLASSES_STELLA_CLASS = ((Surrogate*)(intern_rigid_symbol_wrt_module("CLASS", NULL, 1)));
    SGT_CLASSES_STELLA_OBJECT = ((Surrogate*)(intern_rigid_symbol_wrt_module("OBJECT", NULL, 1)));
    KWD_CLASSES_PUBLICp = ((Keyword*)(intern_rigid_symbol_wrt_module("PUBLIC?", NULL, 2)));
    SYM_CLASSES_STELLA_TRUE = ((Symbol*)(intern_rigid_symbol_wrt_module("TRUE", NULL, 0)));
    SYM_CLASSES_STELLA_FALSE = ((Symbol*)(intern_rigid_symbol_wrt_module("FALSE", NULL, 0)));
    SGT_CLASSES_STELLA_UNKNOWN = ((Surrogate*)(intern_rigid_symbol_wrt_module("UNKNOWN", NULL, 1)));
    KWD_CLASSES_PARAMETERS = ((Keyword*)(intern_rigid_symbol_wrt_module("PARAMETERS", NULL, 2)));
    SYM_CLASSES_STELLA_SLOT_TYPE_SPECIFIER = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-TYPE-SPECIFIER", NULL, 0)));
    KWD_CLASSES_REQUIREDp = ((Keyword*)(intern_rigid_symbol_wrt_module("REQUIRED?", NULL, 2)));
    KWD_CLASSES_COMPONENTp = ((Keyword*)(intern_rigid_symbol_wrt_module("COMPONENT?", NULL, 2)));
    KWD_CLASSES_READ_ONLYp = ((Keyword*)(intern_rigid_symbol_wrt_module("READ-ONLY?", NULL, 2)));
    KWD_CLASSES_ACTIVEp = ((Keyword*)(intern_rigid_symbol_wrt_module("ACTIVE?", NULL, 2)));
    SYM_CLASSES_STELLA_STORED_ACTIVEp = ((Symbol*)(intern_rigid_symbol_wrt_module("STORED-ACTIVE?", NULL, 0)));
    KWD_CLASSES_CONTEXT_SENSITIVEp = ((Keyword*)(intern_rigid_symbol_wrt_module("CONTEXT-SENSITIVE?", NULL, 2)));
    KWD_CLASSES_HARDWIREDp = ((Keyword*)(intern_rigid_symbol_wrt_module("HARDWIRED?", NULL, 2)));
    KWD_CLASSES_ABSTRACTp = ((Keyword*)(intern_rigid_symbol_wrt_module("ABSTRACT?", NULL, 2)));
    KWD_CLASSES_ALLOCATION = ((Keyword*)(intern_rigid_symbol_wrt_module("ALLOCATION", NULL, 2)));
    KWD_CLASSES_CLASS = ((Keyword*)(intern_rigid_symbol_wrt_module("CLASS", NULL, 2)));
    SYM_CLASSES_STELLA_SLOT_ALLOCATION = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-ALLOCATION", NULL, 0)));
    KWD_CLASSES_INITIALLY = ((Keyword*)(intern_rigid_symbol_wrt_module("INITIALLY", NULL, 2)));
    SYM_CLASSES_STELLA_SLOT_INITIAL_VALUE = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-INITIAL-VALUE", NULL, 0)));
    KWD_CLASSES_DEFAULT = ((Keyword*)(intern_rigid_symbol_wrt_module("DEFAULT", NULL, 2)));
    SYM_CLASSES_STELLA_SLOT_DEFAULT_EXPRESSION = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-DEFAULT-EXPRESSION", NULL, 0)));
    KWD_CLASSES_READER = ((Keyword*)(intern_rigid_symbol_wrt_module("READER", NULL, 2)));
    SYM_CLASSES_STELLA_SLOT_READER = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-READER", NULL, 0)));
    KWD_CLASSES_WRITER = ((Keyword*)(intern_rigid_symbol_wrt_module("WRITER", NULL, 2)));
    SYM_CLASSES_STELLA_SLOT_WRITER = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-WRITER", NULL, 0)));
    KWD_CLASSES_INVERSE = ((Keyword*)(intern_rigid_symbol_wrt_module("INVERSE", NULL, 2)));
    SYM_CLASSES_STELLA_SLOT_INVERSE = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-INVERSE", NULL, 0)));
    KWD_CLASSES_RENAMES = ((Keyword*)(intern_rigid_symbol_wrt_module("RENAMES", NULL, 2)));
    SYM_CLASSES_STELLA_SLOT_RENAMES = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-RENAMES", NULL, 0)));
    KWD_CLASSES_DOCUMENTATION = ((Keyword*)(intern_rigid_symbol_wrt_module("DOCUMENTATION", NULL, 2)));
    SYM_CLASSES_STELLA_DOCUMENTATION = ((Symbol*)(intern_rigid_symbol_wrt_module("DOCUMENTATION", NULL, 0)));
    KWD_CLASSES_PROPERTIES = ((Keyword*)(intern_rigid_symbol_wrt_module("PROPERTIES", NULL, 2)));
    SYM_CLASSES_STELLA_PROPERTIES = ((Symbol*)(intern_rigid_symbol_wrt_module("PROPERTIES", NULL, 0)));
    KWD_CLASSES_META_ATTRIBUTES = ((Keyword*)(intern_rigid_symbol_wrt_module("META-ATTRIBUTES", NULL, 2)));
    SYM_CLASSES_STELLA_META_ATTRIBUTES = ((Symbol*)(intern_rigid_symbol_wrt_module("META-ATTRIBUTES", NULL, 0)));
    KWD_CLASSES_OPTION_KEYWORD = ((Keyword*)(intern_rigid_symbol_wrt_module("OPTION-KEYWORD", NULL, 2)));
    SYM_CLASSES_STELLA_SLOT_OPTION_KEYWORD = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-OPTION-KEYWORD", NULL, 0)));
    KWD_CLASSES_OPTION_HANDLER = ((Keyword*)(intern_rigid_symbol_wrt_module("OPTION-HANDLER", NULL, 2)));
    SYM_CLASSES_STELLA_SLOT_OPTION_HANDLER = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-OPTION-HANDLER", NULL, 0)));
    SGT_CLASSES_STELLA_BOOLEAN = ((Surrogate*)(intern_rigid_symbol_wrt_module("BOOLEAN", NULL, 1)));
    KWD_CLASSES_BIT = ((Keyword*)(intern_rigid_symbol_wrt_module("BIT", NULL, 2)));
    KWD_CLASSES_CL_NATIVE_TYPE = ((Keyword*)(intern_rigid_symbol_wrt_module("CL-NATIVE-TYPE", NULL, 2)));
    SYM_CLASSES_STELLA_CLASS_CL_NATIVE_TYPE = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-CL-NATIVE-TYPE", NULL, 0)));
    KWD_CLASSES_CPP_NATIVE_TYPE = ((Keyword*)(intern_rigid_symbol_wrt_module("CPP-NATIVE-TYPE", NULL, 2)));
    SYM_CLASSES_STELLA_CLASS_CPP_NATIVE_TYPE = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-CPP-NATIVE-TYPE", NULL, 0)));
    KWD_CLASSES_IDL_NATIVE_TYPE = ((Keyword*)(intern_rigid_symbol_wrt_module("IDL-NATIVE-TYPE", NULL, 2)));
    SYM_CLASSES_STELLA_CLASS_IDL_NATIVE_TYPE = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-IDL-NATIVE-TYPE", NULL, 0)));
    KWD_CLASSES_JAVA_NATIVE_TYPE = ((Keyword*)(intern_rigid_symbol_wrt_module("JAVA-NATIVE-TYPE", NULL, 2)));
    SYM_CLASSES_STELLA_CLASS_JAVA_NATIVE_TYPE = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-JAVA-NATIVE-TYPE", NULL, 0)));
    KWD_CLASSES_CL_STRUCTp = ((Keyword*)(intern_rigid_symbol_wrt_module("CL-STRUCT?", NULL, 2)));
    KWD_CLASSES_MIXINp = ((Keyword*)(intern_rigid_symbol_wrt_module("MIXIN?", NULL, 2)));
    KWD_CLASSES_RECYCLE_METHOD = ((Keyword*)(intern_rigid_symbol_wrt_module("RECYCLE-METHOD", NULL, 2)));
    SYM_CLASSES_STELLA_CLASS_RECYCLE_METHOD = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-RECYCLE-METHOD", NULL, 0)));
    KWD_CLASSES_EXTENSION = ((Keyword*)(intern_rigid_symbol_wrt_module("EXTENSION", NULL, 2)));
    SYM_CLASSES_STELLA_CLASS_EXTENSION_NAME = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-EXTENSION-NAME", NULL, 0)));
    KWD_CLASSES_CREATOR = ((Keyword*)(intern_rigid_symbol_wrt_module("CREATOR", NULL, 2)));
    SYM_CLASSES_STELLA_CLASS_CREATOR = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-CREATOR", NULL, 0)));
    KWD_CLASSES_INITIALIZER = ((Keyword*)(intern_rigid_symbol_wrt_module("INITIALIZER", NULL, 2)));
  }
}

void help_startup_classes2() {
  {
    SYM_CLASSES_STELLA_CLASS_INITIALIZER = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-INITIALIZER", NULL, 0)));
    KWD_CLASSES_TERMINATOR = ((Keyword*)(intern_rigid_symbol_wrt_module("TERMINATOR", NULL, 2)));
    SYM_CLASSES_STELLA_CLASS_TERMINATOR = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-TERMINATOR", NULL, 0)));
    KWD_CLASSES_DESTRUCTOR = ((Keyword*)(intern_rigid_symbol_wrt_module("DESTRUCTOR", NULL, 2)));
    SYM_CLASSES_STELLA_CLASS_DESTRUCTOR = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-DESTRUCTOR", NULL, 0)));
    KWD_CLASSES_INITIAL_VALUE = ((Keyword*)(intern_rigid_symbol_wrt_module("INITIAL-VALUE", NULL, 2)));
    SYM_CLASSES_STELLA_CLASS_INITIAL_VALUE = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-INITIAL-VALUE", NULL, 0)));
    KWD_CLASSES_PRINT_FORM = ((Keyword*)(intern_rigid_symbol_wrt_module("PRINT-FORM", NULL, 2)));
    SYM_CLASSES_STELLA_PRINT_FORM = ((Symbol*)(intern_rigid_symbol_wrt_module("PRINT-FORM", NULL, 0)));
    KWD_CLASSES_EQUALITY_TEST = ((Keyword*)(intern_rigid_symbol_wrt_module("EQUALITY-TEST", NULL, 2)));
    KWD_CLASSES_KEY = ((Keyword*)(intern_rigid_symbol_wrt_module("KEY", NULL, 2)));
    SYM_CLASSES_STELLA_CLASS_KEY = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-KEY", NULL, 0)));
    SYM_CLASSES_STELLA_CLASS_PARAMETERS = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-PARAMETERS", NULL, 0)));
    KWD_CLASSES_SYNONYMS = ((Keyword*)(intern_rigid_symbol_wrt_module("SYNONYMS", NULL, 2)));
    SYM_CLASSES_STELLA_CLASS_SYNONYMS = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-SYNONYMS", NULL, 0)));
    KWD_CLASSES_CHILDREN = ((Keyword*)(intern_rigid_symbol_wrt_module("CHILDREN", NULL, 2)));
    SGT_CLASSES_STELLA_METHOD_SLOT = ((Surrogate*)(intern_rigid_symbol_wrt_module("METHOD-SLOT", NULL, 1)));
    SYM_CLASSES_STELLA_SLOT_GUARD_DEMONS = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-GUARD-DEMONS", NULL, 0)));
    SYM_CLASSES_STELLA_SLOT_DEMONS = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-DEMONS", NULL, 0)));
    SYM_CLASSES_STELLA_CLASS_CONSTRUCTOR_DEMONS = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-CONSTRUCTOR-DEMONS", NULL, 0)));
    SYM_CLASSES_STELLA_CLASS_GUARD_CONSTRUCTOR_DEMONS = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-GUARD-CONSTRUCTOR-DEMONS", NULL, 0)));
    SYM_CLASSES_STELLA_CLASS_DESTRUCTOR_DEMONS = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-DESTRUCTOR-DEMONS", NULL, 0)));
    SYM_CLASSES_STELLA_CLASS_GUARD_DESTRUCTOR_DEMONS = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-GUARD-DESTRUCTOR-DEMONS", NULL, 0)));
    SGT_CLASSES_STELLA_ACTIVE_OBJECT = ((Surrogate*)(intern_rigid_symbol_wrt_module("ACTIVE-OBJECT", NULL, 1)));
    SGT_CLASSES_STELLA_STANDARD_OBJECT = ((Surrogate*)(intern_rigid_symbol_wrt_module("STANDARD-OBJECT", NULL, 1)));
    SYM_CLASSES_STELLA_PRIMARY_TYPE = ((Symbol*)(intern_rigid_symbol_wrt_module("PRIMARY-TYPE", NULL, 0)));
    SYM_CLASSES_STELLA_SELF = ((Symbol*)(intern_rigid_symbol_wrt_module("SELF", NULL, 0)));
    KWD_CLASSES_TYPE = ((Keyword*)(intern_rigid_symbol_wrt_module("TYPE", NULL, 2)));
    SYM_CLASSES_STELLA_TYPE = ((Symbol*)(intern_rigid_symbol_wrt_module("TYPE", NULL, 0)));
    KWD_CLASSES_AUXILIARYp = ((Keyword*)(intern_rigid_symbol_wrt_module("AUXILIARY?", NULL, 2)));
    SYM_CLASSES_STELLA_RETURN = ((Symbol*)(intern_rigid_symbol_wrt_module("RETURN", NULL, 0)));
    SYM_CLASSES_STELLA_CLASS_EXTENSION = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-EXTENSION", NULL, 0)));
    SYM_CLASSES_STELLA_BADp = ((Symbol*)(intern_rigid_symbol_wrt_module("BAD?", NULL, 0)));
    SGT_CLASSES_STELLA_SET = ((Surrogate*)(intern_rigid_symbol_wrt_module("SET", NULL, 1)));
    SGT_CLASSES_STELLA_ACTIVE_SET = ((Surrogate*)(intern_rigid_symbol_wrt_module("ACTIVE-SET", NULL, 1)));
    SGT_CLASSES_STELLA_LIST = ((Surrogate*)(intern_rigid_symbol_wrt_module("LIST", NULL, 1)));
    SGT_CLASSES_STELLA_ACTIVE_LIST = ((Surrogate*)(intern_rigid_symbol_wrt_module("ACTIVE-LIST", NULL, 1)));
    SGT_CLASSES_STELLA_COLLECTION = ((Surrogate*)(intern_rigid_symbol_wrt_module("COLLECTION", NULL, 1)));
    SGT_CLASSES_STELLA_SET_MIXIN = ((Surrogate*)(intern_rigid_symbol_wrt_module("SET-MIXIN", NULL, 1)));
    SYM_CLASSES_STELLA_INVERSE = ((Symbol*)(intern_rigid_symbol_wrt_module("INVERSE", NULL, 0)));
    KWD_CLASSES_DYNAMIC = ((Keyword*)(intern_rigid_symbol_wrt_module("DYNAMIC", NULL, 2)));
    SGT_CLASSES_STELLA_DYNAMIC_SLOTS_MIXIN = ((Surrogate*)(intern_rigid_symbol_wrt_module("DYNAMIC-SLOTS-MIXIN", NULL, 1)));
    KWD_CLASSES_INSTANCE = ((Keyword*)(intern_rigid_symbol_wrt_module("INSTANCE", NULL, 2)));
    SGT_CLASSES_STELLA_CONTEXT_SENSITIVE_OBJECT = ((Surrogate*)(intern_rigid_symbol_wrt_module("CONTEXT-SENSITIVE-OBJECT", NULL, 1)));
    SGT_CLASSES_STELLA_CONTEXT_SENSITIVE_MIXIN = ((Surrogate*)(intern_rigid_symbol_wrt_module("CONTEXT-SENSITIVE-MIXIN", NULL, 1)));
    SGT_CLASSES_STELLA_VOID = ((Surrogate*)(intern_rigid_symbol_wrt_module("VOID", NULL, 1)));
    SGT_CLASSES_STELLA_TABLE = ((Surrogate*)(intern_rigid_symbol_wrt_module("TABLE", NULL, 1)));
    SYM_CLASSES_STELLA_CLASS_REQUIRED_SLOT_NAMES = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-REQUIRED-SLOT-NAMES", NULL, 0)));
    SGT_CLASSES_STELLA_ALL_CLASS_SLOTS_ITERATOR = ((Surrogate*)(intern_rigid_symbol_wrt_module("ALL-CLASS-SLOTS-ITERATOR", NULL, 1)));
    SYM_CLASSES_STELLA_ITERATOR_CONS_LIST = ((Symbol*)(intern_rigid_symbol_wrt_module("ITERATOR-CONS-LIST", NULL, 0)));
    SYM_CLASSES_STELLA_ITERATOR_OBJECT = ((Symbol*)(intern_rigid_symbol_wrt_module("ITERATOR-OBJECT", NULL, 0)));
    SGT_CLASSES_STELLA_GENERALIZED_SYMBOL = ((Surrogate*)(intern_rigid_symbol_wrt_module("GENERALIZED-SYMBOL", NULL, 1)));
    SYM_CLASSES_STELLA_ANY = ((Symbol*)(intern_rigid_symbol_wrt_module("ANY", NULL, 0)));
    SGT_CLASSES_STELLA_ANY = ((Surrogate*)(intern_rigid_symbol_wrt_module("ANY", NULL, 1)));
    SGT_CLASSES_STELLA_SLOT = ((Surrogate*)(intern_rigid_symbol_wrt_module("SLOT", NULL, 1)));
    SYM_CLASSES_STELLA_STARTUP_CLASSES = ((Symbol*)(intern_rigid_symbol_wrt_module("STARTUP-CLASSES", NULL, 0)));
    SYM_CLASSES_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-STARTUP-CLASSNAME", NULL, 0)));
  }
}

void help_startup_classes3() {
  {
    define_function_object("CLASS-NAME", "(DEFUN (CLASS-NAME STRING) ((CLASS CLASS)) :PUBLIC? TRUE)", ((cpp_function_code)(&class_name)), NULL);
    define_function_object("CLASS-SYMBOL", "(DEFUN (CLASS-SYMBOL SYMBOL) ((CLASS CLASS)) :PUBLIC? TRUE)", ((cpp_function_code)(&class_symbol)), NULL);
    define_method_object("(DEFMETHOD (PRIMARY-CLASS CLASS) ((SELF OBJECT)) :PUBLIC? TRUE)", ((cpp_method_code)(&Object::primary_class)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (LOOKUP-CLASS CLASS) ((NAME STRING)) :DOCUMENTATION \"Return a class with name `name'.  Scan all\nvisible surrogates looking for one that has a class defined for it.\" :PUBLIC? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (LOOKUP-CLASS CLASS) ((NAME SYMBOL)) :DOCUMENTATION \"Return a class with name `name'.  Scan all\nvisible surrogates looking for one that has a class defined for it.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Symbol::lookup_class)), ((cpp_method_code)(NULL)));
    define_function_object("TYPE-TO-CLASS", "(DEFUN (TYPE-TO-CLASS CLASS) ((TYPE TYPE)))", ((cpp_function_code)(&type_to_class)), NULL);
    define_method_object("(DEFMETHOD (GET-STELLA-CLASS CLASS) ((CLASS-NAME TYPE) (ERROR? BOOLEAN)) :DOCUMENTATION \"Return a class with name `class-name'.  If none exists, break\nif `error?', else return `null'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Surrogate::get_stella_class)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (GET-STELLA-CLASS CLASS) ((CLASS-NAME STRING) (ERROR? BOOLEAN)) :DOCUMENTATION \"Return a class with name `class-name'.  If none exists, break\nif `error?', else return `null'.\" :PUBLIC? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (GET-STELLA-CLASS CLASS) ((CLASS-NAME SYMBOL) (ERROR? BOOLEAN)) :DOCUMENTATION \"Return a class with name `class-name'.  If non exists, break\nif `error?', else return `null'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Symbol::get_stella_class)), ((cpp_method_code)(NULL)));
    define_function_object("BIND-TO-SURROGATE?", "(DEFUN (BIND-TO-SURROGATE? BOOLEAN OBJECT SURROGATE) ((SELF OBJECT) (NAME STRING) (CLIPOLDVALUE? BOOLEAN) (ASKFORPERMISSION? BOOLEAN)) :PUBLIC? TRUE)", ((cpp_function_code)(&bind_to_surrogateP)), NULL);
    define_method_object("(DEFMETHOD UNBIND-FROM-SURROGATE ((SELF CLASS)))", ((cpp_method_code)(&Class::unbind_from_surrogate)), ((cpp_method_code)(NULL)));
    define_function_object("SHADOW-SYMBOL", "(DEFUN (SHADOW-SYMBOL SYMBOL) ((NAME STRING)) :PUBLIC? TRUE)", ((cpp_function_code)(&shadow_symbol)), NULL);
    define_function_object("SHADOW-SURROGATE", "(DEFUN (SHADOW-SURROGATE SURROGATE) ((NAME STRING)) :PUBLIC? TRUE)", ((cpp_function_code)(&shadow_surrogate)), NULL);
    define_function_object("SHADOWED-SYMBOL?", "(DEFUN (SHADOWED-SYMBOL? BOOLEAN) ((SYMBOL GENERALIZED-SYMBOL)) :DOCUMENTATION \"Return `true' if `symbol' is shadowed in its home module.\" :PUBLIC? TRUE)", ((cpp_function_code)(&shadowed_symbolP)), NULL);
    define_function_object("GET-CONSTRUCTOR", "(DEFUN (GET-CONSTRUCTOR FUNCTION-CODE) ((CLASS CLASS) (WARN? BOOLEAN)))", ((cpp_function_code)(&get_constructor)), NULL);
    define_function_object("CREATE-OBJECT", "(DEFUN (CREATE-OBJECT OBJECT) ((TYPE TYPE) |&REST| (INITIAL-VALUE-PAIRS OBJECT)) :DOCUMENTATION \"Funcallable version of the `new' operator.\nReturn an instance of the class named by `type'.  If `initial-value-pairs'\nis supplied, it has to be a key/value list similar to what's accepted by `new'\nand the named slots will be initialized with the supplied values.  Similar to\n`new', all required arguments for `type' must be included.  Since all the\nslot initialization, etc. is handled dynamically at run time, `create-object'\nis much slower than `new'; therefore, it should only be used if `type' cannot\nbe known at translation time.\" :PUBLIC? TRUE)", ((cpp_function_code)(&create_object)), NULL);
    define_method_object("(DEFMETHOD FREE ((SELF OBJECT)) :DOCUMENTATION \"Default method.  Deallocate storage for `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Object::free)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD FREE ((SELF ACTIVE-OBJECT)) :DOCUMENTATION \"Remove all pointers between `self' and other objects,\nand then deallocate the storage for self.\")", ((cpp_method_code)(&Active_Object::free)), ((cpp_method_code)(NULL)));
    define_function_object("COERCE-TO-BOOLEAN", "(DEFUN (COERCE-TO-BOOLEAN BOOLEAN-WRAPPER) ((OBJECT OBJECT)) :PUBLIC? TRUE)", ((cpp_function_code)(&coerce_to_boolean)), NULL);
    define_function_object("DEFINE-STELLA-CLASS", "(DEFUN (DEFINE-STELLA-CLASS CLASS) ((NAME TYPE) (SUPERS (LIST OF TYPE)) (SLOTS (LIST OF SLOT)) (OPTIONS KEYWORD-KEY-VALUE-LIST)) :DOCUMENTATION \"Return a Stella class with name `name'.\nCaution:  If the class already exists, the Stella class object gets\nredefined, but the native C++ class is not redefined.\" :PUBLIC? TRUE)", ((cpp_function_code)(&define_stella_class)), NULL);
    define_function_object("DEFINE-STELLA-SLOT", "(DEFUN (DEFINE-STELLA-SLOT SLOT) ((NAME SYMBOL) (OWNER TYPE) (BASETYPE TYPE) (TYPESPECIFIER CONS) (OPTIONS KEYWORD-KEY-VALUE-LIST)))", ((cpp_function_code)(&define_stella_slot)), NULL);
    define_function_object("INCORPORATE-CLASS-OPTIONS", "(DEFUN INCORPORATE-CLASS-OPTIONS ((CLASS CLASS) (OPTIONS KEYWORD-KEY-VALUE-LIST)))", ((cpp_function_code)(&incorporate_class_options)), NULL);
    define_function_object("INLINE-METHOD?", "(DEFUN (INLINE-METHOD? BOOLEAN) ((SLOT SLOT)))", ((cpp_function_code)(&inline_methodP)), NULL);
    define_function_object("TRANSFER-EXTERNAL-SLOT-TO-NEW-CLASS", "(DEFUN TRANSFER-EXTERNAL-SLOT-TO-NEW-CLASS ((EXTERNALSLOT SLOT) (NEWCLASS CLASS)))", ((cpp_function_code)(&transfer_external_slot_to_new_class)), NULL);
    define_function_object("TRANSFER-DEMONS-FROM-OLDCLASS", "(DEFUN TRANSFER-DEMONS-FROM-OLDCLASS ((OLDCLASS CLASS) (NEWCLASS CLASS)))", ((cpp_function_code)(&transfer_demons_from_oldclass)), NULL);
    define_function_object("UNDEFINE-OLD-CLASS", "(DEFUN UNDEFINE-OLD-CLASS ((OLDCLASS CLASS) (NEWCLASS CLASS)))", ((cpp_function_code)(&undefine_old_class)), NULL);
    define_method_object("(DEFMETHOD DESTROY-CLASS ((SELF CLASS)) :DOCUMENTATION \"Destroy the Stella class `self'.  \nUnfinalize its subclasses (if it has any).\" :PUBLIC? TRUE)", ((cpp_method_code)(&Class::destroy_class)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD DESTROY-CLASS ((SELF TYPE)) :PUBLIC? TRUE)", ((cpp_method_code)(&Surrogate::destroy_class)), ((cpp_method_code)(NULL)));
    define_function_object("DESTROY-CLASS-AND-SUBCLASSES", "(DEFUN DESTROY-CLASS-AND-SUBCLASSES ((SELF CLASS)) :DOCUMENTATION \"Destroy the Stella class `self' and all its subclasses.\")", ((cpp_function_code)(&destroy_class_and_subclasses)), NULL);
    define_method_object("(DEFMETHOD (PRIMITIVE? BOOLEAN) ((SELF RELATION)) :DOCUMENTATION \"Return `true' if `self' is not a defined relation.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Relation::primitiveP)), ((cpp_method_code)(NULL)));
    define_function_object("INSERT-PRIMITIVE-SUPER", "(DEFUN INSERT-PRIMITIVE-SUPER ((NEWSUPER TYPE) (SUPERS (LIST OF TYPE))))", ((cpp_function_code)(&insert_primitive_super)), NULL);
    define_function_object("COLLECT-MOST-SPECIFIC-PRIMITIVE-SUPERS", "(DEFUN COLLECT-MOST-SPECIFIC-PRIMITIVE-SUPERS ((CLASS CLASS) (SUPERS (LIST OF TYPE))))", ((cpp_function_code)(&collect_most_specific_primitive_supers)), NULL);
    define_function_object("MOST-SPECIFIC-PRIMITIVE-SUPERS", "(DEFUN (MOST-SPECIFIC-PRIMITIVE-SUPERS (LIST OF TYPE)) ((CLASS CLASS)))", ((cpp_function_code)(&most_specific_primitive_supers)), NULL);
    define_function_object("CLASS-NATIVE-SUPERS", "(DEFUN (CLASS-NATIVE-SUPERS (LIST OF TYPE)) ((CLASS CLASS)))", ((cpp_function_code)(&class_native_supers)), NULL);
    define_function_object("REMEMBER-UNFINALIZED-CLASS", "(DEFUN REMEMBER-UNFINALIZED-CLASS ((CLASS CLASS) (FORCE? BOOLEAN)))", ((cpp_function_code)(&remember_unfinalized_class)), NULL);
    define_function_object("FORGET-UNFINALIZED-CLASS", "(DEFUN FORGET-UNFINALIZED-CLASS ((CLASS CLASS)))", ((cpp_function_code)(&forget_unfinalized_class)), NULL);
    define_function_object("FINALIZE-CLASSES", "(DEFUN FINALIZE-CLASSES () :DOCUMENTATION \"Finalize all currently unfinalized classes.\" :PUBLIC? TRUE)", ((cpp_function_code)(&finalize_classes)), NULL);
    define_function_object("RESOLVE-CHILDREN-REFERENCES", "(DEFUN RESOLVE-CHILDREN-REFERENCES ())", ((cpp_function_code)(&resolve_children_references)), NULL);
    define_function_object("FINALIZE-SLOTS", "(DEFUN FINALIZE-SLOTS () :PUBLIC? TRUE :DOCUMENTATION \"Finalize all currently unfinalized slots.\")", ((cpp_function_code)(&finalize_slots)), NULL);
    define_function_object("CLEANUP-UNFINALIZED-CLASSES", "(DEFUN CLEANUP-UNFINALIZED-CLASSES () :PUBLIC? TRUE :DOCUMENTATION \"Remove all finalized classes from `*UNFINALIZED-CLASSES*',\nand set `*NEWLY-UNFINALIZED-CLASSES?*' to `false'.\")", ((cpp_function_code)(&cleanup_unfinalized_classes)), NULL);
    define_function_object("FINALIZE-CLASSES-AND-SLOTS", "(DEFUN FINALIZE-CLASSES-AND-SLOTS () :PUBLIC? TRUE :DOCUMENTATION \"Finalize all currently unfinalized classes and slots.\")", ((cpp_function_code)(&finalize_classes_and_slots)), NULL);
    define_function_object("ACTIVATE-CLASS", "(DEFUN ACTIVATE-CLASS ((CLASS CLASS)))", ((cpp_function_code)(&activate_class)), NULL);
    define_function_object("ADD-PRIMARY-TYPE", "(DEFUN ADD-PRIMARY-TYPE ((CLASS CLASS)))", ((cpp_function_code)(&add_primary_type)), NULL);
    define_function_object("FINALIZE-ONE-CLASS", "(DEFUN FINALIZE-ONE-CLASS ((CLASS CLASS)))", ((cpp_function_code)(&finalize_one_class)), NULL);
    define_function_object("FINALIZE-CLASS", "(DEFUN (FINALIZE-CLASS BOOLEAN) ((CLASS CLASS)))", ((cpp_function_code)(&finalize_class)), NULL);
    define_function_object("HELP-FINALIZE-CLASS", "(DEFUN (HELP-FINALIZE-CLASS BOOLEAN) ((CLASS CLASS) (FINALIZEDPARENT TYPE)))", ((cpp_function_code)(&help_finalize_class)), NULL);
    define_function_object("UNFINALIZE-CLASS-AND-SUBCLASSES", "(DEFUN UNFINALIZE-CLASS-AND-SUBCLASSES ((CLASS CLASS)))", ((cpp_function_code)(&unfinalize_class_and_subclasses)), NULL);
    define_function_object("ATTACH-SLOT-TO-OWNER", "(DEFUN (ATTACH-SLOT-TO-OWNER SLOT) ((NEWSLOT SLOT)) :PUBLIC? TRUE)", ((cpp_function_code)(&attach_slot_to_owner)), NULL);
    define_function_object("COMPUTE-SLOT-DIRECT-EQUIVALENT", "(DEFUN COMPUTE-SLOT-DIRECT-EQUIVALENT ((SELF SLOT)))", ((cpp_function_code)(&compute_slot_direct_equivalent)), NULL);
    define_function_object("CHECK-CONFORMANCE-OF-SLOT-SIGNATURE", "(DEFUN CHECK-CONFORMANCE-OF-SLOT-SIGNATURE ((SELF SLOT)))", ((cpp_function_code)(&check_conformance_of_slot_signature)), NULL);
    define_function_object("COLLECTION-TO-ACTIVE-COLLECTION", "(DEFUN (COLLECTION-TO-ACTIVE-COLLECTION TYPE) ((TYPE TYPE)))", ((cpp_function_code)(&collection_to_active_collection)), NULL);
    define_function_object("ACTIVATE-SLOT", "(DEFUN ACTIVATE-SLOT ((SELF STORAGE-SLOT)))", ((cpp_function_code)(&activate_slot)), NULL);
    define_method_object("(DEFMETHOD FINALIZE-SLOT-TYPE-COMPUTATIONS ((SELF SLOT)))", ((cpp_method_code)(&Slot::finalize_slot_type_computations)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD FINALIZE-SLOT-TYPE-COMPUTATIONS ((SELF STORAGE-SLOT)))", ((cpp_method_code)(&Storage_Slot::finalize_slot_type_computations)), ((cpp_method_code)(NULL)));
    define_function_object("MULTI-VALUED-SLOT-WITH-DUPLICATES?", "(DEFUN (MULTI-VALUED-SLOT-WITH-DUPLICATES? BOOLEAN) ((SELF SLOT)))", ((cpp_function_code)(&multi_valued_slot_with_duplicatesP)), NULL);
    define_function_object("COMPUTE-SLOT-INVERSES", "(DEFUN COMPUTE-SLOT-INVERSES ((SELF SLOT)))", ((cpp_function_code)(&compute_slot_inverses)), NULL);
    define_method_object("(DEFMETHOD HELP-FINALIZE-LOCAL-SLOT ((SELF SLOT)))", ((cpp_method_code)(&Slot::help_finalize_local_slot)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD HELP-FINALIZE-LOCAL-SLOT ((SELF STORAGE-SLOT)))", ((cpp_method_code)(&Storage_Slot::help_finalize_local_slot)), ((cpp_method_code)(NULL)));
    define_function_object("FINALIZE-LOCAL-SLOT", "(DEFUN FINALIZE-LOCAL-SLOT ((SELF SLOT)))", ((cpp_function_code)(&finalize_local_slot)), NULL);
    define_method_object("(DEFMETHOD UNFINALIZE-LOCAL-SLOT ((SELF SLOT)))", ((cpp_method_code)(&Slot::unfinalize_local_slot)), ((cpp_method_code)(NULL)));
  }
}

void help_startup_classes4() {
  {
    define_method_object("(DEFMETHOD UNFINALIZE-LOCAL-SLOT ((SELF STORAGE-SLOT)))", ((cpp_method_code)(&Storage_Slot::unfinalize_local_slot)), ((cpp_method_code)(NULL)));
    define_function_object("FINALIZE-CLASS-SLOTS", "(DEFUN FINALIZE-CLASS-SLOTS ((CLASS CLASS)))", ((cpp_function_code)(&finalize_class_slots)), NULL);
    define_function_object("UNFINALIZE-CLASS-SLOTS", "(DEFUN UNFINALIZE-CLASS-SLOTS ((CLASS CLASS)))", ((cpp_function_code)(&unfinalize_class_slots)), NULL);
    define_function_object("HELP-UNFINALIZE-CLASS-SLOTS", "(DEFUN HELP-UNFINALIZE-CLASS-SLOTS ((CLASS CLASS)))", ((cpp_function_code)(&help_unfinalize_class_slots)), NULL);
    define_method_object("(DEFMETHOD (MULTIPLE-PARENTS? BOOLEAN) ((CLASS CLASS)) :DOCUMENTATION \"Return `true' if `class' has more than one direct superclass.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Class::multiple_parentsP)), ((cpp_method_code)(NULL)));
    define_function_object("ADD-DIRECT-SUPERS-BACK-LINKS", "(DEFUN ADD-DIRECT-SUPERS-BACK-LINKS ((CLASS CLASS)))", ((cpp_function_code)(&add_direct_supers_back_links)), NULL);
    define_function_object("ADD-DIRECT-SUBS-BACK-LINKS", "(DEFUN ADD-DIRECT-SUBS-BACK-LINKS ((CLASS CLASS)))", ((cpp_function_code)(&add_direct_subs_back_links)), NULL);
    define_function_object("REMOVE-DIRECT-SUPERS-BACK-LINKS", "(DEFUN REMOVE-DIRECT-SUPERS-BACK-LINKS ((CLASS CLASS)))", ((cpp_function_code)(&remove_direct_supers_back_links)), NULL);
    define_function_object("COLLECT-DIRECT-SUPER-CLASSES", "(DEFUN COLLECT-DIRECT-SUPER-CLASSES ((CLASS CLASS) (PARENTS (LIST OF CLASS))))", ((cpp_function_code)(&collect_direct_super_classes)), NULL);
    define_function_object("INHERIT-SUPERCLASSES", "(DEFUN INHERIT-SUPERCLASSES ((CLASS CLASS)))", ((cpp_function_code)(&inherit_superclasses)), NULL);
    define_function_object("UNINHERIT-SUPERCLASSES", "(DEFUN UNINHERIT-SUPERCLASSES ((CLASS CLASS)))", ((cpp_function_code)(&uninherit_superclasses)), NULL);
    define_function_object("TWO-ARGUMENT-LEAST-COMMON-SUPERCLASS", "(DEFUN (TWO-ARGUMENT-LEAST-COMMON-SUPERCLASS CLASS) ((CLASS1 CLASS) (CLASS2 CLASS)) :DOCUMENTATION \"Return the most specific class that is a superclass of\nboth `class1' and `class2'.  If there is more than one, arbitrarily pick one.\nIf there is none, return `null'.\")", ((cpp_function_code)(&two_argument_least_common_superclass)), NULL);
    define_function_object("TWO-ARGUMENT-LEAST-COMMON-SUPERTYPE", "(DEFUN (TWO-ARGUMENT-LEAST-COMMON-SUPERTYPE TYPE-SPEC) ((TYPE1 TYPE-SPEC) (TYPE2 TYPE-SPEC)) :DOCUMENTATION \"Return the most specific type that is a supertype of\nboth `type1' and `type2'.  If there is more than one, arbitrarily pick one.\nIf there is none, return @VOID.  If one or both types are parametric,\nalso try to generalize parameter types if necessary.\")", ((cpp_function_code)(&two_argument_least_common_supertype)), NULL);
    define_function_object("INHERIT-SLOTS", "(DEFUN INHERIT-SLOTS ((CLASS CLASS)))", ((cpp_function_code)(&inherit_slots)), NULL);
    define_function_object("LOCAL-SLOT?", "(DEFUN (LOCAL-SLOT? BOOLEAN) ((SLOT SLOT) (CLASS CLASS)))", ((cpp_function_code)(&local_slotP)), NULL);
    define_function_object("UNINHERIT-SLOTS", "(DEFUN UNINHERIT-SLOTS ((CLASS CLASS)))", ((cpp_function_code)(&uninherit_slots)), NULL);
    define_function_object("INITIALIZE-SLOT-AND-METHOD-CACHE", "(DEFUN INITIALIZE-SLOT-AND-METHOD-CACHE ((CLASS CLASS)))", ((cpp_function_code)(&initialize_slot_and_method_cache)), NULL);
    define_function_object("REGISTER-SLOT-NAME", "(DEFUN REGISTER-SLOT-NAME ((SLOT SLOT)) :PUBLIC? TRUE :DOCUMENTATION \"Register the name symbol of `slot' as a slot name, and\ninitialize its `symbol-slot-offset' so that it can be used by `lookup-slot'.\")", ((cpp_function_code)(&register_slot_name)), NULL);
    define_function_object("UNREGISTER-SLOT-NAME", "(DEFUN UNREGISTER-SLOT-NAME ((SLOT SLOT)) :PUBLIC? TRUE :DOCUMENTATION \"Unregister the name symbol of `slot' as a slot name,\nso that it can no longer be used by `lookup-slot'.\")", ((cpp_function_code)(&unregister_slot_name)), NULL);
    define_function_object("REGISTERED-SLOT-NAME?", "(DEFUN (REGISTERED-SLOT-NAME? BOOLEAN) ((SLOT-NAME SYMBOL)) :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE :DOCUMENTATION \"Return `true' if `slot-name' is the name of some registered;\nslot in the system.\" (RETURN (DEFINED? (SYMBOL-SLOT-OFFSET SLOT-NAME))))", ((cpp_function_code)(&registered_slot_nameP)), NULL);
    define_function_object("WARN-ABOUT-SLOT-SHADOWED-BY-A-FUNCTION?", "(DEFUN (WARN-ABOUT-SLOT-SHADOWED-BY-A-FUNCTION? BOOLEAN) ((SLOT SLOT)))", ((cpp_function_code)(&warn_about_slot_shadowed_by_a_functionP)), NULL);
    define_function_object("WARN-ABOUT-FUNCTION-SHADOWING-SLOTS?", "(DEFUN (WARN-ABOUT-FUNCTION-SHADOWING-SLOTS? BOOLEAN) ((FUNCTION METHOD-SLOT)))", ((cpp_function_code)(&warn_about_function_shadowing_slotsP)), NULL);
    define_function_object("LOOKUP-SLOT", "(DEFUN (LOOKUP-SLOT SLOT) ((CLASS CLASS) (SLOT-NAME SYMBOL)) :DOCUMENTATION \"Return a slot owned by the class `class' with name `slot-name'.\nMultiply inherited slots are disambiguated by a left-to-right class\nprecedence order for classes with multiple parents (similar to CLOS).\" :PUBLIC? TRUE)", ((cpp_function_code)(&lookup_slot)), NULL);
    define_function_object("SAFE-LOOKUP-SLOT", "(DEFUN (SAFE-LOOKUP-SLOT SLOT) ((CLASS CLASS) (SLOT-NAME SYMBOL)) :DOCUMENTATION \"Alias for `lookup-slot'.  Kept for backwards compatibility.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (LOOKUP-SLOT CLASS SLOT-NAME)))", ((cpp_function_code)(&safe_lookup_slot)), NULL);
    define_function_object("LOOKUP-VISIBLE-SLOT", "(DEFUN (LOOKUP-VISIBLE-SLOT SLOT) ((CLASS CLASS) (SLOT-NAME OBJECT)))", ((cpp_function_code)(&lookup_visible_slot)), NULL);
    define_function_object("LOOKUP-LOCAL-SLOT", "(DEFUN (LOOKUP-LOCAL-SLOT SLOT) ((CLASS CLASS) (SLOT-NAME SYMBOL)) :DOCUMENTATION \"Lookup a local slot with `slot-name' on `class'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&lookup_local_slot)), NULL);
    define_function_object("CLEAR-SLOT-AND-METHOD-CACHE", "(DEFUN CLEAR-SLOT-AND-METHOD-CACHE ((CLASS CLASS)))", ((cpp_function_code)(&clear_slot_and_method_cache)), NULL);
    define_function_object("RESIZE-SLOT-CACHES", "(DEFUN RESIZE-SLOT-CACHES ((SIZE INTEGER)) :DOCUMENTATION \"Reset all slot caches to have size `size'.\")", ((cpp_function_code)(&resize_slot_caches)), NULL);
    define_function_object("GET-SLOT", "(DEFUN (GET-SLOT SLOT) ((SELF STANDARD-OBJECT) (SLOT-NAME SYMBOL)) :DOCUMENTATION \"Return the slot named `slot-name' on the class\nrepresenting the type of `self'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&get_slot)), NULL);
    define_function_object("CONFORMING-TYPE-SPEC?", "(DEFUN (CONFORMING-TYPE-SPEC? BOOLEAN) ((SUB-TYPE-SPEC TYPE-SPEC) (SUPER-TYPE-SPEC TYPE-SPEC)))", ((cpp_function_code)(&conforming_type_specP)), NULL);
    define_method_object("(DEFMETHOD (CONFORMING-SIGNATURES? BOOLEAN) ((SELF SLOT) (SUPERSLOT SLOT)))", ((cpp_method_code)(&Slot::conforming_signaturesP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (CONFORMING-SIGNATURES? BOOLEAN) ((SELF STORAGE-SLOT) (SUPERSLOT SLOT)))", ((cpp_method_code)(&Storage_Slot::conforming_signaturesP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (CONFORMING-SIGNATURES? BOOLEAN) ((SELF METHOD-SLOT) (SUPERSLOT SLOT)))", ((cpp_method_code)(&Method_Slot::conforming_signaturesP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (CONFORMING-SIGNATURES? BOOLEAN) ((SELF TABLE) (SUPERSLOT SLOT)))", ((cpp_method_code)(&Table::conforming_signaturesP)), ((cpp_method_code)(NULL)));
    define_function_object("IDENTICAL-SIGNATURES?", "(DEFUN (IDENTICAL-SIGNATURES? BOOLEAN) ((METHOD1 METHOD-SLOT) (METHOD2 METHOD-SLOT)))", ((cpp_function_code)(&identical_signaturesP)), NULL);
    define_function_object("COMPUTE-REQUIRED-SLOT-NAMES", "(DEFUN COMPUTE-REQUIRED-SLOT-NAMES ((CLASS CLASS)))", ((cpp_function_code)(&compute_required_slot_names)), NULL);
    define_function_object("FREE-REQUIRED-SLOT-NAMES", "(DEFUN FREE-REQUIRED-SLOT-NAMES ((CLASS CLASS)))", ((cpp_function_code)(&free_required_slot_names)), NULL);
    define_method_object("(DEFMETHOD (DYNAMIC-STORAGE? BOOLEAN) ((SELF STORAGE-SLOT)))", ((cpp_method_code)(&Storage_Slot::dynamic_storageP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (PRIVATE? BOOLEAN) ((SELF RELATION)) :DOCUMENTATION \"Return `true' if `self' is not public.\" :PUBLIC? TRUE :PUBLIC? TRUE)", ((cpp_method_code)(&Relation::privateP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (PUBLIC? BOOLEAN) ((SELF SLOT)) :DOCUMENTATION \"True if `self' or one it its ancestors is marked public.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Slot::publicP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (PUBLIC-SLOTS (ITERATOR OF SLOT)) ((SELF CLASS)) :DOCUMENTATION \"Return an iterator over public slots of `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Class::public_slots)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (PUBLIC-SLOTS (ITERATOR OF SLOT)) ((SELF OBJECT)) :DOCUMENTATION \"Return an iterator over public slots of `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Object::public_slots)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (ALL-CLASS-SLOTS (ITERATOR OF SLOT)) ((SELF CLASS)))", ((cpp_method_code)(&Class::all_class_slots)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NEXT? BOOLEAN) ((SELF ALL-CLASS-SLOTS-ITERATOR)))", ((cpp_method_code)(&All_Class_Slots_Iterator::nextP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (CLASS-SLOTS (ITERATOR OF SLOT)) ((SELF CLASS)))", ((cpp_method_code)(&Class::class_slots)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (LOCAL-SLOTS (LIST OF SLOT)) ((SELF CLASS)))", ((cpp_method_code)(&Class::local_slots)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (DIRECT-SUPER-CLASSES (ITERATOR OF CLASS)) ((SELF CLASS)) :PUBLIC? TRUE :DOCUMENTATION \"Returns an iterator that generates all direct\nsuper classes of `self'.\")", ((cpp_method_code)(&Class::direct_super_classes)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (SUPER-CLASSES (ITERATOR OF CLASS)) ((SELF CLASS)) :PUBLIC? TRUE :DOCUMENTATION \"Returns an iterator that generates all super classes\nof `self'.  Non-reflexive.\")", ((cpp_method_code)(&Class::super_classes)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (ACTIVE? BOOLEAN) ((SLOT SLOT)))", ((cpp_method_code)(&Slot::activeP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (INITIAL-VALUE OBJECT) ((SELF STORAGE-SLOT)) :DOCUMENTATION \"Return an initial value for `self', or `null'.  The\ninitial value can be defined by the slot itself, inherited from an\nequivalent slot, or inherit from the :initial-value option for the \nclass representing the type of `self'.\")", ((cpp_method_code)(&Storage_Slot::initial_value)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (SYSTEM-DEFAULT-VALUE OBJECT) ((SELF SLOT)) :DOCUMENTATION \"Return a default value expression, or if `self'\nhas dynamic storage, an initial value expression.\")", ((cpp_method_code)(&Slot::system_default_value)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (SYSTEM-DEFAULT-VALUE OBJECT) ((SELF STORAGE-SLOT)) :DOCUMENTATION \"Return a default value expression, or if `self'\nhas dynamic storage, an initial value expression.\")", ((cpp_method_code)(&Storage_Slot::system_default_value)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (INITIALLY OBJECT) ((SELF STORAGE-SLOT)) :DOCUMENTATION \"Defines the value of a slot before it has been assigned\na value.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Storage_Slot::initially)), ((cpp_method_code)(NULL)));
    define_function_object("REPAIR-SLOTS", "(DEFUN REPAIR-SLOTS ())", ((cpp_function_code)(&repair_slots)), NULL);
    define_function_object("DISCONNECT-CLASSES", "(DEFUN DISCONNECT-CLASSES ())", ((cpp_function_code)(&disconnect_classes)), NULL);
    define_function_object("FILTER-UNBOUND-SURROGATE?", "(DEFUN (FILTER-UNBOUND-SURROGATE? BOOLEAN) ((SELF SURROGATE) (ITERATOR ALL-PURPOSE-ITERATOR)))", ((cpp_function_code)(&filter_unbound_surrogateP)), NULL);
    define_function_object("UNBOUND-SURROGATES", "(DEFUN (UNBOUND-SURROGATES (ITERATOR OF SURROGATE)) ((MODULE MODULE) (LOCAL? BOOLEAN)) :PUBLIC? TRUE :DOCUMENTATION \"Iterate over all unbound surrogates visible from `module'.\nLook at all modules if `module' is `null'.  If `local?', only consider\nsurrogates interned in `module'.\")", ((cpp_function_code)(&unbound_surrogates)), NULL);
    define_function_object("NAME-TO-STRING", "(DEFUN (NAME-TO-STRING STRING) ((NAME OBJECT)) :PUBLIC? TRUE :DOCUMENTATION \"Return the string represented by `name'.  Return `null'\nif `name' is undefined or does not represent a string.\")", ((cpp_function_code)(&name_to_string)), NULL);
    define_function_object("PRINT-UNBOUND-SURROGATES", "(DEFUN PRINT-UNBOUND-SURROGATES (|&REST| (ARGS OBJECT)) :COMMAND? TRUE :PUBLIC? TRUE :EVALUATE-ARGUMENTS? FALSE :DOCUMENTATION \"Print all unbound surrogates visible from the module named by the first\nargument (a symbol or string).  Look at all modules if no module name or\n`null' was supplied.  If the second argument is `true', only consider\nsurrogates interned in the specified module.\")", ((cpp_function_code)(&print_unbound_surrogates)), ((cpp_function_code)(&print_unbound_surrogates_evaluator_wrapper)));
    define_function_object("COERCE-TO-SYMBOL", "(DEFUN (COERCE-TO-SYMBOL GENERALIZED-SYMBOL) ((NAME NAME)) :PUBLIC? TRUE :DOCUMENTATION \"Return the (generalized) symbol represented by `name'.\nReturn `null' if `name' is undefined or does not represent a string.\")", ((cpp_function_code)(&coerce_to_symbol)), NULL);
  }
}

void startup_classes() {
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (current_startup_time_phaseP(2)) {
      help_startup_classes1();
      help_startup_classes2();
    }
    if (current_startup_time_phaseP(4)) {
      oUNFINALIZED_CLASSESo = new_list();
      oCLASSES_WITH_UNRESOLVED_CHILDREN_REFERENCESo = new_list();
      oCLASS_TAXONOMY_GRAPHo = new_taxonomy_graph();
    }
    if (current_startup_time_phaseP(5)) {
      { Class* renamed_Class = define_class_from_stringified_source("ALL-CLASS-SLOTS-ITERATOR", "(DEFCLASS ALL-CLASS-SLOTS-ITERATOR (ITERATOR) :PARAMETERS ((ANY-VALUE :TYPE SLOT)) :SLOTS ((ITERATOR-CONS-LIST :TYPE CONS) (ITERATOR-OBJECT :TYPE OBJECT)))");

        renamed_Class->class_constructor_code = ((cpp_function_code)(&new_all_class_slots_iterator));
        renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_all_class_slots_iterator_slot_value));
      }
      define_stella_type_from_stringified_source("(DEFTYPE ANY UNKNOWN)");
    }
    if (current_startup_time_phaseP(6)) {
      finalize_classes();
    }
    if (current_startup_time_phaseP(7)) {
      help_startup_classes3();
      help_startup_classes4();
      define_function_object("PRINT-UNDEFINED-SUPER-CLASSES", "(DEFUN PRINT-UNDEFINED-SUPER-CLASSES ((CLASS NAME)) :COMMAND? TRUE :PUBLIC? TRUE :EVALUATE-ARGUMENTS? FALSE :DOCUMENTATION \"Print all undefined or bad (indirect) super classes of `class'.\")", ((cpp_function_code)(&print_undefined_super_classes)), NULL);
      define_function_object("COLLECT-BAD-SUPER-CLASSES", "(DEFUN (COLLECT-BAD-SUPER-CLASSES (LIST OF TYPE)) ((TYPE TYPE) (BADONES (LIST OF TYPE))) :PUBLIC? TRUE)", ((cpp_function_code)(&collect_bad_super_classes)), NULL);
      define_function_object("ROOT-CLASS?", "(DEFUN (ROOT-CLASS? BOOLEAN) ((CLASS CLASS)))", ((cpp_function_code)(&root_classP)), NULL);
      define_function_object("CREATE-CLASS-TAXONOMY", "(DEFUN CREATE-CLASS-TAXONOMY ())", ((cpp_function_code)(&create_class_taxonomy)), NULL);
      define_function_object("SUBCLASS-OF?", "(DEFUN (SUBCLASS-OF? BOOLEAN) ((SUBCLASS CLASS) (SUPERCLASS CLASS)) :DOCUMENTATION \"Return `true' if `subClass' is a subclass of `superClass'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&subclass_ofP)), NULL);
      define_function_object("FAST-SUBCLASS-OF?", "(DEFUN (FAST-SUBCLASS-OF? BOOLEAN) ((SUBCLASS CLASS) (SUPERCLASS CLASS)))", ((cpp_function_code)(&fast_subclass_ofP)), NULL);
      define_function_object("SUBTYPE-OF?", "(DEFUN (SUBTYPE-OF? BOOLEAN) ((SUB-TYPE TYPE) (SUPER-TYPE TYPE)) :DOCUMENTATION \"Return `true' iff the class named `sub-type' is a subclass \nof the class named `super-type'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&subtype_ofP)), NULL);
      define_function_object("ISA?", "(DEFUN (ISA? BOOLEAN) ((OBJECT OBJECT) (TYPE TYPE)) :DOCUMENTATION \"Return `true' iff `object' is an instance of the class named `type'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&isaP)), NULL);
      define_function_object("TAXONOMY-SUBCLASS-OF?", "(DEFUN (TAXONOMY-SUBCLASS-OF? BOOLEAN) ((SUBCLASS CLASS) (SUPERCLASS CLASS)) :GLOBALLY-INLINE? TRUE (RETURN (SUBCLASS-OF? SUBCLASS SUPERCLASS)))", ((cpp_function_code)(&taxonomy_subclass_ofP)), NULL);
      define_function_object("TAXONOMY-ISA?", "(DEFUN (TAXONOMY-ISA? BOOLEAN) ((OBJECT OBJECT) (TYPE TYPE)) :GLOBALLY-INLINE? TRUE (RETURN (ISA? OBJECT TYPE)))", ((cpp_function_code)(&taxonomy_isaP)), NULL);
      define_function_object("INTERN-SLOTREF", "(DEFUN (INTERN-SLOTREF SLOTREF) ((CLASSNAME STRING) (SLOTNAME STRING)))", ((cpp_function_code)(&intern_slotref)), NULL);
      define_function_object("LOOKUP-SLOTREF", "(DEFUN (LOOKUP-SLOTREF SLOTREF) ((SELF TYPE) (SLOTNAME SYMBOL)) :PUBLIC? TRUE)", ((cpp_function_code)(&lookup_slotref)), NULL);
      define_function_object("SLOTREF?", "(DEFUN (SLOTREF? BOOLEAN) ((SELF SURROGATE)))", ((cpp_function_code)(&slotrefP)), NULL);
      define_function_object("LOOKUP-SLOT-FROM-SLOTREF", "(DEFUN (LOOKUP-SLOT-FROM-SLOTREF SLOT) ((SLOTREF SLOTREF)))", ((cpp_function_code)(&lookup_slot_from_slotref)), NULL);
      define_function_object("SLOTREF-TYPE", "(DEFUN (SLOTREF-TYPE TYPE) ((SLOTREF SLOTREF)))", ((cpp_function_code)(&slotref_type)), NULL);
      define_function_object("STARTUP-CLASSES", "(DEFUN STARTUP-CLASSES () :PUBLIC? TRUE)", ((cpp_function_code)(&startup_classes)), NULL);
      { Method_Slot* function = lookup_function(SYM_CLASSES_STELLA_STARTUP_CLASSES);

        set_dynamic_slot_value(function->dynamic_slots, SYM_CLASSES_STELLA_METHOD_STARTUP_CLASSNAME, wrap_string("Startup-Classes"), NULL_STRING_WRAPPER);
      }
    }
    if (current_startup_time_phaseP(8)) {
      finalize_slots();
      cleanup_unfinalized_classes();
    }
    if (current_startup_time_phaseP(9)) {
      define_stella_global_variable_from_stringified_source("(DEFSPECIAL *WARNIFREDEFINE?* BOOLEAN TRUE :DOCUMENTATION \"If set, warn about each redefinition.\" :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *UNFINALIZED-CLASSES* (LIST OF CLASS) (NEW LIST) :DOCUMENTATION \"List of classes whose class or slot inheritance is\ncurrently unfinalized.\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *NEWLY-UNFINALIZED-CLASSES?* BOOLEAN FALSE :DOCUMENTATION \"Set to `true' by `remember-unfinalized-class'; set\nto `false' by `cleanup-unfinalized-classes'.  Minimizes the time that\n`finalize-classes' spends searching for classes to finalize.\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *CLASSES-WITH-UNRESOLVED-CHILDREN-REFERENCES* (LIST OF CLASS) (NEW LIST) :DOCUMENTATION \"List of classes defined with a :children option\nwhich still have some of their children references unresolved.\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *SYMBOL-SLOT-OFFSET-COUNTER* INTEGER 0)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *SLOT-CACHE-SIZE* INTEGER 20)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *CLASS-TAXONOMY-GRAPH* TAXONOMY-GRAPH (NEW TAXONOMY-GRAPH))");
    }
  }
}

Surrogate* SGT_CLASSES_STELLA_CLASS = NULL;

Surrogate* SGT_CLASSES_STELLA_OBJECT = NULL;

Keyword* KWD_CLASSES_PUBLICp = NULL;

Symbol* SYM_CLASSES_STELLA_TRUE = NULL;

Symbol* SYM_CLASSES_STELLA_FALSE = NULL;

Surrogate* SGT_CLASSES_STELLA_UNKNOWN = NULL;

Keyword* KWD_CLASSES_PARAMETERS = NULL;

Symbol* SYM_CLASSES_STELLA_SLOT_TYPE_SPECIFIER = NULL;

Keyword* KWD_CLASSES_REQUIREDp = NULL;

Keyword* KWD_CLASSES_COMPONENTp = NULL;

Keyword* KWD_CLASSES_READ_ONLYp = NULL;

Keyword* KWD_CLASSES_ACTIVEp = NULL;

Symbol* SYM_CLASSES_STELLA_STORED_ACTIVEp = NULL;

Keyword* KWD_CLASSES_CONTEXT_SENSITIVEp = NULL;

Keyword* KWD_CLASSES_HARDWIREDp = NULL;

Keyword* KWD_CLASSES_ABSTRACTp = NULL;

Keyword* KWD_CLASSES_ALLOCATION = NULL;

Keyword* KWD_CLASSES_CLASS = NULL;

Symbol* SYM_CLASSES_STELLA_SLOT_ALLOCATION = NULL;

Keyword* KWD_CLASSES_INITIALLY = NULL;

Symbol* SYM_CLASSES_STELLA_SLOT_INITIAL_VALUE = NULL;

Keyword* KWD_CLASSES_DEFAULT = NULL;

Symbol* SYM_CLASSES_STELLA_SLOT_DEFAULT_EXPRESSION = NULL;

Keyword* KWD_CLASSES_READER = NULL;

Symbol* SYM_CLASSES_STELLA_SLOT_READER = NULL;

Keyword* KWD_CLASSES_WRITER = NULL;

Symbol* SYM_CLASSES_STELLA_SLOT_WRITER = NULL;

Keyword* KWD_CLASSES_INVERSE = NULL;

Symbol* SYM_CLASSES_STELLA_SLOT_INVERSE = NULL;

Keyword* KWD_CLASSES_RENAMES = NULL;

Symbol* SYM_CLASSES_STELLA_SLOT_RENAMES = NULL;

Keyword* KWD_CLASSES_DOCUMENTATION = NULL;

Symbol* SYM_CLASSES_STELLA_DOCUMENTATION = NULL;

Keyword* KWD_CLASSES_PROPERTIES = NULL;

Symbol* SYM_CLASSES_STELLA_PROPERTIES = NULL;

Keyword* KWD_CLASSES_META_ATTRIBUTES = NULL;

Symbol* SYM_CLASSES_STELLA_META_ATTRIBUTES = NULL;

Keyword* KWD_CLASSES_OPTION_KEYWORD = NULL;

Symbol* SYM_CLASSES_STELLA_SLOT_OPTION_KEYWORD = NULL;

Keyword* KWD_CLASSES_OPTION_HANDLER = NULL;

Symbol* SYM_CLASSES_STELLA_SLOT_OPTION_HANDLER = NULL;

Surrogate* SGT_CLASSES_STELLA_BOOLEAN = NULL;

Keyword* KWD_CLASSES_BIT = NULL;

Keyword* KWD_CLASSES_CL_NATIVE_TYPE = NULL;

Symbol* SYM_CLASSES_STELLA_CLASS_CL_NATIVE_TYPE = NULL;

Keyword* KWD_CLASSES_CPP_NATIVE_TYPE = NULL;

Symbol* SYM_CLASSES_STELLA_CLASS_CPP_NATIVE_TYPE = NULL;

Keyword* KWD_CLASSES_IDL_NATIVE_TYPE = NULL;

Symbol* SYM_CLASSES_STELLA_CLASS_IDL_NATIVE_TYPE = NULL;

Keyword* KWD_CLASSES_JAVA_NATIVE_TYPE = NULL;

Symbol* SYM_CLASSES_STELLA_CLASS_JAVA_NATIVE_TYPE = NULL;

Keyword* KWD_CLASSES_CL_STRUCTp = NULL;

Keyword* KWD_CLASSES_MIXINp = NULL;

Keyword* KWD_CLASSES_RECYCLE_METHOD = NULL;

Symbol* SYM_CLASSES_STELLA_CLASS_RECYCLE_METHOD = NULL;

Keyword* KWD_CLASSES_EXTENSION = NULL;

Symbol* SYM_CLASSES_STELLA_CLASS_EXTENSION_NAME = NULL;

Keyword* KWD_CLASSES_CREATOR = NULL;

Symbol* SYM_CLASSES_STELLA_CLASS_CREATOR = NULL;

Keyword* KWD_CLASSES_INITIALIZER = NULL;

Symbol* SYM_CLASSES_STELLA_CLASS_INITIALIZER = NULL;

Keyword* KWD_CLASSES_TERMINATOR = NULL;

Symbol* SYM_CLASSES_STELLA_CLASS_TERMINATOR = NULL;

Keyword* KWD_CLASSES_DESTRUCTOR = NULL;

Symbol* SYM_CLASSES_STELLA_CLASS_DESTRUCTOR = NULL;

Keyword* KWD_CLASSES_INITIAL_VALUE = NULL;

Symbol* SYM_CLASSES_STELLA_CLASS_INITIAL_VALUE = NULL;

Keyword* KWD_CLASSES_PRINT_FORM = NULL;

Symbol* SYM_CLASSES_STELLA_PRINT_FORM = NULL;

Keyword* KWD_CLASSES_EQUALITY_TEST = NULL;

Keyword* KWD_CLASSES_KEY = NULL;

Symbol* SYM_CLASSES_STELLA_CLASS_KEY = NULL;

Symbol* SYM_CLASSES_STELLA_CLASS_PARAMETERS = NULL;

Keyword* KWD_CLASSES_SYNONYMS = NULL;

Symbol* SYM_CLASSES_STELLA_CLASS_SYNONYMS = NULL;

Keyword* KWD_CLASSES_CHILDREN = NULL;

Surrogate* SGT_CLASSES_STELLA_METHOD_SLOT = NULL;

Symbol* SYM_CLASSES_STELLA_SLOT_GUARD_DEMONS = NULL;

Symbol* SYM_CLASSES_STELLA_SLOT_DEMONS = NULL;

Symbol* SYM_CLASSES_STELLA_CLASS_CONSTRUCTOR_DEMONS = NULL;

Symbol* SYM_CLASSES_STELLA_CLASS_GUARD_CONSTRUCTOR_DEMONS = NULL;

Symbol* SYM_CLASSES_STELLA_CLASS_DESTRUCTOR_DEMONS = NULL;

Symbol* SYM_CLASSES_STELLA_CLASS_GUARD_DESTRUCTOR_DEMONS = NULL;

Surrogate* SGT_CLASSES_STELLA_ACTIVE_OBJECT = NULL;

Surrogate* SGT_CLASSES_STELLA_STANDARD_OBJECT = NULL;

Symbol* SYM_CLASSES_STELLA_PRIMARY_TYPE = NULL;

Symbol* SYM_CLASSES_STELLA_SELF = NULL;

Keyword* KWD_CLASSES_TYPE = NULL;

Symbol* SYM_CLASSES_STELLA_TYPE = NULL;

Keyword* KWD_CLASSES_AUXILIARYp = NULL;

Symbol* SYM_CLASSES_STELLA_RETURN = NULL;

Symbol* SYM_CLASSES_STELLA_CLASS_EXTENSION = NULL;

Symbol* SYM_CLASSES_STELLA_BADp = NULL;

Surrogate* SGT_CLASSES_STELLA_SET = NULL;

Surrogate* SGT_CLASSES_STELLA_ACTIVE_SET = NULL;

Surrogate* SGT_CLASSES_STELLA_LIST = NULL;

Surrogate* SGT_CLASSES_STELLA_ACTIVE_LIST = NULL;

Surrogate* SGT_CLASSES_STELLA_COLLECTION = NULL;

Surrogate* SGT_CLASSES_STELLA_SET_MIXIN = NULL;

Symbol* SYM_CLASSES_STELLA_INVERSE = NULL;

Keyword* KWD_CLASSES_DYNAMIC = NULL;

Surrogate* SGT_CLASSES_STELLA_DYNAMIC_SLOTS_MIXIN = NULL;

Keyword* KWD_CLASSES_INSTANCE = NULL;

Surrogate* SGT_CLASSES_STELLA_CONTEXT_SENSITIVE_OBJECT = NULL;

Surrogate* SGT_CLASSES_STELLA_CONTEXT_SENSITIVE_MIXIN = NULL;

Surrogate* SGT_CLASSES_STELLA_VOID = NULL;

Surrogate* SGT_CLASSES_STELLA_TABLE = NULL;

Symbol* SYM_CLASSES_STELLA_CLASS_REQUIRED_SLOT_NAMES = NULL;

Surrogate* SGT_CLASSES_STELLA_ALL_CLASS_SLOTS_ITERATOR = NULL;

Symbol* SYM_CLASSES_STELLA_ITERATOR_CONS_LIST = NULL;

Symbol* SYM_CLASSES_STELLA_ITERATOR_OBJECT = NULL;

Surrogate* SGT_CLASSES_STELLA_GENERALIZED_SYMBOL = NULL;

Symbol* SYM_CLASSES_STELLA_ANY = NULL;

Surrogate* SGT_CLASSES_STELLA_ANY = NULL;

Surrogate* SGT_CLASSES_STELLA_SLOT = NULL;

Symbol* SYM_CLASSES_STELLA_STARTUP_CLASSES = NULL;

Symbol* SYM_CLASSES_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella
