// contexts.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

// Eliminate pointers to discarded contexts while
// accessing a CS-VALUE context table.
boolean oUNLINK_DISCARDED_CONTEXTS_ON_READpo = TRUE;

// Eliminate pointers to discarded contexts while
// inserting into a CS-VALUE context table.
boolean oUNLINK_DISCARDED_CONTEXTS_ON_WRITEpo = TRUE;

// If true, indicates that contexts are being allocated
// and deallocated in depth-first fashion, and that deallocation of
// CS-VALUE entries is taken care of during context destruction.
boolean oCONTEXT_BACKTRACKING_MODEo = FALSE;

boolean cs_valueP(Object* self) {
  return (((boolean)(self)) &&
      (self->primary_type() == SGT_CONTEXTS_STELLA_CS_VALUE));
}

boolean subcontextP(Context* subcontext, Context* supercontext) {
  return ((subcontext == supercontext) ||
      subcontext->all_super_contexts->membP(supercontext));
}

boolean discarded_contextP(Context* context) {
  return (((context->context_number) % 2));
}

boolean Context::deletedP() {
  { Context* context = this;

    return (((context->context_number) % 2));
  }
}

Context* help_find_context_by_number(int number, List* siblings) {
  { Context* c = NULL;
    Cons* iter_000 = siblings->the_cons_list;

    while (!(iter_000 == NIL)) {
      c = ((Context*)(iter_000->value));
      iter_000 = iter_000->rest;
      if (c->context_number == number) {
        return (c);
      }
      else if (c->context_number < number) {
        return (help_find_context_by_number(number, c->child_contexts));
      }
      else {
        continue;
      }
    }
  }
  return (NULL);
}

Context* find_context_by_number(int contextnumber) {
  return (help_find_context_by_number(contextnumber, oROOT_MODULEo->child_contexts));
}

Context* string_get_stella_context_slowly(char* self) {
  { Context* context = NULL;

    { Context* cxt = NULL;
      Cons* iter_000 = oROOT_MODULEo->child_contexts->the_cons_list;

      while (!(iter_000 == NIL)) {
        cxt = ((Context*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (string_eqlP(cxt->context_name(), self)) {
          return (cxt);
        }
      }
    }
    { Context* cxt = NULL;
      Iterator* iter_001 = all_contexts();

      while (iter_001->nextP()) {
        cxt = ((Context*)(iter_001->value));
        if (string_eqlP(cxt->context_name(), self) ||
            (isaP(cxt, SGT_CONTEXTS_STELLA_MODULE) &&
             (((boolean)(((Module*)(cxt))->nicknames)) &&
              ((Module*)(cxt))->nicknames->memberP(wrap_string(self))))) {
          if (((boolean)(context))) {
            cout << "More than one context has the name or nickname" << "`" << self << "'" << endl;
            return (NULL);
          }
          context = cxt;
        }
      }
    }
    return (context);
  }
}

Context* integer_get_stella_context_slowly(int self) {
  return (find_context_by_number(self));
}

Context* Symbol::get_stella_context_slowly() {
  { Symbol* self = this;

    return (string_get_stella_context_slowly(self->relative_name()));
  }
}

Context* change_context_slowly(Context* self) {
  if (((boolean)(self))) {
    return (self->change_context());
  }
  else {
    return (oCONTEXTo.get());
  }
}

Context* cc(Cons* name) {
  // Change the current context to the one named `name'.  Return the
  // value of the new current context.  If no `name' is supplied, return
  // the pre-existing value of the current context.  `cc' is a no-op if the
  // context reference cannot be successfully evaluated.
  { Context* context = oCONTEXTo.get();
    Object* namespec = name->value;

    if (((boolean)(namespec))) {
      if (subtype_of_integerP(safe_primary_type(namespec))) {
        { Object* namespec_000 = namespec;
          Integer_Wrapper* namespec = ((Integer_Wrapper*)(namespec_000));

          context = integer_get_stella_context_slowly(namespec->wrapper_value);
        }
      }
      else {
        { char* contextname = coerce_to_module_name(namespec, TRUE);

          if (contextname != NULL) {
            context = get_stella_context(contextname, TRUE);
          }
        }
      }
    }
    return (change_context_slowly(context));
  }
}

Context* cc_evaluator_wrapper(Cons* arguments) {
  return (cc(arguments));
}

void print_context(Context* self, ostream* stream) {
  { char* typestring = NULL;
    char* name = NULL;
    int number = self->context_number;

    if (!oCLASS_HIERARCHY_BOOTEDpo) {
      *(stream) << "|MDL|" << ((Module*)(self))->module_name;
      return;
    }
    { Surrogate* test_value_000 = safe_primary_type(self);

      if (subtype_ofP(test_value_000, SGT_CONTEXTS_STELLA_MODULE)) {
        { Context* self_000 = self;
          Module* self = ((Module*)(self_000));

          name = self->module_full_name;
          if (((self->context_number) % 2)) {
            typestring = "|DeLeTeD MDL|";
          }
          else {
            typestring = "|MDL|";
          }
        }
      }
      else if (subtype_ofP(test_value_000, SGT_CONTEXTS_STELLA_WORLD)) {
        { Context* self_001 = self;
          World* self = ((World*)(self_001));

          name = ((String_Wrapper*)(dynamic_slot_value(self->dynamic_slots, SYM_CONTEXTS_STELLA_WORLD_NAME, NULL_STRING_WRAPPER)))->wrapper_value;
          if (((self->context_number) % 2)) {
            typestring = "|DeLeTeD WLD|";
          }
          else {
            typestring = "|WLD|";
          }
        }
      }
      else {
        { Output_String_Stream* stream_000 = new_output_string_stream();

          *(stream_000->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
          throw *new_stella_exception(stream_000->the_string_reader());
        }
      }
    }
    if (((self->context_number) % 2)) {
      number = number + 1;
    }
    if (oPRINTREADABLYpo.get()) {
      if (name != NULL) {
        *(stream) << name;
      }
      else {
        *(stream) << "#<" << typestring << number << ">";
      }
    }
    else {
      if (name != NULL) {
        *(stream) << typestring << name;
      }
      else {
        *(stream) << typestring << number;
      }
    }
  }
}

void help_print_context_tree(List* list, int level) {
  { Context* c = NULL;
    Cons* iter_000 = list->the_cons_list;

    while (!(iter_000 == NIL)) {
      c = ((Context*)(iter_000->value));
      iter_000 = iter_000->rest;
      cout << endl;
      { int i = NULL_INTEGER;
        int iter_001 = 1;
        int upper_bound_000 = 3 * level;

        while (iter_001 <= upper_bound_000) {
          i = iter_001;
          iter_001 = iter_001 + 1;
                    cout << " ";
        }
      }
      cout << c->context_number;
      if (c->context_name() != NULL) {
        cout << "   " << c->context_name();
      }
      cout << endl;
      help_print_context_tree(c->child_contexts, level + 1);
    }
  }
}

void print_context_tree(Context* root) {
  if (!((boolean)(root))) {
    root = oROOT_MODULEo;
  }
  { List* toplist = list(1, root);

    help_print_context_tree(toplist, 0);
    toplist->free();
  }
}

Object* access_in_context(Object* value, Context* homecontext, boolean dontinheritP) {
  if ((!((boolean)(value))) ||
      (!(((boolean)(value)) &&
      (value->primary_type() == SGT_CONTEXTS_STELLA_CS_VALUE)))) {
    if (dontinheritP) {
      if (oCONTEXTo.get() == homecontext) {
        return (value);
      }
    }
    else {
      if ((!((boolean)(homecontext))) ||
          (((oCONTEXTo.get() == homecontext) ||
          oCONTEXTo.get()->all_super_contexts->membP(homecontext)) ||
           (isaP(homecontext, SGT_CONTEXTS_STELLA_MODULE) &&
            visible_fromP(homecontext, oMODULEo.get())))) {
        return (value);
      }
    }
    return (NULL);
  }
  { Kv_Cons* kvcons = ((Cs_Value*)(value))->the_kv_list;
    int contextnumber = NULL_INTEGER;

    { Context* target = oCONTEXTo.get();

      contextnumber = target->context_number;
      while (((boolean)(kvcons)) &&
          (contextnumber < ((Context*)(kvcons->key))->context_number)) {
        if (((((Context*)(kvcons->key))->context_number) % 2) &&
            ((!oCONTEXT_BACKTRACKING_MODEo) &&
             oUNLINK_DISCARDED_CONTEXTS_ON_READpo)) {
          if (((boolean)(kvcons->rest))) {
            { Kv_Cons* nextkvcons = kvcons->rest;

              kvcons->key = nextkvcons->key;
              kvcons->value = nextkvcons->value;
              kvcons->rest = nextkvcons->rest;
              nextkvcons->free();
            }
          }
          else {
            {
              ((Cs_Value*)(value))->remove_at(((Context*)(kvcons->key)));
              return (NULL);
            }
          }
        }
        else {
          kvcons = kvcons->rest;
        }
      }
      if (!((boolean)(kvcons))) {
        return (NULL);
      }
      if (target == kvcons->key) {
        return (kvcons->value);
      }
      if (dontinheritP) {
        return (NULL);
      }
    }
    { Context* target = NULL;
      Cons* iter_000 = oCONTEXTo.get()->all_super_contexts;

      while (!(iter_000 == NIL)) {
        target = ((Context*)(iter_000->value));
        iter_000 = iter_000->rest;
        contextnumber = target->context_number;
        while (((boolean)(kvcons)) &&
            (contextnumber < ((Context*)(kvcons->key))->context_number)) {
          if (((((Context*)(kvcons->key))->context_number) % 2) &&
              ((!oCONTEXT_BACKTRACKING_MODEo) &&
               oUNLINK_DISCARDED_CONTEXTS_ON_READpo)) {
            if (((boolean)(kvcons->rest))) {
              { Kv_Cons* nextkvcons = kvcons->rest;

                kvcons->key = nextkvcons->key;
                kvcons->value = nextkvcons->value;
                kvcons->rest = nextkvcons->rest;
                nextkvcons->free();
              }
            }
            else {
              {
                ((Cs_Value*)(value))->remove_at(((Context*)(kvcons->key)));
                return (NULL);
              }
            }
          }
          else {
            kvcons = kvcons->rest;
          }
        }
        if (!((boolean)(kvcons))) {
          return (NULL);
        }
        if (target == kvcons->key) {
          return (kvcons->value);
        }
        if (dontinheritP) {
          return (NULL);
        }
      }
    }
    return (NULL);
  }
}

void help_insert_a_cs_value(Kv_Cons* kvcons, Object* newvalue, Context* target, boolean overwriteP) {
  { int contextnumber = target->context_number;

    for (;;) {
      if ((!oCONTEXT_BACKTRACKING_MODEo) &&
          (oUNLINK_DISCARDED_CONTEXTS_ON_WRITEpo &&
           ((((Context*)(kvcons->key))->context_number) % 2))) {
        if (((boolean)(kvcons->rest))) {
          { Kv_Cons* nextkvcons = kvcons->rest;

            kvcons->key = nextkvcons->key;
            kvcons->value = nextkvcons->value;
            kvcons->rest = nextkvcons->rest;
            nextkvcons->free();
          }
        }
        else {
          {
            kvcons->key = target;
            kvcons->value = newvalue;
            return;
          }
        }
      }
      if (kvcons->key == target) {
        if (overwriteP) {
          kvcons->value = newvalue;
        }
        return;
      }
      else if (((Context*)(kvcons->key))->context_number < contextnumber) {
        kvcons->rest = kv_cons(kvcons->key, kvcons->value, kvcons->rest);
        kvcons->key = target;
        kvcons->value = newvalue;
        return;
      }
      else if (!((boolean)(kvcons->rest))) {
        kvcons->rest = kv_cons(target, newvalue, NULL);
        return;
      }
      else {
        kvcons = kvcons->rest;
      }
    }
  }
}

void Cs_Value::insert_at(Context* context, Object* newvalue) {
  { Cs_Value* self = this;

    if (!((boolean)(self->the_kv_list))) {
      { Kv_Cons* kvcons = new_kv_cons();

        kvcons->key = context;
        kvcons->value = newvalue;
        self->the_kv_list = kvcons;
        return;
      }
    }
    help_insert_a_cs_value(self->the_kv_list, newvalue, context, TRUE);
  }
}

Object* update_in_context(Object* oldvalue, Object* newvalue, Context* homecontext, boolean copytochildrenP) {
  { Cs_Value* csvalue = NULL;

    if (((boolean)(oldvalue)) &&
        (oldvalue->primary_type() == SGT_CONTEXTS_STELLA_CS_VALUE)) {
      csvalue = ((Cs_Value*)(oldvalue));
    }
    else {
      if ((homecontext == oCONTEXTo.get()) &&
          (oCONTEXTo.get()->child_contexts->emptyP() ||
           (!copytochildrenP))) {
        return (newvalue);
      }
      else {
        {
          csvalue = new_cs_value();
          if (((boolean)(oldvalue))) {
            csvalue->insert_at(homecontext, oldvalue);
          }
        }
      }
    }
    csvalue->insert_at(oCONTEXTo.get(), newvalue);
    if (copytochildrenP) {
      copy_current_value_to_children(csvalue, homecontext, newvalue);
    }
    return (csvalue);
  }
}

void copy_current_value_to_children(Cs_Value* csvalue, Context* homecontext, Object* parentvalue) {
  { Context* childcxt = NULL;
    Cons* iter_000 = oCONTEXTo.get()->child_contexts->the_cons_list;

    while (!(iter_000 == NIL)) {
      childcxt = ((Context*)(iter_000->value));
      iter_000 = iter_000->rest;
      { 
        BIND_STELLA_SPECIAL(oCONTEXTo, Context*, childcxt);
        { Object* currentvalue = access_in_context(csvalue, homecontext, FALSE);

          if (!eqlP(currentvalue, parentvalue)) {
            help_insert_a_cs_value(csvalue->the_kv_list, currentvalue, childcxt, FALSE);
          }
        }
      }
    }
  }
}

void startup_contexts() {
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (current_startup_time_phaseP(2)) {
      SGT_CONTEXTS_STELLA_CS_VALUE = ((Surrogate*)(intern_rigid_symbol_wrt_module("CS-VALUE", NULL, 1)));
      SGT_CONTEXTS_STELLA_MODULE = ((Surrogate*)(intern_rigid_symbol_wrt_module("MODULE", NULL, 1)));
      SGT_CONTEXTS_STELLA_WORLD = ((Surrogate*)(intern_rigid_symbol_wrt_module("WORLD", NULL, 1)));
      SYM_CONTEXTS_STELLA_WORLD_NAME = ((Symbol*)(intern_rigid_symbol_wrt_module("WORLD-NAME", NULL, 0)));
      SYM_CONTEXTS_STELLA_STARTUP_CONTEXTS = ((Symbol*)(intern_rigid_symbol_wrt_module("STARTUP-CONTEXTS", NULL, 0)));
      SYM_CONTEXTS_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-STARTUP-CLASSNAME", NULL, 0)));
    }
    if (current_startup_time_phaseP(6)) {
      finalize_classes();
    }
    if (current_startup_time_phaseP(7)) {
      define_function_object("CS-VALUE?", "(DEFUN (CS-VALUE? BOOLEAN) ((SELF OBJECT)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (AND (DEFINED? SELF) (EQL? (PRIMARY-TYPE SELF) @CS-VALUE))))", ((cpp_function_code)(&cs_valueP)), NULL);
      define_function_object("SUBCONTEXT?", "(DEFUN (SUBCONTEXT? BOOLEAN) ((SUBCONTEXT CONTEXT) (SUPERCONTEXT CONTEXT)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (OR (EQL? SUBCONTEXT SUPERCONTEXT) (MEMB? (ALL-SUPER-CONTEXTS SUBCONTEXT) SUPERCONTEXT))))", ((cpp_function_code)(&subcontextP)), NULL);
      define_function_object("DISCARDED-CONTEXT?", "(DEFUN (DISCARDED-CONTEXT? BOOLEAN) ((CONTEXT CONTEXT)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (ODD? (CONTEXT-NUMBER CONTEXT))))", ((cpp_function_code)(&discarded_contextP)), NULL);
      define_method_object("(DEFMETHOD (DELETED? BOOLEAN) ((CONTEXT CONTEXT)) :PUBLIC? TRUE)", ((cpp_method_code)(&Context::deletedP)), ((cpp_method_code)(NULL)));
      define_function_object("HELP-FIND-CONTEXT-BY-NUMBER", "(DEFUN (HELP-FIND-CONTEXT-BY-NUMBER CONTEXT) ((NUMBER INTEGER) (SIBLINGS (LIST OF CONTEXT))))", ((cpp_function_code)(&help_find_context_by_number)), NULL);
      define_function_object("FIND-CONTEXT-BY-NUMBER", "(DEFUN (FIND-CONTEXT-BY-NUMBER CONTEXT) ((CONTEXTNUMBER INTEGER)))", ((cpp_function_code)(&find_context_by_number)), NULL);
      define_method_object("(DEFMETHOD (GET-STELLA-CONTEXT-SLOWLY CONTEXT) ((SELF STRING)))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (GET-STELLA-CONTEXT-SLOWLY CONTEXT) ((SELF INTEGER)))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (GET-STELLA-CONTEXT-SLOWLY CONTEXT) ((SELF SYMBOL)))", ((cpp_method_code)(&Symbol::get_stella_context_slowly)), ((cpp_method_code)(NULL)));
      define_function_object("CHANGE-CONTEXT-SLOWLY", "(DEFUN (CHANGE-CONTEXT-SLOWLY CONTEXT) ((SELF CONTEXT)))", ((cpp_function_code)(&change_context_slowly)), NULL);
      define_function_object("CC", "(DEFUN (CC CONTEXT) (|&REST| (NAME NAME)) :COMMAND? TRUE :PUBLIC? TRUE :EVALUATE-ARGUMENTS? FALSE :DOCUMENTATION \"Change the current context to the one named `name'.  Return the\nvalue of the new current context.  If no `name' is supplied, return\nthe pre-existing value of the current context.  `cc' is a no-op if the\ncontext reference cannot be successfully evaluated.\")", ((cpp_function_code)(&cc)), ((cpp_function_code)(&cc_evaluator_wrapper)));
      define_function_object("PRINT-CONTEXT", "(DEFUN PRINT-CONTEXT ((SELF CONTEXT) (STREAM NATIVE-OUTPUT-STREAM)))", ((cpp_function_code)(&print_context)), NULL);
      define_function_object("HELP-PRINT-CONTEXT-TREE", "(DEFUN HELP-PRINT-CONTEXT-TREE ((LIST (LIST OF CONTEXT)) (LEVEL INTEGER)))", ((cpp_function_code)(&help_print_context_tree)), NULL);
      define_function_object("PRINT-CONTEXT-TREE", "(DEFUN PRINT-CONTEXT-TREE ((ROOT CONTEXT)))", ((cpp_function_code)(&print_context_tree)), NULL);
      define_function_object("ACCESS-IN-CONTEXT", "(DEFUN (ACCESS-IN-CONTEXT OBJECT) ((VALUE OBJECT) (HOMECONTEXT CONTEXT) (DONTINHERIT? BOOLEAN)) :PUBLIC? TRUE)", ((cpp_function_code)(&access_in_context)), NULL);
      define_function_object("HELP-INSERT-A-CS-VALUE", "(DEFUN HELP-INSERT-A-CS-VALUE ((KVCONS KV-CONS) (NEWVALUE OBJECT) (TARGET CONTEXT) (OVERWRITE? BOOLEAN)))", ((cpp_function_code)(&help_insert_a_cs_value)), NULL);
      define_method_object("(DEFMETHOD INSERT-AT ((SELF CS-VALUE) (CONTEXT CONTEXT) (NEWVALUE OBJECT)) :PUBLIC? TRUE)", ((cpp_method_code)(&Cs_Value::insert_at)), ((cpp_method_code)(NULL)));
      define_function_object("UPDATE-IN-CONTEXT", "(DEFUN (UPDATE-IN-CONTEXT OBJECT) ((OLDVALUE OBJECT) (NEWVALUE OBJECT) (HOMECONTEXT CONTEXT) (COPYTOCHILDREN? BOOLEAN)) :PUBLIC? TRUE)", ((cpp_function_code)(&update_in_context)), NULL);
      define_function_object("COPY-CURRENT-VALUE-TO-CHILDREN", "(DEFUN COPY-CURRENT-VALUE-TO-CHILDREN ((CSVALUE CS-VALUE) (HOMECONTEXT CONTEXT) (PARENTVALUE OBJECT)))", ((cpp_function_code)(&copy_current_value_to_children)), NULL);
      define_function_object("STARTUP-CONTEXTS", "(DEFUN STARTUP-CONTEXTS () :PUBLIC? TRUE)", ((cpp_function_code)(&startup_contexts)), NULL);
      { Method_Slot* function = lookup_function(SYM_CONTEXTS_STELLA_STARTUP_CONTEXTS);

        set_dynamic_slot_value(function->dynamic_slots, SYM_CONTEXTS_STELLA_METHOD_STARTUP_CLASSNAME, wrap_string("Startup-Contexts"), NULL_STRING_WRAPPER);
      }
    }
    if (current_startup_time_phaseP(8)) {
      finalize_slots();
      cleanup_unfinalized_classes();
    }
    if (current_startup_time_phaseP(9)) {
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *UNLINK-DISCARDED-CONTEXTS-ON-READ?* BOOLEAN TRUE :DOCUMENTATION \"Eliminate pointers to discarded contexts while\naccessing a CS-VALUE context table.\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *UNLINK-DISCARDED-CONTEXTS-ON-WRITE?* BOOLEAN TRUE :DOCUMENTATION \"Eliminate pointers to discarded contexts while\ninserting into a CS-VALUE context table.\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *CONTEXT-BACKTRACKING-MODE* BOOLEAN FALSE :DOCUMENTATION \"If true, indicates that contexts are being allocated\nand deallocated in depth-first fashion, and that deallocation of\nCS-VALUE entries is taken care of during context destruction.\")");
    }
  }
}

Surrogate* SGT_CONTEXTS_STELLA_CS_VALUE = NULL;

Surrogate* SGT_CONTEXTS_STELLA_MODULE = NULL;

Surrogate* SGT_CONTEXTS_STELLA_WORLD = NULL;

Symbol* SYM_CONTEXTS_STELLA_WORLD_NAME = NULL;

Symbol* SYM_CONTEXTS_STELLA_STARTUP_CONTEXTS = NULL;

Symbol* SYM_CONTEXTS_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella
