// date-time-parser.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

Cons* WHITESPACE_CHARS = NULL;

Cons* TIME_DIVIDERS = NULL;

Cons* DATE_DIVIDERS = NULL;

// If TRUE, an error will be signalled if parse-date-time is unable
//    to determine the time/date format of the string.
DEFINE_STELLA_SPECIAL(oERROR_ON_MISMATCHo, boolean , FALSE);

Object* hashlist_expander(Cons* list, Hash_Table* table) {
  return (listO(7, SYM_DATE_TIME_PARSER_STELLA_FOREACH, SYM_DATE_TIME_PARSER_STELLA_ITEM, SYM_DATE_TIME_PARSER_STELLA_IN, listO(3, SYM_DATE_TIME_PARSER_STELLA_CAST, list, cons(listO(4, SYM_DATE_TIME_PARSER_STELLA_CONS, SYM_DATE_TIME_PARSER_STELLA_OF, SYM_DATE_TIME_PARSER_STELLA_CONS, NIL), NIL)), SYM_DATE_TIME_PARSER_STELLA_DO, listO(3, SYM_DATE_TIME_PARSER_STELLA_INSERT_AT, table, listO(3, listO(3, SYM_DATE_TIME_PARSER_STELLA_FIRST, SYM_DATE_TIME_PARSER_STELLA_ITEM, NIL), listO(3, SYM_DATE_TIME_PARSER_STELLA_SECOND, SYM_DATE_TIME_PARSER_STELLA_ITEM, NIL), NIL)), NIL));
}

DEFINE_STELLA_SPECIAL(oWEEKDAY_STRINGSo, String_Hash_Table* , NULL);

DEFINE_STELLA_SPECIAL(oMONTH_STRINGSo, String_To_Integer_Hash_Table* , NULL);

DEFINE_STELLA_SPECIAL(oZONE_STRINGSo, String_Hash_Table* , NULL);

DEFINE_STELLA_SPECIAL(oSPECIAL_STRINGSo, String_Hash_Table* , NULL);

DEFINE_STELLA_SPECIAL(oSPECIAL_SYMBOLSo, Cons* , NULL);

DEFINE_STELLA_SPECIAL(oDATE_TIME_PATTERNSo, Cons* , NULL);

void yesterday(Decoded_Date_Time* parsed_values) {
  today(parsed_values);
  { int jd = compute_julian_day(parsed_values->date_time_year, parsed_values->date_time_month, parsed_values->date_time_day);

    { int year = NULL_INTEGER;
      int month = NULL_INTEGER;
      int day = NULL_INTEGER;
      Keyword* dow = NULL;

      year = compute_calendar_date(jd - 1, month, day, dow);
      parsed_values->date_time_day = day;
      parsed_values->date_time_month = month;
      parsed_values->date_time_year = year;
      parsed_values->date_time_dow = dow;
    }
  }
}

void today(Decoded_Date_Time* parsed_values) {
  { int year = NULL_INTEGER;
    int month = NULL_INTEGER;
    int day = NULL_INTEGER;
    Keyword* dow = NULL;

    year = make_current_date_time()->get_calendar_date(parsed_values->date_time_zone, month, day, dow);
    parsed_values->date_time_day = day;
    parsed_values->date_time_month = month;
    parsed_values->date_time_year = year;
    parsed_values->date_time_dow = dow;
  }
}

void tomorrow(Decoded_Date_Time* parsed_values) {
  today(parsed_values);
  { int jd = compute_julian_day(parsed_values->date_time_year, parsed_values->date_time_month, parsed_values->date_time_day);

    { int year = NULL_INTEGER;
      int month = NULL_INTEGER;
      int day = NULL_INTEGER;
      Keyword* dow = NULL;

      year = compute_calendar_date(jd + 1, month, day, dow);
      parsed_values->date_time_day = day;
      parsed_values->date_time_month = month;
      parsed_values->date_time_year = year;
      parsed_values->date_time_dow = dow;
    }
  }
}

void now(Decoded_Date_Time* parsed_values) {
  { int hour = NULL_INTEGER;
    int minute = NULL_INTEGER;
    int second = NULL_INTEGER;
    int millis = NULL_INTEGER;

    hour = make_current_date_time()->get_time(parsed_values->date_time_zone, minute, second, millis);
    parsed_values->date_time_millis = millis;
    parsed_values->date_time_second = second;
    parsed_values->date_time_minute = minute;
    parsed_values->date_time_hour = hour;
  }
}

Symbol* am_pm(Object* string) {
  if (subtype_of_stringP(safe_primary_type(string))) {
    { Object* string_000 = string;
      String_Wrapper* string = ((String_Wrapper*)(string_000));

      if (string_eqlP(string->wrapper_value, "am")) {
        return (SYM_DATE_TIME_PARSER_STELLA_AM);
      }
      else if (string_eqlP(string->wrapper_value, "pm")) {
        return (SYM_DATE_TIME_PARSER_STELLA_PM);
      }
      else {
        return (NULL);
      }
    }
  }
  else {
    return (NULL);
  }
}

Symbol* noon_midn(Object* string) {
  if (subtype_of_stringP(safe_primary_type(string))) {
    { Object* string_000 = string;
      String_Wrapper* string = ((String_Wrapper*)(string_000));

      if (string_eqlP(string->wrapper_value, "noon")) {
        return (SYM_DATE_TIME_PARSER_STELLA_NOON);
      }
      else if (string_eqlP(string->wrapper_value, "midnight")) {
        return (SYM_DATE_TIME_PARSER_STELLA_MIDN);
      }
      else {
        return (NULL);
      }
    }
  }
  else {
    return (NULL);
  }
}

Keyword* weekday(Object* thing) {
  if (subtype_of_stringP(safe_primary_type(thing))) {
    { Object* thing_000 = thing;
      String_Wrapper* thing = ((String_Wrapper*)(thing_000));

      return (((Keyword*)(oWEEKDAY_STRINGSo.get()->lookup(thing->wrapper_value))));
    }
  }
  else {
    return (NULL);
  }
}

Integer_Wrapper* month(Object* thing) {
  { Surrogate* test_value_000 = safe_primary_type(thing);

    if (subtype_of_stringP(test_value_000)) {
      { Object* thing_000 = thing;
        String_Wrapper* thing = ((String_Wrapper*)(thing_000));

        { int month_number = oMONTH_STRINGSo.get()->lookup(thing->wrapper_value);

          if (month_number != NULL_INTEGER) {
            return (wrap_integer(month_number));
          }
          else {
            return (NULL);
          }
        }
      }
    }
    else if (subtype_of_integerP(test_value_000)) {
      { Object* thing_001 = thing;
        Integer_Wrapper* thing = ((Integer_Wrapper*)(thing_001));

        if ((1 <= thing->wrapper_value) &&
            (thing->wrapper_value <= 12)) {
          return (thing);
        }
        else {
          return (NULL);
        }
      }
    }
    else {
      return (NULL);
    }
  }
}

Float_Wrapper* zone(Object* thing) {
  { Surrogate* test_value_000 = safe_primary_type(thing);

    if (subtype_of_stringP(test_value_000)) {
      { Object* thing_000 = thing;
        String_Wrapper* thing = ((String_Wrapper*)(thing_000));

        { Float_Wrapper* zone_number = ((Float_Wrapper*)(oZONE_STRINGSo.get()->lookup(thing->wrapper_value)));

          if (((boolean)(zone_number))) {
            return (zone_number);
          }
          else {
            return (NULL);
          }
        }
      }
    }
    else if (subtype_of_floatP(test_value_000)) {
      { Object* thing_001 = thing;
        Float_Wrapper* thing = ((Float_Wrapper*)(thing_001));

        if ((-12.0 <= thing->wrapper_value) &&
            (thing->wrapper_value <= 14.0)) {
          return (thing);
        }
        else {
          return (NULL);
        }
      }
    }
    else if (subtype_of_integerP(test_value_000)) {
      { Object* thing_002 = thing;
        Integer_Wrapper* thing = ((Integer_Wrapper*)(thing_002));

        { int offset = thing->wrapper_value;

          if ((-12 <= offset) &&
              (offset <= 14)) {
            return (wrap_float(((double)(offset))));
          }
          else if ((-1200 <= offset) &&
              (offset <= 1400)) {
            return (wrap_float(floor_(offset / 100.0) + ((offset % 100) / 60.0)));
          }
          else {
            return (NULL);
          }
        }
      }
    }
    else {
      return (NULL);
    }
  }
}

Symbol* specialp(Object* string) {
  if (subtype_of_stringP(safe_primary_type(string))) {
    { Object* string_000 = string;
      String_Wrapper* string = ((String_Wrapper*)(string_000));

      return (((Symbol*)(oSPECIAL_STRINGSo.get()->lookup(string->wrapper_value))));
    }
  }
  else {
    return (NULL);
  }
}

Boolean_Wrapper* secondp(Object* number) {
  { Surrogate* test_value_000 = safe_primary_type(number);

    if (subtype_of_integerP(test_value_000)) {
      { Object* number_000 = number;
        Integer_Wrapper* number = ((Integer_Wrapper*)(number_000));

        return ((((0 <= number->wrapper_value) &&
            (number->wrapper_value <= 59)) ? TRUE_WRAPPER : FALSE_WRAPPER));
      }
    }
    else if (subtype_of_floatP(test_value_000)) {
      { Object* number_001 = number;
        Float_Wrapper* number = ((Float_Wrapper*)(number_001));

        return ((((0.0 <= number->wrapper_value) &&
            (number->wrapper_value < 60.0)) ? TRUE_WRAPPER : FALSE_WRAPPER));
      }
    }
    else {
      return (FALSE_WRAPPER);
    }
  }
}

Boolean_Wrapper* minute(Object* number) {
  if (subtype_of_integerP(safe_primary_type(number))) {
    { Object* number_000 = number;
      Integer_Wrapper* number = ((Integer_Wrapper*)(number_000));

      return ((((0 <= number->wrapper_value) &&
          (number->wrapper_value <= 59)) ? TRUE_WRAPPER : FALSE_WRAPPER));
    }
  }
  else {
    return (FALSE_WRAPPER);
  }
}

Boolean_Wrapper* hour(Object* number) {
  if (subtype_of_integerP(safe_primary_type(number))) {
    { Object* number_000 = number;
      Integer_Wrapper* number = ((Integer_Wrapper*)(number_000));

      return ((((0 <= number->wrapper_value) &&
          (number->wrapper_value <= 23)) ? TRUE_WRAPPER : FALSE_WRAPPER));
    }
  }
  else {
    return (FALSE_WRAPPER);
  }
}

Boolean_Wrapper* day(Object* number) {
  if (subtype_of_integerP(safe_primary_type(number))) {
    { Object* number_000 = number;
      Integer_Wrapper* number = ((Integer_Wrapper*)(number_000));

      return ((((1 <= number->wrapper_value) &&
          (number->wrapper_value <= 31)) ? TRUE_WRAPPER : FALSE_WRAPPER));
    }
  }
  else {
    return (FALSE_WRAPPER);
  }
}

Boolean_Wrapper* year(Object* number) {
  if (subtype_of_integerP(safe_primary_type(number))) {
    { Object* number_000 = number;
      Integer_Wrapper* number = ((Integer_Wrapper*)(number_000));

      return ((((-4713 <= number->wrapper_value) ||
          (number->wrapper_value <= 5000)) ? TRUE_WRAPPER : FALSE_WRAPPER));
    }
  }
  else {
    return (FALSE_WRAPPER);
  }
}

Boolean_Wrapper* time_divider(Object* thecharacter) {
  if (subtype_of_characterP(safe_primary_type(thecharacter))) {
    { Object* thecharacter_000 = thecharacter;
      Character_Wrapper* thecharacter = ((Character_Wrapper*)(thecharacter_000));

      return ((TIME_DIVIDERS->memberP(thecharacter) ? TRUE_WRAPPER : FALSE_WRAPPER));
    }
  }
  else {
    return (FALSE_WRAPPER);
  }
}

Boolean_Wrapper* date_divider(Object* thecharacter) {
  if (subtype_of_characterP(safe_primary_type(thecharacter))) {
    { Object* thecharacter_000 = thecharacter;
      Character_Wrapper* thecharacter = ((Character_Wrapper*)(thecharacter_000));

      return ((DATE_DIVIDERS->memberP(thecharacter) ? TRUE_WRAPPER : FALSE_WRAPPER));
    }
  }
  else {
    return (FALSE_WRAPPER);
  }
}

Cons* match_substring(char* substring) {
  { char* teststring = string_downcase(substring);
    Object* test_value = NULL;

    if (((boolean)(((Integer_Wrapper*)(test_value = month(wrap_string(teststring))))))) {
      return (cons_list(2, SYM_DATE_TIME_PARSER_STELLA_MONTH, test_value));
    }
    if (((boolean)(((Keyword*)(test_value = weekday(wrap_string(teststring))))))) {
      return (cons_list(2, SYM_DATE_TIME_PARSER_STELLA_WEEKDAY, test_value));
    }
    if (((boolean)(((Symbol*)(test_value = am_pm(wrap_string(teststring))))))) {
      return (cons_list(2, SYM_DATE_TIME_PARSER_STELLA_AM_PM, test_value));
    }
    if (((boolean)(((Symbol*)(test_value = noon_midn(wrap_string(teststring))))))) {
      return (cons_list(2, SYM_DATE_TIME_PARSER_STELLA_NOON_MIDN, test_value));
    }
    if (((boolean)(((Float_Wrapper*)(test_value = zone(wrap_string(teststring))))))) {
      return (cons_list(2, SYM_DATE_TIME_PARSER_STELLA_ZONE, test_value));
    }
    if (((boolean)(((Symbol*)(test_value = specialp(wrap_string(teststring))))))) {
      return (cons_list(2, SYM_DATE_TIME_PARSER_STELLA_SPECIAL, test_value));
    }
    if (oERROR_ON_MISMATCHo.get()) {
      { Output_String_Stream* stream_000 = new_output_string_stream();

        *(stream_000->native_stream) << "\"~A\" is not a recognized word or abbreviation." << "`" << substring << "'";
        throw *new_stella_exception(stream_000->the_string_reader());
      }
    }
    else {
      return (NIL);
    }
  }
}

int digit_value(char renamed_Char) {
  return ((int)(unsigned char) renamed_Char - (int)(unsigned char) '0');
}

Cons* decompose_date_time_string(char* string, int start, int end) {
  { int string_index = start;
    boolean next_negativeP = FALSE;
    Cons* parts_list = NIL;
    char next_char = NULL_CHARACTER;
    char prev_char = NULL_CHARACTER;

    if (end == NULL_INTEGER) {
      end = strlen(string);
    }
    while (string_index < end) {
      next_char = string[string_index];
      prev_char = ((string_index == start) ? NULL_CHARACTER : string[(string_index - 1)]);
      if (oCHARACTER_TYPE_TABLEo[(int)(unsigned char) next_char] == KWD_DATE_TIME_PARSER_LETTER) {
        { int scan_index = NULL_INTEGER;
          int iter_000 = string_index + 1;

          while (TRUE) {
            scan_index = iter_000;
            iter_000 = iter_000 + 1;
            if ((scan_index == end) ||
                (!(oCHARACTER_TYPE_TABLEo[(int)(unsigned char) (string[scan_index])] == KWD_DATE_TIME_PARSER_LETTER))) {
              { Cons* match_symbol = match_substring(string_subsequence(string, string_index, scan_index));

                if (match_symbol == NIL) {
                  return (NIL);
                }
                else {
                  parts_list = cons(match_symbol, parts_list);
                }
              }
              string_index = scan_index;
              break;
            }
          }
        }
      }
      else if (oCHARACTER_TYPE_TABLEo[(int)(unsigned char) next_char] == KWD_DATE_TIME_PARSER_DIGIT) {
        { int numeric_value = digit_value(next_char);
          int fractional_value = 0;
          double fractional_divisor = 1.0;
          boolean in_fractionP = FALSE;

          { int scan_index = NULL_INTEGER;
            int iter_001 = string_index + 1;

            while (TRUE) {
              scan_index = iter_001;
              iter_001 = iter_001 + 1;
              if ((scan_index == end) ||
                  (!((oCHARACTER_TYPE_TABLEo[(int)(unsigned char) (string[scan_index])] == KWD_DATE_TIME_PARSER_DIGIT) ||
                  ((!in_fractionP) &&
                   (string[scan_index] == '.'))))) {
                if (next_negativeP) {
                  next_negativeP = FALSE;
                  numeric_value = 0 - numeric_value;
                  fractional_value = 0 - fractional_value;
                }
                if (in_fractionP) {
                  parts_list = cons(wrap_float(numeric_value + (fractional_value / fractional_divisor)), parts_list);
                }
                else {
                  parts_list = cons(wrap_integer(numeric_value), parts_list);
                }
                string_index = scan_index;
                break;
              }
              if (in_fractionP) {
                fractional_value = (fractional_value * 10) + digit_value(string[scan_index]);
                fractional_divisor = fractional_divisor * 10.0;
              }
              else if (string[scan_index] == '.') {
                in_fractionP = TRUE;
              }
              else {
                numeric_value = (numeric_value * 10) + digit_value(string[scan_index]);
              }
            }
          }
        }
      }
      else if ((next_char == '-') &&
          ((prev_char == NULL_CHARACTER) ||
           (WHITESPACE_CHARS->memberP(wrap_character(prev_char)) ||
            DATE_DIVIDERS->memberP(wrap_character(prev_char))))) {
        next_negativeP = TRUE;
        string_index = string_index + 1;
      }
      else if (TIME_DIVIDERS->memberP(wrap_character(next_char))) {
        parts_list = cons(cons_list(2, SYM_DATE_TIME_PARSER_STELLA_TIME_DIVIDER, wrap_character(next_char)), parts_list);
        string_index = string_index + 1;
      }
      else if (DATE_DIVIDERS->memberP(wrap_character(next_char))) {
        parts_list = cons(cons_list(2, SYM_DATE_TIME_PARSER_STELLA_DATE_DIVIDER, wrap_character(next_char)), parts_list);
        string_index = string_index + 1;
      }
      else if (WHITESPACE_CHARS->memberP(wrap_character(next_char))) {
        string_index = string_index + 1;
      }
      else if (next_char == '+') {
        parts_list = cons(cons_list(2, SYM_DATE_TIME_PARSER_STELLA_DATE_DIVIDER, wrap_character(next_char)), parts_list);
        string_index = string_index + 1;
      }
      else if (next_char == '(') {
        while ((!(string_index == end)) &&
            (!(string[string_index] == ')'))) {
          string_index = string_index + 1;
        }
      }
      else {
        if (oERROR_ON_MISMATCHo.get()) {
          { Output_String_Stream* stream_000 = new_output_string_stream();

            *(stream_000->native_stream) << ">>> Bogus character at position " << "`" << string_index << "'" << " in \"" << "`" << string << "'" << "\"";
            throw *new_stella_exception(stream_000->the_string_reader());
          }
        }
        else {
          return (NIL);
        }
      }
    }
    return (parts_list->reverse());
  }
}

Cons* match_pattern_element(Symbol* pattern_element, Object* datum_element) {
  if (isaP(datum_element, SGT_DATE_TIME_PARSER_STELLA_CONS)) {
    { Object* datum_type = ((((Cons*)(datum_element))->value == SYM_DATE_TIME_PARSER_STELLA_SPECIAL) ? ((Cons*)(datum_element))->rest->value : ((Cons*)(datum_element))->value);

      if (datum_type == pattern_element) {
        return (((Cons*)(datum_element)));
      }
      else {
        return (NIL);
      }
    }
  }
  else if (oSPECIAL_SYMBOLSo.get()->memberP(pattern_element)) {
    return (NIL);
  }
  else {
    { cpp_function_code function = lookup_function(pattern_element)->function_code;
      Object* match = ((Object*  (*) (Object*))function)(datum_element);

      if (subtype_of_booleanP(safe_primary_type(match))) {
        { Object* match_000 = match;
          Boolean_Wrapper* match = ((Boolean_Wrapper*)(match_000));

          if (coerce_wrapped_boolean_to_boolean(match)) {
            return (cons_list(2, pattern_element, datum_element));
          }
          else {
            return (NIL);
          }
        }
      }
      else {
        if (((boolean)(match))) {
          return (cons_list(2, pattern_element, match));
        }
        else {
          return (NIL);
        }
      }
    }
  }
}

Cons* match_pattern(Cons* pattern, Cons* datum, int datum_length) {
  if (pattern->length() < datum_length) {
    return (NIL);
  }
  else {
    { Cons* form_list = NIL;
      Cons* pattern_cursor = pattern;
      Cons* datum_cursor = datum;
      Object* pattern_element = NULL;
      Object* datum_element = NULL;
      boolean optionalP = FALSE;
      Cons* matching = NIL;

      while (!((pattern_cursor == NIL) ||
          (datum_cursor == NIL))) {
        pattern_element = pattern_cursor->value;
        datum_element = datum_cursor->value;
        optionalP = isaP(pattern_element, SGT_DATE_TIME_PARSER_STELLA_CONS);
        matching = match_pattern_element((optionalP ? ((Symbol*)(((Cons*)(pattern_element))->value)) : ((Symbol*)(pattern_element))), datum_element);
        if (!(matching == NIL)) {
          { Object* form_type = matching->value;

            if (!((form_type == SYM_DATE_TIME_PARSER_STELLA_TIME_DIVIDER) ||
                (form_type == SYM_DATE_TIME_PARSER_STELLA_DATE_DIVIDER))) {
              form_list = cons(matching, form_list);
            }
          }
        }
        else if (optionalP) {
          datum_cursor = cons(datum_element, datum_cursor);
        }
        else {
          return (NIL);
        }
        pattern_cursor = pattern_cursor->rest;
        datum_cursor = datum_cursor->rest;
      }
      if ((pattern_cursor == NIL) &&
          (datum_cursor == NIL)) {
        return (form_list->reverse());
      }
      else if (pattern_cursor == NIL) {
        return (NIL);
      }
      else if (datum_cursor == NIL) {
        { Object* element = NULL;
          Cons* iter_000 = pattern_cursor;

          while (!(iter_000 == NIL)) {
            element = iter_000->value;
            iter_000 = iter_000->rest;
            if (!isaP(element, SGT_DATE_TIME_PARSER_STELLA_CONS)) {
              return (NIL);
            }
          }
        }
        return (form_list->reverse());
      }
      else {
        throw *new_stella_exception("Shouldn't get here!");
      }
    }
  }
}

void deal_with_noon_midn(Object* form_value, Decoded_Date_Time* parsed_values) {
  if (form_value == SYM_DATE_TIME_PARSER_STELLA_NOON) {
    parsed_values->date_time_hour = 12;
  }
  else if (form_value == SYM_DATE_TIME_PARSER_STELLA_MIDN) {
    parsed_values->date_time_hour = 0;
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "Unrecognized symbol: ~A" << "`" << form_value << "'";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  parsed_values->date_time_minute = 0;
  parsed_values->date_time_second = 0;
}

void deal_with_am_pm(Object* form_value, Decoded_Date_Time* parsed_values) {
  { int hour = parsed_values->date_time_hour;

    if (form_value == SYM_DATE_TIME_PARSER_STELLA_AM) {
      if (hour == 12) {
        parsed_values->date_time_hour = 0;
      }
      else if (!((0 <= hour) &&
          (hour <= 12))) {
        if (oERROR_ON_MISMATCHo.get()) {
          { Output_String_Stream* stream_000 = new_output_string_stream();

            *(stream_000->native_stream) << "`" << hour << "'" << " is not an AM hour.";
            throw *new_stella_exception(stream_000->the_string_reader());
          }
        }
      }
    }
    else if (form_value == SYM_DATE_TIME_PARSER_STELLA_PM) {
      if ((0 <= hour) &&
          (hour <= 11)) {
        parsed_values->date_time_hour = hour + 12;
      }
    }
    else {
      { Output_String_Stream* stream_001 = new_output_string_stream();

        *(stream_001->native_stream) << "`" << form_value << "'" << "isn't AM/PM - this shouldn't happen.";
        throw *new_stella_exception(stream_001->the_string_reader());
      }
    }
  }
}

void set_time_values(Cons* string_form, Decoded_Date_Time* parsed_values) {
  { Object* form_type = NULL;
    Object* form_value = NULL;

    { Cons* form_part = NULL;
      Cons* iter_000 = string_form;

      while (!(iter_000 == NIL)) {
        form_part = ((Cons*)(iter_000->value));
        iter_000 = iter_000->rest;
        form_type = form_part->value;
        form_value = form_part->rest->value;
        { Generalized_Symbol* test_value_000 = ((Generalized_Symbol*)(form_type));

          if (test_value_000 == SYM_DATE_TIME_PARSER_STELLA_SECONDP) {
            { Surrogate* test_value_001 = safe_primary_type(form_value);

              if (subtype_of_integerP(test_value_001)) {
                { Object* form_value_000 = form_value;
                  Integer_Wrapper* form_value = ((Integer_Wrapper*)(form_value_000));

                  parsed_values->date_time_second = form_value->wrapper_value;
                }
              }
              else if (subtype_of_floatP(test_value_001)) {
                { Object* form_value_001 = form_value;
                  Float_Wrapper* form_value = ((Float_Wrapper*)(form_value_001));

                  parsed_values->date_time_second = floor_(form_value->wrapper_value);
                  parsed_values->date_time_millis = round(1000.0 * (form_value->wrapper_value - floor_(form_value->wrapper_value)));
                }
              }
              else {
                { Output_String_Stream* stream_000 = new_output_string_stream();

                  *(stream_000->native_stream) << "`" << test_value_001 << "'" << " is not a valid case option";
                  throw *new_stella_exception(stream_000->the_string_reader());
                }
              }
            }
          }
          else if (test_value_000 == SYM_DATE_TIME_PARSER_STELLA_MINUTE) {
            parsed_values->date_time_minute = ((Integer_Wrapper*)(form_value))->wrapper_value;
          }
          else if (test_value_000 == SYM_DATE_TIME_PARSER_STELLA_HOUR) {
            parsed_values->date_time_hour = ((Integer_Wrapper*)(form_value))->wrapper_value;
          }
          else if (test_value_000 == SYM_DATE_TIME_PARSER_STELLA_DAY) {
            parsed_values->date_time_day = ((Integer_Wrapper*)(form_value))->wrapper_value;
          }
          else if (test_value_000 == SYM_DATE_TIME_PARSER_STELLA_MONTH) {
            parsed_values->date_time_month = ((Integer_Wrapper*)(form_value))->wrapper_value;
          }
          else if (test_value_000 == SYM_DATE_TIME_PARSER_STELLA_YEAR) {
            parsed_values->date_time_year = ((Integer_Wrapper*)(form_value))->wrapper_value;
          }
          else if (test_value_000 == SYM_DATE_TIME_PARSER_STELLA_ZONE) {
            parsed_values->date_time_zone = ((Float_Wrapper*)(form_value))->wrapper_value;
          }
          else if (test_value_000 == SYM_DATE_TIME_PARSER_STELLA_WEEKDAY) {
            parsed_values->date_time_dow = ((Keyword*)(form_value));
          }
          else if (test_value_000 == SYM_DATE_TIME_PARSER_STELLA_AM_PM) {
            deal_with_am_pm(form_value, parsed_values);
          }
          else if (test_value_000 == SYM_DATE_TIME_PARSER_STELLA_NOON_MIDN) {
            deal_with_noon_midn(form_value, parsed_values);
          }
          else if (test_value_000 == SYM_DATE_TIME_PARSER_STELLA_SPECIAL) {
            ((void  (*) (Decoded_Date_Time*))lookup_function(((Symbol*)(form_value)))->function_code)(parsed_values);
          }
          else {
            { Output_String_Stream* stream_001 = new_output_string_stream();

              *(stream_001->native_stream) << "Unrecognized symbol in form list: ~A." << "`" << form_type << "'";
              throw *new_stella_exception(stream_001->the_string_reader());
            }
          }
        }
      }
    }
  }
}

Decoded_Date_Time* parse_date_time(char* date_time_string, int start, int end, boolean error_on_mismatchP) {
  // Tries very hard to make sense out of the argument `date-time-string' and
  // returns a time structure if successful.  If not, it returns `null'.
  // If `error-on-mismatch?' is true, parse-date-time will signal an error instead of
  // returning `null'.  
  // Default values are 00:00:00 UTC on the current date
  { 
    BIND_STELLA_SPECIAL(oERROR_ON_MISMATCHo, boolean, error_on_mismatchP);
    { Cons* string_parts = decompose_date_time_string(date_time_string, start, end);
      int parts_length = string_parts->length();
      Cons* string_form = NIL;

      { Object* pattern = NULL;
        Cons* iter_000 = oDATE_TIME_PATTERNSo.get();

        while (!(iter_000 == NIL)) {
          pattern = iter_000->value;
          iter_000 = iter_000->rest;
          string_form = match_pattern(((Cons*)(pattern)), string_parts, parts_length);
          if (!(string_form == NIL)) {
            break;
          }
        }
      }
      if (string_form == NIL) {
        if (oERROR_ON_MISMATCHo.get()) {
          { Output_String_Stream* stream_000 = new_output_string_stream();

            *(stream_000->native_stream) << "\"" << "`" << date_time_string << "'" << "\" is not a recognized time/date format.";
            throw *new_stella_exception(stream_000->the_string_reader());
          }
        }
        else {
          return (NULL);
        }
      }
      else {
        { Decoded_Date_Time* parsed_values = make_current_date_time()->decode_calendar_date(get_local_time_zone());

          parsed_values->date_time_millis = 0;
          parsed_values->date_time_second = 0;
          parsed_values->date_time_minute = 0;
          parsed_values->date_time_hour = 0;
          set_time_values(string_form, parsed_values);
          return (parsed_values);
        }
      }
    }
  }
}

Calendar_Date* string_to_calendar_date(char* input) {
  // Returns a calendar date object representing the date and time
  // parsed from the `input' string.  If no valid parse is found,
  // `null' is returned.
  { Decoded_Date_Time* renamed_Do = parse_date_time(input, 0, NULL_INTEGER, FALSE);

    if (!((boolean)(renamed_Do))) {
      return (NULL);
    }
    else {
      return (renamed_Do->encode_calendar_date());
    }
  }
}

void help_startup_date_time_parser1() {
  {
    SYM_DATE_TIME_PARSER_STELLA_FOREACH = ((Symbol*)(intern_rigid_symbol_wrt_module("FOREACH", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_ITEM = ((Symbol*)(intern_rigid_symbol_wrt_module("ITEM", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_IN = ((Symbol*)(intern_rigid_symbol_wrt_module("IN", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_CAST = ((Symbol*)(intern_rigid_symbol_wrt_module("CAST", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_CONS = ((Symbol*)(intern_rigid_symbol_wrt_module("CONS", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_OF = ((Symbol*)(intern_rigid_symbol_wrt_module("OF", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_DO = ((Symbol*)(intern_rigid_symbol_wrt_module("DO", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_INSERT_AT = ((Symbol*)(intern_rigid_symbol_wrt_module("INSERT-AT", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_FIRST = ((Symbol*)(intern_rigid_symbol_wrt_module("FIRST", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_SECOND = ((Symbol*)(intern_rigid_symbol_wrt_module("SECOND", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_HASHLIST = ((Symbol*)(intern_rigid_symbol_wrt_module("HASHLIST", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_HASHLIST_EXPANDER = ((Symbol*)(intern_rigid_symbol_wrt_module("HASHLIST-EXPANDER", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_AM = ((Symbol*)(intern_rigid_symbol_wrt_module("AM", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_PM = ((Symbol*)(intern_rigid_symbol_wrt_module("PM", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_NOON = ((Symbol*)(intern_rigid_symbol_wrt_module("NOON", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_MIDN = ((Symbol*)(intern_rigid_symbol_wrt_module("MIDN", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_MONTH = ((Symbol*)(intern_rigid_symbol_wrt_module("MONTH", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_WEEKDAY = ((Symbol*)(intern_rigid_symbol_wrt_module("WEEKDAY", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_AM_PM = ((Symbol*)(intern_rigid_symbol_wrt_module("AM-PM", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_NOON_MIDN = ((Symbol*)(intern_rigid_symbol_wrt_module("NOON-MIDN", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_ZONE = ((Symbol*)(intern_rigid_symbol_wrt_module("ZONE", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_SPECIAL = ((Symbol*)(intern_rigid_symbol_wrt_module("SPECIAL", NULL, 0)));
    KWD_DATE_TIME_PARSER_LETTER = ((Keyword*)(intern_rigid_symbol_wrt_module("LETTER", NULL, 2)));
    KWD_DATE_TIME_PARSER_DIGIT = ((Keyword*)(intern_rigid_symbol_wrt_module("DIGIT", NULL, 2)));
    SYM_DATE_TIME_PARSER_STELLA_TIME_DIVIDER = ((Symbol*)(intern_rigid_symbol_wrt_module("TIME-DIVIDER", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_DATE_DIVIDER = ((Symbol*)(intern_rigid_symbol_wrt_module("DATE-DIVIDER", NULL, 0)));
    SGT_DATE_TIME_PARSER_STELLA_CONS = ((Surrogate*)(intern_rigid_symbol_wrt_module("CONS", NULL, 1)));
    SYM_DATE_TIME_PARSER_STELLA_SECONDP = ((Symbol*)(intern_rigid_symbol_wrt_module("SECONDP", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_MINUTE = ((Symbol*)(intern_rigid_symbol_wrt_module("MINUTE", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_HOUR = ((Symbol*)(intern_rigid_symbol_wrt_module("HOUR", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_DAY = ((Symbol*)(intern_rigid_symbol_wrt_module("DAY", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_YEAR = ((Symbol*)(intern_rigid_symbol_wrt_module("YEAR", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_STARTUP_DATE_TIME_PARSER = ((Symbol*)(intern_rigid_symbol_wrt_module("STARTUP-DATE-TIME-PARSER", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-STARTUP-CLASSNAME", NULL, 0)));
  }
}

void startup_date_time_parser() {
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (current_startup_time_phaseP(2)) {
      help_startup_date_time_parser1();
    }
    if (current_startup_time_phaseP(4)) {
      WHITESPACE_CHARS = get_quoted_tree("((#\\  #\\Tab #\\Linefeed #\\, #\\' #\\`) \"/STELLA\")", "/STELLA");
      TIME_DIVIDERS = get_quoted_tree("((#\\: #\\.) \"/STELLA\")", "/STELLA");
      DATE_DIVIDERS = get_quoted_tree("((#\\\\ #\\/ #\\-) \"/STELLA\")", "/STELLA");
      oWEEKDAY_STRINGSo.set(new_string_hash_table());
      oMONTH_STRINGSo.set(new_string_to_integer_hash_table());
      oZONE_STRINGSo.set(new_string_hash_table());
      oSPECIAL_STRINGSo.set(new_string_hash_table());
      oSPECIAL_SYMBOLSo.set(get_quoted_tree("((YESTERDAY TODAY TOMORROW NOW) \"/STELLA\")", "/STELLA"));
      { Cons* item = NULL;
        Cons* iter_039 = ((Cons*)(get_quoted_tree("(((\"monday\" :MONDAY) (\"mon\" :MONDAY) (\"tuesday\" :TUESDAY) (\"tues\" :TUESDAY) (\"tue\" :TUESDAY) (\"wednesday\" :WEDNESDAY) (\"wednes\" :WEDNESDAY) (\"wed\" :WEDNESDAY) (\"thursday\" :THURSDAY) (\"thurs\" :THURSDAY) (\"thu\" :THURSDAY) (\"friday\" :FRIDAY) (\"fri\" :FRIDAY) (\"saturday\" :SATURDAY) (\"sat\" :SATURDAY) (\"sunday\" :SUNDAY) (\"sun\" :SUNDAY)) \"/STELLA\")", "/STELLA")));

        while (!(iter_039 == NIL)) {
          item = ((Cons*)(iter_039->value));
          iter_039 = iter_039->rest;
          oWEEKDAY_STRINGSo.get()->insert_at(((String_Wrapper*)(item->value))->wrapper_value, item->rest->value);
        }
      }
      { Cons* item = NULL;
        Cons* iter_040 = ((Cons*)(get_quoted_tree("(((\"january\" 1) (\"jan\" 1) (\"february\" 2) (\"feb\" 2) (\"march\" 3) (\"mar\" 3) (\"april\" 4) (\"apr\" 4) (\"may\" 5) (\"june\" 6) (\"jun\" 6) (\"july\" 7) (\"jul\" 7) (\"august\" 8) (\"aug\" 8) (\"september\" 9) (\"sept\" 9) (\"sep\" 9) (\"october\" 10) (\"oct\" 10) (\"november\" 11) (\"nov\" 11) (\"december\" 12) (\"dec\" 12)) \"/STELLA\")", "/STELLA")));

        while (!(iter_040 == NIL)) {
          item = ((Cons*)(iter_040->value));
          iter_040 = iter_040->rest;
          oMONTH_STRINGSo.get()->insert_at(((String_Wrapper*)(item->value))->wrapper_value, ((Integer_Wrapper*)(item->rest->value))->wrapper_value);
        }
      }
      { Cons* item = NULL;
        Cons* iter_041 = ((Cons*)(get_quoted_tree("(((\"gmt\" 0.0) (\"utc\" 0.0) (\"est\" -5.0) (\"edt\" -4.0) (\"cst\" -6.0) (\"cdt\" -5.0) (\"mst\" -7.0) (\"mdt\" -6.0) (\"pst\" -8.0) (\"pdt\" -7.0) (\"A\" 1.0) (\"B\" 2.0) (\"C\" 3.0) (\"C*\" 3.5) (\"D\" 4.0) (\"D*\" 4.5) (\"E\" 5.0) (\"E*\" 4.5) (\"F\" 6.0) (\"F*\" 6.5) (\"G\" 7.0) (\"H\" 7.0) (\"I\" 9.0) (\"I*\" 9.5) (\"K\" 10.0) (\"K*\" 10.5) (\"L\" 11.0) (\"L*\" 11.5) (\"M\" 12.0) (\"N\" -1.0) (\"O\" -2.0) (\"P\" -3.0) (\"P*\" -3.5) (\"Q\" -4.0) (\"R\" -5.0) (\"S\" -6.0) (\"T\" -7.0) (\"U\" -8.0) (\"U*\" -8.5) (\"V\" -9.0) (\"V*\" -9.5) (\"W\" -10.0) (\"X\" -11.0) (\"Y\" -12.0) (\"Z\" 0.0)) \"/STELLA\")", "/STELLA")));

        while (!(iter_041 == NIL)) {
          item = ((Cons*)(iter_041->value));
          iter_041 = iter_041->rest;
          oZONE_STRINGSo.get()->insert_at(((String_Wrapper*)(item->value))->wrapper_value, item->rest->value);
        }
      }
      { Symbol* symbol = NULL;
        Cons* iter_042 = oSPECIAL_SYMBOLSo.get();

        while (!(iter_042 == NIL)) {
          symbol = ((Symbol*)(iter_042->value));
          iter_042 = iter_042->rest;
          oSPECIAL_STRINGSo.get()->insert_at(string_downcase(symbol->symbol_name), symbol);
        }
      }
      oDATE_TIME_PATTERNSo.set(get_quoted_tree("(((YEAR (DATE-DIVIDER) MONTH (DATE-DIVIDER) DAY (NOON-MIDN)) ((WEEKDAY) MONTH (DATE-DIVIDER) DAY (DATE-DIVIDER) YEAR (NOON-MIDN) (DATE-DIVIDER) (ZONE)) ((WEEKDAY) DAY (DATE-DIVIDER) MONTH (DATE-DIVIDER) YEAR (NOON-MIDN) (DATE-DIVIDER) (ZONE)) ((WEEKDAY) MONTH (DATE-DIVIDER) DAY (NOON-MIDN) (DATE-DIVIDER) (ZONE)) (MONTH (DATE-DIVIDER) YEAR (NOON-MIDN) (DATE-DIVIDER) (ZONE)) (YEAR (DATE-DIVIDER) MONTH (NOON-MIDN) (DATE-DIVIDER) (ZONE)) ((NOON-MIDN) (WEEKDAY) MONTH (DATE-DIVIDER) DAY (DATE-DIVIDER) YEAR) ((NOON-MIDN) (WEEKDAY) DAY (DATE-DIVIDER) MONTH (DATE-DIVIDER) YEAR) ((NOON-MIDN) (WEEKDAY) MONTH (DATE-DIVIDER) DAY) ((NOON-MIDN) YEAR (DATE-DIVIDER) MONTH (DATE-DIVIDER) DAY) ((NOON-MIDN) MONTH (DATE-DIVIDER) YEAR) ((NOON-MIDN) YEAR (DATE-DIVIDER) MONTH) (HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (NOON-MIDN (DATE-DIVIDER) (ZONE)) (HOUR (NOON-MIDN)) (YEAR (DATE-DIVIDER) MONTH (DATE-DIVIDER) DAY HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (YEAR (DATE-DIVIDER) MONTH (DATE-DIVIDER) DAY NOON-MIDN (DATE-DIVIDER) (ZONE)) (YEAR (DATE-DIVIDER) MONTH (DATE-DIVIDER) DAY NOW) ((WEEKDAY) MONTH (DATE-DIVIDER) DAY (DATE-DIVIDER) YEAR HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) ((WEEKDAY) DAY (DATE-DIVIDER) MONTH (DATE-DIVIDER) YEAR HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) ((WEEKDAY) MONTH (DATE-DIVIDER) DAY HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (MONTH (DATE-DIVIDER) YEAR HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (YEAR (DATE-DIVIDER) MONTH HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE) (WEEKDAY) MONTH (DATE-DIVIDER) DAY (DATE-DIVIDER) YEAR) (HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE) (WEEKDAY) DAY (DATE-DIVIDER) MONTH (DATE-DIVIDER) YEAR) (HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE) (WEEKDAY) MONTH (DATE-DIVIDER) DAY) (HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE) YEAR (DATE-DIVIDER) MONTH (DATE-DIVIDER) DAY) (HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE) MONTH (DATE-DIVIDER) YEAR) (HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE) YEAR (DATE-DIVIDER) MONTH) (WEEKDAY MONTH DAY HOUR (TIME-DIVIDER) MINUTE (TIME-DIVIDER) SECONDP (AM-PM) (ZONE) YEAR) ((WEEKDAY) DAY (DATE-DIVIDER) MONTH (DATE-DIVIDER) YEAR HOUR (TIME-DIVIDER) MINUTE (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) ((WEEKDAY) MONTH (DATE-DIVIDER) DAY (DATE-DIVIDER) YEAR HOUR (TIME-DIVIDER) MINUTE (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (YEAR (DATE-DIVIDER) (MONTH)) (MONTH (DATE-DIVIDER) (YEAR)) ((WEEKDAY) MONTH DAY HOUR TIME-DIVIDER MINUTE (TIME-DIVIDER) (SECONDP) YEAR) (NOW (YESTERDAY)) ((YESTERDAY) NOW) (NOW (TODAY)) ((TODAY) NOW) (NOW (TOMORROW)) ((TOMORROW) NOW) (YESTERDAY (NOON-MIDN)) ((NOON-MIDN) YESTERDAY) (TODAY (NOON-MIDN)) ((NOON-MIDN) TODAY) (TOMORROW (NOON-MIDN)) ((NOON-MIDN) TOMORROW) (YESTERDAY HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (TOMORROW HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (TODAY HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE))) \"/STELLA\")", "/STELLA"));
    }
    if (current_startup_time_phaseP(6)) {
      finalize_classes();
    }
    if (current_startup_time_phaseP(7)) {
      define_function_object("HASHLIST-EXPANDER", "(DEFUN (HASHLIST-EXPANDER OBJECT) ((LIST (CONS OF CONS)) (TABLE HASH-TABLE)))", ((cpp_function_code)(&hashlist_expander)), NULL);
      define_function_object("YESTERDAY", "(DEFUN YESTERDAY ((PARSED-VALUES DECODED-DATE-TIME)))", ((cpp_function_code)(&yesterday)), NULL);
      define_function_object("TODAY", "(DEFUN TODAY ((PARSED-VALUES DECODED-DATE-TIME)))", ((cpp_function_code)(&today)), NULL);
      define_function_object("TOMORROW", "(DEFUN TOMORROW ((PARSED-VALUES DECODED-DATE-TIME)))", ((cpp_function_code)(&tomorrow)), NULL);
      define_function_object("NOW", "(DEFUN NOW ((PARSED-VALUES DECODED-DATE-TIME)))", ((cpp_function_code)(&now)), NULL);
      define_function_object("AM-PM", "(DEFUN (AM-PM SYMBOL) ((STRING OBJECT)))", ((cpp_function_code)(&am_pm)), NULL);
      define_function_object("NOON-MIDN", "(DEFUN (NOON-MIDN SYMBOL) ((STRING OBJECT)))", ((cpp_function_code)(&noon_midn)), NULL);
      define_function_object("WEEKDAY", "(DEFUN (WEEKDAY KEYWORD) ((THING OBJECT)))", ((cpp_function_code)(&weekday)), NULL);
      define_function_object("MONTH", "(DEFUN (MONTH INTEGER-WRAPPER) ((THING OBJECT)))", ((cpp_function_code)(&month)), NULL);
      define_function_object("ZONE", "(DEFUN (ZONE FLOAT-WRAPPER) ((THING OBJECT)))", ((cpp_function_code)(&zone)), NULL);
      define_function_object("SPECIALP", "(DEFUN (SPECIALP SYMBOL) ((STRING OBJECT)))", ((cpp_function_code)(&specialp)), NULL);
      define_function_object("SECONDP", "(DEFUN (SECONDP BOOLEAN-WRAPPER) ((NUMBER OBJECT)))", ((cpp_function_code)(&secondp)), NULL);
      define_function_object("MINUTE", "(DEFUN (MINUTE BOOLEAN-WRAPPER) ((NUMBER OBJECT)))", ((cpp_function_code)(&minute)), NULL);
      define_function_object("HOUR", "(DEFUN (HOUR BOOLEAN-WRAPPER) ((NUMBER OBJECT)))", ((cpp_function_code)(&hour)), NULL);
      define_function_object("DAY", "(DEFUN (DAY BOOLEAN-WRAPPER) ((NUMBER OBJECT)))", ((cpp_function_code)(&day)), NULL);
      define_function_object("YEAR", "(DEFUN (YEAR BOOLEAN-WRAPPER) ((NUMBER OBJECT)))", ((cpp_function_code)(&year)), NULL);
      define_function_object("TIME-DIVIDER", "(DEFUN (TIME-DIVIDER BOOLEAN-WRAPPER) ((THECHARACTER OBJECT)))", ((cpp_function_code)(&time_divider)), NULL);
      define_function_object("DATE-DIVIDER", "(DEFUN (DATE-DIVIDER BOOLEAN-WRAPPER) ((THECHARACTER OBJECT)))", ((cpp_function_code)(&date_divider)), NULL);
      define_function_object("MATCH-SUBSTRING", "(DEFUN (MATCH-SUBSTRING CONS) ((SUBSTRING STRING)))", ((cpp_function_code)(&match_substring)), NULL);
      define_function_object("DIGIT-VALUE", "(DEFUN (DIGIT-VALUE INTEGER) ((CHAR CHARACTER)))", ((cpp_function_code)(&digit_value)), NULL);
      define_function_object("DECOMPOSE-DATE-TIME-STRING", "(DEFUN (DECOMPOSE-DATE-TIME-STRING CONS) ((STRING STRING) (START INTEGER) (END INTEGER)))", ((cpp_function_code)(&decompose_date_time_string)), NULL);
      define_function_object("MATCH-PATTERN-ELEMENT", "(DEFUN (MATCH-PATTERN-ELEMENT CONS) ((PATTERN-ELEMENT SYMBOL) (DATUM-ELEMENT OBJECT)))", ((cpp_function_code)(&match_pattern_element)), NULL);
      define_function_object("MATCH-PATTERN", "(DEFUN (MATCH-PATTERN CONS) ((PATTERN CONS) (DATUM CONS) (DATUM-LENGTH INTEGER)))", ((cpp_function_code)(&match_pattern)), NULL);
      define_function_object("DEAL-WITH-NOON-MIDN", "(DEFUN DEAL-WITH-NOON-MIDN ((FORM-VALUE OBJECT) (PARSED-VALUES DECODED-DATE-TIME)))", ((cpp_function_code)(&deal_with_noon_midn)), NULL);
      define_function_object("DEAL-WITH-AM-PM", "(DEFUN DEAL-WITH-AM-PM ((FORM-VALUE OBJECT) (PARSED-VALUES DECODED-DATE-TIME)))", ((cpp_function_code)(&deal_with_am_pm)), NULL);
      define_function_object("SET-TIME-VALUES", "(DEFUN SET-TIME-VALUES ((STRING-FORM (CONS OF CONS)) (PARSED-VALUES DECODED-DATE-TIME)))", ((cpp_function_code)(&set_time_values)), NULL);
      define_function_object("PARSE-DATE-TIME", "(DEFUN (PARSE-DATE-TIME DECODED-DATE-TIME) ((DATE-TIME-STRING STRING) (START INTEGER) (END INTEGER) (ERROR-ON-MISMATCH? BOOLEAN)) :PUBLIC? TRUE :DOCUMENTATION \"Tries very hard to make sense out of the argument `date-time-string' and\nreturns a time structure if successful.  If not, it returns `null'.\nIf `error-on-mismatch?' is true, parse-date-time will signal an error instead of\nreturning `null'.  \nDefault values are 00:00:00 UTC on the current date\")", ((cpp_function_code)(&parse_date_time)), NULL);
      define_function_object("STRING-TO-CALENDAR-DATE", "(DEFUN (STRING-TO-CALENDAR-DATE CALENDAR-DATE) ((INPUT STRING)) :PUBLIC? TRUE :DOCUMENTATION \"Returns a calendar date object representing the date and time\nparsed from the `input' string.  If no valid parse is found,\n`null' is returned.\")", ((cpp_function_code)(&string_to_calendar_date)), NULL);
      define_function_object("STARTUP-DATE-TIME-PARSER", "(DEFUN STARTUP-DATE-TIME-PARSER () :PUBLIC? TRUE)", ((cpp_function_code)(&startup_date_time_parser)), NULL);
      { Method_Slot* function = lookup_function(SYM_DATE_TIME_PARSER_STELLA_STARTUP_DATE_TIME_PARSER);

        set_dynamic_slot_value(function->dynamic_slots, SYM_DATE_TIME_PARSER_STELLA_METHOD_STARTUP_CLASSNAME, wrap_string("Startup-Date-Time-Parser"), NULL_STRING_WRAPPER);
      }
    }
    if (current_startup_time_phaseP(8)) {
      finalize_slots();
      cleanup_unfinalized_classes();
    }
    if (current_startup_time_phaseP(9)) {
      define_stella_global_variable_from_stringified_source("(DEFCONSTANT WHITESPACE-CHARS CONS (QUOTE (#\\  #\\Tab #\\Linefeed #\\, #\\' #\\`)))");
      define_stella_global_variable_from_stringified_source("(DEFCONSTANT TIME-DIVIDERS CONS (QUOTE (#\\: #\\.)))");
      define_stella_global_variable_from_stringified_source("(DEFCONSTANT DATE-DIVIDERS CONS (QUOTE (#\\\\ #\\/ #\\-)))");
      define_stella_global_variable_from_stringified_source("(DEFSPECIAL *ERROR-ON-MISMATCH* BOOLEAN FALSE :PUBLIC? TRUE :DOCUMENTATION \"If TRUE, an error will be signalled if parse-date-time is unable\n   to determine the time/date format of the string.\")");
      store_macro(SYM_DATE_TIME_PARSER_STELLA_HASHLIST, SYM_DATE_TIME_PARSER_STELLA_HASHLIST_EXPANDER, ((cpp_function_code)(&hashlist_expander)));
      define_stella_global_variable_from_stringified_source("(DEFSPECIAL *WEEKDAY-STRINGS* (STRING-HASH-TABLE OF STRING KEYWORD) (NEW (STRING-HASH-TABLE OF STRING KEYWORD)))");
      define_stella_global_variable_from_stringified_source("(DEFSPECIAL *MONTH-STRINGS* STRING-TO-INTEGER-HASH-TABLE (NEW STRING-TO-INTEGER-HASH-TABLE))");
      define_stella_global_variable_from_stringified_source("(DEFSPECIAL *ZONE-STRINGS* (STRING-HASH-TABLE OF STRING FLOAT-WRAPPER) (NEW (STRING-HASH-TABLE OF STRING FLOAT-WRAPPER)))");
      define_stella_global_variable_from_stringified_source("(DEFSPECIAL *SPECIAL-STRINGS* STRING-HASH-TABLE (NEW STRING-HASH-TABLE))");
      define_stella_global_variable_from_stringified_source("(DEFSPECIAL *SPECIAL-SYMBOLS* (CONS OF SYMBOL) (QUOTE (YESTERDAY TODAY TOMORROW NOW)))");
      define_stella_global_variable_from_stringified_source("(DEFSPECIAL *DATE-TIME-PATTERNS* CONS (QUOTE ((YEAR (DATE-DIVIDER) MONTH (DATE-DIVIDER) DAY (NOON-MIDN)) ((WEEKDAY) MONTH (DATE-DIVIDER) DAY (DATE-DIVIDER) YEAR (NOON-MIDN) (DATE-DIVIDER) (ZONE)) ((WEEKDAY) DAY (DATE-DIVIDER) MONTH (DATE-DIVIDER) YEAR (NOON-MIDN) (DATE-DIVIDER) (ZONE)) ((WEEKDAY) MONTH (DATE-DIVIDER) DAY (NOON-MIDN) (DATE-DIVIDER) (ZONE)) (MONTH (DATE-DIVIDER) YEAR (NOON-MIDN) (DATE-DIVIDER) (ZONE)) (YEAR (DATE-DIVIDER) MONTH (NOON-MIDN) (DATE-DIVIDER) (ZONE)) ((NOON-MIDN) (WEEKDAY) MONTH (DATE-DIVIDER) DAY (DATE-DIVIDER) YEAR) ((NOON-MIDN) (WEEKDAY) DAY (DATE-DIVIDER) MONTH (DATE-DIVIDER) YEAR) ((NOON-MIDN) (WEEKDAY) MONTH (DATE-DIVIDER) DAY) ((NOON-MIDN) YEAR (DATE-DIVIDER) MONTH (DATE-DIVIDER) DAY) ((NOON-MIDN) MONTH (DATE-DIVIDER) YEAR) ((NOON-MIDN) YEAR (DATE-DIVIDER) MONTH) (HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (NOON-MIDN (DATE-DIVIDER) (ZONE)) (HOUR (NOON-MIDN)) (YEAR (DATE-DIVIDER) MONTH (DATE-DIVIDER) DAY HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (YEAR (DATE-DIVIDER) MONTH (DATE-DIVIDER) DAY NOON-MIDN (DATE-DIVIDER) (ZONE)) (YEAR (DATE-DIVIDER) MONTH (DATE-DIVIDER) DAY NOW) ((WEEKDAY) MONTH (DATE-DIVIDER) DAY (DATE-DIVIDER) YEAR HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) ((WEEKDAY) DAY (DATE-DIVIDER) MONTH (DATE-DIVIDER) YEAR HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) ((WEEKDAY) MONTH (DATE-DIVIDER) DAY HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (MONTH (DATE-DIVIDER) YEAR HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (YEAR (DATE-DIVIDER) MONTH HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE) (WEEKDAY) MONTH (DATE-DIVIDER) DAY (DATE-DIVIDER) YEAR) (HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE) (WEEKDAY) DAY (DATE-DIVIDER) MONTH (DATE-DIVIDER) YEAR) (HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE) (WEEKDAY) MONTH (DATE-DIVIDER) DAY) (HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE) YEAR (DATE-DIVIDER) MONTH (DATE-DIVIDER) DAY) (HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE) MONTH (DATE-DIVIDER) YEAR) (HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE) YEAR (DATE-DIVIDER) MONTH) (WEEKDAY MONTH DAY HOUR (TIME-DIVIDER) MINUTE (TIME-DIVIDER) SECONDP (AM-PM) (ZONE) YEAR) ((WEEKDAY) DAY (DATE-DIVIDER) MONTH (DATE-DIVIDER) YEAR HOUR (TIME-DIVIDER) MINUTE (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) ((WEEKDAY) MONTH (DATE-DIVIDER) DAY (DATE-DIVIDER) YEAR HOUR (TIME-DIVIDER) MINUTE (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (YEAR (DATE-DIVIDER) (MONTH)) (MONTH (DATE-DIVIDER) (YEAR)) ((WEEKDAY) MONTH DAY HOUR TIME-DIVIDER MINUTE (TIME-DIVIDER) (SECONDP) YEAR) (NOW (YESTERDAY)) ((YESTERDAY) NOW) (NOW (TODAY)) ((TODAY) NOW) (NOW (TOMORROW)) ((TOMORROW) NOW) (YESTERDAY (NOON-MIDN)) ((NOON-MIDN) YESTERDAY) (TODAY (NOON-MIDN)) ((NOON-MIDN) TODAY) (TOMORROW (NOON-MIDN)) ((NOON-MIDN) TOMORROW) (YESTERDAY HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (TOMORROW HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (TODAY HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)))))");
    }
  }
}

Symbol* SYM_DATE_TIME_PARSER_STELLA_FOREACH = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_ITEM = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_IN = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_CAST = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_CONS = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_OF = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_DO = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_INSERT_AT = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_FIRST = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_SECOND = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_HASHLIST = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_HASHLIST_EXPANDER = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_AM = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_PM = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_NOON = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_MIDN = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_MONTH = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_WEEKDAY = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_AM_PM = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_NOON_MIDN = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_ZONE = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_SPECIAL = NULL;

Keyword* KWD_DATE_TIME_PARSER_LETTER = NULL;

Keyword* KWD_DATE_TIME_PARSER_DIGIT = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_TIME_DIVIDER = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_DATE_DIVIDER = NULL;

Surrogate* SGT_DATE_TIME_PARSER_STELLA_CONS = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_SECONDP = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_MINUTE = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_HOUR = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_DAY = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_YEAR = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_STARTUP_DATE_TIME_PARSER = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella
