// methods.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

Object* apply(cpp_function_code code, Cons* arguments) {
  // Apply `code' to `arguments', returning a value of type
  // OBJECT.
  switch (arguments->length()) {
    case 0: 
      return (((Object*  (*) ())code)());
    case 1: 
      return (((Object*  (*) (Object*))code)(arguments->value));
    case 2: 
      return (((Object*  (*) (Object*, Object*))code)(arguments->value, arguments->rest->value));
    case 3: 
      return (((Object*  (*) (Object*, Object*, Object*))code)(arguments->value, arguments->rest->value, arguments->nth(2)));
    case 4: 
      return (((Object*  (*) (Object*, Object*, Object*, Object*))code)(arguments->value, arguments->rest->value, arguments->nth(2), arguments->nth(3)));
    case 5: 
      return (((Object*  (*) (Object*, Object*, Object*, Object*, Object*))code)(arguments->value, arguments->rest->value, arguments->nth(2), arguments->nth(3), arguments->nth(4)));
    default:
      throw *new_stella_exception("Too many function arguments in `apply'.  Max is 5.");
    break;
  }
}

Object* apply_method(cpp_method_code code, Cons* arguments) {
  // Apply `code' to `arguments', returning a value of type
  // OBJECT.
  switch (arguments->length()) {
    case 0: 
      throw *new_stella_exception("Can't call method code on 0 arguments.");
    break;
    case 1: 
      return ((arguments->value->*((Object*  (Object::*) ()) code)) ());
    case 2: 
      return ((arguments->value->*((Object*  (Object::*) (Object*)) code)) (arguments->rest->value));
    case 3: 
      return ((arguments->value->*((Object*  (Object::*) (Object*, Object*)) code)) (arguments->rest->value, arguments->nth(2)));
    case 4: 
      return ((arguments->value->*((Object*  (Object::*) (Object*, Object*, Object*)) code)) (arguments->rest->value, arguments->nth(2), arguments->nth(3)));
    case 5: 
      return ((arguments->value->*((Object*  (Object::*) (Object*, Object*, Object*, Object*)) code)) (arguments->rest->value, arguments->nth(2), arguments->nth(3), arguments->nth(4)));
    default:
      throw *new_stella_exception("Too many function arguments in `apply'.  Max is 5.");
    break;
  }
}

int apply_integer_method(cpp_method_code code, Cons* arguments) {
  // Apply `code' to `arguments', returning a value of type
  // INTEGER.
  switch (arguments->length()) {
    case 0: 
      throw *new_stella_exception("Can't call method code on 0 arguments.");
    break;
    case 1: 
      return ((arguments->value->*((int  (Object::*) ()) code)) ());
    case 2: 
      return ((arguments->value->*((int  (Object::*) (Object*)) code)) (arguments->rest->value));
    case 3: 
      return ((arguments->value->*((int  (Object::*) (Object*, Object*)) code)) (arguments->rest->value, arguments->nth(2)));
    case 4: 
      return ((arguments->value->*((int  (Object::*) (Object*, Object*, Object*)) code)) (arguments->rest->value, arguments->nth(2), arguments->nth(3)));
    case 5: 
      return ((arguments->value->*((int  (Object::*) (Object*, Object*, Object*, Object*)) code)) (arguments->rest->value, arguments->nth(2), arguments->nth(3), arguments->nth(4)));
    default:
      throw *new_stella_exception("Too many function arguments in `apply'.  Max is 5.");
    break;
  }
}

char* apply_string_method(cpp_method_code code, Cons* arguments) {
  // Apply `code' to `arguments', returning a value of type
  // STRING.
  switch (arguments->length()) {
    case 0: 
      throw *new_stella_exception("Can't call method code on 0 arguments.");
    break;
    case 1: 
      return ((arguments->value->*((char*  (Object::*) ()) code)) ());
    case 2: 
      return ((arguments->value->*((char*  (Object::*) (Object*)) code)) (arguments->rest->value));
    case 3: 
      return ((arguments->value->*((char*  (Object::*) (Object*, Object*)) code)) (arguments->rest->value, arguments->nth(2)));
    case 4: 
      return ((arguments->value->*((char*  (Object::*) (Object*, Object*, Object*)) code)) (arguments->rest->value, arguments->nth(2), arguments->nth(3)));
    case 5: 
      return ((arguments->value->*((char*  (Object::*) (Object*, Object*, Object*, Object*)) code)) (arguments->rest->value, arguments->nth(2), arguments->nth(3), arguments->nth(4)));
    default:
      throw *new_stella_exception("Too many function arguments in `apply'.  Max is 5.");
    break;
  }
}

boolean apply_boolean_method(cpp_method_code code, Cons* arguments) {
  // Apply `code' to `arguments', returning a value of type
  // BOOLEAN.
  switch (arguments->length()) {
    case 0: 
      throw *new_stella_exception("Can't call method code on 0 arguments.");
    break;
    case 1: 
      return ((arguments->value->*((boolean  (Object::*) ()) code)) ());
    case 2: 
      return ((arguments->value->*((boolean  (Object::*) (Object*)) code)) (arguments->rest->value));
    case 3: 
      return ((arguments->value->*((boolean  (Object::*) (Object*, Object*)) code)) (arguments->rest->value, arguments->nth(2)));
    case 4: 
      return ((arguments->value->*((boolean  (Object::*) (Object*, Object*, Object*)) code)) (arguments->rest->value, arguments->nth(2), arguments->nth(3)));
    case 5: 
      return ((arguments->value->*((boolean  (Object::*) (Object*, Object*, Object*, Object*)) code)) (arguments->rest->value, arguments->nth(2), arguments->nth(3), arguments->nth(4)));
    default:
      throw *new_stella_exception("Too many function arguments in `apply'.  Max is 5.");
    break;
  }
}

double apply_float_method(cpp_method_code code, Cons* arguments) {
  // Apply `code' to `arguments', returning a value of type
  // FLOAT.
  switch (arguments->length()) {
    case 0: 
      throw *new_stella_exception("Can't call method code on 0 arguments.");
    break;
    case 1: 
      return ((arguments->value->*((double  (Object::*) ()) code)) ());
    case 2: 
      return ((arguments->value->*((double  (Object::*) (Object*)) code)) (arguments->rest->value));
    case 3: 
      return ((arguments->value->*((double  (Object::*) (Object*, Object*)) code)) (arguments->rest->value, arguments->nth(2)));
    case 4: 
      return ((arguments->value->*((double  (Object::*) (Object*, Object*, Object*)) code)) (arguments->rest->value, arguments->nth(2), arguments->nth(3)));
    case 5: 
      return ((arguments->value->*((double  (Object::*) (Object*, Object*, Object*, Object*)) code)) (arguments->rest->value, arguments->nth(2), arguments->nth(3), arguments->nth(4)));
    default:
      throw *new_stella_exception("Too many function arguments in `apply'.  Max is 5.");
    break;
  }
}

boolean defined_typeP(Surrogate* type) {
  { Object* value = type->surrogate_value;
    Surrogate* valuetype = NULL;

    if (((boolean)(value))) {
      valuetype = value->primary_type();
      return ((SGT_METHODS_STELLA_SURROGATE == valuetype) ||
          ((SGT_METHODS_STELLA_PARAMETRIC_TYPE_SPECIFIER == valuetype) ||
           (SGT_METHODS_STELLA_ANCHORED_TYPE_SPECIFIER == valuetype)));
    }
    return (FALSE);
  }
}

Standard_Object* real_type_specifier(Surrogate* type) {
  { Object* value = type->surrogate_value;

    if (defined_typeP(type)) {
      if (typeP(value)) {
        return (real_type_specifier(((Surrogate*)(value))));
      }
      else {
        return (((Standard_Object*)(value)));
      }
    }
    else {
      return (type);
    }
  }
}

Surrogate* canonical_type(Surrogate* type) {
  { Class* renamed_Class = ((Class*)(type->surrogate_value));

    if (((boolean)(renamed_Class))) {
      return (renamed_Class->class_type);
    }
    else {
      return (type);
    }
  }
}

Standard_Object* Surrogate::yield_type_specifier() {
  { Surrogate* typespectree = this;

    return (real_type_specifier(typespectree));
  }
}

Standard_Object* Symbol::yield_type_specifier() {
  { Symbol* typespectree = this;

    return (real_type_specifier(typify(typespectree)));
  }
}

Standard_Object* Parametric_Type_Specifier::yield_type_specifier() {
  { Parametric_Type_Specifier* typespectree = this;

    return (typespectree);
  }
}

Standard_Object* Anchored_Type_Specifier::yield_type_specifier() {
  { Anchored_Type_Specifier* typespectree = this;

    return (typespectree);
  }
}

Standard_Object* Object::yield_type_specifier() {
  { Object* typespectree = this;

    cerr << "Illegal type specification " << "`" << typespectree << "'";
    return (SGT_METHODS_STELLA_OBJECT);
  }
}

Standard_Object* Cons::yield_type_specifier() {
  { Cons* typespectree = this;

    if (typespectree == NIL) {
      return (SGT_METHODS_STELLA_VOID);
    }
    else if (typespectree->value == SYM_METHODS_STELLA_LIKE) {
      return (yield_anchored_type_specifier(typespectree));
    }
    else {
      return (yield_parametric_type_specifier(typespectree));
    }
  }
}

Parametric_Type_Specifier* yield_parametric_type_specifier(Cons* typespectree) {
  { Parametric_Type_Specifier* typespec = new_parametric_type_specifier();
    Standard_Object* parametertypespec = NULL;
    Cons* cursor = typespectree;

    { Object* head_000 = cursor->value;

      cursor = cursor->rest;
      { Object* basetype = head_000;

        { Object* head_001 = cursor->value;

          cursor = cursor->rest;
          { Object* renamed_Operator = head_001;
            boolean arraytypeP = FALSE;

            typespec->specifier_base_type = verify_type(typify(basetype));
            if (array_type_specifierP(typespec)) {
              if (renamed_Operator == SYM_METHODS_STELLA_SIZE) {
                { Object* head_002 = cursor->value;

                  cursor = cursor->rest;
                  renamed_Operator = head_002;
                }
              }
              arraytypeP = TRUE;
              typespec->specifier_dimensions = parse_array_dimensions_spec(renamed_Operator);
              { Object* head_003 = cursor->value;

                cursor = cursor->rest;
                renamed_Operator = head_003;
              }
            }
            if (arraytypeP &&
                (!((boolean)(typespec->specifier_dimensions)))) {
              { 
                BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
                signal_translation_error();
                if (!(suppress_warningsP())) {
                  print_error_context(">> ERROR: ", STANDARD_ERROR);
                  *(STANDARD_ERROR->native_stream) << endl << " " << "Illegal array type specifier dimensions: " << "`" << de_uglify_parse_tree(typespectree) << "'" << "." << endl;
                }
              }
              return (yield_parametric_type_specifier(listO(3, SYM_METHODS_STELLA_UNKNOWN, SYM_METHODS_STELLA_OF, NIL)));
            }
            if (!(renamed_Operator == SYM_METHODS_STELLA_OF)) {
              { 
                BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
                signal_translation_error();
                if (!(suppress_warningsP())) {
                  print_error_context(">> ERROR: ", STANDARD_ERROR);
                  *(STANDARD_ERROR->native_stream) << endl << " " << "Bad type specifier " << "`" << de_uglify_parse_tree(typespectree) << "'" << "." << endl;
                }
              }
              return (yield_parametric_type_specifier(listO(3, SYM_METHODS_STELLA_UNKNOWN, SYM_METHODS_STELLA_OF, NIL)));
            }
            { Object* p = NULL;
              Cons* iter_000 = cursor;
              List* into_000 = typespec->specifier_parameter_types;
              Cons* collect_000 = NULL;

              while (!(iter_000 == NIL)) {
                p = iter_000->value;
                iter_000 = iter_000->rest;
                parametertypespec = p->yield_type_specifier();
                if (array_type_specifierP(parametertypespec)) {
                  { 
                    BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
                    signal_translation_error();
                    if (!(suppress_warningsP())) {
                      print_error_context(">> ERROR: ", STANDARD_ERROR);
                      *(STANDARD_ERROR->native_stream) << endl << " " << "Illegal nested array type specifier: " << endl << "   " << "`" << de_uglify_parse_tree(typespectree) << "'" << endl << "   Use multi-dimensional arrays instead" << "." << endl;
                    }
                  }
                  return (yield_parametric_type_specifier(listO(3, SYM_METHODS_STELLA_UNKNOWN, SYM_METHODS_STELLA_OF, NIL)));
                }
                if (!((boolean)(collect_000))) {
                  {
                    collect_000 = cons(parametertypespec, NIL);
                    if (into_000->the_cons_list == NIL) {
                      into_000->the_cons_list = collect_000;
                    }
                    else {
                      add_cons_to_end_of_cons_list(into_000->the_cons_list, collect_000);
                    }
                  }
                }
                else {
                  {
                    collect_000->rest = cons(parametertypespec, NIL);
                    collect_000 = collect_000->rest;
                  }
                }
              }
            }
            return (typespec);
          }
        }
      }
    }
  }
}

List* parse_array_dimensions_spec(Object* dimensionsspec) {
  { List* dimensions = new_list();

    if (!((boolean)(dimensionsspec))) {
      return (NULL);
    }
    if (!(consP(dimensionsspec) &&
        (!(dimensionsspec == NIL)))) {
      dimensionsspec = cons(dimensionsspec, NIL);
    }
    { Object* dimspec = NULL;
      Cons* iter_000 = ((Cons*)(dimensionsspec));
      Cons* collect_000 = NULL;

      while (!(iter_000 == NIL)) {
        dimspec = iter_000->value;
        iter_000 = iter_000->rest;
        if (consP(dimspec)) {
          dimspec = ((Cons*)(dimspec))->value;
        }
        if ((dimspec == NULL) ||
            (dimspec == SYM_METHODS_STELLA_NULL)) {
          dimspec = NULL;
        }
        else if (integerP(dimspec)) {
        }
        else {
          return (NULL);
        }
        if (!((boolean)(collect_000))) {
          {
            collect_000 = cons(dimspec, NIL);
            if (dimensions->the_cons_list == NIL) {
              dimensions->the_cons_list = collect_000;
            }
            else {
              add_cons_to_end_of_cons_list(dimensions->the_cons_list, collect_000);
            }
          }
        }
        else {
          {
            collect_000->rest = cons(dimspec, NIL);
            collect_000 = collect_000->rest;
          }
        }
      }
    }
    return (dimensions);
  }
}

Cons* yield_array_dimensions_tree(Parametric_Type_Specifier* arraytype) {
  { List* dimensions = arraytype->specifier_dimensions;
    Cons* result = NIL;

    if (!((boolean)(dimensions))) {
      return (NULL);
    }
    { Object* dim = NULL;
      Cons* iter_000 = dimensions->the_cons_list;
      Cons* collect_000 = NULL;

      while (!(iter_000 == NIL)) {
        dim = iter_000->value;
        iter_000 = iter_000->rest;
        if (!((boolean)(collect_000))) {
          {
            collect_000 = cons(((!((boolean)(dim))) ? NIL : cons(dim, NIL)), NIL);
            if (result == NIL) {
              result = collect_000;
            }
            else {
              add_cons_to_end_of_cons_list(result, collect_000);
            }
          }
        }
        else {
          {
            collect_000->rest = cons(((!((boolean)(dim))) ? NIL : cons(dim, NIL)), NIL);
            collect_000 = collect_000->rest;
          }
        }
      }
    }
    if (result->length() == 1) {
      return (((Cons*)(result->value)));
    }
    else {
      return (result);
    }
  }
}

boolean array_type_specifierP(Standard_Object* typespec) {
  if (subtype_of_parametric_type_specifierP(safe_primary_type(typespec))) {
    { Standard_Object* typespec_000 = typespec;
      Parametric_Type_Specifier* typespec = ((Parametric_Type_Specifier*)(typespec_000));

      { Surrogate* basetype = typespec->specifier_base_type;

        if (basetype == SGT_METHODS_STELLA_ARRAY) {
          return (TRUE);
        }
      }
    }
  }
  else {
  }
  return (FALSE);
}

int array_type_rank(Parametric_Type_Specifier* arraytype) {
  { List* dimensions = arraytype->specifier_dimensions;

    if (!((boolean)(dimensions))) {
      return (NULL_INTEGER);
    }
    else {
      return (dimensions->length());
    }
  }
}

List* array_type_dimensions(Parametric_Type_Specifier* arraytype) {
  return (((List*)(arraytype->specifier_dimensions)));
}

boolean indefinite_array_typeP(Parametric_Type_Specifier* arraytype) {
  { List* dimensions = arraytype->specifier_dimensions;

    if (!((boolean)(dimensions))) {
      return (TRUE);
    }
    else {
      { Object* dim = NULL;
        Cons* iter_000 = dimensions->the_cons_list;

        while (!(iter_000 == NIL)) {
          dim = iter_000->value;
          iter_000 = iter_000->rest;
          if (!((boolean)(dim))) {
            return (TRUE);
          }
        }
      }
    }
    return (FALSE);
  }
}

Anchored_Type_Specifier* yield_anchored_type_specifier(Cons* typespectree) {
  { Anchored_Type_Specifier* typespec = new_anchored_type_specifier();
    Object* likeargument = typespectree->rest->value;

    if (!(likeargument == SYM_METHODS_STELLA_SELF)) {
      typespec->specifier_parameter_name = ((Symbol*)(((Cons*)(likeargument))->value->permanentify()));
    }
    return (typespec);
  }
}

void incorporate_input_parameters(Method_Slot* method, Cons* parameters) {
  if (parameters == NIL) {
    return;
  }
  { Symbol* name = NULL;
    Standard_Object* ts = NULL;
    Symbol* direction = NULL;
    boolean variableargsP = FALSE;

    method->method_parameter_names = new_list();
    method->method_parameter_type_specifiers = new_list();
    { Object* p = NULL;
      Cons* iter_000 = parameters;

      while (!(iter_000 == NIL)) {
        p = iter_000->value;
        iter_000 = iter_000->rest;
        { Surrogate* test_value_000 = safe_primary_type(p);

          if (test_value_000 == SGT_METHODS_STELLA_CONS) {
            { Object* p_000 = p;
              Cons* p = ((Cons*)(p_000));

              switch (p->length()) {
                case 2: 
                  direction = SYM_METHODS_STELLA_IN;
                break;
                case 3: 
                  if ((p->value == SYM_METHODS_STELLA_IN) ||
                      ((p->value == SYM_METHODS_STELLA_OUT) ||
                       (p->value == SYM_METHODS_STELLA_INOUT))) {
                    direction = ((Symbol*)(p->value));
                    if (null_listP(method->method_parameter_directions_reader())) {
                      set_dynamic_slot_value(method->dynamic_slots, SYM_METHODS_STELLA_METHOD_PARAMETER_DIRECTIONS, new_list(), NULL);
                      { Object* par = NULL;
                        Cons* iter_001 = parameters;

                        while (!(iter_001 == NIL)) {
                          par = iter_001->value;
                          iter_001 = iter_001->rest;
                          if (par == p) {
                            break;
                          }
                          else {
                            method->method_parameter_directions_reader()->insert(SYM_METHODS_STELLA_IN);
                          }
                        }
                      }
                    }
                  }
                  else {
                    { 
                      BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
                      signal_translation_error();
                      if (!(suppress_warningsP())) {
                        print_error_context(">> ERROR: ", STANDARD_ERROR);
                        *(STANDARD_ERROR->native_stream) << endl << " " << "Illegal direction for method parameter: " << "`" << de_uglify_parse_tree(p) << "'" << "." << endl;
                      }
                    }
                    direction = SYM_METHODS_STELLA_IN;
                  }
                  p = p->rest;
                break;
                default:
                  { 
                    BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
                    signal_translation_error();
                    if (!(suppress_warningsP())) {
                      print_error_context(">> ERROR: ", STANDARD_ERROR);
                      *(STANDARD_ERROR->native_stream) << endl << " " << "Illegal method parameter: " << "`" << de_uglify_parse_tree(p) << "'" << "." << endl;
                    }
                  }
                  continue;
                break;
              }
              if (variableargsP) {
                { Cons* varargtype = listO(4, SYM_METHODS_STELLA_ARGUMENT_LIST, SYM_METHODS_STELLA_OF, p->rest->value, NIL);

                  name = ((Symbol*)(p->value));
                  ts = varargtype->yield_type_specifier();
                }
              }
              else {
                {
                  if (!(symbolP(p->value) &&
                      (symbolP(p->rest->value) ||
                       (surrogateP(p->rest->value) ||
                        consP(p->rest->value))))) {
                    { 
                      BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
                      signal_translation_error();
                      if (!(suppress_warningsP())) {
                        print_error_context(">> ERROR: ", STANDARD_ERROR);
                        *(STANDARD_ERROR->native_stream) << endl << " " << "Illegal method parameter: " << "`" << de_uglify_parse_tree(p) << "'" << "." << endl;
                      }
                    }
                    continue;
                  }
                  name = ((Symbol*)(p->value));
                  ts = p->rest->value->yield_type_specifier();
                }
              }
            }
          }
          else if (subtype_of_symbolP(test_value_000)) {
            { Object* p_001 = p;
              Symbol* p = ((Symbol*)(p_001));

              if (p == SYM_METHODS_STELLA_aREST) {
                set_dynamic_slot_value(method->dynamic_slots, SYM_METHODS_STELLA_METHOD_VARIABLE_ARGUMENTSp, (TRUE ? TRUE_WRAPPER : FALSE_WRAPPER), FALSE_WRAPPER);
                variableargsP = TRUE;
                continue;
              }
              else if (p == SYM_METHODS_STELLA_aBODY) {
                set_dynamic_slot_value(method->dynamic_slots, SYM_METHODS_STELLA_METHOD_BODY_ARGUMENTp, (TRUE ? TRUE_WRAPPER : FALSE_WRAPPER), FALSE_WRAPPER);
                continue;
              }
              else {
                { 
                  BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
                  signal_translation_error();
                  if (!(suppress_warningsP())) {
                    print_error_context(">> ERROR: ", STANDARD_ERROR);
                    *(STANDARD_ERROR->native_stream) << endl << " " << "Untyped parameter in parameter list: " << "`" << de_uglify_parse_tree(parameters) << "'" << "." << endl;
                  }
                }
                name = p;
                ts = SGT_METHODS_STELLA_UNKNOWN;
              }
            }
          }
          else {
            { 
              BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
              signal_translation_error();
              if (!(suppress_warningsP())) {
                print_error_context(">> ERROR: ", STANDARD_ERROR);
                *(STANDARD_ERROR->native_stream) << endl << " " << "Illegal parameter list: " << "`" << de_uglify_parse_tree(parameters) << "'" << "." << endl;
              }
            }
            continue;
          }
        }
        method->method_parameter_type_specifiers_reader()->insert(ts);
        method->method_parameter_names_reader()->insert(((Symbol*)(name->permanentify())));
        if (defined_listP(method->method_parameter_directions_reader())) {
          method->method_parameter_directions_reader()->insert(direction);
        }
      }
    }
    method->method_parameter_names_reader()->reverse();
    method->method_parameter_type_specifiers_reader()->reverse();
    method->method_parameter_directions_reader()->reverse();
  }
}

void incorporate_first_input_parameter(Method_Slot* method) {
  { List* parametertypespecs = method->method_parameter_type_specifiers_reader();
    Standard_Object* firstargtype = ((Standard_Object*)(parametertypespecs->first()));

    if (!method->method_functionP) {
      if (parametertypespecs->emptyP()) {
        { 
          BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
          signal_translation_error();
          if (!(suppress_warningsP())) {
            print_error_context(">> ERROR: ", STANDARD_ERROR);
            *(STANDARD_ERROR->native_stream) << endl << " " << "Method has no input parameters, converting it into a function" << "." << endl;
          }
        }
        method->method_functionP = TRUE;
        return;
      }
      else if (((boolean)(firstargtype)) &&
          (!typeP(firstargtype))) {
        { 
          BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
          signal_translation_error();
          if (!(suppress_warningsP())) {
            print_error_context(">> ERROR: ", STANDARD_ERROR);
            *(STANDARD_ERROR->native_stream) << endl << " " << "Illegal complex argument type for the first method parameter: " << endl << "   " << "`" << de_uglify_parse_tree(firstargtype) << "'" << "." << endl;
          }
        }
        if (anchored_type_specifierP(firstargtype)) {
          method->method_parameter_type_specifiers_reader()->first_setter(SGT_METHODS_STELLA_UNKNOWN);
          firstargtype = SGT_METHODS_STELLA_UNKNOWN;
        }
      }
    }
    method->slot_owner = (((boolean)(firstargtype)) ? type_spec_to_base_type(firstargtype) : NULL);
  }
}

Method_Slot* define_stella_method_slot(Symbol* inputname, Cons* returntypes, boolean functionP, Cons* inputparameters, Keyword_Key_Value_List* options) {
  // Define a new Stella method object (a slot), and attach it
  // to the class identified by the first parameter in 'inputParameters'.
  { Method_Slot* newmethod = new_method_slot();
    Symbol* name = ((Symbol*)(inputname->permanentify()));

    newmethod->slot_name = name;
    newmethod->method_functionP = functionP;
    incorporate_input_parameters(newmethod, inputparameters);
    incorporate_first_input_parameter(newmethod);
    if (returntypes == NIL) {
      if (((boolean)(options->lookup(KWD_METHODS_TYPE)))) {
        returntypes = local_cons(options->lookup(KWD_METHODS_TYPE), NIL);
      }
      else if (((boolean)(options->lookup(KWD_METHODS_RETURN_TYPES)))) {
        returntypes = local_cons(options->lookup(KWD_METHODS_RETURN_TYPES), NIL);
      }
    }
    { Standard_Object* typespec = ((returntypes == NIL) ? SGT_METHODS_STELLA_VOID : ((Standard_Object*)(returntypes->value->yield_type_specifier())));

      if (typeP(typespec)) {
        newmethod->slot_base_type = ((Surrogate*)(typespec));
      }
      else {
        set_dynamic_slot_value(newmethod->dynamic_slots, SYM_METHODS_STELLA_SLOT_TYPE_SPECIFIER, ((Compound_Type_Specifier*)(typespec)), NULL);
      }
    }
    if (!(returntypes == NIL)) {
      newmethod->method_return_type_specifiers = new_list();
    }
    { Object* ts = NULL;
      Cons* iter_000 = returntypes;
      List* into_000 = newmethod->method_return_type_specifiers_reader();
      Cons* collect_000 = NULL;

      while (!(iter_000 == NIL)) {
        ts = iter_000->value;
        iter_000 = iter_000->rest;
        if (!((boolean)(collect_000))) {
          {
            collect_000 = cons(ts->yield_type_specifier(), NIL);
            if (into_000->the_cons_list == NIL) {
              into_000->the_cons_list = collect_000;
            }
            else {
              add_cons_to_end_of_cons_list(into_000->the_cons_list, collect_000);
            }
          }
        }
        else {
          {
            collect_000->rest = cons(ts->yield_type_specifier(), NIL);
            collect_000 = collect_000->rest;
          }
        }
      }
    }
    { Keyword* key = NULL;
      Object* value = NULL;
      Kv_List_Iterator* iter_001 = ((Kv_List_Iterator*)(options->allocate_iterator()));

      while (iter_001->nextP()) {
        key = ((Keyword*)(iter_001->key));
        value = iter_001->value;
        { Keyword* test_value_000 = ((Keyword*)(key));

          if (test_value_000 == KWD_METHODS_PUBLICp) {
            newmethod->slot_publicP = ((Boolean_Wrapper*)(value))->wrapper_value;
          }
          else if (test_value_000 == KWD_METHODS_ABSTRACTp) {
            newmethod->abstractP = ((Boolean_Wrapper*)(value))->wrapper_value;
          }
          else if (test_value_000 == KWD_METHODS_NATIVEp) {
            set_dynamic_slot_value(newmethod->dynamic_slots, SYM_METHODS_STELLA_METHOD_NATIVEp, (((Boolean_Wrapper*)(value))->wrapper_value ? TRUE_WRAPPER : FALSE_WRAPPER), FALSE_WRAPPER);
          }
          else if (test_value_000 == KWD_METHODS_CONSTRUCTORp) {
            set_dynamic_slot_value(newmethod->dynamic_slots, SYM_METHODS_STELLA_METHOD_CONSTRUCTORp, (((Boolean_Wrapper*)(value))->wrapper_value ? TRUE_WRAPPER : FALSE_WRAPPER), FALSE_WRAPPER);
          }
          else if (test_value_000 == KWD_METHODS_GLOBALLY_INLINEp) {
            set_dynamic_slot_value(newmethod->dynamic_slots, SYM_METHODS_STELLA_METHOD_GLOBALLY_INLINEp, (((Boolean_Wrapper*)(value))->wrapper_value ? TRUE_WRAPPER : FALSE_WRAPPER), FALSE_WRAPPER);
          }
          else if (test_value_000 == KWD_METHODS_INLINE) {
            if (newmethod->method_inlined_functions_reader()->emptyP()) {
              set_dynamic_slot_value(newmethod->dynamic_slots, SYM_METHODS_STELLA_METHOD_INLINED_FUNCTIONS, new_list(), NULL);
            }
            { Surrogate* test_value_001 = safe_primary_type(value);

              if (test_value_001 == SGT_METHODS_STELLA_CONS) {
                { Object* value_000 = value;
                  Cons* value = ((Cons*)(value_000));

                  { Object* fnname = NULL;
                    Cons* iter_002 = value;

                    while (!(iter_002 == NIL)) {
                      fnname = iter_002->value;
                      iter_002 = iter_002->rest;
                      newmethod->method_inlined_functions_reader()->insert(((Symbol*)(fnname)));
                    }
                  }
                }
              }
              else if (subtype_of_symbolP(test_value_001)) {
                { Object* value_001 = value;
                  Symbol* value = ((Symbol*)(value_001));

                  newmethod->method_inlined_functions_reader()->insert(value);
                }
              }
              else {
                *(STANDARD_WARNING->native_stream) << "Warning: " << "Illegal :inline argument: " << "`" << value << "'" << endl;
              }
            }
          }
          else if (test_value_000 == KWD_METHODS_AUXILIARYp) {
            set_dynamic_slot_value(newmethod->dynamic_slots, SYM_METHODS_STELLA_SLOT_AUXILIARYp, (((Boolean_Wrapper*)(value))->wrapper_value ? TRUE_WRAPPER : FALSE_WRAPPER), FALSE_WRAPPER);
          }
          else if (test_value_000 == KWD_METHODS_DOCUMENTATION) {
            set_dynamic_slot_value(newmethod->dynamic_slots, SYM_METHODS_STELLA_DOCUMENTATION, wrap_string(((String_Wrapper*)(value))->wrapper_value), NULL_STRING_WRAPPER);
          }
          else if ((test_value_000 == KWD_METHODS_TYPE) ||
              (test_value_000 == KWD_METHODS_RETURNS)) {
          }
          else if (test_value_000 == KWD_METHODS_STORAGE_SLOT) {
            set_dynamic_slot_value(newmethod->dynamic_slots, SYM_METHODS_STELLA_STORAGE_SLOT, ((Symbol*)(((Symbol*)(value))->permanentify())), NULL);
          }
          else if (test_value_000 == KWD_METHODS_INHERITS_THROUGH) {
            set_dynamic_slot_value(newmethod->dynamic_slots, SYM_METHODS_STELLA_METHOD_INHERITS_THROUGH, ((Symbol*)(((Symbol*)(value))->permanentify())), NULL);
          }
          else if (test_value_000 == KWD_METHODS_PROPERTIES) {
            set_dynamic_slot_value(newmethod->dynamic_slots, SYM_METHODS_STELLA_PROPERTIES, ((List*)(value)), NULL);
          }
          else if (test_value_000 == KWD_METHODS_META_ATTRIBUTES) {
            set_dynamic_slot_value(newmethod->dynamic_slots, SYM_METHODS_STELLA_META_ATTRIBUTES, ((Key_Value_List*)(value)), NULL);
          }
          else if (test_value_000 == KWD_METHODS_COMMANDp) {
            set_dynamic_slot_value(newmethod->dynamic_slots, SYM_METHODS_STELLA_METHOD_COMMANDp, (((Boolean_Wrapper*)(value))->wrapper_value ? TRUE_WRAPPER : FALSE_WRAPPER), FALSE_WRAPPER);
          }
          else if (test_value_000 == KWD_METHODS_LISP_MACROp) {
            set_dynamic_slot_value(newmethod->dynamic_slots, SYM_METHODS_STELLA_METHOD_LISP_MACROp, (((Boolean_Wrapper*)(value))->wrapper_value ? TRUE_WRAPPER : FALSE_WRAPPER), NULL);
          }
          else if (test_value_000 == KWD_METHODS_EVALUATE_ARGUMENTSp) {
            set_dynamic_slot_value(newmethod->dynamic_slots, SYM_METHODS_STELLA_METHOD_EVALUATE_ARGUMENTSp, (((Boolean_Wrapper*)(value))->wrapper_value ? TRUE_WRAPPER : FALSE_WRAPPER), FALSE_WRAPPER);
          }
          else {
            if (!(run_option_handlerP(newmethod, key, value))) {
              { 
                BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
                signal_translation_warning();
                if (!(suppress_warningsP())) {
                  print_error_context(">> WARNING: ", STANDARD_WARNING);
                  *(STANDARD_WARNING->native_stream) << endl << " " << "Skipping invalid method option " << "`" << de_uglify_parse_tree(key) << "'" << endl << "in the definition of method " << "`" << newmethod->slot_owner << "'" << "." << "`" << de_uglify_parse_tree(name) << "'" << "." << endl;
                }
              }
            }
          }
        }
      }
    }
    return (newmethod);
  }
}

Method_Slot* attach_method_slot_to_owner(Method_Slot* newmethod) {
  if (newmethod->method_functionP) {
    return (attach_function(newmethod));
  }
  if (!((boolean)(newmethod->slot_owner->surrogate_value))) {
    return (newmethod);
  }
  if (!undefine_conflicting_definitionsP(newmethod)) {
    return (NULL);
  }
  return (((Method_Slot*)(attach_slot_to_owner(newmethod))));
}

// Maximum size for a string constant in the target language.
int oMAXIMUM_STRING_CONSTANT_SIZEo = 4000;

Object* yield_string_constant_tree(char* string) {
  { int length = strlen(string);
    Cons* chunks = NIL;
    int start = 0;
    Object* stringtree = NULL;

    if (length <= oMAXIMUM_STRING_CONSTANT_SIZEo) {
      return (wrap_string(string));
    }
    while ((start + oMAXIMUM_STRING_CONSTANT_SIZEo) < length) {
      chunks = cons(wrap_string(string_subsequence(string, start, start + oMAXIMUM_STRING_CONSTANT_SIZEo)), chunks);
      start = start + oMAXIMUM_STRING_CONSTANT_SIZEo;
    }
    if (start < length) {
      chunks = cons(wrap_string(string_subsequence(string, start, length)), chunks);
    }
    stringtree = chunks->value;
    { Object* chunk = NULL;
      Cons* iter_000 = chunks->rest;

      while (!(iter_000 == NIL)) {
        chunk = iter_000->value;
        iter_000 = iter_000->rest;
        stringtree = listO(3, SYM_METHODS_STELLA_CONCATENATE, chunk, cons(stringtree, NIL));
      }
    }
    return (stringtree);
  }
}

Cons* yield_define_stella_method_object(Method_Slot* method, Method_Slot* codemethod, Method_Slot* wrappermethod) {
  { char* name = stringify(method->slot_name);

    if (method->method_functionP) {
      return (listO(3, SYM_METHODS_STELLA_DEFINE_FUNCTION_OBJECT, (string_eqlP(name, method->slot_name->symbol_name) ? wrap_string(name) : wrap_string(string_concatenate(" ", name, 0))), cons(yield_string_constant_tree(method->method_stringified_source), cons((((boolean)(codemethod)) ? ((Standard_Object*)(listO(4, SYM_METHODS_STELLA_THE_CODE, KWD_METHODS_FUNCTION, codemethod->slot_name, NIL))) : ((Standard_Object*)(SYM_METHODS_STELLA_NULL))), cons((((boolean)(wrappermethod)) ? ((Standard_Object*)(listO(4, SYM_METHODS_STELLA_THE_CODE, KWD_METHODS_FUNCTION, wrappermethod->slot_name, cons(wrappermethod, NIL)))) : ((Standard_Object*)(SYM_METHODS_STELLA_NULL))), NIL)))));
    }
    else {
      return (listO(3, SYM_METHODS_STELLA_DEFINE_METHOD_OBJECT, yield_string_constant_tree(method->method_stringified_source), cons((((boolean)(codemethod)) ? ((Standard_Object*)(listO(4, SYM_METHODS_STELLA_THE_CODE, KWD_METHODS_METHOD, codemethod->slot_owner, cons(codemethod->slot_name, NIL)))) : ((Standard_Object*)(SYM_METHODS_STELLA_NULL))), cons(SYM_METHODS_STELLA_NULL, NIL))));
    }
  }
}

Cons* yield_class_parameter_types(Class* renamed_Class) {
  { Cons* types = NIL;

    { Symbol* pname = NULL;
      Cons* iter_000 = renamed_Class->parameters()->the_cons_list;
      Cons* collect_000 = NULL;

      while (!(iter_000 == NIL)) {
        pname = ((Symbol*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (!((boolean)(collect_000))) {
          {
            collect_000 = cons(lookup_slot(renamed_Class, pname)->slot_base_type, NIL);
            if (types == NIL) {
              types = collect_000;
            }
            else {
              add_cons_to_end_of_cons_list(types, collect_000);
            }
          }
        }
        else {
          {
            collect_000->rest = cons(lookup_slot(renamed_Class, pname)->slot_base_type, NIL);
            collect_000 = collect_000->rest;
          }
        }
      }
    }
    return (types);
  }
}

boolean optimistic_subtype_ofP(Surrogate* subtype, Surrogate* supertype) {
  if (((boolean)(((Class*)(subtype->surrogate_value)))) &&
      (((boolean)(((Class*)(supertype->surrogate_value)))) &&
       ((Class*)(subtype->surrogate_value))->class_finalizedP)) {
    return (subtype_ofP(subtype, supertype));
  }
  else {
    return (TRUE);
  }
}

void warn_of_parameter_mismatch(Parametric_Type_Specifier* self, char* message) {
  { Cons* inheritedtypes = yield_class_parameter_types(((Class*)(self->specifier_base_type->surrogate_value)));

    { 
      BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
      *(STANDARD_WARNING->native_stream) << "Warning: " << "`" << message << "'" << endl << "Type " << "`" << self->specifier_base_type << "'" << " takes parameters of type " << endl << "`" << inheritedtypes << "'" << " but was modified by a list of types " << "`" << self->specifier_parameter_types->the_cons_list << "'" << endl;
    }
  }
}

Surrogate* validate_type_specifier(Standard_Object* self, Class* anchorclass, boolean skipanchoredtypeP) {
  { Surrogate* test_value_000 = safe_primary_type(self);

    if (subtype_of_parametric_type_specifierP(test_value_000)) {
      { Standard_Object* self_000 = self;
        Parametric_Type_Specifier* self = ((Parametric_Type_Specifier*)(self_000));

        if (!((boolean)(((Class*)(self->specifier_base_type->surrogate_value))))) {
          return (SGT_METHODS_STELLA_UNKNOWN);
        }
        { int parameterdifferential = self->specifier_parameter_types->length() - ((Class*)(self->specifier_base_type->surrogate_value))->parameters()->length();

          if (parameterdifferential > 0) {
            warn_of_parameter_mismatch(self, "Too many parameters in type specification.");
          }
          else if (parameterdifferential < 0) {
            warn_of_parameter_mismatch(self, "Not enough parameters in type specification.");
          }
          else {
            { Class* baseclass = ((Class*)(type_spec_to_base_type(self)->surrogate_value));

              if (baseclass->class_slots_finalizedP) {
                { Cons* inheritedtypes = yield_class_parameter_types(baseclass);

                  { Surrogate* supertype = NULL;
                    Standard_Object* subtypespec = NULL;
                    Cons* iter_000 = inheritedtypes;
                    Cons* iter_001 = self->specifier_parameter_types->the_cons_list;

                    while ((!(iter_000 == NIL)) &&
                        (!(iter_001 == NIL))) {
                      supertype = ((Surrogate*)(iter_000->value));
                      iter_000 = iter_000->rest;
                      subtypespec = ((Standard_Object*)(iter_001->value));
                      iter_001 = iter_001->rest;
                      if ((!anchored_type_specifierP(subtypespec)) &&
                          ((!(supertype == SGT_METHODS_STELLA_UNKNOWN)) &&
                           (!optimistic_subtype_ofP(type_spec_to_base_type(subtypespec), supertype)))) {
                        warn_of_parameter_mismatch(self, "Parametric subtype(s) don't specialize supertype(s)");
                      }
                    }
                  }
                }
              }
            }
          }
          return (self->specifier_base_type);
        }
      }
    }
    else if (subtype_of_anchored_type_specifierP(test_value_000)) {
      { Standard_Object* self_001 = self;
        Anchored_Type_Specifier* self = ((Anchored_Type_Specifier*)(self_001));

        if (skipanchoredtypeP) {
          return (NULL);
        }
        if (!((boolean)(anchorclass))) {
          *(STANDARD_WARNING->native_stream) << "Warning: " << "Illegal use of anchored type for function that has no first\nargument (and hence nothing to anchor onto)." << endl;
          return (NULL);
        }
        { Symbol* slotname = self->specifier_parameter_name;
          Slot* anchorslot = NULL;

          if (!((boolean)(slotname))) {
            return (anchorclass->class_type);
          }
          { 
            anchorslot = lookup_slot(anchorclass, slotname);
            if (!(((boolean)(anchorslot)))) {
              cerr << "Can't find a slot named " << "`" << slotname << "'" << " on the class " << "`" << class_name(anchorclass) << "'";
            }
          }
          return (anchorslot->slot_base_type);
        }
      }
    }
    else if (subtype_of_surrogateP(test_value_000)) {
      { Standard_Object* self_002 = self;
        Surrogate* self = ((Surrogate*)(self_002));

        return (self);
      }
    }
    else {
      { Output_String_Stream* stream_000 = new_output_string_stream();

        *(stream_000->native_stream) << "validate-type-specifier: Not defined on " << "`" << self << "'";
        throw *new_stella_exception(stream_000->the_string_reader());
      }
    }
  }
}

Cons* synthesize_method_body(Method_Slot* method) {
  if (!((boolean)(((Class*)(SGT_METHODS_STELLA_BOOLEAN->surrogate_value))))) {
    return (NIL);
  }
  { Symbol* storageslotname = ((Symbol*)(dynamic_slot_value(method->dynamic_slots, SYM_METHODS_STELLA_STORAGE_SLOT, NULL)));
    Symbol* inheritanceslotname = ((Symbol*)(dynamic_slot_value(method->dynamic_slots, SYM_METHODS_STELLA_METHOD_INHERITS_THROUGH, NULL)));
    Storage_Slot* storageslot = NULL;
    Slot* inheritanceslot = NULL;
    Object* defaultvalue = NULL;

    if (inheritanceslotname == SYM_METHODS_STELLA_SUPER_CLASSES) {
      inheritanceslotname = SYM_METHODS_STELLA_CLASS_ALL_SUPER_CLASSES;
    }
    else if (inheritanceslotname == SYM_METHODS_STELLA_EQUIVALENT_SLOT) {
      inheritanceslotname = SYM_METHODS_STELLA_SLOT_DIRECT_EQUIVALENT;
    }
    else {
    }
    storageslot = ((Storage_Slot*)(lookup_slot(((Class*)(method->slot_owner->surrogate_value)), storageslotname)));
    if (!((boolean)(storageslot))) {
      *(STANDARD_WARNING->native_stream) << "Warning: " << ":storage-slot references non-existent slot " << "`" << storageslotname << "'" << " on the class " << "`" << method->slot_owner << "'" << endl;
      return (NIL);
    }
    inheritanceslot = lookup_slot(((Class*)(method->slot_owner->surrogate_value)), inheritanceslotname);
    if (!((boolean)(inheritanceslot))) {
      *(STANDARD_WARNING->native_stream) << "Warning: " << ":inherits-through references non-existent slot " << "`" << inheritanceslotname << "'" << " on the class " << "`" << method->slot_owner << "'" << endl;
      return (NIL);
    }
    defaultvalue = storageslot->system_default_value();
    return (cons(listO(4, SYM_METHODS_STELLA_LET, listO(3, listO(3, SYM_METHODS_STELLA_CURSOR, SYM_METHODS_STELLA_SELF, NIL), listO(3, SYM_METHODS_STELLA_VALUE, listO(4, SYM_METHODS_STELLA_SLOT_VALUE, SYM_METHODS_STELLA_CURSOR, storageslotname, NIL), NIL), NIL), listO(5, SYM_METHODS_STELLA_LOOP, cons(SYM_METHODS_STELLA_WHEN, ((subtype_ofP(storageslot->type(), SGT_METHODS_STELLA_BOOLEAN) ? cons(SYM_METHODS_STELLA_VALUE, NIL) : cons(listO(3, SYM_METHODS_STELLA_DEFINEDp, SYM_METHODS_STELLA_VALUE, NIL), NIL)))->concatenate(cons(listO(3, SYM_METHODS_STELLA_RETURN, SYM_METHODS_STELLA_VALUE, NIL), NIL), 0)), listO(4, SYM_METHODS_STELLA_SETQ, SYM_METHODS_STELLA_CURSOR, ((subtype_ofP(inheritanceslot->type(), SGT_METHODS_STELLA_COLLECTION) ||
        subtype_ofP(inheritanceslot->type(), SGT_METHODS_STELLA_CONS)) ? listO(3, SYM_METHODS_STELLA_FIRST, cons(inheritanceslotname, cons(SYM_METHODS_STELLA_CURSOR, NIL)), NIL) : cons(inheritanceslotname, cons(SYM_METHODS_STELLA_CURSOR, NIL))), NIL), listO(5, SYM_METHODS_STELLA_IF, listO(3, SYM_METHODS_STELLA_DEFINEDp, SYM_METHODS_STELLA_CURSOR, NIL), listO(4, SYM_METHODS_STELLA_SETQ, SYM_METHODS_STELLA_VALUE, listO(4, SYM_METHODS_STELLA_SLOT_VALUE, SYM_METHODS_STELLA_CURSOR, storageslotname, NIL), NIL), listO(3, SYM_METHODS_STELLA_RETURN, (((boolean)(defaultvalue)) ? defaultvalue : SYM_METHODS_STELLA_NULL), NIL), NIL), NIL), NIL), NIL));
  }
}

void Method_Slot::finalize_slot_type_computations() {
  { Method_Slot* self = this;

    { Class* ownerclass = (((boolean)(self->slot_owner)) ? ((Class*)(self->slot_owner->surrogate_value)) : ((Class*)(NULL)));

      { Standard_Object* ts = NULL;
        Cons* iter_000 = self->method_parameter_type_specifiers_reader()->the_cons_list;

        while (!(iter_000 == NIL)) {
          ts = ((Standard_Object*)(iter_000->value));
          iter_000 = iter_000->rest;
          validate_type_specifier(ts, ownerclass, FALSE);
        }
      }
      if (((boolean)(((Compound_Type_Specifier*)(dynamic_slot_value(self->dynamic_slots, SYM_METHODS_STELLA_SLOT_TYPE_SPECIFIER, NULL)))))) {
        self->slot_base_type = validate_type_specifier(((Compound_Type_Specifier*)(dynamic_slot_value(self->dynamic_slots, SYM_METHODS_STELLA_SLOT_TYPE_SPECIFIER, NULL))), ownerclass, FALSE);
      }
      { Standard_Object* ts = NULL;
        Cons* iter_001 = self->method_return_type_specifiers_reader()->rest();

        while (!(iter_001 == NIL)) {
          ts = ((Standard_Object*)(iter_001->value));
          iter_001 = iter_001->rest;
          validate_type_specifier(ts, ownerclass, FALSE);
        }
      }
    }
  }
}

void Method_Slot::help_finalize_local_slot() {
  { Method_Slot* self = this;

  }
}

void Method_Slot::unfinalize_local_slot() {
  { Method_Slot* self = this;

  }
}

void undefine_external_slots(Symbol* name) {
  { Cons* slots = NIL;

    { Module* module = NULL;
      Iterator* iter_000 = all_modules();

      while (iter_000->nextP()) {
        module = ((Module*)(iter_000->value));
        { Slot* slot = NULL;
          Iterator* iter_001 = all_slots(module, TRUE);
          Cons* collect_000 = NULL;

          while (iter_001->nextP()) {
            slot = ((Slot*)(iter_001->value));
            if ((slot->slot_name == name) &&
                slot->slot_externalP) {
              if (!((boolean)(collect_000))) {
                {
                  collect_000 = cons(slot, NIL);
                  if (slots == NIL) {
                    slots = collect_000;
                  }
                  else {
                    add_cons_to_end_of_cons_list(slots, collect_000);
                  }
                }
              }
              else {
                {
                  collect_000->rest = cons(slot, NIL);
                  collect_000 = collect_000->rest;
                }
              }
            }
          }
        }
      }
    }
    { Object* slot = NULL;
      Cons* iter_002 = slots;

      while (!(iter_002 == NIL)) {
        slot = iter_002->value;
        iter_002 = iter_002->rest;
        destroy_external_slot(((Slot*)(slot)));
      }
    }
  }
}

void destroy_external_slot(Slot* self) {
  if (!(self->deletedP() ||
      (coerce_wrapped_boolean_to_boolean(self->badP_reader()) ||
       (!self->slot_externalP)))) {
    { Class* ownerclass = ((Class*)(self->slot_owner->surrogate_value));

      if (((boolean)(ownerclass))) {
        ownerclass->class_local_slots->remove(self);
        unfinalize_class_slots(ownerclass);
        self->deletedP_setter(TRUE);
      }
    }
  }
}

void destroy_method(Method_Slot* self) {
  destroy_external_slot(self);
}

int Slot::method_argument_count() {
  { Slot* self = this;

    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "method-argument-count: Not defined on " << "`" << self << "'";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
}

int Storage_Slot::method_argument_count() {
  { Storage_Slot* self = this;

    return (1);
  }
}

int Method_Slot::method_argument_count() {
  { Method_Slot* self = this;

    return (self->method_parameter_names_reader()->length());
  }
}

int Table::method_argument_count() {
  { Table* self = this;

    return (self->tuple_domains->length() - 1);
  }
}

boolean Storage_Slot::method_variable_argumentsP() {
  { Storage_Slot* self = this;

    return (FALSE);
  }
}

boolean Storage_Slot::method_body_argumentP() {
  { Storage_Slot* self = this;

    return (FALSE);
  }
}

int Storage_Slot::arity() {
  { Storage_Slot* self = this;

    return (2);
  }
}

int Method_Slot::arity() {
  { Method_Slot* self = this;

    if (((Boolean_Wrapper*)(dynamic_slot_value(self->dynamic_slots, SYM_METHODS_STELLA_METHOD_VARIABLE_ARGUMENTSp, FALSE_WRAPPER)))->wrapper_value) {
      return (NULL_INTEGER);
    }
    { int returntypecount = self->method_return_type_specifiers_reader()->length();
      int result = self->method_parameter_type_specifiers_reader()->length() + returntypecount;

      switch (returntypecount) {
        case 1: 
          if (((Standard_Object*)(self->method_return_type_specifiers_reader()->first())) == SGT_METHODS_STELLA_BOOLEAN) {
            return (result - 1);
          }
          else {
            return (result);
          }
        break;
        case 0: 
          return (result);
        default:
          if (((Standard_Object*)(self->method_return_type_specifiers_reader()->last())) == SGT_METHODS_STELLA_BOOLEAN) {
            return (result - 1);
          }
          else {
            return (result);
          }
        break;
      }
    }
  }
}

int Table::arity() {
  { Table* self = this;

    if (((Boolean_Wrapper*)(dynamic_slot_value(self->dynamic_slots, SYM_METHODS_STELLA_VARIABLE_ARITY_TABLEp, FALSE_WRAPPER)))->wrapper_value) {
      return (NULL_INTEGER);
    }
    else {
      return (self->tuple_domains->length());
    }
  }
}

Surrogate* type_spec_to_base_type(Standard_Object* self) {
  { Surrogate* test_value_000 = safe_primary_type(self);

    if (subtype_of_surrogateP(test_value_000)) {
      { Standard_Object* self_000 = self;
        Surrogate* self = ((Surrogate*)(self_000));

        return (self);
      }
    }
    else if (subtype_of_parametric_type_specifierP(test_value_000)) {
      { Standard_Object* self_001 = self;
        Parametric_Type_Specifier* self = ((Parametric_Type_Specifier*)(self_001));

        return (self->specifier_base_type);
      }
    }
    else {
      { Output_String_Stream* stream_000 = new_output_string_stream();

        *(stream_000->native_stream) << "type-spec-to-base-type: Not defined on " << "`" << self << "'";
        throw *new_stella_exception(stream_000->the_string_reader());
      }
    }
  }
}

Standard_Object* base_type_to_type_spec(Standard_Object* self) {
  { Surrogate* test_value_000 = safe_primary_type(self);

    if (subtype_of_surrogateP(test_value_000)) {
      { Standard_Object* self_000 = self;
        Surrogate* self = ((Surrogate*)(self_000));

        { Class* renamed_Class = ((Class*)(self->surrogate_value));
          List* parameters = renamed_Class->parameters();
          Standard_Object* parametertype = NULL;
          List* parametertypes = NULL;

          if (parameters->non_emptyP()) {
            parametertypes = new_list();
            { Symbol* parameter = NULL;
              Cons* iter_000 = parameters->the_cons_list;
              Cons* collect_000 = NULL;

              while (!(iter_000 == NIL)) {
                parameter = ((Symbol*)(iter_000->value));
                iter_000 = iter_000->rest;
                parametertype = lookup_slot(renamed_Class, parameter)->compute_return_type_spec(self);
                if (unknown_typeP(((Surrogate*)(parametertype)))) {
                  return (self);
                }
                if (!((boolean)(collect_000))) {
                  {
                    collect_000 = cons(parametertype, NIL);
                    if (parametertypes->the_cons_list == NIL) {
                      parametertypes->the_cons_list = collect_000;
                    }
                    else {
                      add_cons_to_end_of_cons_list(parametertypes->the_cons_list, collect_000);
                    }
                  }
                }
                else {
                  {
                    collect_000->rest = cons(parametertype, NIL);
                    collect_000 = collect_000->rest;
                  }
                }
              }
            }
            { Parametric_Type_Specifier* self_003 = new_parametric_type_specifier();

              self_003->specifier_base_type = self;
              self_003->specifier_parameter_types = parametertypes;
              { Parametric_Type_Specifier* value_000 = self_003;

                return (value_000);
              }
            }
          }
          return (self);
        }
      }
    }
    else if (subtype_of_parametric_type_specifierP(test_value_000)) {
      { Standard_Object* self_001 = self;
        Parametric_Type_Specifier* self = ((Parametric_Type_Specifier*)(self_001));

        return (self);
      }
    }
    else {
      { Output_String_Stream* stream_000 = new_output_string_stream();

        *(stream_000->native_stream) << "base-type-to-type-spec: Not defined on " << "`" << self << "'";
        throw *new_stella_exception(stream_000->the_string_reader());
      }
    }
  }
}

Class* type_spec_to_class(Standard_Object* self) {
  return (((Class*)(type_spec_to_base_type(self)->surrogate_value)));
}

boolean compatible_parameter_typesP(Standard_Object* subtype, Standard_Object* supertype) {
  { boolean dummy1;
    boolean dummy2;

    if (typeP(subtype) ||
        typeP(supertype)) {
      return (TRUE);
    }
    { Class* subclass = NULL;
      Class* superclass = NULL;

      { Surrogate* test_value_000 = safe_primary_type(subtype);

        if (subtype_of_surrogateP(test_value_000)) {
          { Standard_Object* subtype_000 = subtype;
            Surrogate* subtype = ((Surrogate*)(subtype_000));

            subclass = ((Class*)(subtype->surrogate_value));
          }
        }
        else if (subtype_of_parametric_type_specifierP(test_value_000)) {
          { Standard_Object* subtype_001 = subtype;
            Parametric_Type_Specifier* subtype = ((Parametric_Type_Specifier*)(subtype_001));

            subclass = ((Class*)(subtype->specifier_base_type->surrogate_value));
          }
        }
        else {
          { Output_String_Stream* stream_000 = new_output_string_stream();

            *(stream_000->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
            throw *new_stella_exception(stream_000->the_string_reader());
          }
        }
      }
      { Surrogate* test_value_001 = safe_primary_type(supertype);

        if (subtype_of_surrogateP(test_value_001)) {
          { Standard_Object* supertype_000 = supertype;
            Surrogate* supertype = ((Surrogate*)(supertype_000));

            superclass = ((Class*)(supertype->surrogate_value));
          }
        }
        else if (subtype_of_parametric_type_specifierP(test_value_001)) {
          { Standard_Object* supertype_001 = supertype;
            Parametric_Type_Specifier* supertype = ((Parametric_Type_Specifier*)(supertype_001));

            superclass = ((Class*)(supertype->specifier_base_type->surrogate_value));
          }
        }
        else {
          { Output_String_Stream* stream_001 = new_output_string_stream();

            *(stream_001->native_stream) << "`" << test_value_001 << "'" << " is not a valid case option";
            throw *new_stella_exception(stream_001->the_string_reader());
          }
        }
      }
      { Symbol* suppar = NULL;
        Cons* iter_000 = superclass->parameters()->the_cons_list;

        while (!(iter_000 == NIL)) {
          suppar = ((Symbol*)(iter_000->value));
          iter_000 = iter_000->rest;
          { boolean foundP_000 = FALSE;

            { Symbol* subpar = NULL;
              Cons* iter_001 = subclass->parameters()->the_cons_list;

              while (!(iter_001 == NIL)) {
                subpar = ((Symbol*)(iter_001->value));
                iter_001 = iter_001->rest;
                if (suppar == subpar) {
                  foundP_000 = TRUE;
                  break;
                }
              }
            }
            if (!(foundP_000)) {
              return (FALSE);
            }
          }
          { Standard_Object* subpartype = extract_parameter_type(subtype, suppar, dummy1);
            Standard_Object* suppartype = extract_parameter_type(supertype, suppar, dummy2);

            if (subpartype == suppartype) {
              continue;
            }
            if ((!anchored_type_specifierP(subpartype)) &&
                (!((boolean)(((Class*)(type_spec_to_base_type(subpartype)->surrogate_value)))))) {
              { 
                BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
                signal_translation_error();
                if (!(suppress_warningsP())) {
                  print_error_context(">> ERROR: ", STANDARD_ERROR);
                  *(STANDARD_ERROR->native_stream) << endl << " " << "Parameter type " << "`" << type_spec_to_base_type(subpartype) << "'" << " of " << "`" << de_uglify_parse_tree(subtype) << "'" << " is not yet defined" << "." << endl;
                }
              }
              return (FALSE);
            }
            if ((!anchored_type_specifierP(suppartype)) &&
                (!((boolean)(((Class*)(type_spec_to_base_type(suppartype)->surrogate_value)))))) {
              { 
                BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
                signal_translation_error();
                if (!(suppress_warningsP())) {
                  print_error_context(">> ERROR: ", STANDARD_ERROR);
                  *(STANDARD_ERROR->native_stream) << endl << " " << "Parameter type " << "`" << type_spec_to_base_type(suppartype) << "'" << " of " << "`" << de_uglify_parse_tree(supertype) << "'" << " is not yet defined" << "." << endl;
                }
              }
              return (FALSE);
            }
            if (!(sub_type_spec_ofP(subpartype, suppartype))) {
              return (FALSE);
            }
          }
        }
      }
      return (TRUE);
    }
  }
}

boolean sub_type_spec_ofP(Standard_Object* subtype, Standard_Object* supertype) {
  if (subtype == supertype) {
    return (TRUE);
  }
  { Surrogate* test_value_000 = safe_primary_type(subtype);

    if (subtype_of_surrogateP(test_value_000)) {
      { Standard_Object* subtype_000 = subtype;
        Surrogate* subtype = ((Surrogate*)(subtype_000));

        { Surrogate* test_value_001 = safe_primary_type(supertype);

          if (subtype_of_surrogateP(test_value_001)) {
            { Standard_Object* supertype_000 = supertype;
              Surrogate* supertype = ((Surrogate*)(supertype_000));

              return (subtype_ofP(subtype, supertype) &&
                  compatible_parameter_typesP(subtype, supertype));
            }
          }
          else if (subtype_of_parametric_type_specifierP(test_value_001)) {
            { Standard_Object* supertype_001 = supertype;
              Parametric_Type_Specifier* supertype = ((Parametric_Type_Specifier*)(supertype_001));

              return (subtype_ofP(subtype, supertype->specifier_base_type) &&
                  compatible_parameter_typesP(subtype, supertype));
            }
          }
          else {
            return (FALSE);
          }
        }
      }
    }
    else if (subtype_of_parametric_type_specifierP(test_value_000)) {
      { Standard_Object* subtype_001 = subtype;
        Parametric_Type_Specifier* subtype = ((Parametric_Type_Specifier*)(subtype_001));

        { Surrogate* test_value_002 = safe_primary_type(supertype);

          if (subtype_of_surrogateP(test_value_002)) {
            { Standard_Object* supertype_002 = supertype;
              Surrogate* supertype = ((Surrogate*)(supertype_002));

              return (subtype_ofP(subtype->specifier_base_type, supertype) &&
                  compatible_parameter_typesP(subtype, supertype));
            }
          }
          else if (subtype_of_parametric_type_specifierP(test_value_002)) {
            { Standard_Object* supertype_003 = supertype;
              Parametric_Type_Specifier* supertype = ((Parametric_Type_Specifier*)(supertype_003));

              return (subtype_ofP(subtype->specifier_base_type, supertype->specifier_base_type) &&
                  compatible_parameter_typesP(subtype, supertype));
            }
          }
          else {
            return (FALSE);
          }
        }
      }
    }
    else if (subtype_of_anchored_type_specifierP(test_value_000)) {
      { Standard_Object* subtype_002 = subtype;
        Anchored_Type_Specifier* subtype = ((Anchored_Type_Specifier*)(subtype_002));

        return (anchored_type_specifierP(supertype) &&
            (subtype->specifier_parameter_name == ((Anchored_Type_Specifier*)(supertype))->specifier_parameter_name));
      }
    }
    else {
      { Output_String_Stream* stream_000 = new_output_string_stream();

        *(stream_000->native_stream) << "sub-type-spec-of?: Not defined on " << "`" << subtype << "'";
        throw *new_stella_exception(stream_000->the_string_reader());
      }
    }
  }
}

boolean voidP(Standard_Object* type) {
  return (type == SGT_METHODS_STELLA_VOID);
}

Standard_Object* extract_parameter_type(Standard_Object* self, Symbol* parameter, boolean& _return1) {
  { Surrogate* test_value_000 = safe_primary_type(self);

    if (subtype_of_surrogateP(test_value_000)) {
      { Standard_Object* self_000 = self;
        Surrogate* self = ((Surrogate*)(self_000));

        { Slot* slot = lookup_slot(((Class*)(self->surrogate_value)), parameter);

          if (((boolean)(slot))) {
            _return1 = TRUE;
            return (slot->type());
          }
          else {
            _return1 = FALSE;
            return (SGT_METHODS_STELLA_OBJECT);
          }
        }
      }
    }
    else if (subtype_of_parametric_type_specifierP(test_value_000)) {
      { Standard_Object* self_001 = self;
        Parametric_Type_Specifier* self = ((Parametric_Type_Specifier*)(self_001));

        { Symbol* pname = NULL;
          Standard_Object* ptype = NULL;
          Cons* iter_000 = ((Class*)(self->specifier_base_type->surrogate_value))->parameters()->the_cons_list;
          Cons* iter_001 = self->specifier_parameter_types->the_cons_list;

          while ((!(iter_000 == NIL)) &&
              (!(iter_001 == NIL))) {
            pname = ((Symbol*)(iter_000->value));
            iter_000 = iter_000->rest;
            ptype = ((Standard_Object*)(iter_001->value));
            iter_001 = iter_001->rest;
            if (pname == parameter) {
              _return1 = TRUE;
              return (ptype);
            }
          }
        }
        _return1 = FALSE;
        return (SGT_METHODS_STELLA_OBJECT);
      }
    }
    else {
      { Output_String_Stream* stream_000 = new_output_string_stream();

        *(stream_000->native_stream) << "extract-parameter-type: Not defined on " << "`" << self << "'";
        throw *new_stella_exception(stream_000->the_string_reader());
      }
    }
  }
}

Standard_Object* compute_anchored_type_spec(Standard_Object* ownertype, Anchored_Type_Specifier* reltype) {
  { Surrogate* test_value_000 = safe_primary_type(ownertype);

    if (subtype_of_surrogateP(test_value_000)) {
      { Standard_Object* ownertype_000 = ownertype;
        Surrogate* ownertype = ((Surrogate*)(ownertype_000));

        if (!((boolean)(reltype->specifier_parameter_name))) {
          return (ownertype);
        }
        { Slot* slot = lookup_slot(((Class*)(ownertype->surrogate_value)), reltype->specifier_parameter_name);

          if (!((boolean)(slot))) {
            { 
              BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
              signal_translation_error();
              if (!(suppress_warningsP())) {
                print_error_context(">> ERROR: ", STANDARD_ERROR);
                *(STANDARD_ERROR->native_stream) << endl << " " << "Missing :parameter named " << "`" << reltype->specifier_parameter_name << "'" << endl << "   for the class " << "`" << de_uglify_parse_tree(ownertype) << "'" << endl << "." << endl;
              }
            }
            return (SGT_METHODS_STELLA_OBJECT);
          }
          return (slot->slot_base_type);
        }
      }
    }
    else if (subtype_of_parametric_type_specifierP(test_value_000)) {
      { Standard_Object* ownertype_001 = ownertype;
        Parametric_Type_Specifier* ownertype = ((Parametric_Type_Specifier*)(ownertype_001));

        if (!((boolean)(reltype->specifier_parameter_name))) {
          return (ownertype);
        }
        { Symbol* pname = NULL;
          Standard_Object* ptype = NULL;
          Cons* iter_000 = ((Class*)(ownertype->specifier_base_type->surrogate_value))->parameters()->the_cons_list;
          Cons* iter_001 = ownertype->specifier_parameter_types->the_cons_list;

          while ((!(iter_000 == NIL)) &&
              (!(iter_001 == NIL))) {
            pname = ((Symbol*)(iter_000->value));
            iter_000 = iter_000->rest;
            ptype = ((Standard_Object*)(iter_001->value));
            iter_001 = iter_001->rest;
            if (pname == reltype->specifier_parameter_name) {
              return (ptype);
            }
          }
        }
        return (compute_anchored_type_spec(ownertype->specifier_base_type, reltype));
      }
    }
    else {
      { Output_String_Stream* stream_000 = new_output_string_stream();

        *(stream_000->native_stream) << "compute-anchored-type-spec: Not defined on " << "`" << ownertype << "'";
        throw *new_stella_exception(stream_000->the_string_reader());
      }
    }
  }
}

Standard_Object* compute_relative_type_spec(Standard_Object* relativetype, Standard_Object* ownertype) {
  { Surrogate* test_value_000 = safe_primary_type(relativetype);

    if (subtype_of_parametric_type_specifierP(test_value_000)) {
      { Standard_Object* relativetype_000 = relativetype;
        Parametric_Type_Specifier* relativetype = ((Parametric_Type_Specifier*)(relativetype_000));

        { boolean alwaysP_000 = TRUE;

          { Standard_Object* ts = NULL;
            Cons* iter_000 = relativetype->specifier_parameter_types->the_cons_list;

            while (!(iter_000 == NIL)) {
              ts = ((Standard_Object*)(iter_000->value));
              iter_000 = iter_000->rest;
              if (!typeP(ts)) {
                alwaysP_000 = FALSE;
                break;
              }
            }
          }
          if (alwaysP_000) {
            return (relativetype);
          }
        }
        { List* typeslist = new_list();
          Standard_Object* relativets = NULL;
          boolean foundrelativistictypeP = FALSE;

          { Standard_Object* ts = NULL;
            Cons* iter_001 = relativetype->specifier_parameter_types->the_cons_list;
            Cons* collect_000 = NULL;

            while (!(iter_001 == NIL)) {
              ts = ((Standard_Object*)(iter_001->value));
              iter_001 = iter_001->rest;
              relativets = compute_relative_type_spec(ts, ownertype);
              if (!(relativets == ts)) {
                foundrelativistictypeP = TRUE;
              }
              if (!((boolean)(collect_000))) {
                {
                  collect_000 = cons(relativets, NIL);
                  if (typeslist->the_cons_list == NIL) {
                    typeslist->the_cons_list = collect_000;
                  }
                  else {
                    add_cons_to_end_of_cons_list(typeslist->the_cons_list, collect_000);
                  }
                }
              }
              else {
                {
                  collect_000->rest = cons(relativets, NIL);
                  collect_000 = collect_000->rest;
                }
              }
            }
          }
          if (foundrelativistictypeP) {
            { Parametric_Type_Specifier* newts = new_parametric_type_specifier();

              newts->specifier_base_type = relativetype->specifier_base_type;
              newts->specifier_parameter_types = typeslist;
              return (newts);
            }
          }
          else {
            {
              typeslist->free();
              return (relativetype);
            }
          }
        }
      }
    }
    else if (subtype_of_anchored_type_specifierP(test_value_000)) {
      { Standard_Object* relativetype_001 = relativetype;
        Anchored_Type_Specifier* relativetype = ((Anchored_Type_Specifier*)(relativetype_001));

        return (compute_anchored_type_spec(ownertype, relativetype));
      }
    }
    else if (subtype_of_surrogateP(test_value_000)) {
      { Standard_Object* relativetype_002 = relativetype;
        Surrogate* relativetype = ((Surrogate*)(relativetype_002));

        return (relativetype);
      }
    }
    else {
      { Output_String_Stream* stream_000 = new_output_string_stream();

        *(stream_000->native_stream) << "compute-relative-type-spec: Not defined on " << "`" << relativetype << "'";
        throw *new_stella_exception(stream_000->the_string_reader());
      }
    }
  }
}

Standard_Object* Slot::compute_return_type_spec(Standard_Object* firstargtype) {
  { Slot* self = this;

    firstargtype = firstargtype;
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "compute-return-type-spec: Not defined on " << "`" << self << "'";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
}

Standard_Object* Method_Slot::compute_return_type_spec(Standard_Object* firstargtype) {
  { Method_Slot* self = this;

    if (!((boolean)(((Compound_Type_Specifier*)(dynamic_slot_value(self->dynamic_slots, SYM_METHODS_STELLA_SLOT_TYPE_SPECIFIER, NULL)))))) {
      return (self->slot_base_type);
    }
    else {
      return (compute_relative_type_spec(((Compound_Type_Specifier*)(dynamic_slot_value(self->dynamic_slots, SYM_METHODS_STELLA_SLOT_TYPE_SPECIFIER, NULL))), firstargtype));
    }
  }
}

Standard_Object* Storage_Slot::compute_return_type_spec(Standard_Object* firstargtype) {
  { Storage_Slot* self = this;

    if (!((boolean)(((Compound_Type_Specifier*)(dynamic_slot_value(self->dynamic_slots, SYM_METHODS_STELLA_SLOT_TYPE_SPECIFIER, NULL)))))) {
      return (self->type());
    }
    else {
      return (compute_relative_type_spec(((Compound_Type_Specifier*)(dynamic_slot_value(self->dynamic_slots, SYM_METHODS_STELLA_SLOT_TYPE_SPECIFIER, NULL))), firstargtype));
    }
  }
}

Object* yield_type_spec_tree(Standard_Object* self) {
  { Surrogate* test_value_000 = safe_primary_type(self);

    if (subtype_of_surrogateP(test_value_000)) {
      { Standard_Object* self_000 = self;
        Surrogate* self = ((Surrogate*)(self_000));

        return (intern_symbol_in_module(self->symbol_name, ((Module*)(self->home_context)), TRUE));
      }
    }
    else if (subtype_of_parametric_type_specifierP(test_value_000)) {
      { Standard_Object* self_001 = self;
        Parametric_Type_Specifier* self = ((Parametric_Type_Specifier*)(self_001));

        { Cons* listoftypes = NIL;

          { Standard_Object* ts = NULL;
            Cons* iter_000 = self->specifier_parameter_types->the_cons_list;
            Cons* collect_000 = NULL;

            while (!(iter_000 == NIL)) {
              ts = ((Standard_Object*)(iter_000->value));
              iter_000 = iter_000->rest;
              if (!((boolean)(collect_000))) {
                {
                  collect_000 = cons(yield_type_spec_tree(ts), NIL);
                  if (listoftypes == NIL) {
                    listoftypes = collect_000;
                  }
                  else {
                    add_cons_to_end_of_cons_list(listoftypes, collect_000);
                  }
                }
              }
              else {
                {
                  collect_000->rest = cons(yield_type_spec_tree(ts), NIL);
                  collect_000 = collect_000->rest;
                }
              }
            }
          }
          return (cons(intern_symbol_in_module(self->specifier_base_type->symbol_name, ((Module*)(self->specifier_base_type->home_context)), TRUE), ((array_type_specifierP(self) ? listO(3, SYM_METHODS_STELLA_SIZE, yield_array_dimensions_tree(self), NIL) : NIL))->concatenate(cons(SYM_METHODS_STELLA_OF, listoftypes->concatenate(NIL, 0)), 0)));
        }
      }
    }
    else if (subtype_of_anchored_type_specifierP(test_value_000)) {
      { Standard_Object* self_002 = self;
        Anchored_Type_Specifier* self = ((Anchored_Type_Specifier*)(self_002));

        if (((boolean)(self->specifier_parameter_name))) {
          return (listO(3, SYM_METHODS_STELLA_LIKE, cons(self->specifier_parameter_name, cons(SYM_METHODS_STELLA_SELF, NIL)), NIL));
        }
        else {
          return (listO(3, SYM_METHODS_STELLA_LIKE, SYM_METHODS_STELLA_SELF, NIL));
        }
      }
    }
    else {
      { Output_String_Stream* stream_000 = new_output_string_stream();

        *(stream_000->native_stream) << "validate-type-specifier: Not defined on " << "`" << self << "'";
        throw *new_stella_exception(stream_000->the_string_reader());
      }
    }
  }
}

// Lookup table for functions.
Hash_Table* oFUNCTION_LOOKUP_TABLEo = NULL;

Method_Slot* attach_function(Method_Slot* newfunction) {
  { Symbol* name = newfunction->slot_name;
    Method_Slot* oldfunction = lookup_function(name);

    if (!undefine_conflicting_definitionsP(newfunction)) {
      return (NULL);
    }
    oFUNCTION_LOOKUP_TABLEo->insert_at(name, newfunction);
    unregister_slot_name(newfunction);
    newfunction->slot_externalP = TRUE;
    newfunction->finalize_slot_type_computations();
    if (((boolean)(oldfunction))) {
      run_hooks(oREDEFINE_RELATION_HOOKSo, list(2, oldfunction, newfunction));
      oldfunction->free();
    }
    return (newfunction);
  }
}

Method_Slot* lookup_function(Symbol* functionsymbol) {
  // Return the function defined for 'functionSymbol', if it exists.
  functionsymbol = functionsymbol->soft_permanentify();
  { Standard_Object* function = ((Standard_Object*)(oFUNCTION_LOOKUP_TABLEo->lookup(functionsymbol)));

    if (((boolean)(function)) &&
        (function->primary_type() == SGT_METHODS_STELLA_VECTOR)) {
      { Vector* record = ((Vector*)(function));
        char* definition = ((String_Wrapper*)((record->the_array)[0]))->wrapper_value;
        Method_Slot* oldfunction = ((Method_Slot*)((record->the_array)[1]));
        char* modulename = ((String_Wrapper*)((record->the_array)[2]))->wrapper_value;
        cpp_function_code code = (((boolean)((record->the_array)[3])) ? ((Function_Code_Wrapper*)((record->the_array)[3]))->wrapper_value : NULL);
        cpp_function_code wrappercode = ((record->length() >= 5) ? ((Function_Code_Wrapper*)((record->the_array)[4]))->wrapper_value : NULL);

        oFUNCTION_LOOKUP_TABLEo->insert_at(functionsymbol, oldfunction);
        { 
          BIND_STELLA_SPECIAL(oMODULEo, Module*, get_stella_module(modulename, TRUE));
          BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
          function = define_method_from_stringified_source(functionsymbol->symbol_name, NULL, definition);
        }
        if (((boolean)(function))) {
          if (code != NULL) {
            ((Method_Slot*)(function))->function_code = code;
          }
          if (wrappercode != NULL) {
            set_dynamic_slot_value(((Method_Slot*)(function))->dynamic_slots, SYM_METHODS_STELLA_EVALUATOR_WRAPPER_CODE, wrap_function_code(wrappercode), NULL_FUNCTION_CODE_WRAPPER);
          }
        }
      }
    }
    return (((Method_Slot*)(function)));
  }
}

Method_Slot* lookup_function_by_name(char* name) {
  // Return a function with name 'name' visible from the current module.
  // Scan all visible symbols looking for one that has a function defined for it.
  { Method_Slot* function = NULL;
    Symbol* symbol = NULL;

    { Module* module = NULL;
      Iterator* iter_000 = visible_modules(oMODULEo.get());

      while (iter_000->nextP()) {
        module = ((Module*)(iter_000->value));
        { boolean test_value_000 = FALSE;

          { 
            symbol = lookup_symbol_in_module(name, module, TRUE);
            test_value_000 = ((boolean)(symbol));
          }
          if (test_value_000) {
            { 
              function = lookup_function(symbol);
              test_value_000 = ((boolean)(function));
            }
          }
          if (test_value_000) {
            return (function);
          }
        }
      }
    }
    return (NULL);
  }
}

Method_Slot* lookup_function_or_stella_function(Symbol* functionsymbol) {
  return (lookup_function(functionsymbol));
}

void undefine_function(Symbol* name) {
  { Method_Slot* function = lookup_function(name);

    if (((boolean)(function))) {
      destroy_function(function);
    }
  }
}

void destroy_function(Method_Slot* function) {
  { Symbol* functionname = function->slot_name;
    Surrogate* slotref = function->slot_slotref;
    Method_Slot* realfunction = lookup_function(functionname);

    if (function == realfunction) {
      oFUNCTION_LOOKUP_TABLEo->remove_at(functionname);
    }
    if (((boolean)(slotref)) &&
        (function == ((Slot*)(slotref->surrogate_value)))) {
      slotref->surrogate_value = NULL;
    }
    function->deletedP_setter(TRUE);
    function->free();
  }
}

boolean undefine_conflicting_definitionsP(Method_Slot* newslot) {
  if (newslot->method_functionP) {
    if (warn_about_function_shadowing_slotsP(newslot)) {
      if (!(y_or_nP("Do you want to remove the conflicting method definitions? "))) {
        return (FALSE);
      }
      undefine_external_slots(newslot->slot_name);
      unregister_slot_name(newslot);
    }
    if (((boolean)(lookup_macro(newslot->slot_name)))) {
      { 
        BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
        signal_translation_warning();
        if (!(suppress_warningsP())) {
          print_error_context(">> WARNING: ", STANDARD_WARNING);
          *(STANDARD_WARNING->native_stream) << endl << " " << "Function " << "`" << de_uglify_parse_tree(newslot) << "'" << " is shadowed by a pre-exisiting macro." << endl << " Removing the preexisting macro definition" << "." << endl;
        }
      }
      undefine_macro(newslot->slot_name);
    }
  }
  else {
    if (warn_about_slot_shadowed_by_a_functionP(newslot)) {
      cout << " Removing the preexisting function definition." << endl;
      undefine_function(newslot->slot_name);
    }
  }
  return (TRUE);
}

void define_function_object(char* name, char* definition, cpp_function_code code, cpp_function_code wrappercode) {
  { Object* namesymbol = ((name[0] == ' ') ? read_s_expression_from_string(name) : intern_symbol_in_module(name, NULL, FALSE));
    Standard_Object* oldfunction = ((Standard_Object*)(oFUNCTION_LOOKUP_TABLEo->lookup(((Symbol*)(namesymbol)))));
    Vector* record = new_vector(((wrappercode != NULL) ? 5 : 4));

    if (((boolean)(oldfunction)) &&
        (oldfunction->primary_type() == SGT_METHODS_STELLA_VECTOR)) {
      oldfunction = NULL;
    }
    (record->the_array)[0] = (wrap_string(definition));
    (record->the_array)[1] = oldfunction;
    (record->the_array)[2] = (wrap_string(oMODULEo.get()->module_full_name));
    if (code != NULL) {
      (record->the_array)[3] = (wrap_function_code(code));
    }
    if (wrappercode != NULL) {
      (record->the_array)[4] = (wrap_function_code(wrappercode));
    }
    oFUNCTION_LOOKUP_TABLEo->insert_at(((Symbol*)(namesymbol)), record);
  }
}

// Lookup table for global variables.
Hash_Table* oGLOBAL_VARIABLE_LOOKUP_TABLEo = NULL;

Global_Variable* intern_global_variable(Global_Variable* global) {
  { Symbol* name = global->variable_name;
    Global_Variable* oldglobal = name->lookup_global_variable();

    if (((boolean)(oldglobal))) {
      if (string_eqlP(global->variable_stringified_source, oldglobal->variable_stringified_source)) {
        global->free();
        return (oldglobal);
      }
      else {
        cout << "Redefining the global variable " << "`" << global->variable_name << "'" << endl;
        oldglobal->free();
      }
    }
    oGLOBAL_VARIABLE_LOOKUP_TABLEo->insert_at(name, global);
    return (global);
  }
}

Global_Variable* Generalized_Symbol::lookup_global_variable() {
  // Return a global variable with name 'self'.
  { Generalized_Symbol* self = this;

    return (((Global_Variable*)(oGLOBAL_VARIABLE_LOOKUP_TABLEo->lookup(((Symbol*)(self))))));
  }
}

Global_Variable* Surrogate::lookup_global_variable() {
  // Return a global variable with name 'self'.
  { Surrogate* self = this;

    { Symbol* symbol = ((Symbol*)(lookup_rigid_symbol_wrt_module(self->symbol_name, ((Module*)(self->home_context)), SYMBOL_SYM)));

      if (((boolean)(symbol))) {
        return (symbol->lookup_global_variable());
      }
      else {
        return (NULL);
      }
    }
  }
}

Global_Variable* string_lookup_global_variable(char* self) {
  // Return a global variable with name 'self'.
  { Symbol* symbol = lookup_symbol(self);

    if (((boolean)(symbol))) {
      return (symbol->lookup_global_variable());
    }
    else {
      return (NULL);
    }
  }
}

Standard_Object* global_variable_type_spec(Global_Variable* global) {
  // Return the type spec for the global variable `global'.
  { Standard_Object* type = ((Standard_Object*)(dynamic_slot_value(global->dynamic_slots, SYM_METHODS_STELLA_VARIABLE_TYPE_SPECIFIER, NULL)));

    if (((boolean)(type))) {
      return (type);
    }
    else {
      return (global->variable_type);
    }
  }
}

Standard_Object* lookup_global_variable_type(Symbol* name) {
  { Global_Variable* global = name->lookup_global_variable();

    if (((boolean)(global))) {
      return (global_variable_type_spec(global));
    }
    else {
      return (NULL);
    }
  }
}

Global_Variable* define_stella_global_variable_from_parse_tree(Cons* tree, char* stringifiedsource, Object*& _return1) {
  { Object* typetree = tree->rest->rest->value;
    Object* initialvaluetree = tree->fourth();
    Cons* optionstree = tree->nth_rest(4);
    boolean specialP = tree->value == SYM_METHODS_STELLA_DEFSPECIAL;
    boolean constantP = tree->value == SYM_METHODS_STELLA_DEFCONSTANT;
    boolean noinitialvalueP = FALSE;
    Global_Variable* global = new_global_variable();
    Global_Variable* internedglobal = NULL;

    if ((tree->length() <= 3) ||
        (get_quoted_tree("((:DOCUMENTATION :PUBLIC? :AUXILIARY?) \"/STELLA\")", "/STELLA")->memberP(initialvaluetree) &&
         ((optionstree->length()) % 2))) {
      noinitialvalueP = TRUE;
      initialvaluetree = (specialP ? KWD_METHODS_UNBOUND_SPECIAL_VARIABLE : NULL);
      optionstree = tree->nth_rest(3);
    }
    global->variable_name = ((Symbol*)(tree->rest->value->permanentify()));
    global->variable_stringified_source = stringifiedsource;
    internedglobal = intern_global_variable(global);
    record_variable_home_module(internedglobal);
    if (noinitialvalueP &&
        (!specialP)) {
      if (constantP) {
        { 
          BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
          signal_translation_error();
          if (!(suppress_warningsP())) {
            print_error_context(">> ERROR: ", STANDARD_ERROR);
            *(STANDARD_ERROR->native_stream) << endl << " " << "Missing value in constant declaration" << "." << endl;
          }
        }
      }
      else {
        { 
          BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
          signal_translation_error();
          if (!(suppress_warningsP())) {
            print_error_context(">> ERROR: ", STANDARD_ERROR);
            *(STANDARD_ERROR->native_stream) << endl << " " << "Missing initial value in variable declaration" << "." << endl;
          }
        }
      }
      _return1 = initialvaluetree;
      return (internedglobal);
    }
    if (!(global == internedglobal)) {
      _return1 = initialvaluetree;
      return (internedglobal);
    }
    typetree = typetree->yield_type_specifier();
    validate_type_specifier(((Standard_Object*)(typetree)), NULL, TRUE);
    if (typeP(typetree)) {
      global->variable_type = ((Surrogate*)(typetree));
    }
    else {
      {
        set_dynamic_slot_value(global->dynamic_slots, SYM_METHODS_STELLA_VARIABLE_TYPE_SPECIFIER, ((Standard_Object*)(typetree)), NULL);
        global->variable_type = type_spec_to_base_type(((Standard_Object*)(typetree)));
      }
    }
    global->variable_specialP = specialP;
    global->variable_constantP = constantP;
    if (!(optionstree == NIL)) {
      { Property_List* options = new_property_list();

        options->the_plist = optionstree;
        { Object* key = NULL;
          Object* value = NULL;
          Cons* iter_000 = options->the_plist;

          while (!(iter_000 == NIL)) {
            key = iter_000->value;
            value = iter_000->rest->value;
            iter_000 = iter_000->rest->rest;
            { Keyword* test_value_000 = ((Keyword*)(key));

              if (test_value_000 == KWD_METHODS_DOCUMENTATION) {
                global->documentation = ((String_Wrapper*)(value))->wrapper_value;
              }
              else if (test_value_000 == KWD_METHODS_PUBLICp) {
                global->variable_publicP = coerce_wrapped_boolean_to_boolean(coerce_to_boolean(value));
              }
              else if (test_value_000 == KWD_METHODS_AUXILIARYp) {
                global->variable_auxiliaryP = coerce_wrapped_boolean_to_boolean(coerce_to_boolean(value));
              }
              else {
                { Output_String_Stream* stream_000 = new_output_string_stream();

                  *(stream_000->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
                  throw *new_stella_exception(stream_000->the_string_reader());
                }
              }
            }
          }
        }
        options->the_plist = NULL;
      }
    }
    _return1 = initialvaluetree;
    return (global);
  }
}

void define_stella_global_variable_from_stringified_source(char* stringifiedsource) {
  { Object* dummy1;

    define_stella_global_variable_from_parse_tree(((Cons*)(unstringify_stella_source(stringifiedsource, NULL))), stringifiedsource, dummy1);
  }
}

Module* record_variable_home_module(Global_Variable* self) {
  { Symbol* name = self->variable_name;
    Module* namemodule = ((Module*)(name->home_context));
    Module* definitionmodule = oMODULEo.get();
    Module* homemodule = namemodule;

    if (explicitly_qualified_nameP(name, definitionmodule)) {
      homemodule = namemodule;
    }
    else {
      homemodule = definitionmodule;
    }
    if (!(homemodule == namemodule)) {
      set_dynamic_slot_value(self->dynamic_slots, SYM_METHODS_STELLA_VARIABLE_HOME_MODULE, homemodule, NULL);
    }
    return (homemodule);
  }
}

Object* get_global_value(Surrogate* self) {
  // Return the (possibly-wrapped) value of the global
  // variable for the surrogate 'self'.
  return (((Object*  (*) ())self->lookup_global_variable()->variable_get_value_code)());
}

Object* set_global_value(Surrogate* self, Object* value) {
  // Set the value of the global variable for the surrogate
  // 'self' to 'value'.
  return (((Object*  (*) (Object*))self->lookup_global_variable()->variable_set_value_code)(value));
}

void destroy_variable(Global_Variable* variable) {
  { Symbol* variablename = variable->variable_name;
    Global_Variable* realvariable = variablename->lookup_global_variable();

    if (variable == realvariable) {
      oGLOBAL_VARIABLE_LOOKUP_TABLEo->remove_at(variablename);
    }
    variable->deletedP_setter(TRUE);
    variable->free();
  }
}

Surrogate* define_stella_type_from_parse_tree(Cons* tree) {
  { Object* renamed_Typename = tree->rest->value;
    Standard_Object* typedefinition = tree->rest->rest->value->yield_type_specifier();
    Surrogate* type = NULL;

    if (subtype_of_symbolP(safe_primary_type(renamed_Typename))) {
      { Object* typename_000 = renamed_Typename;
        Symbol* renamed_Typename = ((Symbol*)(typename_000));

        type = typify(renamed_Typename);
        if ((!defined_typeP(type)) &&
            ((boolean)(type->surrogate_value))) {
          { 
            BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
            signal_translation_error();
            if (!(suppress_warningsP())) {
              print_error_context(">> ERROR: ", STANDARD_ERROR);
              *(STANDARD_ERROR->native_stream) << endl << " " << "Cannot define type " << "`" << de_uglify_parse_tree(renamed_Typename) << "'" << ", since " << "`" << de_uglify_parse_tree(type) << "'" << endl << "   already points to " << "`" << type->surrogate_value << "'" << "." << endl;
            }
          }
          return (NULL);
        }
        type->surrogate_value = typedefinition;
        return (type);
      }
    }
    else {
      { 
        BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
        signal_translation_error();
        if (!(suppress_warningsP())) {
          print_error_context(">> ERROR: ", STANDARD_ERROR);
          *(STANDARD_ERROR->native_stream) << endl << " " << "Illegal type name: " << "`" << de_uglify_parse_tree(renamed_Typename) << "'" << "." << endl;
        }
      }
      return (NULL);
    }
  }
}

void define_stella_type_from_stringified_source(char* stringifiedsource) {
  define_stella_type_from_parse_tree(((Cons*)(unstringify_stella_source(stringifiedsource, NULL))));
}

void help_startup_methods1() {
  {
    SGT_METHODS_STELLA_SURROGATE = ((Surrogate*)(intern_rigid_symbol_wrt_module("SURROGATE", NULL, 1)));
    SGT_METHODS_STELLA_PARAMETRIC_TYPE_SPECIFIER = ((Surrogate*)(intern_rigid_symbol_wrt_module("PARAMETRIC-TYPE-SPECIFIER", NULL, 1)));
    SGT_METHODS_STELLA_ANCHORED_TYPE_SPECIFIER = ((Surrogate*)(intern_rigid_symbol_wrt_module("ANCHORED-TYPE-SPECIFIER", NULL, 1)));
    SGT_METHODS_STELLA_OBJECT = ((Surrogate*)(intern_rigid_symbol_wrt_module("OBJECT", NULL, 1)));
    SGT_METHODS_STELLA_VOID = ((Surrogate*)(intern_rigid_symbol_wrt_module("VOID", NULL, 1)));
    SYM_METHODS_STELLA_LIKE = ((Symbol*)(intern_rigid_symbol_wrt_module("LIKE", NULL, 0)));
    SYM_METHODS_STELLA_SIZE = ((Symbol*)(intern_rigid_symbol_wrt_module("SIZE", NULL, 0)));
    SYM_METHODS_STELLA_UNKNOWN = ((Symbol*)(intern_rigid_symbol_wrt_module("UNKNOWN", NULL, 0)));
    SYM_METHODS_STELLA_OF = ((Symbol*)(intern_rigid_symbol_wrt_module("OF", NULL, 0)));
    SYM_METHODS_STELLA_NULL = ((Symbol*)(intern_rigid_symbol_wrt_module("NULL", NULL, 0)));
    SGT_METHODS_STELLA_ARRAY = ((Surrogate*)(intern_rigid_symbol_wrt_module("ARRAY", NULL, 1)));
    SYM_METHODS_STELLA_SELF = ((Symbol*)(intern_rigid_symbol_wrt_module("SELF", NULL, 0)));
    SGT_METHODS_STELLA_CONS = ((Surrogate*)(intern_rigid_symbol_wrt_module("CONS", NULL, 1)));
    SYM_METHODS_STELLA_IN = ((Symbol*)(intern_rigid_symbol_wrt_module("IN", NULL, 0)));
    SYM_METHODS_STELLA_OUT = ((Symbol*)(intern_rigid_symbol_wrt_module("OUT", NULL, 0)));
    SYM_METHODS_STELLA_INOUT = ((Symbol*)(intern_rigid_symbol_wrt_module("INOUT", NULL, 0)));
    SYM_METHODS_STELLA_METHOD_PARAMETER_DIRECTIONS = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-PARAMETER-DIRECTIONS", NULL, 0)));
    SYM_METHODS_STELLA_ARGUMENT_LIST = ((Symbol*)(intern_rigid_symbol_wrt_module("ARGUMENT-LIST", NULL, 0)));
    SYM_METHODS_STELLA_aREST = ((Symbol*)(intern_rigid_symbol_wrt_module("&REST", NULL, 0)));
    SYM_METHODS_STELLA_METHOD_VARIABLE_ARGUMENTSp = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-VARIABLE-ARGUMENTS?", NULL, 0)));
    SYM_METHODS_STELLA_aBODY = ((Symbol*)(intern_rigid_symbol_wrt_module("&BODY", NULL, 0)));
    SYM_METHODS_STELLA_METHOD_BODY_ARGUMENTp = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-BODY-ARGUMENT?", NULL, 0)));
    SGT_METHODS_STELLA_UNKNOWN = ((Surrogate*)(intern_rigid_symbol_wrt_module("UNKNOWN", NULL, 1)));
    KWD_METHODS_TYPE = ((Keyword*)(intern_rigid_symbol_wrt_module("TYPE", NULL, 2)));
    KWD_METHODS_RETURN_TYPES = ((Keyword*)(intern_rigid_symbol_wrt_module("RETURN-TYPES", NULL, 2)));
    SYM_METHODS_STELLA_SLOT_TYPE_SPECIFIER = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-TYPE-SPECIFIER", NULL, 0)));
    KWD_METHODS_PUBLICp = ((Keyword*)(intern_rigid_symbol_wrt_module("PUBLIC?", NULL, 2)));
    KWD_METHODS_ABSTRACTp = ((Keyword*)(intern_rigid_symbol_wrt_module("ABSTRACT?", NULL, 2)));
    KWD_METHODS_NATIVEp = ((Keyword*)(intern_rigid_symbol_wrt_module("NATIVE?", NULL, 2)));
    SYM_METHODS_STELLA_METHOD_NATIVEp = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-NATIVE?", NULL, 0)));
    KWD_METHODS_CONSTRUCTORp = ((Keyword*)(intern_rigid_symbol_wrt_module("CONSTRUCTOR?", NULL, 2)));
    SYM_METHODS_STELLA_METHOD_CONSTRUCTORp = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-CONSTRUCTOR?", NULL, 0)));
    KWD_METHODS_GLOBALLY_INLINEp = ((Keyword*)(intern_rigid_symbol_wrt_module("GLOBALLY-INLINE?", NULL, 2)));
    SYM_METHODS_STELLA_METHOD_GLOBALLY_INLINEp = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-GLOBALLY-INLINE?", NULL, 0)));
    KWD_METHODS_INLINE = ((Keyword*)(intern_rigid_symbol_wrt_module("INLINE", NULL, 2)));
    SYM_METHODS_STELLA_METHOD_INLINED_FUNCTIONS = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-INLINED-FUNCTIONS", NULL, 0)));
    KWD_METHODS_AUXILIARYp = ((Keyword*)(intern_rigid_symbol_wrt_module("AUXILIARY?", NULL, 2)));
    SYM_METHODS_STELLA_SLOT_AUXILIARYp = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-AUXILIARY?", NULL, 0)));
    KWD_METHODS_DOCUMENTATION = ((Keyword*)(intern_rigid_symbol_wrt_module("DOCUMENTATION", NULL, 2)));
    SYM_METHODS_STELLA_DOCUMENTATION = ((Symbol*)(intern_rigid_symbol_wrt_module("DOCUMENTATION", NULL, 0)));
    KWD_METHODS_RETURNS = ((Keyword*)(intern_rigid_symbol_wrt_module("RETURNS", NULL, 2)));
    KWD_METHODS_STORAGE_SLOT = ((Keyword*)(intern_rigid_symbol_wrt_module("STORAGE-SLOT", NULL, 2)));
    SYM_METHODS_STELLA_STORAGE_SLOT = ((Symbol*)(intern_rigid_symbol_wrt_module("STORAGE-SLOT", NULL, 0)));
    KWD_METHODS_INHERITS_THROUGH = ((Keyword*)(intern_rigid_symbol_wrt_module("INHERITS-THROUGH", NULL, 2)));
    SYM_METHODS_STELLA_METHOD_INHERITS_THROUGH = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-INHERITS-THROUGH", NULL, 0)));
    KWD_METHODS_PROPERTIES = ((Keyword*)(intern_rigid_symbol_wrt_module("PROPERTIES", NULL, 2)));
    SYM_METHODS_STELLA_PROPERTIES = ((Symbol*)(intern_rigid_symbol_wrt_module("PROPERTIES", NULL, 0)));
    KWD_METHODS_META_ATTRIBUTES = ((Keyword*)(intern_rigid_symbol_wrt_module("META-ATTRIBUTES", NULL, 2)));
    SYM_METHODS_STELLA_META_ATTRIBUTES = ((Symbol*)(intern_rigid_symbol_wrt_module("META-ATTRIBUTES", NULL, 0)));
    KWD_METHODS_COMMANDp = ((Keyword*)(intern_rigid_symbol_wrt_module("COMMAND?", NULL, 2)));
    SYM_METHODS_STELLA_METHOD_COMMANDp = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-COMMAND?", NULL, 0)));
    KWD_METHODS_LISP_MACROp = ((Keyword*)(intern_rigid_symbol_wrt_module("LISP-MACRO?", NULL, 2)));
    SYM_METHODS_STELLA_METHOD_LISP_MACROp = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-LISP-MACRO?", NULL, 0)));
    KWD_METHODS_EVALUATE_ARGUMENTSp = ((Keyword*)(intern_rigid_symbol_wrt_module("EVALUATE-ARGUMENTS?", NULL, 2)));
    SYM_METHODS_STELLA_METHOD_EVALUATE_ARGUMENTSp = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-EVALUATE-ARGUMENTS?", NULL, 0)));
    SYM_METHODS_STELLA_CONCATENATE = ((Symbol*)(intern_rigid_symbol_wrt_module("CONCATENATE", NULL, 0)));
    SYM_METHODS_STELLA_DEFINE_FUNCTION_OBJECT = ((Symbol*)(intern_rigid_symbol_wrt_module("DEFINE-FUNCTION-OBJECT", NULL, 0)));
    SYM_METHODS_STELLA_THE_CODE = ((Symbol*)(intern_rigid_symbol_wrt_module("THE-CODE", NULL, 0)));
    KWD_METHODS_FUNCTION = ((Keyword*)(intern_rigid_symbol_wrt_module("FUNCTION", NULL, 2)));
    SYM_METHODS_STELLA_DEFINE_METHOD_OBJECT = ((Symbol*)(intern_rigid_symbol_wrt_module("DEFINE-METHOD-OBJECT", NULL, 0)));
  }
}

void help_startup_methods2() {
  {
    KWD_METHODS_METHOD = ((Keyword*)(intern_rigid_symbol_wrt_module("METHOD", NULL, 2)));
    SGT_METHODS_STELLA_BOOLEAN = ((Surrogate*)(intern_rigid_symbol_wrt_module("BOOLEAN", NULL, 1)));
    SYM_METHODS_STELLA_SUPER_CLASSES = ((Symbol*)(intern_rigid_symbol_wrt_module("SUPER-CLASSES", NULL, 0)));
    SYM_METHODS_STELLA_EQUIVALENT_SLOT = ((Symbol*)(intern_rigid_symbol_wrt_module("EQUIVALENT-SLOT", NULL, 0)));
    SYM_METHODS_STELLA_CLASS_ALL_SUPER_CLASSES = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-ALL-SUPER-CLASSES", NULL, 0)));
    SYM_METHODS_STELLA_SLOT_DIRECT_EQUIVALENT = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-DIRECT-EQUIVALENT", NULL, 0)));
    SYM_METHODS_STELLA_LET = ((Symbol*)(intern_rigid_symbol_wrt_module("LET", NULL, 0)));
    SYM_METHODS_STELLA_CURSOR = ((Symbol*)(intern_rigid_symbol_wrt_module("CURSOR", NULL, 0)));
    SYM_METHODS_STELLA_VALUE = ((Symbol*)(intern_rigid_symbol_wrt_module("VALUE", NULL, 0)));
    SYM_METHODS_STELLA_SLOT_VALUE = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-VALUE", NULL, 0)));
    SYM_METHODS_STELLA_LOOP = ((Symbol*)(intern_rigid_symbol_wrt_module("LOOP", NULL, 0)));
    SYM_METHODS_STELLA_WHEN = ((Symbol*)(intern_rigid_symbol_wrt_module("WHEN", NULL, 0)));
    SYM_METHODS_STELLA_RETURN = ((Symbol*)(intern_rigid_symbol_wrt_module("RETURN", NULL, 0)));
    SYM_METHODS_STELLA_SETQ = ((Symbol*)(intern_rigid_symbol_wrt_module("SETQ", NULL, 0)));
    SYM_METHODS_STELLA_IF = ((Symbol*)(intern_rigid_symbol_wrt_module("IF", NULL, 0)));
    SYM_METHODS_STELLA_DEFINEDp = ((Symbol*)(intern_rigid_symbol_wrt_module("DEFINED?", NULL, 0)));
    SGT_METHODS_STELLA_COLLECTION = ((Surrogate*)(intern_rigid_symbol_wrt_module("COLLECTION", NULL, 1)));
    SYM_METHODS_STELLA_FIRST = ((Symbol*)(intern_rigid_symbol_wrt_module("FIRST", NULL, 0)));
    SYM_METHODS_STELLA_VARIABLE_ARITY_TABLEp = ((Symbol*)(intern_rigid_symbol_wrt_module("VARIABLE-ARITY-TABLE?", NULL, 0)));
    SGT_METHODS_STELLA_VECTOR = ((Surrogate*)(intern_rigid_symbol_wrt_module("VECTOR", NULL, 1)));
    SYM_METHODS_STELLA_EVALUATOR_WRAPPER_CODE = ((Symbol*)(intern_rigid_symbol_wrt_module("EVALUATOR-WRAPPER-CODE", NULL, 0)));
    SYM_METHODS_STELLA_VARIABLE_TYPE_SPECIFIER = ((Symbol*)(intern_rigid_symbol_wrt_module("VARIABLE-TYPE-SPECIFIER", NULL, 0)));
    SYM_METHODS_STELLA_DEFSPECIAL = ((Symbol*)(intern_rigid_symbol_wrt_module("DEFSPECIAL", NULL, 0)));
    SYM_METHODS_STELLA_DEFCONSTANT = ((Symbol*)(intern_rigid_symbol_wrt_module("DEFCONSTANT", NULL, 0)));
    KWD_METHODS_UNBOUND_SPECIAL_VARIABLE = ((Keyword*)(intern_rigid_symbol_wrt_module("UNBOUND-SPECIAL-VARIABLE", NULL, 2)));
    SYM_METHODS_STELLA_VARIABLE_HOME_MODULE = ((Symbol*)(intern_rigid_symbol_wrt_module("VARIABLE-HOME-MODULE", NULL, 0)));
    SYM_METHODS_STELLA_STARTUP_METHODS = ((Symbol*)(intern_rigid_symbol_wrt_module("STARTUP-METHODS", NULL, 0)));
    SYM_METHODS_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-STARTUP-CLASSNAME", NULL, 0)));
  }
}

void help_startup_methods3() {
  {
    define_function_object("APPLY", "(DEFUN (APPLY OBJECT) ((CODE FUNCTION-CODE) (ARGUMENTS (CONS OF OBJECT))) :DOCUMENTATION \"Apply `code' to `arguments', returning a value of type\nOBJECT.\" :PUBLIC? TRUE)", ((cpp_function_code)(&apply)), NULL);
    define_function_object("APPLY-METHOD", "(DEFUN (APPLY-METHOD OBJECT) ((CODE METHOD-CODE) (ARGUMENTS (CONS OF OBJECT))) :DOCUMENTATION \"Apply `code' to `arguments', returning a value of type\nOBJECT.\" :PUBLIC? TRUE)", ((cpp_function_code)(&apply_method)), NULL);
    define_function_object("APPLY-INTEGER-METHOD", "(DEFUN (APPLY-INTEGER-METHOD INTEGER) ((CODE METHOD-CODE) (ARGUMENTS (CONS OF OBJECT))) :DOCUMENTATION \"Apply `code' to `arguments', returning a value of type\nINTEGER.\" :PUBLIC? TRUE)", ((cpp_function_code)(&apply_integer_method)), NULL);
    define_function_object("APPLY-STRING-METHOD", "(DEFUN (APPLY-STRING-METHOD STRING) ((CODE METHOD-CODE) (ARGUMENTS (CONS OF OBJECT))) :DOCUMENTATION \"Apply `code' to `arguments', returning a value of type\nSTRING.\" :PUBLIC? TRUE)", ((cpp_function_code)(&apply_string_method)), NULL);
    define_function_object("APPLY-BOOLEAN-METHOD", "(DEFUN (APPLY-BOOLEAN-METHOD BOOLEAN) ((CODE METHOD-CODE) (ARGUMENTS (CONS OF OBJECT))) :DOCUMENTATION \"Apply `code' to `arguments', returning a value of type\nBOOLEAN.\" :PUBLIC? TRUE)", ((cpp_function_code)(&apply_boolean_method)), NULL);
    define_function_object("APPLY-FLOAT-METHOD", "(DEFUN (APPLY-FLOAT-METHOD FLOAT) ((CODE METHOD-CODE) (ARGUMENTS (CONS OF OBJECT))) :DOCUMENTATION \"Apply `code' to `arguments', returning a value of type\nFLOAT.\" :PUBLIC? TRUE)", ((cpp_function_code)(&apply_float_method)), NULL);
    define_function_object("DEFINED-TYPE?", "(DEFUN (DEFINED-TYPE? BOOLEAN) ((TYPE TYPE)) :PUBLIC? TRUE)", ((cpp_function_code)(&defined_typeP)), NULL);
    define_function_object("REAL-TYPE-SPECIFIER", "(DEFUN (REAL-TYPE-SPECIFIER TYPE-SPEC) ((TYPE TYPE)))", ((cpp_function_code)(&real_type_specifier)), NULL);
    define_function_object("CANONICAL-TYPE", "(DEFUN (CANONICAL-TYPE TYPE) ((TYPE TYPE)))", ((cpp_function_code)(&canonical_type)), NULL);
    define_method_object("(DEFMETHOD (YIELD-TYPE-SPECIFIER TYPE-SPEC) ((TYPESPECTREE TYPE)))", ((cpp_method_code)(&Surrogate::yield_type_specifier)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (YIELD-TYPE-SPECIFIER TYPE-SPEC) ((TYPESPECTREE SYMBOL)))", ((cpp_method_code)(&Symbol::yield_type_specifier)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (YIELD-TYPE-SPECIFIER TYPE-SPEC) ((TYPESPECTREE PARAMETRIC-TYPE-SPECIFIER)))", ((cpp_method_code)(&Parametric_Type_Specifier::yield_type_specifier)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (YIELD-TYPE-SPECIFIER TYPE-SPEC) ((TYPESPECTREE ANCHORED-TYPE-SPECIFIER)))", ((cpp_method_code)(&Anchored_Type_Specifier::yield_type_specifier)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (YIELD-TYPE-SPECIFIER TYPE-SPEC) ((TYPESPECTREE OBJECT)))", ((cpp_method_code)(&Object::yield_type_specifier)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (YIELD-TYPE-SPECIFIER TYPE-SPEC) ((TYPESPECTREE CONS)))", ((cpp_method_code)(&Cons::yield_type_specifier)), ((cpp_method_code)(NULL)));
    define_function_object("YIELD-PARAMETRIC-TYPE-SPECIFIER", "(DEFUN (YIELD-PARAMETRIC-TYPE-SPECIFIER PARAMETRIC-TYPE-SPECIFIER) ((TYPESPECTREE CONS)))", ((cpp_function_code)(&yield_parametric_type_specifier)), NULL);
    define_function_object("PARSE-ARRAY-DIMENSIONS-SPEC", "(DEFUN (PARSE-ARRAY-DIMENSIONS-SPEC (LIST OF INTEGER-WRAPPER)) ((DIMENSIONSSPEC OBJECT)))", ((cpp_function_code)(&parse_array_dimensions_spec)), NULL);
    define_function_object("YIELD-ARRAY-DIMENSIONS-TREE", "(DEFUN (YIELD-ARRAY-DIMENSIONS-TREE CONS) ((ARRAYTYPE PARAMETRIC-TYPE-SPECIFIER)))", ((cpp_function_code)(&yield_array_dimensions_tree)), NULL);
    define_function_object("ARRAY-TYPE-SPECIFIER?", "(DEFUN (ARRAY-TYPE-SPECIFIER? BOOLEAN) ((TYPESPEC TYPE-SPEC)))", ((cpp_function_code)(&array_type_specifierP)), NULL);
    define_function_object("ARRAY-TYPE-RANK", "(DEFUN (ARRAY-TYPE-RANK INTEGER) ((ARRAYTYPE PARAMETRIC-TYPE-SPECIFIER)))", ((cpp_function_code)(&array_type_rank)), NULL);
    define_function_object("ARRAY-TYPE-DIMENSIONS", "(DEFUN (ARRAY-TYPE-DIMENSIONS (LIST OF INTEGER-WRAPPER)) ((ARRAYTYPE PARAMETRIC-TYPE-SPECIFIER)))", ((cpp_function_code)(&array_type_dimensions)), NULL);
    define_function_object("INDEFINITE-ARRAY-TYPE?", "(DEFUN (INDEFINITE-ARRAY-TYPE? BOOLEAN) ((ARRAYTYPE PARAMETRIC-TYPE-SPECIFIER)))", ((cpp_function_code)(&indefinite_array_typeP)), NULL);
    define_function_object("YIELD-ANCHORED-TYPE-SPECIFIER", "(DEFUN (YIELD-ANCHORED-TYPE-SPECIFIER ANCHORED-TYPE-SPECIFIER) ((TYPESPECTREE CONS)))", ((cpp_function_code)(&yield_anchored_type_specifier)), NULL);
    define_function_object("INCORPORATE-INPUT-PARAMETERS", "(DEFUN INCORPORATE-INPUT-PARAMETERS ((METHOD METHOD-SLOT) (PARAMETERS CONS)))", ((cpp_function_code)(&incorporate_input_parameters)), NULL);
    define_function_object("INCORPORATE-FIRST-INPUT-PARAMETER", "(DEFUN INCORPORATE-FIRST-INPUT-PARAMETER ((METHOD METHOD-SLOT)))", ((cpp_function_code)(&incorporate_first_input_parameter)), NULL);
    define_function_object("DEFINE-STELLA-METHOD-SLOT", "(DEFUN (DEFINE-STELLA-METHOD-SLOT METHOD-SLOT) ((INPUTNAME SYMBOL) (RETURNTYPES CONS) (FUNCTION? BOOLEAN) (INPUTPARAMETERS CONS) (OPTIONS KEYWORD-KEY-VALUE-LIST)) :DOCUMENTATION \"Define a new Stella method object (a slot), and attach it\nto the class identified by the first parameter in 'inputParameters'.\")", ((cpp_function_code)(&define_stella_method_slot)), NULL);
    define_function_object("ATTACH-METHOD-SLOT-TO-OWNER", "(DEFUN (ATTACH-METHOD-SLOT-TO-OWNER METHOD-SLOT) ((NEWMETHOD METHOD-SLOT)))", ((cpp_function_code)(&attach_method_slot_to_owner)), NULL);
    define_function_object("YIELD-STRING-CONSTANT-TREE", "(DEFUN (YIELD-STRING-CONSTANT-TREE OBJECT) ((STRING STRING)))", ((cpp_function_code)(&yield_string_constant_tree)), NULL);
    define_function_object("YIELD-DEFINE-STELLA-METHOD-OBJECT", "(DEFUN (YIELD-DEFINE-STELLA-METHOD-OBJECT CONS) ((METHOD METHOD-SLOT) (CODEMETHOD METHOD-SLOT) (WRAPPERMETHOD METHOD-SLOT)))", ((cpp_function_code)(&yield_define_stella_method_object)), NULL);
    define_function_object("YIELD-CLASS-PARAMETER-TYPES", "(DEFUN (YIELD-CLASS-PARAMETER-TYPES (CONS OF TYPE)) ((CLASS CLASS)))", ((cpp_function_code)(&yield_class_parameter_types)), NULL);
    define_function_object("OPTIMISTIC-SUBTYPE-OF?", "(DEFUN (OPTIMISTIC-SUBTYPE-OF? BOOLEAN) ((SUBTYPE TYPE) (SUPERTYPE TYPE)))", ((cpp_function_code)(&optimistic_subtype_ofP)), NULL);
    define_function_object("WARN-OF-PARAMETER-MISMATCH", "(DEFUN WARN-OF-PARAMETER-MISMATCH ((SELF PARAMETRIC-TYPE-SPECIFIER) (MESSAGE STRING)))", ((cpp_function_code)(&warn_of_parameter_mismatch)), NULL);
    define_function_object("VALIDATE-TYPE-SPECIFIER", "(DEFUN (VALIDATE-TYPE-SPECIFIER TYPE) ((SELF TYPE-SPEC) (ANCHORCLASS CLASS) (SKIPANCHOREDTYPE? BOOLEAN)))", ((cpp_function_code)(&validate_type_specifier)), NULL);
    define_function_object("SYNTHESIZE-METHOD-BODY", "(DEFUN (SYNTHESIZE-METHOD-BODY CONS) ((METHOD METHOD-SLOT)))", ((cpp_function_code)(&synthesize_method_body)), NULL);
    define_method_object("(DEFMETHOD FINALIZE-SLOT-TYPE-COMPUTATIONS ((SELF METHOD-SLOT)))", ((cpp_method_code)(&Method_Slot::finalize_slot_type_computations)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD HELP-FINALIZE-LOCAL-SLOT ((SELF METHOD-SLOT)))", ((cpp_method_code)(&Method_Slot::help_finalize_local_slot)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD UNFINALIZE-LOCAL-SLOT ((SELF METHOD-SLOT)))", ((cpp_method_code)(&Method_Slot::unfinalize_local_slot)), ((cpp_method_code)(NULL)));
    define_function_object("UNDEFINE-EXTERNAL-SLOTS", "(DEFUN UNDEFINE-EXTERNAL-SLOTS ((NAME SYMBOL)))", ((cpp_function_code)(&undefine_external_slots)), NULL);
    define_function_object("DESTROY-EXTERNAL-SLOT", "(DEFUN DESTROY-EXTERNAL-SLOT ((SELF SLOT)))", ((cpp_function_code)(&destroy_external_slot)), NULL);
    define_function_object("DESTROY-METHOD", "(DEFUN DESTROY-METHOD ((SELF METHOD-SLOT)))", ((cpp_function_code)(&destroy_method)), NULL);
    define_method_object("(DEFMETHOD (METHOD-ARGUMENT-COUNT INTEGER) ((SELF SLOT)))", ((cpp_method_code)(&Slot::method_argument_count)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (METHOD-ARGUMENT-COUNT INTEGER) ((SELF STORAGE-SLOT)))", ((cpp_method_code)(&Storage_Slot::method_argument_count)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (METHOD-ARGUMENT-COUNT INTEGER) ((SELF METHOD-SLOT)))", ((cpp_method_code)(&Method_Slot::method_argument_count)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (METHOD-ARGUMENT-COUNT INTEGER) ((SELF TABLE)))", ((cpp_method_code)(&Table::method_argument_count)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (METHOD-VARIABLE-ARGUMENTS? BOOLEAN) ((SELF STORAGE-SLOT)))", ((cpp_method_code)(&Storage_Slot::method_variable_argumentsP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (METHOD-BODY-ARGUMENT? BOOLEAN) ((SELF STORAGE-SLOT)))", ((cpp_method_code)(&Storage_Slot::method_body_argumentP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (ARITY INTEGER) ((SELF STORAGE-SLOT)) :PUBLIC? TRUE)", ((cpp_method_code)(&Storage_Slot::arity)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (ARITY INTEGER) ((SELF METHOD-SLOT)) :PUBLIC? TRUE)", ((cpp_method_code)(&Method_Slot::arity)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (ARITY INTEGER) ((SELF TABLE)) :PUBLIC? TRUE)", ((cpp_method_code)(&Table::arity)), ((cpp_method_code)(NULL)));
    define_function_object("TYPE-SPEC-TO-BASE-TYPE", "(DEFUN (TYPE-SPEC-TO-BASE-TYPE TYPE) ((SELF TYPE-SPEC)) :PUBLIC? TRUE)", ((cpp_function_code)(&type_spec_to_base_type)), NULL);
    define_function_object("BASE-TYPE-TO-TYPE-SPEC", "(DEFUN (BASE-TYPE-TO-TYPE-SPEC TYPE-SPEC) ((SELF TYPE-SPEC)))", ((cpp_function_code)(&base_type_to_type_spec)), NULL);
    define_function_object("TYPE-SPEC-TO-CLASS", "(DEFUN (TYPE-SPEC-TO-CLASS CLASS) ((SELF TYPE-SPEC)) :PUBLIC? TRUE)", ((cpp_function_code)(&type_spec_to_class)), NULL);
    define_function_object("COMPATIBLE-PARAMETER-TYPES?", "(DEFUN (COMPATIBLE-PARAMETER-TYPES? BOOLEAN) ((SUBTYPE TYPE-SPEC) (SUPERTYPE TYPE-SPEC)) :PUBLIC? TRUE)", ((cpp_function_code)(&compatible_parameter_typesP)), NULL);
    define_function_object("SUB-TYPE-SPEC-OF?", "(DEFUN (SUB-TYPE-SPEC-OF? BOOLEAN) ((SUBTYPE TYPE-SPEC) (SUPERTYPE TYPE-SPEC)))", ((cpp_function_code)(&sub_type_spec_ofP)), NULL);
    define_function_object("VOID?", "(DEFUN (VOID? BOOLEAN) ((TYPE TYPE-SPEC)) :PUBLIC? TRUE)", ((cpp_function_code)(&voidP)), NULL);
    define_function_object("EXTRACT-PARAMETER-TYPE", "(DEFUN (EXTRACT-PARAMETER-TYPE TYPE-SPEC BOOLEAN) ((SELF TYPE-SPEC) (PARAMETER SYMBOL)) :PUBLIC? TRUE)", ((cpp_function_code)(&extract_parameter_type)), NULL);
    define_function_object("COMPUTE-ANCHORED-TYPE-SPEC", "(DEFUN (COMPUTE-ANCHORED-TYPE-SPEC TYPE-SPEC) ((OWNERTYPE TYPE-SPEC) (RELTYPE ANCHORED-TYPE-SPECIFIER)))", ((cpp_function_code)(&compute_anchored_type_spec)), NULL);
    define_function_object("COMPUTE-RELATIVE-TYPE-SPEC", "(DEFUN (COMPUTE-RELATIVE-TYPE-SPEC TYPE-SPEC) ((RELATIVETYPE TYPE-SPEC) (OWNERTYPE TYPE-SPEC)))", ((cpp_function_code)(&compute_relative_type_spec)), NULL);
    define_method_object("(DEFMETHOD (COMPUTE-RETURN-TYPE-SPEC TYPE-SPEC) ((SELF SLOT) (FIRSTARGTYPE TYPE-SPEC)))", ((cpp_method_code)(&Slot::compute_return_type_spec)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (COMPUTE-RETURN-TYPE-SPEC TYPE-SPEC) ((SELF METHOD-SLOT) (FIRSTARGTYPE TYPE-SPEC)))", ((cpp_method_code)(&Method_Slot::compute_return_type_spec)), ((cpp_method_code)(NULL)));
  }
}

void startup_methods() {
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (current_startup_time_phaseP(2)) {
      help_startup_methods1();
      help_startup_methods2();
    }
    if (current_startup_time_phaseP(4)) {
      oFUNCTION_LOOKUP_TABLEo = new_hash_table();
      oGLOBAL_VARIABLE_LOOKUP_TABLEo = new_hash_table();
    }
    if (current_startup_time_phaseP(6)) {
      finalize_classes();
    }
    if (current_startup_time_phaseP(7)) {
      help_startup_methods3();
      define_method_object("(DEFMETHOD (COMPUTE-RETURN-TYPE-SPEC TYPE-SPEC) ((SELF STORAGE-SLOT) (FIRSTARGTYPE TYPE-SPEC)))", ((cpp_method_code)(&Storage_Slot::compute_return_type_spec)), ((cpp_method_code)(NULL)));
      define_function_object("YIELD-TYPE-SPEC-TREE", "(DEFUN (YIELD-TYPE-SPEC-TREE OBJECT) ((SELF TYPE-SPEC)))", ((cpp_function_code)(&yield_type_spec_tree)), NULL);
      define_function_object("ATTACH-FUNCTION", "(DEFUN (ATTACH-FUNCTION METHOD-SLOT) ((NEWFUNCTION METHOD-SLOT)))", ((cpp_function_code)(&attach_function)), NULL);
      define_function_object("LOOKUP-FUNCTION", "(DEFUN (LOOKUP-FUNCTION FUNCTION) ((FUNCTIONSYMBOL SYMBOL)) :DOCUMENTATION \"Return the function defined for 'functionSymbol', if it exists.\" :PUBLIC? TRUE)", ((cpp_function_code)(&lookup_function)), NULL);
      define_function_object("LOOKUP-FUNCTION-BY-NAME", "(DEFUN (LOOKUP-FUNCTION-BY-NAME FUNCTION) ((NAME STRING)) :DOCUMENTATION \"Return a function with name 'name' visible from the current module.\nScan all visible symbols looking for one that has a function defined for it.\" :PUBLIC? TRUE)", ((cpp_function_code)(&lookup_function_by_name)), NULL);
      define_function_object("LOOKUP-FUNCTION-OR-STELLA-FUNCTION", "(DEFUN (LOOKUP-FUNCTION-OR-STELLA-FUNCTION FUNCTION) ((FUNCTIONSYMBOL SYMBOL)) :GLOBALLY-INLINE? TRUE (RETURN (LOOKUP-FUNCTION FUNCTIONSYMBOL)))", ((cpp_function_code)(&lookup_function_or_stella_function)), NULL);
      define_function_object("UNDEFINE-FUNCTION", "(DEFUN UNDEFINE-FUNCTION ((NAME SYMBOL)))", ((cpp_function_code)(&undefine_function)), NULL);
      define_function_object("DESTROY-FUNCTION", "(DEFUN DESTROY-FUNCTION ((FUNCTION METHOD-SLOT)))", ((cpp_function_code)(&destroy_function)), NULL);
      define_function_object("UNDEFINE-CONFLICTING-DEFINITIONS?", "(DEFUN (UNDEFINE-CONFLICTING-DEFINITIONS? BOOLEAN) ((NEWSLOT METHOD-SLOT)))", ((cpp_function_code)(&undefine_conflicting_definitionsP)), NULL);
      define_function_object("DEFINE-FUNCTION-OBJECT", "(DEFUN DEFINE-FUNCTION-OBJECT ((NAME STRING) (DEFINITION STRING) (CODE FUNCTION-CODE) (WRAPPERCODE FUNCTION-CODE)))", ((cpp_function_code)(&define_function_object)), NULL);
      define_function_object("INTERN-GLOBAL-VARIABLE", "(DEFUN (INTERN-GLOBAL-VARIABLE GLOBAL-VARIABLE) ((GLOBAL GLOBAL-VARIABLE)))", ((cpp_function_code)(&intern_global_variable)), NULL);
      define_method_object("(DEFMETHOD (LOOKUP-GLOBAL-VARIABLE GLOBAL-VARIABLE) ((SELF GENERALIZED-SYMBOL)) :PUBLIC? TRUE :DOCUMENTATION \"Return a global variable with name 'self'.\")", ((cpp_method_code)(&Generalized_Symbol::lookup_global_variable)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (LOOKUP-GLOBAL-VARIABLE GLOBAL-VARIABLE) ((SELF SURROGATE)) :PUBLIC? TRUE :DOCUMENTATION \"Return a global variable with name 'self'.\")", ((cpp_method_code)(&Surrogate::lookup_global_variable)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (LOOKUP-GLOBAL-VARIABLE GLOBAL-VARIABLE) ((SELF STRING)) :PUBLIC? TRUE :DOCUMENTATION \"Return a global variable with name 'self'.\")", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
      define_function_object("GLOBAL-VARIABLE-TYPE-SPEC", "(DEFUN (GLOBAL-VARIABLE-TYPE-SPEC TYPE-SPEC) ((GLOBAL GLOBAL-VARIABLE)) :DOCUMENTATION \"Return the type spec for the global variable `global'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&global_variable_type_spec)), NULL);
      define_function_object("LOOKUP-GLOBAL-VARIABLE-TYPE", "(DEFUN (LOOKUP-GLOBAL-VARIABLE-TYPE TYPE-SPEC) ((NAME SYMBOL)) :PUBLIC? TRUE)", ((cpp_function_code)(&lookup_global_variable_type)), NULL);
      define_function_object("DEFINE-STELLA-GLOBAL-VARIABLE-FROM-PARSE-TREE", "(DEFUN (DEFINE-STELLA-GLOBAL-VARIABLE-FROM-PARSE-TREE GLOBAL-VARIABLE OBJECT) ((TREE CONS) (STRINGIFIEDSOURCE STRING)))", ((cpp_function_code)(&define_stella_global_variable_from_parse_tree)), NULL);
      define_function_object("DEFINE-STELLA-GLOBAL-VARIABLE-FROM-STRINGIFIED-SOURCE", "(DEFUN DEFINE-STELLA-GLOBAL-VARIABLE-FROM-STRINGIFIED-SOURCE ((STRINGIFIEDSOURCE STRING)) :PUBLIC? TRUE)", ((cpp_function_code)(&define_stella_global_variable_from_stringified_source)), NULL);
      define_function_object("RECORD-VARIABLE-HOME-MODULE", "(DEFUN (RECORD-VARIABLE-HOME-MODULE MODULE) ((SELF GLOBAL-VARIABLE)))", ((cpp_function_code)(&record_variable_home_module)), NULL);
      define_function_object("GET-GLOBAL-VALUE", "(DEFUN (GET-GLOBAL-VALUE OBJECT) ((SELF SURROGATE)) :DOCUMENTATION \"Return the (possibly-wrapped) value of the global\nvariable for the surrogate 'self'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&get_global_value)), NULL);
      define_function_object("SET-GLOBAL-VALUE", "(DEFUN (SET-GLOBAL-VALUE OBJECT) ((SELF SURROGATE) (VALUE OBJECT)) :DOCUMENTATION \"Set the value of the global variable for the surrogate\n'self' to 'value'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&set_global_value)), NULL);
      define_function_object("DESTROY-VARIABLE", "(DEFUN DESTROY-VARIABLE ((VARIABLE GLOBAL-VARIABLE)))", ((cpp_function_code)(&destroy_variable)), NULL);
      define_function_object("DEFINE-STELLA-TYPE-FROM-PARSE-TREE", "(DEFUN (DEFINE-STELLA-TYPE-FROM-PARSE-TREE TYPE) ((TREE CONS)) :PUBLIC? TRUE)", ((cpp_function_code)(&define_stella_type_from_parse_tree)), NULL);
      define_function_object("DEFINE-STELLA-TYPE-FROM-STRINGIFIED-SOURCE", "(DEFUN DEFINE-STELLA-TYPE-FROM-STRINGIFIED-SOURCE ((STRINGIFIEDSOURCE STRING)) :PUBLIC? TRUE)", ((cpp_function_code)(&define_stella_type_from_stringified_source)), NULL);
      define_function_object("STARTUP-METHODS", "(DEFUN STARTUP-METHODS () :PUBLIC? TRUE)", ((cpp_function_code)(&startup_methods)), NULL);
      { Method_Slot* function = lookup_function(SYM_METHODS_STELLA_STARTUP_METHODS);

        set_dynamic_slot_value(function->dynamic_slots, SYM_METHODS_STELLA_METHOD_STARTUP_CLASSNAME, wrap_string("Startup-Methods"), NULL_STRING_WRAPPER);
      }
    }
    if (current_startup_time_phaseP(8)) {
      finalize_slots();
      cleanup_unfinalized_classes();
    }
    if (current_startup_time_phaseP(9)) {
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *MAXIMUM-STRING-CONSTANT-SIZE* INTEGER 4000 :DOCUMENTATION \"Maximum size for a string constant in the target language.\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *FUNCTION-LOOKUP-TABLE* (HASH-TABLE OF SYMBOL STANDARD-OBJECT) (NEW (HASH-TABLE OF SYMBOL FUNCTION)) :DOCUMENTATION \"Lookup table for functions.\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *GLOBAL-VARIABLE-LOOKUP-TABLE* (HASH-TABLE OF SYMBOL GLOBAL-VARIABLE) (NEW (HASH-TABLE OF SYMBOL GLOBAL-VARIABLE)) :DOCUMENTATION \"Lookup table for global variables.\" :PUBLIC? TRUE)");
    }
  }
}

Surrogate* SGT_METHODS_STELLA_SURROGATE = NULL;

Surrogate* SGT_METHODS_STELLA_PARAMETRIC_TYPE_SPECIFIER = NULL;

Surrogate* SGT_METHODS_STELLA_ANCHORED_TYPE_SPECIFIER = NULL;

Surrogate* SGT_METHODS_STELLA_OBJECT = NULL;

Surrogate* SGT_METHODS_STELLA_VOID = NULL;

Symbol* SYM_METHODS_STELLA_LIKE = NULL;

Symbol* SYM_METHODS_STELLA_SIZE = NULL;

Symbol* SYM_METHODS_STELLA_UNKNOWN = NULL;

Symbol* SYM_METHODS_STELLA_OF = NULL;

Symbol* SYM_METHODS_STELLA_NULL = NULL;

Surrogate* SGT_METHODS_STELLA_ARRAY = NULL;

Symbol* SYM_METHODS_STELLA_SELF = NULL;

Surrogate* SGT_METHODS_STELLA_CONS = NULL;

Symbol* SYM_METHODS_STELLA_IN = NULL;

Symbol* SYM_METHODS_STELLA_OUT = NULL;

Symbol* SYM_METHODS_STELLA_INOUT = NULL;

Symbol* SYM_METHODS_STELLA_METHOD_PARAMETER_DIRECTIONS = NULL;

Symbol* SYM_METHODS_STELLA_ARGUMENT_LIST = NULL;

Symbol* SYM_METHODS_STELLA_aREST = NULL;

Symbol* SYM_METHODS_STELLA_METHOD_VARIABLE_ARGUMENTSp = NULL;

Symbol* SYM_METHODS_STELLA_aBODY = NULL;

Symbol* SYM_METHODS_STELLA_METHOD_BODY_ARGUMENTp = NULL;

Surrogate* SGT_METHODS_STELLA_UNKNOWN = NULL;

Keyword* KWD_METHODS_TYPE = NULL;

Keyword* KWD_METHODS_RETURN_TYPES = NULL;

Symbol* SYM_METHODS_STELLA_SLOT_TYPE_SPECIFIER = NULL;

Keyword* KWD_METHODS_PUBLICp = NULL;

Keyword* KWD_METHODS_ABSTRACTp = NULL;

Keyword* KWD_METHODS_NATIVEp = NULL;

Symbol* SYM_METHODS_STELLA_METHOD_NATIVEp = NULL;

Keyword* KWD_METHODS_CONSTRUCTORp = NULL;

Symbol* SYM_METHODS_STELLA_METHOD_CONSTRUCTORp = NULL;

Keyword* KWD_METHODS_GLOBALLY_INLINEp = NULL;

Symbol* SYM_METHODS_STELLA_METHOD_GLOBALLY_INLINEp = NULL;

Keyword* KWD_METHODS_INLINE = NULL;

Symbol* SYM_METHODS_STELLA_METHOD_INLINED_FUNCTIONS = NULL;

Keyword* KWD_METHODS_AUXILIARYp = NULL;

Symbol* SYM_METHODS_STELLA_SLOT_AUXILIARYp = NULL;

Keyword* KWD_METHODS_DOCUMENTATION = NULL;

Symbol* SYM_METHODS_STELLA_DOCUMENTATION = NULL;

Keyword* KWD_METHODS_RETURNS = NULL;

Keyword* KWD_METHODS_STORAGE_SLOT = NULL;

Symbol* SYM_METHODS_STELLA_STORAGE_SLOT = NULL;

Keyword* KWD_METHODS_INHERITS_THROUGH = NULL;

Symbol* SYM_METHODS_STELLA_METHOD_INHERITS_THROUGH = NULL;

Keyword* KWD_METHODS_PROPERTIES = NULL;

Symbol* SYM_METHODS_STELLA_PROPERTIES = NULL;

Keyword* KWD_METHODS_META_ATTRIBUTES = NULL;

Symbol* SYM_METHODS_STELLA_META_ATTRIBUTES = NULL;

Keyword* KWD_METHODS_COMMANDp = NULL;

Symbol* SYM_METHODS_STELLA_METHOD_COMMANDp = NULL;

Keyword* KWD_METHODS_LISP_MACROp = NULL;

Symbol* SYM_METHODS_STELLA_METHOD_LISP_MACROp = NULL;

Keyword* KWD_METHODS_EVALUATE_ARGUMENTSp = NULL;

Symbol* SYM_METHODS_STELLA_METHOD_EVALUATE_ARGUMENTSp = NULL;

Symbol* SYM_METHODS_STELLA_CONCATENATE = NULL;

Symbol* SYM_METHODS_STELLA_DEFINE_FUNCTION_OBJECT = NULL;

Symbol* SYM_METHODS_STELLA_THE_CODE = NULL;

Keyword* KWD_METHODS_FUNCTION = NULL;

Symbol* SYM_METHODS_STELLA_DEFINE_METHOD_OBJECT = NULL;

Keyword* KWD_METHODS_METHOD = NULL;

Surrogate* SGT_METHODS_STELLA_BOOLEAN = NULL;

Symbol* SYM_METHODS_STELLA_SUPER_CLASSES = NULL;

Symbol* SYM_METHODS_STELLA_EQUIVALENT_SLOT = NULL;

Symbol* SYM_METHODS_STELLA_CLASS_ALL_SUPER_CLASSES = NULL;

Symbol* SYM_METHODS_STELLA_SLOT_DIRECT_EQUIVALENT = NULL;

Symbol* SYM_METHODS_STELLA_LET = NULL;

Symbol* SYM_METHODS_STELLA_CURSOR = NULL;

Symbol* SYM_METHODS_STELLA_VALUE = NULL;

Symbol* SYM_METHODS_STELLA_SLOT_VALUE = NULL;

Symbol* SYM_METHODS_STELLA_LOOP = NULL;

Symbol* SYM_METHODS_STELLA_WHEN = NULL;

Symbol* SYM_METHODS_STELLA_RETURN = NULL;

Symbol* SYM_METHODS_STELLA_SETQ = NULL;

Symbol* SYM_METHODS_STELLA_IF = NULL;

Symbol* SYM_METHODS_STELLA_DEFINEDp = NULL;

Surrogate* SGT_METHODS_STELLA_COLLECTION = NULL;

Symbol* SYM_METHODS_STELLA_FIRST = NULL;

Symbol* SYM_METHODS_STELLA_VARIABLE_ARITY_TABLEp = NULL;

Surrogate* SGT_METHODS_STELLA_VECTOR = NULL;

Symbol* SYM_METHODS_STELLA_EVALUATOR_WRAPPER_CODE = NULL;

Symbol* SYM_METHODS_STELLA_VARIABLE_TYPE_SPECIFIER = NULL;

Symbol* SYM_METHODS_STELLA_DEFSPECIAL = NULL;

Symbol* SYM_METHODS_STELLA_DEFCONSTANT = NULL;

Keyword* KWD_METHODS_UNBOUND_SPECIAL_VARIABLE = NULL;

Symbol* SYM_METHODS_STELLA_VARIABLE_HOME_MODULE = NULL;

Symbol* SYM_METHODS_STELLA_STARTUP_METHODS = NULL;

Symbol* SYM_METHODS_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella
