// stella-in.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

// If `true', always redefine classes, even if an
//                   identical class exists
boolean oREDEFINE_IDENTICAL_CLASSESpo = FALSE;

Object* permanentify_form(Object* form) {
  { Surrogate* test_value_000 = safe_primary_type(form);

    if (subtype_of_symbolP(test_value_000)) {
      { Object* form_000 = form;
        Symbol* form = ((Symbol*)(form_000));

        if (form == SYM_STELLA_IN_STELLA_TRUE) {
          return (form);
        }
        else if (form == SYM_STELLA_IN_STELLA_FALSE) {
          return (form);
        }
        else {
          return (((Symbol*)(form->permanentify())));
        }
      }
    }
    else if (test_value_000 == SGT_STELLA_IN_STELLA_CONS) {
      { Object* form_001 = form;
        Cons* form = ((Cons*)(form_001));

        return (permanent_copy(form));
      }
    }
    else {
      return (form->permanentify());
    }
  }
}

Object* transientify_form(Object* form) {
  return (copy_cons_tree(form));
}

char* stringify_in_module(Object* tree, Module* module) {
  // Stringify a parse `tree' relative to `module', or
  // `*module*' if no module is specified.
  if (!((boolean)(module))) {
    module = oMODULEo.get();
  }
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, module);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    return (stringify(tree));
  }
}

Object* unstringify_in_module(char* string, Module* module) {
  // Unstringify relative to `module', or `*MODULE*' if no
  // module is specified.
  if (!((boolean)(module))) {
    module = oMODULEo.get();
  }
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, module);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    return (read_s_expression_from_string(string));
  }
}

Object* unstringify_stella_source(char* source, Module* module) {
  // Unstringify a STELLA `source' string relative to `module',
  // or `*MODULE*' if no module is specified.  This function allocates transient
  // objects as opposed to `unstringify-in-module' or the regular `unstringify'.
  { Object* result = NULL;

    { 
      BIND_STELLA_SPECIAL(oTRANSIENTOBJECTSpo, boolean, TRUE);
      result = unstringify_in_module(source, module);
    }
    return (result);
  }
}

List* collect_feature_list(Object* tree) {
  if (safe_primary_type(tree) == SGT_STELLA_IN_STELLA_CONS) {
    { Object* tree_000 = tree;
      Cons* tree = ((Cons*)(tree_000));

      { List* result = NULL;

        if (tree->length() > 0) {
          result = new_list();
        }
        { Object* feature = NULL;
          Cons* iter_000 = tree;
          Cons* collect_000 = NULL;

          while (!(iter_000 == NIL)) {
            feature = iter_000->value;
            iter_000 = iter_000->rest;
            if (!isaP(feature, SGT_STELLA_IN_STELLA_GENERALIZED_SYMBOL)) {
              { 
                BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
                signal_translation_error();
                if (!(suppress_warningsP())) {
                  print_error_context(">> ERROR: ", STANDARD_ERROR);
                  *(STANDARD_ERROR->native_stream) << endl << " " << "Ignoring illegal feature " << "`" << de_uglify_parse_tree(feature) << "'" << " in feature list: " << "`" << de_uglify_parse_tree(tree) << "'" << "." << endl;
                }
              }
              continue;
            }
            if (!((boolean)(collect_000))) {
              {
                collect_000 = cons(feature->permanentify(), NIL);
                if (result->the_cons_list == NIL) {
                  result->the_cons_list = collect_000;
                }
                else {
                  add_cons_to_end_of_cons_list(result->the_cons_list, collect_000);
                }
              }
            }
            else {
              {
                collect_000->rest = cons(feature->permanentify(), NIL);
                collect_000 = collect_000->rest;
              }
            }
          }
        }
        return (result);
      }
    }
  }
  else {
    { 
      BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
      signal_translation_error();
      if (!(suppress_warningsP())) {
        print_error_context(">> ERROR: ", STANDARD_ERROR);
        *(STANDARD_ERROR->native_stream) << endl << " " << "Illegal atomic feature list: " << "`" << de_uglify_parse_tree(tree) << "'" << "." << endl;
      }
    }
    return (NULL);
  }
}

Key_Value_List* collect_key_value_list(Object* tree) {
  if (safe_primary_type(tree) == SGT_STELLA_IN_STELLA_CONS) {
    { Object* tree_000 = tree;
      Cons* tree = ((Cons*)(tree_000));

      { int length = tree->length();
        Cons* cursor = tree;
        Object* key = NULL;
        Object* value = NULL;
        Key_Value_List* result = NULL;

        if (length == 0) {
          return (NULL);
        }
        if ((length % 2)) {
          { 
            BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
            signal_translation_error();
            if (!(suppress_warningsP())) {
              print_error_context(">> ERROR: ", STANDARD_ERROR);
              *(STANDARD_ERROR->native_stream) << endl << " " << "Unbalanced keys and values in key-value list: " << "`" << de_uglify_parse_tree(tree) << "'" << "." << endl;
            }
          }
          return (NULL);
        }
        result = new_key_value_list();
        while (!(cursor == NIL)) {
          key = cursor->value;
          value = cursor->rest->value;
          cursor = cursor->rest->rest;
          if (!isaP(key, SGT_STELLA_IN_STELLA_GENERALIZED_SYMBOL)) {
            { 
              BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
              signal_translation_error();
              if (!(suppress_warningsP())) {
                print_error_context(">> ERROR: ", STANDARD_ERROR);
                *(STANDARD_ERROR->native_stream) << endl << " " << "Ignoring illegal key " << "`" << de_uglify_parse_tree(key) << "'" << " in key-value list: " << "`" << de_uglify_parse_tree(tree) << "'" << "." << endl;
              }
            }
            continue;
          }
          if ((value == SYM_STELLA_IN_STELLA_TRUE) ||
              (value == SYM_STELLA_IN_STELLA_FALSE)) {
            value = coerce_to_boolean(value);
          }
          else {
            value = permanent_copy(value);
          }
          result->insert_at(key->permanentify(), value);
        }
        return (result);
      }
    }
  }
  else {
    { 
      BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
      signal_translation_error();
      if (!(suppress_warningsP())) {
        print_error_context(">> ERROR: ", STANDARD_ERROR);
        *(STANDARD_ERROR->native_stream) << endl << " " << "Illegal atomic key-value list: " << "`" << de_uglify_parse_tree(tree) << "'" << "." << endl;
      }
    }
    return (NULL);
  }
}

Method_Slot* define_inline_method(Symbol* name, Cons* parametersandbody) {
  { boolean dummy1;

    { Cons* parsetree = NULL;
      Method_Slot* method = NULL;

      parsetree = listO(3, SYM_STELLA_IN_STELLA_DEFMETHOD, name, parametersandbody->concatenate(NIL, 0));
      method = help_define_method_from_parse_tree(parsetree, stringify(parsetree), dummy1);
      return (method);
    }
  }
}

Class* help_define_class_from_parse_tree(Cons* parsetree, char* stringifiedsource) {
  { Cons* treedefinition = parsetree->rest;
    Cons* thesupers = ((Cons*)(treedefinition->rest->value));
    Property_List* theoptions = new_property_list();
    Surrogate* classtype = typify(treedefinition->value);
    List* directsupers = new_list();
    List* slots = new_list();
    Keyword_Key_Value_List* classoptions = new_keyword_key_value_list();
    Keyword* key = NULL;
    List* parameters = NULL;
    Cons* slottree = NULL;

    theoptions->the_plist = treedefinition->rest->rest;
    { Object* s = NULL;
      Cons* iter_000 = thesupers;
      Cons* collect_000 = NULL;

      while (!(iter_000 == NIL)) {
        s = iter_000->value;
        iter_000 = iter_000->rest;
        if (!((boolean)(collect_000))) {
          {
            collect_000 = cons(typify(s), NIL);
            if (directsupers->the_cons_list == NIL) {
              directsupers->the_cons_list = collect_000;
            }
            else {
              add_cons_to_end_of_cons_list(directsupers->the_cons_list, collect_000);
            }
          }
        }
        else {
          {
            collect_000->rest = cons(typify(s), NIL);
            collect_000 = collect_000->rest;
          }
        }
      }
    }
    { Object* thekey = NULL;
      Object* thevalue = NULL;
      Cons* iter_001 = theoptions->the_plist;

      while (!(iter_001 == NIL)) {
        thekey = iter_001->value;
        thevalue = iter_001->rest->value;
        iter_001 = iter_001->rest->rest;
        key = ((Keyword*)(thekey->permanentify()));
        if ((key == KWD_STELLA_IN_PUBLIC_SLOTS) ||
            (key == KWD_STELLA_IN_SLOTS)) {
          { Slot* localslot = NULL;

            { Object* slotdef = NULL;
              Cons* iter_002 = ((Cons*)(thevalue));
              Cons* collect_001 = NULL;

              while (!(iter_002 == NIL)) {
                slotdef = iter_002->value;
                iter_002 = iter_002->rest;
                if (!consP(slotdef)) {
                  { 
                    BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
                    signal_translation_warning();
                    if (!(suppress_warningsP())) {
                      print_error_context(">> WARNING: ", STANDARD_OUTPUT);
                      cout << endl << " " << "Illegal slot definition:" << endl << "   " << "`" << de_uglify_parse_tree(slotdef) << "'" << endl << "." << endl;
                    }
                  }
                  continue;
                }
                localslot = define_storage_slot_from_parse_tree(((Cons*)(slotdef)), classtype);
                if (key == KWD_STELLA_IN_PUBLIC_SLOTS) {
                  localslot->slot_publicP = TRUE;
                }
                if (!((boolean)(collect_001))) {
                  {
                    collect_001 = cons(localslot, NIL);
                    if (slots->the_cons_list == NIL) {
                      slots->the_cons_list = collect_001;
                    }
                    else {
                      add_cons_to_end_of_cons_list(slots->the_cons_list, collect_001);
                    }
                  }
                }
                else {
                  {
                    collect_001->rest = cons(localslot, NIL);
                    collect_001 = collect_001->rest;
                  }
                }
              }
            }
          }
        }
        else if ((key == KWD_STELLA_IN_PUBLIC_METHODS) ||
            (key == KWD_STELLA_IN_METHODS)) {
          { Slot* localslot = NULL;

            { Cons* methoddef = NULL;
              Cons* iter_003 = ((Cons*)(thevalue));
              Cons* collect_002 = NULL;

              while (!(iter_003 == NIL)) {
                methoddef = ((Cons*)(iter_003->value));
                iter_003 = iter_003->rest;
                if (!consP(methoddef)) {
                  { 
                    BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
                    signal_translation_warning();
                    if (!(suppress_warningsP())) {
                      print_error_context(">> WARNING: ", STANDARD_OUTPUT);
                      cout << endl << " " << "Illegal inline method definition:" << endl << "   " << "`" << de_uglify_parse_tree(methoddef) << "'" << endl << "." << endl;
                    }
                  }
                  continue;
                }
                localslot = define_inline_method(((Symbol*)(methoddef->value)), methoddef->rest);
                if (key == KWD_STELLA_IN_PUBLIC_METHODS) {
                  localslot->slot_publicP = TRUE;
                }
                if (!(localslot->slot_owner == classtype)) {
                  { 
                    BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
                    signal_translation_error();
                    if (!(suppress_warningsP())) {
                      print_error_context(">> ERROR: ", STANDARD_ERROR);
                      *(STANDARD_ERROR->native_stream) << endl << " " << "Defining a method for the class " << "`" << localslot->slot_owner << "'" << " inside of the class " << "`" << de_uglify_parse_tree(classtype) << "'" << "." << endl;
                    }
                  }
                }
                if (!((boolean)(collect_002))) {
                  {
                    collect_002 = cons(localslot, NIL);
                    if (slots->the_cons_list == NIL) {
                      slots->the_cons_list = collect_002;
                    }
                    else {
                      add_cons_to_end_of_cons_list(slots->the_cons_list, collect_002);
                    }
                  }
                }
                else {
                  {
                    collect_002->rest = cons(localslot, NIL);
                    collect_002 = collect_002->rest;
                  }
                }
              }
            }
          }
        }
        else if (key == KWD_STELLA_IN_PARAMETERS) {
          parameters = new_list();
          { Object* slotdef = NULL;
            Cons* iter_004 = ((Cons*)(thevalue));
            Cons* collect_003 = NULL;

            while (!(iter_004 == NIL)) {
              slotdef = iter_004->value;
              iter_004 = iter_004->rest;
              if (!((boolean)(collect_003))) {
                {
                  collect_003 = cons(define_storage_slot_from_parse_tree(((Cons*)(slotdef)), classtype), NIL);
                  if (parameters->the_cons_list == NIL) {
                    parameters->the_cons_list = collect_003;
                  }
                  else {
                    add_cons_to_end_of_cons_list(parameters->the_cons_list, collect_003);
                  }
                }
              }
              else {
                {
                  collect_003->rest = cons(define_storage_slot_from_parse_tree(((Cons*)(slotdef)), classtype), NIL);
                  collect_003 = collect_003->rest;
                }
              }
            }
          }
          classoptions->insert_at(key, parameters);
        }
        else if (key == KWD_STELLA_IN_KEY) {
          if (!consP(thevalue)) {
            thevalue = local_cons(thekey, NIL);
          }
          in_place_objects_to_symbols(((Cons*)(thevalue)));
          classoptions->insert_at(key, ((Cons*)(thevalue)));
        }
        else if (key == KWD_STELLA_IN_SYNONYMS) {
          if (!consP(thevalue)) {
            thevalue = local_cons(thekey, NIL);
          }
          in_place_objects_to_types(((Cons*)(thevalue)));
          classoptions->insert_at(key, ((Cons*)(thevalue)));
        }
        else if ((key == KWD_STELLA_IN_DOCUMENTATION) ||
            ((key == KWD_STELLA_IN_CL_NATIVE_TYPE) ||
             ((key == KWD_STELLA_IN_CPP_NATIVE_TYPE) ||
              ((key == KWD_STELLA_IN_JAVA_NATIVE_TYPE) ||
               (key == KWD_STELLA_IN_IDL_NATIVE_TYPE))))) {
          classoptions->insert_at(key, thevalue);
        }
        else if ((key == KWD_STELLA_IN_ABSTRACTp) ||
            ((key == KWD_STELLA_IN_ACTIVEp) ||
             ((key == KWD_STELLA_IN_PUBLICp) ||
              ((key == KWD_STELLA_IN_CL_STRUCTp) ||
               (key == KWD_STELLA_IN_MIXINp))))) {
          classoptions->insert_at(key, coerce_to_boolean(thevalue));
        }
        else if (key == KWD_STELLA_IN_RECYCLE_METHOD) {
          thevalue = thevalue->permanentify();
          classoptions->insert_at(key, thevalue);
          if (get_quoted_tree("((:SWEEP-LIST :FREE-AND-SWEEP-LIST) \"/STELLA\")", "/STELLA")->memberP(thevalue)) {
            slottree = listO(4, SYM_STELLA_IN_STELLA_NEXT_SWEEP_LIST_OBJECT, KWD_STELLA_IN_TYPE, treedefinition->value, NIL);
            slots->push(define_storage_slot_from_parse_tree(slottree, classtype));
          }
        }
        else if ((key == KWD_STELLA_IN_EXTENSION) ||
            ((key == KWD_STELLA_IN_CREATOR) ||
             ((key == KWD_STELLA_IN_DESTRUCTOR) ||
              ((key == KWD_STELLA_IN_INITIALIZER) ||
               ((key == KWD_STELLA_IN_TERMINATOR) ||
                (key == KWD_STELLA_IN_EQUALITY_TEST)))))) {
          classoptions->insert_at(key, thevalue->permanentify());
        }
        else if ((key == KWD_STELLA_IN_INITIAL_VALUE) ||
            (key == KWD_STELLA_IN_PRINT_FORM)) {
          classoptions->insert_at(key, permanentify_form(thevalue));
        }
        else if (key == KWD_STELLA_IN_PROPERTIES) {
          classoptions->insert_at(key, collect_feature_list(thevalue));
        }
        else if (key == KWD_STELLA_IN_META_ATTRIBUTES) {
          classoptions->insert_at(key, collect_key_value_list(thevalue));
        }
        else if (key == KWD_STELLA_IN_CHILDREN) {
          if (safe_primary_type(thevalue) == SGT_STELLA_IN_STELLA_CONS) {
            { Object* thevalue_000 = thevalue;
              Cons* thevalue = ((Cons*)(thevalue_000));

              { List* children = new_list();

                { Object* child = NULL;
                  Cons* iter_005 = thevalue;
                  Cons* collect_004 = NULL;

                  while (!(iter_005 == NIL)) {
                    child = iter_005->value;
                    iter_005 = iter_005->rest;
                    if (symbolP(child)) {
                      if (!((boolean)(collect_004))) {
                        {
                          collect_004 = cons(typify(child), NIL);
                          if (children->the_cons_list == NIL) {
                            children->the_cons_list = collect_004;
                          }
                          else {
                            add_cons_to_end_of_cons_list(children->the_cons_list, collect_004);
                          }
                        }
                      }
                      else {
                        {
                          collect_004->rest = cons(typify(child), NIL);
                          collect_004 = collect_004->rest;
                        }
                      }
                    }
                  }
                }
                classoptions->insert_at(key, children);
              }
            }
          }
          else {
            { 
              BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
              signal_translation_error();
              if (!(suppress_warningsP())) {
                print_error_context(">> ERROR: ", STANDARD_ERROR);
                *(STANDARD_ERROR->native_stream) << endl << " " << "Illegal :children specification: " << "`" << de_uglify_parse_tree(thevalue) << "'" << "." << endl;
              }
            }
          }
        }
        else {
          classoptions->insert_at(key, thevalue);
        }
      }
    }
    theoptions->the_plist = NULL;
    { Class* renamed_Class = define_stella_class(classtype, directsupers, slots, ((Keyword_Key_Value_List*)(classoptions->reverse())));

      if (((boolean)(renamed_Class))) {
        renamed_Class->class_stringified_source = stringifiedsource;
      }
      return (renamed_Class);
    }
  }
}

Storage_Slot* define_storage_slot_from_parse_tree(Cons* slotdef, Surrogate* owner) {
  { Object* name = slotdef->value->permanentify();
    Surrogate* basetype = NULL;
    Cons* typeexpression = NIL;
    Property_List* theoptions = new_property_list();
    Keyword_Key_Value_List* slotoptions = new_keyword_key_value_list();
    Keyword* key = NULL;
    Storage_Slot* slot = NULL;

    theoptions->the_plist = slotdef->rest;
    { Object* thekey = NULL;
      Object* thevalue = NULL;
      Cons* iter_000 = theoptions->the_plist;

      while (!(iter_000 == NIL)) {
        thekey = iter_000->value;
        thevalue = iter_000->rest->value;
        iter_000 = iter_000->rest->rest;
        key = ((Keyword*)(thekey->permanentify()));
        if ((key == KWD_STELLA_IN_INITIALLY) ||
            (key == KWD_STELLA_IN_DEFAULT)) {
          slotoptions->insert_at(key, permanentify_form(thevalue));
        }
        else if (key == KWD_STELLA_IN_TYPE) {
          if (safe_primary_type(thevalue) == SGT_STELLA_IN_STELLA_CONS) {
            { Object* thevalue_000 = thevalue;
              Cons* thevalue = ((Cons*)(thevalue_000));

              typeexpression = thevalue;
              if (thevalue->value == SYM_STELLA_IN_STELLA_LIKE) {
                basetype = NULL;
              }
              else {
                basetype = typify(thevalue->value);
              }
            }
          }
          else {
            basetype = typify(thevalue);
            if (defined_typeP(basetype)) {
              { Standard_Object* realtype = thevalue->yield_type_specifier();

                basetype = type_spec_to_base_type(realtype);
                if (!(typeP(realtype))) {
                  typeexpression = ((Cons*)(yield_type_spec_tree(realtype)));
                }
              }
            }
          }
        }
        else if (key == KWD_STELLA_IN_ALLOCATION) {
          slotoptions->insert_at(key, thevalue->permanentify());
        }
        else if (key == KWD_STELLA_IN_READER) {
          slotoptions->insert_at(key, thevalue->permanentify());
        }
        else if (key == KWD_STELLA_IN_WRITER) {
          slotoptions->insert_at(key, thevalue->permanentify());
        }
        else if ((key == KWD_STELLA_IN_INVERSE) ||
            (key == KWD_STELLA_IN_RENAMES)) {
          slotoptions->insert_at(key, thevalue->permanentify());
        }
        else if ((key == KWD_STELLA_IN_PUBLICp) ||
            ((key == KWD_STELLA_IN_REQUIREDp) ||
             ((key == KWD_STELLA_IN_COMPONENTp) ||
              ((key == KWD_STELLA_IN_READ_ONLYp) ||
               ((key == KWD_STELLA_IN_ACTIVEp) ||
                ((key == KWD_STELLA_IN_CONTEXT_SENSITIVEp) ||
                 ((key == KWD_STELLA_IN_HARDWIREDp) ||
                  (key == KWD_STELLA_IN_ABSTRACTp)))))))) {
          slotoptions->insert_at(key, coerce_to_boolean(thevalue));
        }
        else if (key == KWD_STELLA_IN_DOCUMENTATION) {
          slotoptions->insert_at(key, thevalue);
        }
        else if (key == KWD_STELLA_IN_PROPERTIES) {
          slotoptions->insert_at(key, collect_feature_list(thevalue));
        }
        else if (key == KWD_STELLA_IN_META_ATTRIBUTES) {
          slotoptions->insert_at(key, collect_key_value_list(thevalue));
        }
        else if ((key == KWD_STELLA_IN_OPTION_KEYWORD) ||
            (key == KWD_STELLA_IN_OPTION_HANDLER)) {
          slotoptions->insert_at(key, thevalue->permanentify());
        }
        else {
          slotoptions->insert_at(key, thevalue);
        }
      }
    }
    theoptions->the_plist = NULL;
    slot = ((Storage_Slot*)(define_stella_slot(((Symbol*)(name)), owner, basetype, typeexpression, ((Keyword_Key_Value_List*)(slotoptions->reverse())))));
    record_slot_home_module(slot);
    return (slot);
  }
}

Module* record_slot_home_module(Slot* self) {
  { Symbol* name = self->slot_name;
    Module* namemodule = ((Module*)(name->home_context));
    Module* definitionmodule = oMODULEo.get();
    Module* homemodule = namemodule;

    if (explicitly_qualified_nameP(name, definitionmodule)) {
      homemodule = namemodule;
    }
    else {
      homemodule = definitionmodule;
    }
    if (!(homemodule == namemodule)) {
      set_dynamic_slot_value(self->dynamic_slots, SYM_STELLA_IN_STELLA_SLOT_HOME_MODULE, homemodule, NULL);
    }
    return (homemodule);
  }
}

boolean explicitly_qualified_nameP(Generalized_Symbol* name, Module* definitionmodule) {
  { Module* namemodule = ((Module*)(name->home_context));
    Generalized_Symbol* shadow = NULL;

    if (namemodule == definitionmodule) {
      {
        return (FALSE);
      }
    }
    else {
      if (!visible_fromP(namemodule, definitionmodule)) {
        {
          return (TRUE);
        }
      }
      else {
        { boolean test_value_000 = FALSE;

          { 
            shadow = lookup_symbol_in_module(name->symbol_name, definitionmodule, TRUE);
            test_value_000 = ((boolean)(shadow));
          }
          if (test_value_000) {
            test_value_000 = !(shadow == name);
          }
          if (test_value_000) {
            {
              return (TRUE);
            }
          }
          else {
            {
              return (FALSE);
            }
          }
        }
      }
    }
  }
}

Class* get_identical_class(char* classname, char* stringifiedsource) {
  { Surrogate* surrogate = lookup_surrogate(classname);
    Object* oldvalue = (((boolean)(surrogate)) ? surrogate->surrogate_value : ((Object*)(NULL)));

    if (((boolean)(oldvalue))) {
      if (subtype_of_classP(safe_primary_type(oldvalue))) {
        { Object* oldvalue_000 = oldvalue;
          Class* oldvalue = ((Class*)(oldvalue_000));

          if ((oldvalue->class_stringified_source != NULL) &&
              (string_eqlP(oldvalue->class_stringified_source, stringifiedsource) &&
               (((Module*)(surrogate->home_context)) == oMODULEo.get()))) {
            return (oldvalue);
          }
        }
      }
      else {
      }
    }
    return (NULL);
  }
}

Class* define_class_from_parse_tree(Cons* parsetree) {
  if (bad_argument_rangeP(parsetree, 2, NULL_INTEGER)) {
    return (NULL);
  }
  if (!(symbolP(parsetree->rest->value) &&
      consP(parsetree->rest->rest->value))) {
    { 
      BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
      signal_translation_error();
      if (!(suppress_warningsP())) {
        print_error_context(">> ERROR: ", STANDARD_ERROR);
        *(STANDARD_ERROR->native_stream) << endl << " " << "Illegal class definition: " << "`" << de_uglify_parse_tree(parsetree) << "'" << "." << endl;
      }
    }
    return (NULL);
  }
  { Symbol* classname = ((Symbol*)(parsetree->rest->value));
    char* stringifiedsource = stringify(parsetree);
    Class* oldclass = get_identical_class(classname->symbol_name, stringifiedsource);

    if (((boolean)(oldclass)) &&
        (!oREDEFINE_IDENTICAL_CLASSESpo)) {
      return (oldclass);
    }
    return (help_define_class_from_parse_tree(parsetree, stringifiedsource));
  }
}

Class* define_class_from_stringified_source(char* classname, char* stringifiedsource) {
  { Class* oldclass = get_identical_class(classname, stringifiedsource);
    Class* newclass = NULL;

    if (((boolean)(oldclass)) &&
        (!oREDEFINE_IDENTICAL_CLASSESpo)) {
      return (oldclass);
    }
    newclass = help_define_class_from_parse_tree(((Cons*)(unstringify_stella_source(stringifiedsource, NULL))), stringifiedsource);
    return (newclass);
  }
}

Cons* extract_options(Cons* tree, Key_Value_List* optionstable) {
  { Cons* options = NULL;
    Cons* tail = NULL;

    while ((!(tree->rest == NIL)) &&
        (!keywordP(tree->rest->value))) {
      tree = tree->rest;
    }
    if ((tree->rest == NIL) ||
        (tree->rest->rest == NIL)) {
      return (NIL);
    }
    if (((boolean)(optionstable))) {
      optionstable->insert_at(tree->rest->value, tree->rest->rest->value);
    }
    options = tree->rest;
    tail = tree->rest->rest;
    while ((!(tail->rest->rest == NIL)) &&
        keywordP(tail->rest->value)) {
      if (((boolean)(optionstable))) {
        optionstable->insert_at(tail->rest->value, tail->rest->rest->value);
      }
      tail = tail->rest->rest;
    }
    tree->rest = tail->rest;
    tail->rest = NIL;
    return (options);
  }
}

Object* destructure_defmethod_tree(Cons* method_tree, Key_Value_List* options_table, Cons*& _return1, Cons*& _return2) {
  // Return three parse trees representing the name,
  // parameters, and code body of the parse tree `method-tree'.  Fill
  // `options-table' with a dictionary of method options.
  // Storage note:  Options are treated specially because the other return
  // values are subtrees of `method-tree', while `options-table' is a newly-created
  // cons tree.  Note also, the parameter and body trees are destructively
  // removed from `method-tree'.
  { Object* nametree = method_tree->rest->value;
    Object* parameterstree = method_tree->rest->rest->value;

    if (!consP(parameterstree)) {
      if (!((boolean)(parameterstree))) {
        { 
          BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
          signal_translation_error();
          if (!(suppress_warningsP())) {
            print_error_context(">> ERROR: ", STANDARD_ERROR);
            *(STANDARD_ERROR->native_stream) << endl << " " << "Missing parameter list" << "." << endl;
          }
        }
      }
      else {
        { 
          BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
          signal_translation_error();
          if (!(suppress_warningsP())) {
            print_error_context(">> ERROR: ", STANDARD_ERROR);
            *(STANDARD_ERROR->native_stream) << endl << " " << "Illegal parameter list: " << "`" << de_uglify_parse_tree(parameterstree) << "'" << "." << endl;
          }
        }
      }
      parameterstree = NIL;
    }
    extract_options(method_tree, options_table);
    { Object* _return_temp = nametree;

      _return1 = ((Cons*)(parameterstree));
      _return2 = method_tree->nth_rest(3);
      return (_return_temp);
    }
  }
}

Symbol* yield_setter_method_name(Symbol* methodname) {
  return (intern_derived_symbol(methodname, string_concatenate(methodname->symbol_name, "-SETTER", 0)));
}

Symbol* destructure_method_name_tree(Object* nametree, Cons*& _return1) {
  { Surrogate* test_value_000 = safe_primary_type(nametree);

    if (test_value_000 == SGT_STELLA_IN_STELLA_CONS) {
      { Object* nametree_000 = nametree;
        Cons* nametree = ((Cons*)(nametree_000));

        if (!symbolP(nametree->value)) {
          { 
            BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
            signal_translation_error();
            if (!(suppress_warningsP())) {
              print_error_context(">> ERROR: ", STANDARD_ERROR);
              *(STANDARD_ERROR->native_stream) << endl << " " << "Illegal method name: " << "`" << de_uglify_parse_tree(nametree) << "'" << "." << endl;
            }
          }
          _return1 = NULL;
          return (NULL);
        }
        if (nametree->rest->value == SYM_STELLA_IN_STELLA_SETTER) {
          _return1 = NIL;
          return (yield_setter_method_name(((Symbol*)(nametree->value))));
        }
        else {
          _return1 = nametree->rest;
          return (((Symbol*)(nametree->value)));
        }
      }
    }
    else if (subtype_of_symbolP(test_value_000)) {
      { Object* nametree_001 = nametree;
        Symbol* nametree = ((Symbol*)(nametree_001));

        _return1 = NIL;
        return (nametree);
      }
    }
    else {
      { 
        BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
        signal_translation_error();
        if (!(suppress_warningsP())) {
          print_error_context(">> ERROR: ", STANDARD_ERROR);
          *(STANDARD_ERROR->native_stream) << endl << " " << "Illegal method name: " << "`" << de_uglify_parse_tree(nametree) << "'" << "." << endl;
        }
      }
      _return1 = NULL;
      return (NULL);
    }
  }
}

void in_place_normalize_method_options(Keyword_Key_Value_List* methodoptions, Symbol* methodname) {
  methodname = methodname;
  { Keyword* key = NULL;
    Object* value = NULL;
    Kv_List_Iterator* iter_000 = ((Kv_List_Iterator*)(methodoptions->allocate_iterator()));

    while (iter_000->nextP()) {
      key = ((Keyword*)(iter_000->key));
      value = iter_000->value;
      if ((key == KWD_STELLA_IN_TYPE) ||
          (key == KWD_STELLA_IN_RETURN_TYPES)) {
      }
      else if ((key == KWD_STELLA_IN_RENAMES) ||
          ((key == KWD_STELLA_IN_STORAGE_SLOT) ||
           (key == KWD_STELLA_IN_INHERITS_THROUGH))) {
        methodoptions->insert_at(key, value->permanentify());
      }
      else if ((key == KWD_STELLA_IN_PUBLICp) ||
          ((key == KWD_STELLA_IN_ABSTRACTp) ||
           ((key == KWD_STELLA_IN_SETTERp) ||
            ((key == KWD_STELLA_IN_NATIVEp) ||
             ((key == KWD_STELLA_IN_CONSTRUCTORp) ||
              ((key == KWD_STELLA_IN_AUXILIARYp) ||
               ((key == KWD_STELLA_IN_GLOBALLY_INLINEp) ||
                ((key == KWD_STELLA_IN_COMMANDp) ||
                 ((key == KWD_STELLA_IN_LISP_MACROp) ||
                  (key == KWD_STELLA_IN_EVALUATE_ARGUMENTSp)))))))))) {
        methodoptions->insert_at(key, coerce_to_boolean(value));
      }
      else if (key == KWD_STELLA_IN_DOCUMENTATION) {
        methodoptions->insert_at(key, value);
      }
      else if (key == KWD_STELLA_IN_INLINE) {
        { Surrogate* test_value_000 = safe_primary_type(value);

          if (subtype_of_symbolP(test_value_000)) {
            { Object* value_000 = value;
              Symbol* value = ((Symbol*)(value_000));

              methodoptions->insert_at(key, ((Symbol*)(value->permanentify())));
            }
          }
          else if (test_value_000 == SGT_STELLA_IN_STELLA_CONS) {
            { Object* value_001 = value;
              Cons* value = ((Cons*)(value_001));

              { Cons* symbolslist = NIL;

                { Object* symbol = NULL;
                  Cons* iter_001 = value;
                  Cons* collect_000 = NULL;

                  while (!(iter_001 == NIL)) {
                    symbol = iter_001->value;
                    iter_001 = iter_001->rest;
                    if (!((boolean)(collect_000))) {
                      {
                        collect_000 = cons(symbol->permanentify(), NIL);
                        if (symbolslist == NIL) {
                          symbolslist = collect_000;
                        }
                        else {
                          add_cons_to_end_of_cons_list(symbolslist, collect_000);
                        }
                      }
                    }
                    else {
                      {
                        collect_000->rest = cons(symbol->permanentify(), NIL);
                        collect_000 = collect_000->rest;
                      }
                    }
                  }
                }
                methodoptions->insert_at(key, symbolslist);
              }
            }
          }
          else {
            { Output_String_Stream* stream_000 = new_output_string_stream();

              *(stream_000->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
              throw *new_stella_exception(stream_000->the_string_reader());
            }
          }
        }
      }
      else if (key == KWD_STELLA_IN_PROPERTIES) {
        methodoptions->insert_at(key, collect_feature_list(value));
      }
      else if (key == KWD_STELLA_IN_META_ATTRIBUTES) {
        methodoptions->insert_at(key, collect_key_value_list(value));
      }
      else {
        methodoptions->insert_at(key, value->permanentify());
      }
    }
  }
  methodoptions->reverse();
}

Method_Slot* get_function_with_identical_signature(Symbol* name, char* stringifiedsource) {
  { Method_Slot* oldmethod = lookup_function(name);

    if (((boolean)(oldmethod)) &&
        string_eqlP(oldmethod->method_stringified_source, stringifiedsource)) {
      return (oldmethod);
    }
    return (NULL);
  }
}

Method_Slot* get_method_with_identical_signature(Symbol* name, Cons* parameterstree, char* stringifiedsource) {
  { Slot* existingslot = NULL;

    if (consP(parameterstree->value)) {
      parameterstree = ((Cons*)(parameterstree->value));
      if (symbolP(parameterstree->last())) {
        existingslot = lookup_slot(type_to_class(typify(((Symbol*)(parameterstree->last())))), name);
        if (((boolean)(existingslot))) {
          if (subtype_of_method_slotP(safe_primary_type(existingslot))) {
            { Slot* existingslot_000 = existingslot;
              Method_Slot* existingslot = ((Method_Slot*)(existingslot_000));

              if (string_eqlP(existingslot->method_stringified_source, stringifiedsource)) {
                return (existingslot);
              }
            }
          }
          else {
          }
        }
      }
    }
    return (NULL);
  }
}

Method_Slot* help_define_method_from_parse_tree(Cons* inputtree, char* stringifiedsource, boolean& _return1) {
  { Object* nametree = NULL;
    Cons* parameterstree = NULL;
    Keyword_Key_Value_List* methodoptions = new_keyword_key_value_list();
    Cons* bodytree = NULL;
    Symbol* name = NULL;
    Cons* returntypestree = NULL;
    boolean functionP = inputtree->value == SYM_STELLA_IN_STELLA_DEFUN;
    Method_Slot* oldmethod = NULL;
    char* signaturestring = ((stringifiedsource == NULL) ? stringify_method_signature(inputtree) : stringifiedsource);

    nametree = destructure_defmethod_tree(inputtree, methodoptions, parameterstree, bodytree);
    name = destructure_method_name_tree(nametree, returntypestree);
    if (!((boolean)(name))) {
      _return1 = FALSE;
      return (NULL);
    }
    oldmethod = (functionP ? get_function_with_identical_signature(name, signaturestring) : get_method_with_identical_signature(name, parameterstree, signaturestring));
    if (((boolean)(oldmethod))) {
      _return1 = TRUE;
      return (oldmethod);
    }
    in_place_normalize_method_options(methodoptions, name);
    { Method_Slot* method = define_stella_method_slot(name, returntypestree, functionP, parameterstree, methodoptions);

      if (stringifiedsource == NULL) {
        stringifiedsource = (stringify_method_bodyP(method) ? stringify(((Cons*)(read_s_expression_from_string(signaturestring)))->concatenate(bodytree, 0)) : signaturestring);
      }
      method->method_stringified_source = stringifiedsource;
      record_slot_home_module(method);
      _return1 = FALSE;
      return (method);
    }
  }
}

Method_Slot* define_method_from_parse_tree(Cons* parsetree) {
  if (bad_argument_rangeP(parsetree, 2, NULL_INTEGER)) {
    return (NULL);
  }
  { Method_Slot* method = NULL;
    boolean identicalmethodP = FALSE;

    method = help_define_method_from_parse_tree(parsetree, NULL, identicalmethodP);
    if (!(identicalmethodP ||
        (!((boolean)(method))))) {
      if (((Boolean_Wrapper*)(dynamic_slot_value(method->dynamic_slots, SYM_STELLA_IN_STELLA_SLOT_AUXILIARYp, FALSE_WRAPPER)))->wrapper_value) {
        method->slot_externalP = TRUE;
      }
      else {
        attach_method_slot_to_owner(method);
      }
    }
    return (method);
  }
}

char* stringify_method_signature(Cons* parsetree) {
  { Cons* signature = NIL;
    char* signaturestring = NULL;
    Cons* optionstree = parsetree->nth_rest(3);

    signature = cons(parsetree->value, signature);
    signature = cons(parsetree->rest->value, signature);
    signature = cons(parsetree->rest->rest->value, signature);
    while (keywordP(optionstree->value)) {
      signature = cons(optionstree->value, signature);
      signature = cons(optionstree->rest->value, signature);
      optionstree = optionstree->rest->rest;
    }
    signaturestring = stringify(signature->reverse());
    return (signaturestring);
  }
}

boolean stringify_method_bodyP(Method_Slot* method) {
  return (((Boolean_Wrapper*)(dynamic_slot_value(method->dynamic_slots, SYM_STELLA_IN_STELLA_METHOD_GLOBALLY_INLINEp, FALSE_WRAPPER)))->wrapper_value ||
      ((!method->method_functionP) &&
       ((!((boolean)(((Class*)(method->slot_owner->surrogate_value))))) ||
        ((Class*)(method->slot_owner->surrogate_value))->mixinP)));
}

Method_Slot* define_method_from_stringified_source(char* methodname, char* classname, char* stringifiedsource) {
  {
    methodname = methodname;
    classname = classname;
  }
  { Method_Slot* method = NULL;
    boolean identicalmethodP = FALSE;

    method = help_define_method_from_parse_tree(((Cons*)(unstringify_stella_source(stringifiedsource, NULL))), stringifiedsource, identicalmethodP);
    if (!(identicalmethodP ||
        ((Boolean_Wrapper*)(dynamic_slot_value(method->dynamic_slots, SYM_STELLA_IN_STELLA_SLOT_AUXILIARYp, FALSE_WRAPPER)))->wrapper_value)) {
      attach_method_slot_to_owner(method);
    }
    return (method);
  }
}

void define_method_object(char* definition, cpp_method_code code, cpp_method_code wrappercode) {
  wrappercode = wrappercode;
  { Method_Slot* method = define_method_from_stringified_source(NULL, NULL, definition);

    if (((boolean)(method))) {
      if (code != NULL) {
        method->method_code = code;
      }
    }
  }
}

List* oUNSUPPORTED_EXTERNAL_SLOT_OPTIONSo = NULL;

Storage_Slot* define_external_slot_from_parse_tree(Cons* parsetree) {
  if (bad_argument_rangeP(parsetree, 2, NULL_INTEGER)) {
    return (NULL);
  }
  if ((!symbolP(parsetree->rest->value)) ||
      ((!symbolP(parsetree->rest->rest->value)) ||
       (!!((parsetree->nth_rest(3)->length()) % 2)))) {
    { 
      BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
      signal_translation_error();
      if (!(suppress_warningsP())) {
        print_error_context(">> ERROR: ", STANDARD_ERROR);
        *(STANDARD_ERROR->native_stream) << endl << " " << "Illegal external slot definition: " << "`" << de_uglify_parse_tree(parsetree) << "'" << "." << endl;
      }
    }
    return (NULL);
  }
  { Symbol* classname = ((Symbol*)(parsetree->rest->value));
    Surrogate* owner = typify(classname);
    Class* ownerclass = ((Class*)(owner->surrogate_value));
    Storage_Slot* slot = NULL;
    Cons* options = parsetree->nth_rest(3);
    Module* marker = oSTELLA_MODULEo;
    boolean abstractP = FALSE;

    if (!((boolean)(ownerclass))) {
      { 
        BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
        signal_translation_error();
        if (!(suppress_warningsP())) {
          print_error_context(">> ERROR: ", STANDARD_ERROR);
          *(STANDARD_ERROR->native_stream) << endl << " " << "Can't define a slot on the non-existent class " << "`" << de_uglify_parse_tree(classname) << "'" << "." << endl;
        }
      }
      return (NULL);
    }
    while (!(options == NIL)) {
      { Object* option = options->value;
        Object* value = options->rest->value;

        if (oUNSUPPORTED_EXTERNAL_SLOT_OPTIONSo->memberP(((Keyword*)(option)))) {
          if ((!(option == KWD_STELLA_IN_ALLOCATION)) ||
              (!(value == KWD_STELLA_IN_DYNAMIC))) {
            { 
              BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
              signal_translation_warning();
              if (!(suppress_warningsP())) {
                print_error_context(">> WARNING: ", STANDARD_WARNING);
                *(STANDARD_WARNING->native_stream) << endl << " " << "Skipping slot option " << "`" << de_uglify_parse_tree(option) << "'" << " which is unsupported on external slots" << "." << endl;
              }
            }
          }
          options->first_setter(marker);
          if (((boolean)(value))) {
            options->second_setter(marker);
          }
        }
        if (((option == KWD_STELLA_IN_ABSTRACTp) &&
            (value == SYM_STELLA_IN_STELLA_TRUE)) ||
            (option == KWD_STELLA_IN_RENAMES)) {
          abstractP = TRUE;
        }
      }
      options = options->rest->rest;
    }
    if ((!abstractP) &&
        (ownerclass->class_finalizedP &&
         (!subtype_ofP(owner, SGT_STELLA_IN_STELLA_DYNAMIC_SLOTS_MIXIN)))) {
      { 
        BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
        signal_translation_error();
        if (!(suppress_warningsP())) {
          print_error_context(">> ERROR: ", STANDARD_ERROR);
          *(STANDARD_ERROR->native_stream) << endl << " " << "Can't define a non-abstract external slot on class " << "`" << de_uglify_parse_tree(classname) << "'" << "," << endl << " since it does not inherit DYNAMIC-SLOTS-MIXIN" << "." << endl;
        }
      }
      return (NULL);
    }
    parsetree = parsetree->remove(marker);
    if (!(abstractP)) {
      parsetree = parsetree->concatenate(listO(3, KWD_STELLA_IN_ALLOCATION, KWD_STELLA_IN_DYNAMIC, NIL), 0);
    }
    slot = define_storage_slot_from_parse_tree(parsetree->rest->rest, owner);
    if (((boolean)(slot))) {
      attach_slot_to_owner(slot);
    }
    return (slot);
  }
}

Storage_Slot* define_external_slot_from_stringified_source(char* stringifiedsource) {
  return (define_external_slot_from_parse_tree(((Cons*)(read_s_expression_from_string(stringifiedsource)))));
}

Storage_Slot* lookup_slot_from_option_keyword(Surrogate* owner, Keyword* keyword) {
  { Class* ownerclass = ((Class*)(owner->surrogate_value));

    if (((boolean)(ownerclass))) {
      { Slot* slot = NULL;
        Iterator* iter_000 = ownerclass->class_slots();

        while (iter_000->nextP()) {
          slot = ((Slot*)(iter_000->value));
          if (storage_slotP(slot) &&
              (((Keyword*)(dynamic_slot_value(((Storage_Slot*)(slot))->dynamic_slots, SYM_STELLA_IN_STELLA_SLOT_OPTION_KEYWORD, NULL))) == keyword)) {
            return (((Storage_Slot*)(slot)));
          }
        }
      }
    }
    return (NULL);
  }
}

Method_Slot* oDEFAULT_OPTION_HANDLERo = NULL;

Method_Slot* lookup_option_handler(Storage_Slot* slot) {
  { Symbol* handlername = ((Symbol*)(dynamic_slot_value(slot->dynamic_slots, SYM_STELLA_IN_STELLA_SLOT_OPTION_HANDLER, NULL)));
    Method_Slot* handler = NULL;

    if (!((boolean)(handlername))) {
      if (((boolean)(oDEFAULT_OPTION_HANDLERo))) {
        return (oDEFAULT_OPTION_HANDLERo);
      }
      else {
        return (lookup_function(SYM_STELLA_IN_STELLA_DEFAULT_OPTION_HANDLER));
      }
    }
    handler = lookup_function(handlername);
    if ((oSAFETYo.get() >= 2) &&
        (((boolean)(handler)) &&
         ((!(handlername == SYM_STELLA_IN_STELLA_DEFAULT_OPTION_HANDLER)) &&
          (!handler->conforming_signaturesP(oDEFAULT_OPTION_HANDLERo))))) {
      { Output_String_Stream* stream_000 = new_output_string_stream();

        *(stream_000->native_stream) << "The signature of slot option handler " << "`" << handlername << "'" << " does not conform to that of 'default-option-handler'.";
        throw *new_stella_exception(stream_000->the_string_reader());
      }
    }
    return (handler);
  }
}

boolean run_option_handlerP(Object* object, Keyword* option, Object* valuetree) {
  { Storage_Slot* slot = lookup_slot_from_option_keyword(object->primary_type(), option);
    Method_Slot* handler = (((boolean)(slot)) ? lookup_option_handler(slot) : ((Method_Slot*)(NULL)));

    if (((boolean)(handler))) {
      ((void  (*) (Object*, Storage_Slot*, Object*))handler->function_code)(object, slot, valuetree);
      return (TRUE);
    }
    return (FALSE);
  }
}

void default_option_handler(Object* self, Storage_Slot* slot, Object* tree) {
  { Object* parsedvalue = NULL;

    if (slot->type() == SGT_STELLA_IN_STELLA_BOOLEAN) {
      parsedvalue = coerce_to_boolean(tree);
    }
    else {
      if (((boolean)(tree))) {
        parsedvalue = permanent_copy(tree);
      }
    }
    if ((oSAFETYo.get() >= 2) &&
        ((boolean)(parsedvalue))) {
      if (!isaP(parsedvalue, slot->type()->type_to_wrapped_type())) {
        { 
          BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
          signal_translation_error();
          if (!(suppress_warningsP())) {
            print_error_context(">> ERROR: ", STANDARD_ERROR);
            *(STANDARD_ERROR->native_stream) << endl << " " << "Illegal value for " << "`" << ((Keyword*)(dynamic_slot_value(slot->dynamic_slots, SYM_STELLA_IN_STELLA_SLOT_OPTION_KEYWORD, NULL))) << "'" << " option: " << "`" << de_uglify_parse_tree(tree) << "'" << "." << endl;
          }
        }
        return;
      }
    }
    write_slot_value(((Standard_Object*)(self)), slot, parsedvalue);
  }
}

void help_startup_stella_in1() {
  {
    SYM_STELLA_IN_STELLA_TRUE = ((Symbol*)(intern_rigid_symbol_wrt_module("TRUE", NULL, 0)));
    SYM_STELLA_IN_STELLA_FALSE = ((Symbol*)(intern_rigid_symbol_wrt_module("FALSE", NULL, 0)));
    SGT_STELLA_IN_STELLA_CONS = ((Surrogate*)(intern_rigid_symbol_wrt_module("CONS", NULL, 1)));
    SGT_STELLA_IN_STELLA_GENERALIZED_SYMBOL = ((Surrogate*)(intern_rigid_symbol_wrt_module("GENERALIZED-SYMBOL", NULL, 1)));
    SYM_STELLA_IN_STELLA_DEFMETHOD = ((Symbol*)(intern_rigid_symbol_wrt_module("DEFMETHOD", NULL, 0)));
    KWD_STELLA_IN_PUBLIC_SLOTS = ((Keyword*)(intern_rigid_symbol_wrt_module("PUBLIC-SLOTS", NULL, 2)));
    KWD_STELLA_IN_SLOTS = ((Keyword*)(intern_rigid_symbol_wrt_module("SLOTS", NULL, 2)));
    KWD_STELLA_IN_PUBLIC_METHODS = ((Keyword*)(intern_rigid_symbol_wrt_module("PUBLIC-METHODS", NULL, 2)));
    KWD_STELLA_IN_METHODS = ((Keyword*)(intern_rigid_symbol_wrt_module("METHODS", NULL, 2)));
    KWD_STELLA_IN_PARAMETERS = ((Keyword*)(intern_rigid_symbol_wrt_module("PARAMETERS", NULL, 2)));
    KWD_STELLA_IN_KEY = ((Keyword*)(intern_rigid_symbol_wrt_module("KEY", NULL, 2)));
    KWD_STELLA_IN_SYNONYMS = ((Keyword*)(intern_rigid_symbol_wrt_module("SYNONYMS", NULL, 2)));
    KWD_STELLA_IN_DOCUMENTATION = ((Keyword*)(intern_rigid_symbol_wrt_module("DOCUMENTATION", NULL, 2)));
    KWD_STELLA_IN_CL_NATIVE_TYPE = ((Keyword*)(intern_rigid_symbol_wrt_module("CL-NATIVE-TYPE", NULL, 2)));
    KWD_STELLA_IN_CPP_NATIVE_TYPE = ((Keyword*)(intern_rigid_symbol_wrt_module("CPP-NATIVE-TYPE", NULL, 2)));
    KWD_STELLA_IN_JAVA_NATIVE_TYPE = ((Keyword*)(intern_rigid_symbol_wrt_module("JAVA-NATIVE-TYPE", NULL, 2)));
    KWD_STELLA_IN_IDL_NATIVE_TYPE = ((Keyword*)(intern_rigid_symbol_wrt_module("IDL-NATIVE-TYPE", NULL, 2)));
    KWD_STELLA_IN_ABSTRACTp = ((Keyword*)(intern_rigid_symbol_wrt_module("ABSTRACT?", NULL, 2)));
    KWD_STELLA_IN_ACTIVEp = ((Keyword*)(intern_rigid_symbol_wrt_module("ACTIVE?", NULL, 2)));
    KWD_STELLA_IN_PUBLICp = ((Keyword*)(intern_rigid_symbol_wrt_module("PUBLIC?", NULL, 2)));
    KWD_STELLA_IN_CL_STRUCTp = ((Keyword*)(intern_rigid_symbol_wrt_module("CL-STRUCT?", NULL, 2)));
    KWD_STELLA_IN_MIXINp = ((Keyword*)(intern_rigid_symbol_wrt_module("MIXIN?", NULL, 2)));
    KWD_STELLA_IN_RECYCLE_METHOD = ((Keyword*)(intern_rigid_symbol_wrt_module("RECYCLE-METHOD", NULL, 2)));
    SYM_STELLA_IN_STELLA_NEXT_SWEEP_LIST_OBJECT = ((Symbol*)(intern_rigid_symbol_wrt_module("NEXT-SWEEP-LIST-OBJECT", NULL, 0)));
    KWD_STELLA_IN_TYPE = ((Keyword*)(intern_rigid_symbol_wrt_module("TYPE", NULL, 2)));
    KWD_STELLA_IN_EXTENSION = ((Keyword*)(intern_rigid_symbol_wrt_module("EXTENSION", NULL, 2)));
    KWD_STELLA_IN_CREATOR = ((Keyword*)(intern_rigid_symbol_wrt_module("CREATOR", NULL, 2)));
    KWD_STELLA_IN_DESTRUCTOR = ((Keyword*)(intern_rigid_symbol_wrt_module("DESTRUCTOR", NULL, 2)));
    KWD_STELLA_IN_INITIALIZER = ((Keyword*)(intern_rigid_symbol_wrt_module("INITIALIZER", NULL, 2)));
    KWD_STELLA_IN_TERMINATOR = ((Keyword*)(intern_rigid_symbol_wrt_module("TERMINATOR", NULL, 2)));
    KWD_STELLA_IN_EQUALITY_TEST = ((Keyword*)(intern_rigid_symbol_wrt_module("EQUALITY-TEST", NULL, 2)));
    KWD_STELLA_IN_INITIAL_VALUE = ((Keyword*)(intern_rigid_symbol_wrt_module("INITIAL-VALUE", NULL, 2)));
    KWD_STELLA_IN_PRINT_FORM = ((Keyword*)(intern_rigid_symbol_wrt_module("PRINT-FORM", NULL, 2)));
    KWD_STELLA_IN_PROPERTIES = ((Keyword*)(intern_rigid_symbol_wrt_module("PROPERTIES", NULL, 2)));
    KWD_STELLA_IN_META_ATTRIBUTES = ((Keyword*)(intern_rigid_symbol_wrt_module("META-ATTRIBUTES", NULL, 2)));
    KWD_STELLA_IN_CHILDREN = ((Keyword*)(intern_rigid_symbol_wrt_module("CHILDREN", NULL, 2)));
    KWD_STELLA_IN_INITIALLY = ((Keyword*)(intern_rigid_symbol_wrt_module("INITIALLY", NULL, 2)));
    KWD_STELLA_IN_DEFAULT = ((Keyword*)(intern_rigid_symbol_wrt_module("DEFAULT", NULL, 2)));
    SYM_STELLA_IN_STELLA_LIKE = ((Symbol*)(intern_rigid_symbol_wrt_module("LIKE", NULL, 0)));
    KWD_STELLA_IN_ALLOCATION = ((Keyword*)(intern_rigid_symbol_wrt_module("ALLOCATION", NULL, 2)));
    KWD_STELLA_IN_READER = ((Keyword*)(intern_rigid_symbol_wrt_module("READER", NULL, 2)));
    KWD_STELLA_IN_WRITER = ((Keyword*)(intern_rigid_symbol_wrt_module("WRITER", NULL, 2)));
    KWD_STELLA_IN_INVERSE = ((Keyword*)(intern_rigid_symbol_wrt_module("INVERSE", NULL, 2)));
    KWD_STELLA_IN_RENAMES = ((Keyword*)(intern_rigid_symbol_wrt_module("RENAMES", NULL, 2)));
    KWD_STELLA_IN_REQUIREDp = ((Keyword*)(intern_rigid_symbol_wrt_module("REQUIRED?", NULL, 2)));
    KWD_STELLA_IN_COMPONENTp = ((Keyword*)(intern_rigid_symbol_wrt_module("COMPONENT?", NULL, 2)));
    KWD_STELLA_IN_READ_ONLYp = ((Keyword*)(intern_rigid_symbol_wrt_module("READ-ONLY?", NULL, 2)));
    KWD_STELLA_IN_CONTEXT_SENSITIVEp = ((Keyword*)(intern_rigid_symbol_wrt_module("CONTEXT-SENSITIVE?", NULL, 2)));
    KWD_STELLA_IN_HARDWIREDp = ((Keyword*)(intern_rigid_symbol_wrt_module("HARDWIRED?", NULL, 2)));
    KWD_STELLA_IN_OPTION_KEYWORD = ((Keyword*)(intern_rigid_symbol_wrt_module("OPTION-KEYWORD", NULL, 2)));
    KWD_STELLA_IN_OPTION_HANDLER = ((Keyword*)(intern_rigid_symbol_wrt_module("OPTION-HANDLER", NULL, 2)));
    SYM_STELLA_IN_STELLA_SLOT_HOME_MODULE = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-HOME-MODULE", NULL, 0)));
    SYM_STELLA_IN_STELLA_SETTER = ((Symbol*)(intern_rigid_symbol_wrt_module("SETTER", NULL, 0)));
    KWD_STELLA_IN_RETURN_TYPES = ((Keyword*)(intern_rigid_symbol_wrt_module("RETURN-TYPES", NULL, 2)));
    KWD_STELLA_IN_STORAGE_SLOT = ((Keyword*)(intern_rigid_symbol_wrt_module("STORAGE-SLOT", NULL, 2)));
    KWD_STELLA_IN_INHERITS_THROUGH = ((Keyword*)(intern_rigid_symbol_wrt_module("INHERITS-THROUGH", NULL, 2)));
    KWD_STELLA_IN_SETTERp = ((Keyword*)(intern_rigid_symbol_wrt_module("SETTER?", NULL, 2)));
    KWD_STELLA_IN_NATIVEp = ((Keyword*)(intern_rigid_symbol_wrt_module("NATIVE?", NULL, 2)));
    KWD_STELLA_IN_CONSTRUCTORp = ((Keyword*)(intern_rigid_symbol_wrt_module("CONSTRUCTOR?", NULL, 2)));
    KWD_STELLA_IN_AUXILIARYp = ((Keyword*)(intern_rigid_symbol_wrt_module("AUXILIARY?", NULL, 2)));
  }
}

void startup_stella_in() {
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (current_startup_time_phaseP(2)) {
      help_startup_stella_in1();
      KWD_STELLA_IN_GLOBALLY_INLINEp = ((Keyword*)(intern_rigid_symbol_wrt_module("GLOBALLY-INLINE?", NULL, 2)));
      KWD_STELLA_IN_COMMANDp = ((Keyword*)(intern_rigid_symbol_wrt_module("COMMAND?", NULL, 2)));
      KWD_STELLA_IN_LISP_MACROp = ((Keyword*)(intern_rigid_symbol_wrt_module("LISP-MACRO?", NULL, 2)));
      KWD_STELLA_IN_EVALUATE_ARGUMENTSp = ((Keyword*)(intern_rigid_symbol_wrt_module("EVALUATE-ARGUMENTS?", NULL, 2)));
      KWD_STELLA_IN_INLINE = ((Keyword*)(intern_rigid_symbol_wrt_module("INLINE", NULL, 2)));
      SYM_STELLA_IN_STELLA_DEFUN = ((Symbol*)(intern_rigid_symbol_wrt_module("DEFUN", NULL, 0)));
      SYM_STELLA_IN_STELLA_SLOT_AUXILIARYp = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-AUXILIARY?", NULL, 0)));
      SYM_STELLA_IN_STELLA_METHOD_GLOBALLY_INLINEp = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-GLOBALLY-INLINE?", NULL, 0)));
      KWD_STELLA_IN_DYNAMIC = ((Keyword*)(intern_rigid_symbol_wrt_module("DYNAMIC", NULL, 2)));
      SGT_STELLA_IN_STELLA_DYNAMIC_SLOTS_MIXIN = ((Surrogate*)(intern_rigid_symbol_wrt_module("DYNAMIC-SLOTS-MIXIN", NULL, 1)));
      SYM_STELLA_IN_STELLA_SLOT_OPTION_KEYWORD = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-OPTION-KEYWORD", NULL, 0)));
      SYM_STELLA_IN_STELLA_DEFAULT_OPTION_HANDLER = ((Symbol*)(intern_rigid_symbol_wrt_module("DEFAULT-OPTION-HANDLER", NULL, 0)));
      SYM_STELLA_IN_STELLA_SLOT_OPTION_HANDLER = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-OPTION-HANDLER", NULL, 0)));
      SGT_STELLA_IN_STELLA_BOOLEAN = ((Surrogate*)(intern_rigid_symbol_wrt_module("BOOLEAN", NULL, 1)));
      SYM_STELLA_IN_STELLA_STARTUP_STELLA_IN = ((Symbol*)(intern_rigid_symbol_wrt_module("STARTUP-STELLA-IN", NULL, 0)));
      SYM_STELLA_IN_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-STARTUP-CLASSNAME", NULL, 0)));
    }
    if (current_startup_time_phaseP(4)) {
      oUNSUPPORTED_EXTERNAL_SLOT_OPTIONSo = list(4, KWD_STELLA_IN_ALLOCATION, KWD_STELLA_IN_INITIALLY, KWD_STELLA_IN_CONTEXT_SENSITIVEp, KWD_STELLA_IN_HARDWIREDp);
    }
    if (current_startup_time_phaseP(6)) {
      finalize_classes();
    }
    if (current_startup_time_phaseP(7)) {
      define_function_object("PERMANENTIFY-FORM", "(DEFUN (PERMANENTIFY-FORM OBJECT) ((FORM OBJECT)) :PUBLIC? TRUE)", ((cpp_function_code)(&permanentify_form)), NULL);
      define_function_object("TRANSIENTIFY-FORM", "(DEFUN (TRANSIENTIFY-FORM OBJECT) ((FORM OBJECT)) :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (COPY-CONS-TREE FORM)))", ((cpp_function_code)(&transientify_form)), NULL);
      define_function_object("STRINGIFY-IN-MODULE", "(DEFUN (STRINGIFY-IN-MODULE STRING) ((TREE OBJECT) (MODULE MODULE)) :DOCUMENTATION \"Stringify a parse `tree' relative to `module', or\n`*module*' if no module is specified.\" :PUBLIC? TRUE)", ((cpp_function_code)(&stringify_in_module)), NULL);
      define_function_object("UNSTRINGIFY-IN-MODULE", "(DEFUN (UNSTRINGIFY-IN-MODULE OBJECT) ((STRING STRING) (MODULE MODULE)) :DOCUMENTATION \"Unstringify relative to `module', or `*MODULE*' if no\nmodule is specified.\" :PUBLIC? TRUE)", ((cpp_function_code)(&unstringify_in_module)), NULL);
      define_function_object("UNSTRINGIFY-STELLA-SOURCE", "(DEFUN (UNSTRINGIFY-STELLA-SOURCE OBJECT) ((SOURCE STRING) (MODULE MODULE)) :DOCUMENTATION \"Unstringify a STELLA `source' string relative to `module',\nor `*MODULE*' if no module is specified.  This function allocates transient\nobjects as opposed to `unstringify-in-module' or the regular `unstringify'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&unstringify_stella_source)), NULL);
      define_function_object("COLLECT-FEATURE-LIST", "(DEFUN (COLLECT-FEATURE-LIST (LIST OF GENERALIZED-SYMBOL)) ((TREE OBJECT)))", ((cpp_function_code)(&collect_feature_list)), NULL);
      define_function_object("COLLECT-KEY-VALUE-LIST", "(DEFUN (COLLECT-KEY-VALUE-LIST (KEY-VALUE-LIST OF GENERALIZED-SYMBOL OBJECT)) ((TREE OBJECT)))", ((cpp_function_code)(&collect_key_value_list)), NULL);
      define_function_object("DEFINE-INLINE-METHOD", "(DEFUN (DEFINE-INLINE-METHOD METHOD-SLOT) ((NAME SYMBOL) (PARAMETERSANDBODY CONS)))", ((cpp_function_code)(&define_inline_method)), NULL);
      define_function_object("HELP-DEFINE-CLASS-FROM-PARSE-TREE", "(DEFUN (HELP-DEFINE-CLASS-FROM-PARSE-TREE CLASS) ((PARSETREE CONS) (STRINGIFIEDSOURCE STRING)))", ((cpp_function_code)(&help_define_class_from_parse_tree)), NULL);
      define_function_object("DEFINE-STORAGE-SLOT-FROM-PARSE-TREE", "(DEFUN (DEFINE-STORAGE-SLOT-FROM-PARSE-TREE STORAGE-SLOT) ((SLOTDEF CONS) (OWNER TYPE)))", ((cpp_function_code)(&define_storage_slot_from_parse_tree)), NULL);
      define_function_object("RECORD-SLOT-HOME-MODULE", "(DEFUN (RECORD-SLOT-HOME-MODULE MODULE) ((SELF SLOT)))", ((cpp_function_code)(&record_slot_home_module)), NULL);
      define_function_object("EXPLICITLY-QUALIFIED-NAME?", "(DEFUN (EXPLICITLY-QUALIFIED-NAME? BOOLEAN) ((NAME GENERALIZED-SYMBOL) (DEFINITIONMODULE MODULE)))", ((cpp_function_code)(&explicitly_qualified_nameP)), NULL);
      define_function_object("GET-IDENTICAL-CLASS", "(DEFUN (GET-IDENTICAL-CLASS CLASS) ((CLASSNAME STRING) (STRINGIFIEDSOURCE STRING)) :PUBLIC? TRUE)", ((cpp_function_code)(&get_identical_class)), NULL);
      define_function_object("DEFINE-CLASS-FROM-PARSE-TREE", "(DEFUN (DEFINE-CLASS-FROM-PARSE-TREE CLASS) ((PARSETREE CONS)) :PUBLIC? TRUE)", ((cpp_function_code)(&define_class_from_parse_tree)), NULL);
      define_function_object("DEFINE-CLASS-FROM-STRINGIFIED-SOURCE", "(DEFUN (DEFINE-CLASS-FROM-STRINGIFIED-SOURCE CLASS) ((CLASSNAME STRING) (STRINGIFIEDSOURCE STRING)) :PUBLIC? TRUE)", ((cpp_function_code)(&define_class_from_stringified_source)), NULL);
      define_function_object("EXTRACT-OPTIONS", "(DEFUN (EXTRACT-OPTIONS CONS) ((TREE CONS) (OPTIONSTABLE KEY-VALUE-LIST)) :PUBLIC? TRUE)", ((cpp_function_code)(&extract_options)), NULL);
      define_function_object("DESTRUCTURE-DEFMETHOD-TREE", "(DEFUN (DESTRUCTURE-DEFMETHOD-TREE OBJECT CONS CONS) ((METHOD-TREE CONS) (OPTIONS-TABLE KEY-VALUE-LIST)) :PUBLIC? TRUE :DOCUMENTATION \"Return three parse trees representing the name,\nparameters, and code body of the parse tree `method-tree'.  Fill\n`options-table' with a dictionary of method options.\nStorage note:  Options are treated specially because the other return\nvalues are subtrees of `method-tree', while `options-table' is a newly-created\ncons tree.  Note also, the parameter and body trees are destructively\nremoved from `method-tree'.\")", ((cpp_function_code)(&destructure_defmethod_tree)), NULL);
      define_function_object("YIELD-SETTER-METHOD-NAME", "(DEFUN (YIELD-SETTER-METHOD-NAME SYMBOL) ((METHODNAME SYMBOL)))", ((cpp_function_code)(&yield_setter_method_name)), NULL);
      define_function_object("DESTRUCTURE-METHOD-NAME-TREE", "(DEFUN (DESTRUCTURE-METHOD-NAME-TREE SYMBOL CONS) ((NAMETREE OBJECT)))", ((cpp_function_code)(&destructure_method_name_tree)), NULL);
      define_function_object("IN-PLACE-NORMALIZE-METHOD-OPTIONS", "(DEFUN IN-PLACE-NORMALIZE-METHOD-OPTIONS ((METHODOPTIONS KEYWORD-KEY-VALUE-LIST) (METHODNAME SYMBOL)))", ((cpp_function_code)(&in_place_normalize_method_options)), NULL);
      define_function_object("GET-FUNCTION-WITH-IDENTICAL-SIGNATURE", "(DEFUN (GET-FUNCTION-WITH-IDENTICAL-SIGNATURE METHOD-SLOT) ((NAME SYMBOL) (STRINGIFIEDSOURCE STRING)))", ((cpp_function_code)(&get_function_with_identical_signature)), NULL);
      define_function_object("GET-METHOD-WITH-IDENTICAL-SIGNATURE", "(DEFUN (GET-METHOD-WITH-IDENTICAL-SIGNATURE METHOD-SLOT) ((NAME SYMBOL) (PARAMETERSTREE CONS) (STRINGIFIEDSOURCE STRING)))", ((cpp_function_code)(&get_method_with_identical_signature)), NULL);
      define_function_object("HELP-DEFINE-METHOD-FROM-PARSE-TREE", "(DEFUN (HELP-DEFINE-METHOD-FROM-PARSE-TREE METHOD-SLOT BOOLEAN) ((INPUTTREE CONS) (STRINGIFIEDSOURCE STRING)))", ((cpp_function_code)(&help_define_method_from_parse_tree)), NULL);
      define_function_object("DEFINE-METHOD-FROM-PARSE-TREE", "(DEFUN (DEFINE-METHOD-FROM-PARSE-TREE METHOD-SLOT) ((PARSETREE CONS)))", ((cpp_function_code)(&define_method_from_parse_tree)), NULL);
      define_function_object("STRINGIFY-METHOD-SIGNATURE", "(DEFUN (STRINGIFY-METHOD-SIGNATURE STRING) ((PARSETREE CONS)))", ((cpp_function_code)(&stringify_method_signature)), NULL);
      define_function_object("STRINGIFY-METHOD-BODY?", "(DEFUN (STRINGIFY-METHOD-BODY? BOOLEAN) ((METHOD METHOD-SLOT)))", ((cpp_function_code)(&stringify_method_bodyP)), NULL);
      define_function_object("DEFINE-METHOD-FROM-STRINGIFIED-SOURCE", "(DEFUN (DEFINE-METHOD-FROM-STRINGIFIED-SOURCE METHOD-SLOT) ((METHODNAME STRING) (CLASSNAME STRING) (STRINGIFIEDSOURCE STRING)) :PUBLIC? TRUE)", ((cpp_function_code)(&define_method_from_stringified_source)), NULL);
      define_function_object("DEFINE-METHOD-OBJECT", "(DEFUN DEFINE-METHOD-OBJECT ((DEFINITION STRING) (CODE METHOD-CODE) (WRAPPERCODE METHOD-CODE)))", ((cpp_function_code)(&define_method_object)), NULL);
      define_function_object("DEFINE-EXTERNAL-SLOT-FROM-PARSE-TREE", "(DEFUN (DEFINE-EXTERNAL-SLOT-FROM-PARSE-TREE STORAGE-SLOT) ((PARSETREE CONS)))", ((cpp_function_code)(&define_external_slot_from_parse_tree)), NULL);
      define_function_object("DEFINE-EXTERNAL-SLOT-FROM-STRINGIFIED-SOURCE", "(DEFUN (DEFINE-EXTERNAL-SLOT-FROM-STRINGIFIED-SOURCE STORAGE-SLOT) ((STRINGIFIEDSOURCE STRING)) :PUBLIC? TRUE)", ((cpp_function_code)(&define_external_slot_from_stringified_source)), NULL);
      define_function_object("LOOKUP-SLOT-FROM-OPTION-KEYWORD", "(DEFUN (LOOKUP-SLOT-FROM-OPTION-KEYWORD STORAGE-SLOT) ((OWNER TYPE) (KEYWORD KEYWORD)))", ((cpp_function_code)(&lookup_slot_from_option_keyword)), NULL);
      define_function_object("LOOKUP-OPTION-HANDLER", "(DEFUN (LOOKUP-OPTION-HANDLER METHOD-SLOT) ((SLOT STORAGE-SLOT)) :PUBLIC? TRUE)", ((cpp_function_code)(&lookup_option_handler)), NULL);
      define_function_object("RUN-OPTION-HANDLER?", "(DEFUN (RUN-OPTION-HANDLER? BOOLEAN) ((OBJECT OBJECT) (OPTION KEYWORD) (VALUETREE OBJECT)) :PUBLIC? TRUE)", ((cpp_function_code)(&run_option_handlerP)), NULL);
      define_function_object("DEFAULT-OPTION-HANDLER", "(DEFUN DEFAULT-OPTION-HANDLER ((SELF OBJECT) (SLOT STORAGE-SLOT) (TREE OBJECT)))", ((cpp_function_code)(&default_option_handler)), NULL);
      define_function_object("STARTUP-STELLA-IN", "(DEFUN STARTUP-STELLA-IN () :PUBLIC? TRUE)", ((cpp_function_code)(&startup_stella_in)), NULL);
      { Method_Slot* function = lookup_function(SYM_STELLA_IN_STELLA_STARTUP_STELLA_IN);

        set_dynamic_slot_value(function->dynamic_slots, SYM_STELLA_IN_STELLA_METHOD_STARTUP_CLASSNAME, wrap_string("Startup-Stella-In"), NULL_STRING_WRAPPER);
      }
    }
    if (current_startup_time_phaseP(8)) {
      finalize_slots();
      cleanup_unfinalized_classes();
    }
    if (current_startup_time_phaseP(9)) {
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *REDEFINE-IDENTICAL-CLASSES?* BOOLEAN FALSE :DOCUMENTATION \"If `true', always redefine classes, even if an\n                  identical class exists\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *UNSUPPORTED-EXTERNAL-SLOT-OPTIONS* (LIST OF KEYWORD) (LIST :ALLOCATION :INITIALLY :CONTEXT-SENSITIVE? :HARDWIRED?))");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *DEFAULT-OPTION-HANDLER* METHOD-SLOT NULL)");
      oDEFAULT_OPTION_HANDLERo = lookup_function(SYM_STELLA_IN_STELLA_DEFAULT_OPTION_HANDLER);
    }
  }
}

Symbol* SYM_STELLA_IN_STELLA_TRUE = NULL;

Symbol* SYM_STELLA_IN_STELLA_FALSE = NULL;

Surrogate* SGT_STELLA_IN_STELLA_CONS = NULL;

Surrogate* SGT_STELLA_IN_STELLA_GENERALIZED_SYMBOL = NULL;

Symbol* SYM_STELLA_IN_STELLA_DEFMETHOD = NULL;

Keyword* KWD_STELLA_IN_PUBLIC_SLOTS = NULL;

Keyword* KWD_STELLA_IN_SLOTS = NULL;

Keyword* KWD_STELLA_IN_PUBLIC_METHODS = NULL;

Keyword* KWD_STELLA_IN_METHODS = NULL;

Keyword* KWD_STELLA_IN_PARAMETERS = NULL;

Keyword* KWD_STELLA_IN_KEY = NULL;

Keyword* KWD_STELLA_IN_SYNONYMS = NULL;

Keyword* KWD_STELLA_IN_DOCUMENTATION = NULL;

Keyword* KWD_STELLA_IN_CL_NATIVE_TYPE = NULL;

Keyword* KWD_STELLA_IN_CPP_NATIVE_TYPE = NULL;

Keyword* KWD_STELLA_IN_JAVA_NATIVE_TYPE = NULL;

Keyword* KWD_STELLA_IN_IDL_NATIVE_TYPE = NULL;

Keyword* KWD_STELLA_IN_ABSTRACTp = NULL;

Keyword* KWD_STELLA_IN_ACTIVEp = NULL;

Keyword* KWD_STELLA_IN_PUBLICp = NULL;

Keyword* KWD_STELLA_IN_CL_STRUCTp = NULL;

Keyword* KWD_STELLA_IN_MIXINp = NULL;

Keyword* KWD_STELLA_IN_RECYCLE_METHOD = NULL;

Symbol* SYM_STELLA_IN_STELLA_NEXT_SWEEP_LIST_OBJECT = NULL;

Keyword* KWD_STELLA_IN_TYPE = NULL;

Keyword* KWD_STELLA_IN_EXTENSION = NULL;

Keyword* KWD_STELLA_IN_CREATOR = NULL;

Keyword* KWD_STELLA_IN_DESTRUCTOR = NULL;

Keyword* KWD_STELLA_IN_INITIALIZER = NULL;

Keyword* KWD_STELLA_IN_TERMINATOR = NULL;

Keyword* KWD_STELLA_IN_EQUALITY_TEST = NULL;

Keyword* KWD_STELLA_IN_INITIAL_VALUE = NULL;

Keyword* KWD_STELLA_IN_PRINT_FORM = NULL;

Keyword* KWD_STELLA_IN_PROPERTIES = NULL;

Keyword* KWD_STELLA_IN_META_ATTRIBUTES = NULL;

Keyword* KWD_STELLA_IN_CHILDREN = NULL;

Keyword* KWD_STELLA_IN_INITIALLY = NULL;

Keyword* KWD_STELLA_IN_DEFAULT = NULL;

Symbol* SYM_STELLA_IN_STELLA_LIKE = NULL;

Keyword* KWD_STELLA_IN_ALLOCATION = NULL;

Keyword* KWD_STELLA_IN_READER = NULL;

Keyword* KWD_STELLA_IN_WRITER = NULL;

Keyword* KWD_STELLA_IN_INVERSE = NULL;

Keyword* KWD_STELLA_IN_RENAMES = NULL;

Keyword* KWD_STELLA_IN_REQUIREDp = NULL;

Keyword* KWD_STELLA_IN_COMPONENTp = NULL;

Keyword* KWD_STELLA_IN_READ_ONLYp = NULL;

Keyword* KWD_STELLA_IN_CONTEXT_SENSITIVEp = NULL;

Keyword* KWD_STELLA_IN_HARDWIREDp = NULL;

Keyword* KWD_STELLA_IN_OPTION_KEYWORD = NULL;

Keyword* KWD_STELLA_IN_OPTION_HANDLER = NULL;

Symbol* SYM_STELLA_IN_STELLA_SLOT_HOME_MODULE = NULL;

Symbol* SYM_STELLA_IN_STELLA_SETTER = NULL;

Keyword* KWD_STELLA_IN_RETURN_TYPES = NULL;

Keyword* KWD_STELLA_IN_STORAGE_SLOT = NULL;

Keyword* KWD_STELLA_IN_INHERITS_THROUGH = NULL;

Keyword* KWD_STELLA_IN_SETTERp = NULL;

Keyword* KWD_STELLA_IN_NATIVEp = NULL;

Keyword* KWD_STELLA_IN_CONSTRUCTORp = NULL;

Keyword* KWD_STELLA_IN_AUXILIARYp = NULL;

Keyword* KWD_STELLA_IN_GLOBALLY_INLINEp = NULL;

Keyword* KWD_STELLA_IN_COMMANDp = NULL;

Keyword* KWD_STELLA_IN_LISP_MACROp = NULL;

Keyword* KWD_STELLA_IN_EVALUATE_ARGUMENTSp = NULL;

Keyword* KWD_STELLA_IN_INLINE = NULL;

Symbol* SYM_STELLA_IN_STELLA_DEFUN = NULL;

Symbol* SYM_STELLA_IN_STELLA_SLOT_AUXILIARYp = NULL;

Symbol* SYM_STELLA_IN_STELLA_METHOD_GLOBALLY_INLINEp = NULL;

Keyword* KWD_STELLA_IN_DYNAMIC = NULL;

Surrogate* SGT_STELLA_IN_STELLA_DYNAMIC_SLOTS_MIXIN = NULL;

Symbol* SYM_STELLA_IN_STELLA_SLOT_OPTION_KEYWORD = NULL;

Symbol* SYM_STELLA_IN_STELLA_DEFAULT_OPTION_HANDLER = NULL;

Symbol* SYM_STELLA_IN_STELLA_SLOT_OPTION_HANDLER = NULL;

Surrogate* SGT_STELLA_IN_STELLA_BOOLEAN = NULL;

Symbol* SYM_STELLA_IN_STELLA_STARTUP_STELLA_IN = NULL;

Symbol* SYM_STELLA_IN_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella
