// symbols.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

int next_free_offset(Extensible_Symbol_Array* self) {
  { int topoffset = self->top_symbol_offset;
    int freeoffset = topoffset + 1;

    { int i = NULL_INTEGER;
      int iter_000 = self->potential_free_symbol_offset;
      int upper_bound_000 = topoffset;
      boolean unboundedP_000 = upper_bound_000 == NULL_INTEGER;

      while (unboundedP_000 ||
          (iter_000 <= upper_bound_000)) {
        i = iter_000;
        iter_000 = iter_000 + 1;
        if (!((boolean)((self->the_array)[i]))) {
          freeoffset = i;
          break;
        }
      }
    }
    self->potential_free_symbol_offset = freeoffset + 1;
    return (freeoffset);
  }
}

void free_symbol_offset(Extensible_Symbol_Array* self, int offset) {
  (self->the_array)[offset] = NULL;
  if (offset < self->potential_free_symbol_offset) {
    self->potential_free_symbol_offset = offset;
  }
}

void add_to_symbol_array_at(Extensible_Symbol_Array* array, int offset, Generalized_Symbol* symbol) {
  array->insert_at(offset, symbol);
  symbol->symbol_id = offset;
}

int add_to_symbol_array(Extensible_Symbol_Array* array, Generalized_Symbol* symbol) {
  { int offset = next_free_offset(array);

    add_to_symbol_array_at(array, offset, symbol);
    return (offset);
  }
}

boolean legal_symbol_array_offsetP(Extensible_Symbol_Array* array, int offset) {
  return (offset <= array->top_symbol_offset);
}

Extensible_Symbol_Array* oSYMBOL_ARRAYo = NULL;

Extensible_Symbol_Array* oSURROGATE_ARRAYo = NULL;

Extensible_Symbol_Array* oKEYWORD_ARRAYo = NULL;

String_To_Integer_Hash_Table* oKEYWORD_OFFSET_TABLEo = NULL;

Extensible_Symbol_Array* oFIXED_SYMBOL_ARRAYo = NULL;

Extensible_Symbol_Array* oFIXED_SURROGATE_ARRAYo = NULL;

Extensible_Symbol_Array* oFIXED_KEYWORD_ARRAYo = NULL;

String_Hash_Table* oTRANSIENT_SYMBOL_LOOKUP_TABLEo = NULL;

String_Hash_Table* oCOMMON_LISP_SYMBOL_LOOKUP_TABLEo = NULL;

int SYMBOL_SYM = 0;

int SURROGATE_SYM = 1;

int KEYWORD_SYM = 2;

// Pointer to the top-most module ROOT-MODULE.
Module* oROOT_MODULEo = NULL;

// Pointer to the module STELLA containing definitions of 
// STELLA classes and methods.
Module* oSTELLA_MODULEo = NULL;

// Pointer to the module COMMON-LISP containing Common
// Lisp symbols (and maybe other things).
Module* oCOMMON_LISP_MODULEo = NULL;

// Pointer to the module ALTERNATE-STELLA containing
// definitions of STELLA classes and methods generated during the
// translation of STELLA files.
Module* oALTERNATE_STELLA_MODULEo = NULL;

// Points to the current context (either a module or a
// world).  Set by calls to `change-world' and `change-module'.
DEFINE_STELLA_SPECIAL(oCONTEXTo, Context* , NULL);

// Points to the current module.  Set by calls to
// `change-context' and `change-module'.
DEFINE_STELLA_SPECIAL(oMODULEo, Module* , NULL);

// Character used to prefix the name of a surrogate.
char SURROGATE_PREFIX_CHARACTER = '@';

// Character used to concatenate a module name
// to a symbol or surrogate name to form a qualified name.
char MODULE_SEPARATOR_CHARACTER = '/';

// String containing character used to concatenate a module name
// to a symbol or surrogate name to form a qualified name.
char* MODULE_SEPARATOR_STRING = "/";

Symbol* get_sym(int offset) {
  return (((Symbol*)((oFIXED_SYMBOL_ARRAYo->the_array)[offset])));
}

Surrogate* get_sgt(int offset) {
  return (((Surrogate*)((oFIXED_SURROGATE_ARRAYo->the_array)[offset])));
}

Keyword* get_kwd(int offset) {
  return (((Keyword*)((oFIXED_KEYWORD_ARRAYo->the_array)[offset])));
}

Generalized_Symbol* get_generalized_symbol_from_offset(Extensible_Symbol_Array* symbolarray, int offset) {
  return (((Generalized_Symbol*)((symbolarray->the_array)[offset])));
}

Symbol* get_sym_from_offset(int offset) {
  return (((Symbol*)(get_generalized_symbol_from_offset(oSYMBOL_ARRAYo, offset))));
}

Surrogate* get_sgt_from_offset(int offset) {
  return (((Surrogate*)(get_generalized_symbol_from_offset(oSURROGATE_ARRAYo, offset))));
}

Keyword* get_kwd_from_offset(int offset) {
  return (((Keyword*)(get_generalized_symbol_from_offset(oKEYWORD_ARRAYo, offset))));
}

String_To_Integer_Hash_Table* select_symbol_offset_table(Module* module, int kindofsym) {
  switch (kindofsym) {
    case 0: 
      return (module->symbol_offset_table);
    case 1: 
      return (module->surrogate_offset_table);
    case 2: 
      return (oKEYWORD_OFFSET_TABLEo);
    default:
      { Output_String_Stream* stream_000 = new_output_string_stream();

        *(stream_000->native_stream) << "`" << kindofsym << "'" << " is not a valid case option";
        throw *new_stella_exception(stream_000->the_string_reader());
      }
    break;
  }
}

Extensible_Symbol_Array* select_symbol_array(int kindofsym) {
  switch (kindofsym) {
    case 0: 
      return (oSYMBOL_ARRAYo);
    case 1: 
      return (oSURROGATE_ARRAYo);
    case 2: 
      return (oKEYWORD_ARRAYo);
    default:
      { Output_String_Stream* stream_000 = new_output_string_stream();

        *(stream_000->native_stream) << "`" << kindofsym << "'" << " is not a valid case option";
        throw *new_stella_exception(stream_000->the_string_reader());
      }
    break;
  }
}

Generalized_Symbol* lookup_rigid_symbol_locally(char* name, Module* module, int kindofsym) {
  { String_To_Integer_Hash_Table* offsettable = select_symbol_offset_table(module, kindofsym);
    int offset = offsettable->lookup(name);

    if (offset != NULL_INTEGER) {
      switch (kindofsym) {
        case 0: 
          return (get_sym_from_offset(offset));
        case 1: 
          return (get_sgt_from_offset(offset));
        case 2: 
          return (get_kwd_from_offset(offset));
        default:
          { Output_String_Stream* stream_000 = new_output_string_stream();

            *(stream_000->native_stream) << "`" << kindofsym << "'" << " is not a valid case option";
            throw *new_stella_exception(stream_000->the_string_reader());
          }
        break;
      }
    }
    return (NULL);
  }
}

Cons* yield_visible_rigid_symbols_wrt_module(char* name, Module* module, int kindofsym) {
  { Cons* listofsymbols = NIL;
    Generalized_Symbol* symbol = NULL;

    { Module* visiblemodule = NULL;
      Iterator* iter_000 = visible_modules(module);

      while (iter_000->nextP()) {
        visiblemodule = ((Module*)(iter_000->value));
        { 
          symbol = lookup_rigid_symbol_locally(name, visiblemodule, kindofsym);
          if (((boolean)(symbol))) {
            listofsymbols = cons(symbol, listofsymbols);
          }
        }
      }
    }
    return (listofsymbols->reverse());
  }
}

Iterator* visible_rigid_symbols_wrt_module(char* name, Module* module, int kindofsym) {
  return (yield_visible_rigid_symbols_wrt_module(name, module, kindofsym)->allocate_destructive_list_iterator());
}

Cons* remove_shadowed_symbols(Cons* visiblesymbols) {
  if (visiblesymbols->rest == NIL) {
    return (visiblesymbols);
  }
  { Cons* shadowingmodules = NIL;

    { Generalized_Symbol* symbol = NULL;
      Cons* iter_000 = visiblesymbols;
      Cons* collect_000 = NULL;

      while (!(iter_000 == NIL)) {
        symbol = ((Generalized_Symbol*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (shadowed_symbolP(symbol)) {
          if (!((boolean)(collect_000))) {
            {
              collect_000 = cons(((Module*)(symbol->home_context)), NIL);
              if (shadowingmodules == NIL) {
                shadowingmodules = collect_000;
              }
              else {
                add_cons_to_end_of_cons_list(shadowingmodules, collect_000);
              }
            }
          }
          else {
            {
              collect_000->rest = cons(((Module*)(symbol->home_context)), NIL);
              collect_000 = collect_000->rest;
            }
          }
        }
      }
    }
    if (!(shadowingmodules == NIL)) {
      { Cons* unshadowedsymbols = NIL;

        { Generalized_Symbol* symbol = NULL;
          Cons* iter_001 = visiblesymbols;
          Cons* collect_001 = NULL;

          while (!(iter_001 == NIL)) {
            symbol = ((Generalized_Symbol*)(iter_001->value));
            iter_001 = iter_001->rest;
            { boolean test_value_000 = FALSE;

              { boolean foundP_000 = FALSE;

                { Module* module = NULL;
                  Cons* iter_002 = shadowingmodules;

                  while (!(iter_002 == NIL)) {
                    module = ((Module*)(iter_002->value));
                    iter_002 = iter_002->rest;
                    if ((!(module == ((Module*)(symbol->home_context)))) &&
                        visible_fromP(((Module*)(symbol->home_context)), module)) {
                      foundP_000 = TRUE;
                      break;
                    }
                  }
                }
                test_value_000 = foundP_000;
              }
              test_value_000 = !test_value_000;
              if (test_value_000) {
                if (!((boolean)(collect_001))) {
                  {
                    collect_001 = cons(symbol, NIL);
                    if (unshadowedsymbols == NIL) {
                      unshadowedsymbols = collect_001;
                    }
                    else {
                      add_cons_to_end_of_cons_list(unshadowedsymbols, collect_001);
                    }
                  }
                }
                else {
                  {
                    collect_001->rest = cons(symbol, NIL);
                    collect_001 = collect_001->rest;
                  }
                }
              }
            }
          }
        }
        visiblesymbols = unshadowedsymbols;
      }
    }
    return (visiblesymbols);
  }
}

int lookup_rigid_symbol_offset_wrt_module(char* name, Module* module, int kindofsym) {
  { String_To_Integer_Hash_Table* offsettable = select_symbol_offset_table(module, kindofsym);
    int offset = offsettable->lookup(name);

    if (offset != NULL_INTEGER) {
      return (offset);
    }
    { Module* super = NULL;
      Iterator* iter_000 = visible_modules(module)->advance(1);

      while (iter_000->nextP()) {
        super = ((Module*)(iter_000->value));
        offset = select_symbol_offset_table(super, kindofsym)->lookup(name);
        if (offset != NULL_INTEGER) {
          return (offset);
        }
      }
    }
    return (NULL_INTEGER);
  }
}

Generalized_Symbol* lookup_rigid_symbol_wrt_module(char* name, Module* module, int kindofsym) {
  { int offset = lookup_rigid_symbol_offset_wrt_module(name, module, kindofsym);

    if (offset != NULL_INTEGER) {
      switch (kindofsym) {
        case 0: 
          return (get_sym_from_offset(offset));
        case 1: 
          return (get_sgt_from_offset(offset));
        case 2: 
          return (get_kwd_from_offset(offset));
        default:
          { Output_String_Stream* stream_000 = new_output_string_stream();

            *(stream_000->native_stream) << "`" << kindofsym << "'" << " is not a valid case option";
            throw *new_stella_exception(stream_000->the_string_reader());
          }
        break;
      }
    }
    else {
      return (NULL);
    }
  }
}

Generalized_Symbol* lookup_rigid_symbol(char* name, int kindofsym) {
  // Return the symbol named `name'.  Case sensitive.
  { Module* module = oMODULEo.get();

    if (qualified_nameP(name)) {
      module = compute_module_and_bare_name(name, name);
    }
    if (((boolean)(module))) {
      return (lookup_rigid_symbol_wrt_module(name, module, kindofsym));
    }
    else {
      return (NULL);
    }
  }
}

Symbol* lookup_symbol(char* name) {
  // Return the first symbol with `name' visible from the current module.
  return (((Symbol*)(lookup_rigid_symbol(name, SYMBOL_SYM))));
}

Symbol* lookup_symbol_in_module(char* name, Module* module, boolean localP) {
  // Return the first symbol with `name' visible from `module'.
  // If `local?' only consider symbols directly interned in `module'.
  // If `module' is `null', use `*MODULE*' instead.
  if (!((boolean)(module))) {
    module = oMODULEo.get();
  }
  if (localP) {
    return (((Symbol*)(lookup_rigid_symbol_locally(name, module, SYMBOL_SYM))));
  }
  else {
    return (((Symbol*)(lookup_rigid_symbol_wrt_module(name, module, SYMBOL_SYM))));
  }
}

Cons* lookup_visible_symbols_in_module(char* name, Module* module, boolean enforceshadowingP) {
  // Return the list of symbols with `name' visible from `module'.
  // More specific symbols (relative to the module precedence order defined by
  // `visible-modules') come earlier in the list.  If `module' is `null', start
  // from `*MODULE*' instead.  If `enforceShadowing?' is true, do not return any
  // symbols that are shadowed due to some :SHADOW declaration.
  { Cons* visiblesymbols = yield_visible_rigid_symbols_wrt_module(name, module, SYMBOL_SYM);

    if (enforceshadowingP &&
        (!(visiblesymbols->rest == NIL))) {
      return (((Cons*)(remove_shadowed_symbols(visiblesymbols))));
    }
    else {
      return (((Cons*)(visiblesymbols)));
    }
  }
}

Surrogate* lookup_surrogate(char* name) {
  // Return the first surrogate with `name' visible from the current module.
  return (((Surrogate*)(lookup_rigid_symbol(name, SURROGATE_SYM))));
}

Surrogate* lookup_surrogate_in_module(char* name, Module* module, boolean localP) {
  // Return the first surrogate with `name' visible from `module'.
  // If `local?' only consider surrogates directly interned in `module'.
  // If `module' is `null', use `*MODULE*' instead.
  if (!((boolean)(module))) {
    module = oMODULEo.get();
  }
  if (localP) {
    return (((Surrogate*)(lookup_rigid_symbol_locally(name, module, SURROGATE_SYM))));
  }
  else {
    return (((Surrogate*)(lookup_rigid_symbol_wrt_module(name, module, SURROGATE_SYM))));
  }
}

Cons* lookup_visible_surrogates_in_module(char* name, Module* module, boolean enforceshadowingP) {
  // Return the list of surrogates with `name' visible from `module'.
  // More specific surrogates (relative to the module precedence order defined by
  // `visible-modules') come earlier in the list.  If `module' is `null', start
  // from `*MODULE*' instead.  If `enforceShadowing?' is true, do not return any
  // surrogates that are shadowed due to some :SHADOW declaration.
  { Cons* visiblesurrogates = yield_visible_rigid_symbols_wrt_module(name, module, SURROGATE_SYM);

    if (enforceshadowingP &&
        (!(visiblesurrogates->rest == NIL))) {
      return (((Cons*)(remove_shadowed_symbols(visiblesurrogates))));
    }
    else {
      return (((Cons*)(visiblesurrogates)));
    }
  }
}

Keyword* lookup_keyword(char* name) {
  // Return the keyword with `name' if it exists.
  return (((Keyword*)(lookup_rigid_symbol(name, KEYWORD_SYM))));
}

Generalized_Symbol* help_intern_generalized_symbol(char* name, int kindofsym, Extensible_Symbol_Array* array, int offset, Module* module) {
  { Generalized_Symbol* symbol = NULL;
    String_To_Integer_Hash_Table* table = select_symbol_offset_table(module, kindofsym);

    switch (kindofsym) {
      case 0: 
        symbol = new_symbol(name);
        table->insert_at(name, offset);
        symbol->home_context = module;
      break;
      case 1: 
        symbol = new_surrogate(name);
        table->insert_at(name, offset);
        symbol->home_context = module;
      break;
      case 2: 
        symbol = new_keyword(name);
        table->insert_at(name, offset);
        symbol->home_context = NULL;
      break;
      default:
        { Output_String_Stream* stream_000 = new_output_string_stream();

          *(stream_000->native_stream) << "`" << kindofsym << "'" << " is not a valid case option";
          throw *new_stella_exception(stream_000->the_string_reader());
        }
      break;
    }
    add_to_symbol_array_at(array, offset, symbol);
    return (symbol);
  }
}

Generalized_Symbol* intern_rigid_symbol_wrt_module(char* name, Module* module, int kindofsym) {
  // Return a newly-created or existing rigid symbol with
  // name `name'.
  if (!((boolean)(module))) {
    module = oMODULEo.get();
  }
  { int offset = lookup_rigid_symbol_offset_wrt_module(name, module, kindofsym);

    if (offset != NULL_INTEGER) {
      switch (kindofsym) {
        case 0: 
          return (get_sym_from_offset(offset));
        case 1: 
          return (get_sgt_from_offset(offset));
        case 2: 
          return (get_kwd_from_offset(offset));
        default:
          { Output_String_Stream* stream_000 = new_output_string_stream();

            *(stream_000->native_stream) << "`" << kindofsym << "'" << " is not a valid case option";
            throw *new_stella_exception(stream_000->the_string_reader());
          }
        break;
      }
    }
    { Extensible_Symbol_Array* array = select_symbol_array(kindofsym);

      offset = next_free_offset(array);
      return (help_intern_generalized_symbol(name, kindofsym, array, offset, module));
    }
  }
}

Generalized_Symbol* intern_rigid_symbol_locally(char* name, Module* module, int kindofsym) {
  // Return a newly-created or existing rigid symbol
  // interned into the module `module' with name `name'.
  if (!((boolean)(module))) {
    module = oMODULEo.get();
  }
  { Generalized_Symbol* symbol = lookup_rigid_symbol_locally(name, module, kindofsym);

    if (((boolean)(symbol))) {
      return (symbol);
    }
    { Extensible_Symbol_Array* array = select_symbol_array(kindofsym);

      return (help_intern_generalized_symbol(name, kindofsym, array, next_free_offset(array), module));
    }
  }
}

Generalized_Symbol* intern_rigid_symbol_case_sensitively(char* name, int kindofsym, boolean tryupcasingP) {
  { Generalized_Symbol* symbol = NULL;

    symbol = lookup_rigid_symbol(name, kindofsym);
    if (((boolean)(symbol))) {
      return (symbol);
    }
    if (oTRANSIENTOBJECTSpo.get() &&
        (kindofsym == SYMBOL_SYM)) {
      symbol = lookup_transient_symbol(name);
      if (((boolean)(symbol))) {
        return (symbol);
      }
    }
    if (tryupcasingP) {
      symbol = lookup_rigid_symbol(string_upcase(name), kindofsym);
      if (((boolean)(symbol))) {
        return (symbol);
      }
    }
    switch (kindofsym) {
      case 0: 
        if (oTRANSIENTOBJECTSpo.get()) {
          return (intern_transient_symbol(name));
        }
        else {
          return (intern_permanent_symbol(name));
        }
      break;
      case 1: 
        return (intern_rigid_symbol_wrt_module(name, oMODULEo.get(), SURROGATE_SYM));
      default:
        { Output_String_Stream* stream_000 = new_output_string_stream();

          *(stream_000->native_stream) << "`" << kindofsym << "'" << " is not a valid case option";
          throw *new_stella_exception(stream_000->the_string_reader());
        }
      break;
    }
  }
}

Symbol* intern_permanent_symbol(char* name) {
  // Return a newly-created or existing permanent symbol with
  // name `name'.
  return (((Symbol*)(intern_rigid_symbol_wrt_module(name, oMODULEo.get(), SYMBOL_SYM))));
}

Symbol* intern_symbol(char* name) {
  // Return a newly-created or existing symbol with name `name'.
  if (oMODULEo.get()->case_sensitiveP) {
    return (((Symbol*)(intern_rigid_symbol_case_sensitively(name, SYMBOL_SYM, FALSE))));
  }
  else {
    if (oTRANSIENTOBJECTSpo.get()) {
      return (intern_transient_symbol(string_upcase(name)));
    }
    else {
      return (intern_permanent_symbol(string_upcase(name)));
    }
  }
}

Symbol* intern_symbol_in_module(char* name, Module* module, boolean localP) {
  // Look for a symbol named `name' in `module' (if `local?' do
  // not consider inherited modules).  If none exists, intern it locally in
  // `module'.  Return the existing or newly-created symbol.
  if (!((boolean)(module))) {
    module = oMODULEo.get();
  }
  if (localP) {
    return (((Symbol*)(intern_rigid_symbol_locally(name, module, SYMBOL_SYM))));
  }
  else {
    return (((Symbol*)(intern_rigid_symbol_wrt_module(name, module, SYMBOL_SYM))));
  }
}

Symbol* intern_derived_symbol(Generalized_Symbol* basesymbol, char* newname) {
  // Return a newly-created or existing symbol with name
  // `newName' which is interned in the same module as `baseSymbol'.
  { Module* currentmodule = oMODULEo.get();

    { 
      BIND_STELLA_SPECIAL(oMODULEo, Module*, ((Module*)(basesymbol->home_context)));
      if (((boolean)(oMODULEo.get()))) {
        return (((Symbol*)(intern_rigid_symbol_locally((oMODULEo.get()->case_sensitiveP ? newname : string_upcase(newname)), oMODULEo.get(), SYMBOL_SYM))));
      }
      oMODULEo.set(currentmodule);
      return (intern_symbol(newname));
    }
  }
}

Surrogate* intern_surrogate(char* name) {
  // Return a newly-created or existing surrogate with name `name'.
  if (oMODULEo.get()->case_sensitiveP) {
    return (((Surrogate*)(intern_rigid_symbol_case_sensitively(name, SURROGATE_SYM, FALSE))));
  }
  else {
    return (((Surrogate*)(intern_rigid_symbol_wrt_module(string_upcase(name), oMODULEo.get(), SURROGATE_SYM))));
  }
}

Surrogate* intern_surrogate_in_module(char* name, Module* module, boolean localP) {
  // Look for a symbol named `name' in `module' (if `local?' do
  // not consider inherited modules).  If none exists, intern it locally in
  // `module'.  Return the existing or newly-created symbol.
  if (!((boolean)(module))) {
    module = oMODULEo.get();
  }
  if (localP) {
    return (((Surrogate*)(intern_rigid_symbol_locally(name, module, SURROGATE_SYM))));
  }
  else {
    return (((Surrogate*)(intern_rigid_symbol_wrt_module(name, module, SURROGATE_SYM))));
  }
}

Surrogate* intern_derived_surrogate(Generalized_Symbol* basesymbol, char* newname) {
  // Return a newly-created or existing surrogate with name
  // `newName' which is interned in the same module as `baseSymbol'.
  { Module* currentmodule = oMODULEo.get();

    { 
      BIND_STELLA_SPECIAL(oMODULEo, Module*, ((Module*)(basesymbol->home_context)));
      if (((boolean)(oMODULEo.get()))) {
        return (((Surrogate*)(intern_rigid_symbol_locally((oMODULEo.get()->case_sensitiveP ? newname : string_upcase(newname)), oMODULEo.get(), SURROGATE_SYM))));
      }
      oMODULEo.set(currentmodule);
      return (intern_surrogate(newname));
    }
  }
}

Keyword* intern_keyword(char* name) {
  // Return a newly-created or existing keyword with name
  // `name'.  Storage note: a COPY of `name' is stored in the keyword
  return (((Keyword*)(intern_rigid_symbol_wrt_module(string_upcase(name), oMODULEo.get(), KEYWORD_SYM))));
}

void unintern_symbol(Symbol* self) {
  // Remove `self' from its home module and the symbol table.
  { Extensible_Symbol_Array* symbolarray = select_symbol_array(SYMBOL_SYM);
    String_To_Integer_Hash_Table* offsettable = (((boolean)(((Module*)(self->home_context)))) ? select_symbol_offset_table(((Module*)(self->home_context)), SYMBOL_SYM) : ((String_To_Integer_Hash_Table*)(NULL)));
    int symbolid = self->symbol_id;
    Symbol* realsymbol = get_sym_from_offset(symbolid);

    if (self == realsymbol) {
      free_symbol_offset(symbolarray, symbolid);
    }
    if (((boolean)(offsettable)) &&
        (symbolid == offsettable->lookup(self->symbol_name))) {
      offsettable->remove_at(self->symbol_name);
    }
    self->symbol_id = -1;
    self->symbol_value_and_plist = NULL;
  }
}

void unintern_surrogate(Surrogate* self) {
  // Remove `self' from its home module and the surrogate table.
  { Extensible_Symbol_Array* surrogatearray = select_symbol_array(SURROGATE_SYM);
    String_To_Integer_Hash_Table* offsettable = (((boolean)(((Module*)(self->home_context)))) ? select_symbol_offset_table(((Module*)(self->home_context)), SURROGATE_SYM) : ((String_To_Integer_Hash_Table*)(NULL)));
    int surrogateid = self->symbol_id;
    Surrogate* realsurrogate = get_sgt_from_offset(surrogateid);

    if (self == realsurrogate) {
      free_symbol_offset(surrogatearray, surrogateid);
    }
    if (((boolean)(offsettable)) &&
        (surrogateid == offsettable->lookup(self->symbol_name))) {
      offsettable->remove_at(self->symbol_name);
    }
    self->symbol_id = -1;
    self->surrogate_value = NULL;
  }
}

boolean Generalized_Symbol::deletedP() {
  { Generalized_Symbol* self = this;

    return (self->symbol_id == -1);
  }
}

Generalized_Symbol* intern_bootstrap_symbol_at(char* name, int offset, int kindofsym) {
  // Interns a symbol, or keyword with
  // `name' and symbol-id `offset'.
  // If a symbol with that ID already existed, an error is signaled.  This
  // function is used to recreate the symbol table in a way where symbol IDs
  // which got hardwired into translated code will refer to the correct objects.
  { Extensible_Symbol_Array* symbolarray = NULL;
    Generalized_Symbol* symbol = NULL;

    switch (kindofsym) {
      case 0: 
        symbol = lookup_symbol(name);
      break;
      case 1: 
        symbol = lookup_surrogate(name);
      break;
      case 2: 
        symbol = lookup_keyword(name);
      break;
      default:
        { Output_String_Stream* stream_000 = new_output_string_stream();

          *(stream_000->native_stream) << "`" << kindofsym << "'" << " is not a valid case option";
          throw *new_stella_exception(stream_000->the_string_reader());
        }
      break;
    }
    if (((boolean)(symbol))) {
      if (!(symbol->symbol_id == offset)) {
        *(STANDARD_WARNING->native_stream) << "Warning: " << "intern-bootstrap-symbol-at: " << "`" << symbol << "'" << " is already interned at offset " << "`" << symbol->symbol_id << "'" << endl;
      }
      return (symbol);
    }
    symbolarray = select_symbol_array(kindofsym);
    if (legal_symbol_array_offsetP(symbolarray, offset) &&
        ((boolean)(get_generalized_symbol_from_offset(symbolarray, offset)))) {
      symbol = get_generalized_symbol_from_offset(symbolarray, offset);
      *(STANDARD_WARNING->native_stream) << "Warning: " << "intern-bootstrap-symbol-at: " << "`" << symbol << "'" << " is already interned at offset " << "`" << offset << "'" << endl;
      return (symbol);
    }
    { Extensible_Symbol_Array* array = select_symbol_array(kindofsym);

      return (help_intern_generalized_symbol(name, kindofsym, array, offset, oMODULEo.get()));
    }
  }
}

Symbol* intern_symbol_at(char* name, int offset) {
  return (((Symbol*)(intern_bootstrap_symbol_at(name, offset, SYMBOL_SYM))));
}

Surrogate* intern_surrogate_at(char* name, int offset) {
  return (((Surrogate*)(intern_bootstrap_symbol_at(name, offset, SURROGATE_SYM))));
}

Keyword* intern_keyword_at(char* name, int offset) {
  return (((Keyword*)(intern_bootstrap_symbol_at(name, offset, KEYWORD_SYM))));
}

// Contains the number of the last-created
// context (all 'live' contexts are even-numbered).
int oCONTEXT_NUMBER_COUNTERo = -2;

void initialize_kernel_module(Module* module, char* name, char* fullname, Module* parent) {
  module->module_name = name;
  module->module_full_name = fullname;
  if (((boolean)(parent))) {
    parent->child_contexts->insert(module);
    module->cardinal_module = module;
  }
  module->symbol_offset_table = new_string_to_integer_hash_table();
  module->surrogate_offset_table = new_string_to_integer_hash_table();
  module->all_super_contexts = NIL;
  module->base_module = module;
  oCONTEXT_NUMBER_COUNTERo = oCONTEXT_NUMBER_COUNTERo + 2;
  module->context_number = oCONTEXT_NUMBER_COUNTERo;
  module->stringified_options = "";
}

void initialize_kernel_modules() {
  oROOT_MODULEo = new_module();
  initialize_kernel_module(oROOT_MODULEo, "ROOT-MODULE", "/", NULL);
  oSTELLA_MODULEo = new_module();
  initialize_kernel_module(oSTELLA_MODULEo, "STELLA", "/STELLA", oROOT_MODULEo);
  oCOMMON_LISP_MODULEo = new_module();
  initialize_kernel_module(oCOMMON_LISP_MODULEo, "COMMON-LISP", "/COMMON-LISP", oROOT_MODULEo);
  set_dynamic_slot_value(oCOMMON_LISP_MODULEo->dynamic_slots, SYM_SYMBOLS_STELLA_MODULE_LISP_PACKAGE, wrap_string("CL"), NULL_STRING_WRAPPER);
  oROOT_MODULEo->cardinal_module = oSTELLA_MODULEo;
  oMODULEo.set(oSTELLA_MODULEo);
  oCONTEXTo.set(oMODULEo.get());
}

void initialize_symbols_and_kernel_modules() {
  oFIXED_SYMBOL_ARRAYo = new_extensible_symbol_array(100);
  oFIXED_SURROGATE_ARRAYo = new_extensible_symbol_array(100);
  oFIXED_KEYWORD_ARRAYo = new_extensible_symbol_array(100);
  oKEYWORD_OFFSET_TABLEo = new_string_to_integer_hash_table();
  oSYMBOL_ARRAYo = oFIXED_SYMBOL_ARRAYo;
  oSURROGATE_ARRAYo = oFIXED_SURROGATE_ARRAYo;
  oKEYWORD_ARRAYo = oFIXED_KEYWORD_ARRAYo;
  initialize_kernel_modules();
}

Symbol* lookup_transient_symbol(char* name) {
  if (((boolean)(oTRANSIENT_SYMBOL_LOOKUP_TABLEo))) {
    return (((Symbol*)(oTRANSIENT_SYMBOL_LOOKUP_TABLEo->lookup(name))));
  }
  else {
    {
      *(STANDARD_WARNING->native_stream) << "Warning: " << "Looking for a transient symbol when the parsing tables have\nnot been initialized." << endl;
      return (NULL);
    }
  }
}

Symbol* intern_transient_symbol(char* name) {
  { Symbol* symbol = lookup_symbol_in_module(name, oMODULEo.get(), FALSE);

    if (((boolean)(symbol))) {
      return (symbol);
    }
    symbol = lookup_transient_symbol(name);
    if (((boolean)(symbol))) {
      return (symbol);
    }
    symbol = new_transient_symbol(name);
    symbol->home_context = NULL;
    oTRANSIENT_SYMBOL_LOOKUP_TABLEo->insert_at(name, symbol);
    return (symbol);
  }
}

void free_transient_symbols() {
  oTRANSIENT_SYMBOL_LOOKUP_TABLEo->initialize_hash_table();
}

Symbol* intern_common_lisp_symbol(char* name) {
  { Symbol* symbol = ((Symbol*)(lookup_rigid_symbol_wrt_module(name, oCOMMON_LISP_MODULEo, SYMBOL_SYM)));

    if (((boolean)(symbol))) {
      return (symbol);
    }
    name = string_upcase(name);
    symbol = ((Symbol*)(lookup_rigid_symbol_wrt_module(name, oCOMMON_LISP_MODULEo, SYMBOL_SYM)));
    if (((boolean)(symbol))) {
      return (symbol);
    }
    symbol = ((Symbol*)(intern_rigid_symbol_wrt_module(name, oCOMMON_LISP_MODULEo, SYMBOL_SYM)));
    return (symbol);
  }
}

boolean symbol_common_lispP(Symbol* self) {
  return (((Module*)(self->home_context)) == oCOMMON_LISP_MODULEo);
}

boolean common_lisp_symbolP(Object* self) {
  return (symbolP(self) &&
      symbol_common_lispP(((Symbol*)(self))));
}

void initialize_parsing_tables() {
  // Create indices for recording transient symbols and
  // Common Lisp symbols while parsing.
  oTRANSIENT_SYMBOL_LOOKUP_TABLEo = new_string_hash_table();
  oCOMMON_LISP_SYMBOL_LOOKUP_TABLEo = new_string_hash_table();
}

// Incremented once for each gensym call.
int oGENSYM_COUNTERo = 0;

// Defines the separator character for gensyms,
// and the fill digits.
char* oGENSYM_MASKo = "-000";

char* yield_gensym_name(char* prefix, int counter) {
  { char* suffix = integer_to_string(counter);
    int maskend = 4 - strlen(suffix);

    if (maskend < 1) {
      maskend = 1;
    }
    return (string_concatenate(prefix, string_subsequence(oGENSYM_MASKo, 0, maskend), 1, suffix));
  }
}

Symbol* gensym(char* prefix) {
  // Return a transient symbol with a name beginning with
  // `prefix' and ending with a globally gensym'd integer.
  return (intern_transient_symbol(yield_gensym_name(prefix, oGENSYM_COUNTERo = oGENSYM_COUNTERo + 1)));
}

Symbol* surrogate_to_symbol(Surrogate* self) {
  return (intern_symbol_in_module(self->symbol_name, oMODULEo.get(), FALSE));
}

Symbol* type_to_symbol(Surrogate* type) {
  // Convert `type' into a symbol with the same name and module.
  return (intern_symbol_in_module(type->symbol_name, ((Module*)(type->home_context)), TRUE));
}

Symbol* symbolize(Surrogate* surrogate) {
  // Convert `surrogate' into a symbol with the same name and module.
  return (intern_symbol_in_module(surrogate->symbol_name, ((Module*)(surrogate->home_context)), TRUE));
}

Surrogate* string_to_surrogate(char* self) {
  { Surrogate* surrogate = intern_surrogate_in_module(self, oMODULEo.get(), FALSE);

    if ((surrogate->home_context == oMODULEo.get()) ||
        ((boolean)(surrogate->surrogate_value))) {
      return (surrogate);
    }
    return (intern_surrogate_in_module(self, oMODULEo.get(), TRUE));
  }
}

Surrogate* symbol_to_surrogate(Symbol* self) {
  { Surrogate* surrogate = lookup_surrogate_in_module(self->symbol_name, ((Module*)(self->home_context)), FALSE);

    if (((boolean)(surrogate)) &&
        ((boolean)(surrogate->surrogate_value))) {
      return (surrogate);
    }
    return (string_to_surrogate(self->symbol_name));
  }
}

Surrogate* Surrogate::surrogatify() {
  { Surrogate* self = this;

    return (self);
  }
}

Surrogate* Symbol::surrogatify() {
  { Symbol* self = this;

    return (symbol_to_surrogate(self));
  }
}

Surrogate* string_surrogatify(char* self) {
  return (intern_surrogate(self));
}

Surrogate* Cons::surrogatify() {
  { Cons* self = this;

    if (self == NIL) {
      return (intern_surrogate("NIL"));
    }
    else {
      throw *new_stella_exception("Cons passed to 'surrogatify'");
    }
  }
}

Surrogate* Object::surrogatify() {
  { Object* self = this;

    throw *new_stella_exception("OBJECT passed to 'surrogatify'");
  }
}

Surrogate* typify(Object* self) {
  return (self->surrogatify());
}

Keyword* Keyword::keywordify() {
  { Keyword* self = this;

    return (self);
  }
}

Keyword* Generalized_Symbol::keywordify() {
  { Generalized_Symbol* self = this;

    return (intern_keyword(self->symbol_name));
  }
}

Keyword* String_Wrapper::keywordify() {
  { String_Wrapper* self = this;

    return (intern_keyword(self->wrapper_value));
  }
}

Keyword* string_keywordify(char* self) {
  return (intern_keyword(self));
}

void in_place_objects_to_symbols(Cons* conslist) {
  { Cons* cursor = conslist;

    while (!(cursor == NIL)) {
      cursor->value = cursor->value->permanentify();
      cursor = cursor->rest;
    }
  }
}

void in_place_objects_to_types(Cons* conslist) {
  { Cons* cursor = conslist;

    while (!(cursor == NIL)) {
      cursor->value = typify(cursor->value);
      cursor = cursor->rest;
    }
  }
}

boolean surrogate_nameP(char* name) {
  return ((strlen(name) > 0) &&
      (name[0] == SURROGATE_PREFIX_CHARACTER));
}

boolean qualified_nameP(char* name) {
  if (strlen(name) == 0) {
    return (FALSE);
  }
  else if (string_memberP(name, MODULE_SEPARATOR_CHARACTER)) {
    if (strlen(name) > 1) {
      if (oCHARACTER_TYPE_TABLEo[(int)(unsigned char) (name[0])] == KWD_SYMBOLS_DIGIT) {
        return (FALSE);
      }
      if ((name[0] == '.') &&
          (oCHARACTER_TYPE_TABLEo[(int)(unsigned char) (name[1])] == KWD_SYMBOLS_DIGIT)) {
        return (FALSE);
      }
      return (TRUE);
    }
  }
  else if (name[0] == '.') {
    switch (strlen(name)) {
      case 1: 
        return (TRUE);
      case 2: 
        if (name[1] == '.') {
          return (TRUE);
        }
      break;
      default:
      break;
    }
  }
  return (FALSE);
}

boolean keyword_nameP(char* name) {
  return ((strlen(name) > 0) &&
      (name[0] == ':'));
}

Generalized_Symbol* intern_generalized_symbol(char* name) {
  if (qualified_nameP(name)) {
    return (intern_qualified_name(name, TRUE));
  }
  else if (keyword_nameP(name)) {
    return (intern_keyword(string_rest(name)));
  }
  else if (surrogate_nameP(name)) {
    return (intern_surrogate(string_rest(name)));
  }
  else {
    return (intern_symbol(name));
  }
}

char* compute_full_name(char* name, Module* module) {
  if (!((boolean)(module))) {
    return (name);
  }
  else if (string_eqlP(name, "")) {
    { char* pathname = "";

      while (((boolean)(module))) {
        pathname = string_concatenate(MODULE_SEPARATOR_STRING, module->module_name, 1, pathname);
        module = ((Module*)(module->parent_modules->first()));
      }
      return (pathname);
    }
  }
  else {
    { char* modulefullname = module->module_full_name;

      if (modulefullname == NULL) {
        modulefullname = compute_full_name("", module);
      }
      return (string_concatenate(modulefullname, MODULE_SEPARATOR_STRING, 1, name));
    }
  }
}

char* Generalized_Symbol::local_print_name() {
  { Generalized_Symbol* self = this;

    return (self->symbol_name);
  }
}

char* Surrogate::local_print_name() {
  { Surrogate* self = this;

    return (string_concatenate("@", self->symbol_name, 0));
  }
}

char* Generalized_Symbol::relative_name() {
  { Generalized_Symbol* self = this;

    { char* string = NULL;

      if (((Module*)(self->home_context)) == oMODULEo.get()) {
        string = self->local_print_name();
      }
      else {
        string = compute_full_name(self->local_print_name(), ((Module*)(self->home_context)));
      }
      if (self->symbol_id == -1) {
        string = string_concatenate("<<UNINTERNED>>/", string, 0);
      }
      return (string);
    }
  }
}

boolean visible_symbolP(Symbol* self) {
  // Return `true' if `self' is visible from the current module.
  return (self == lookup_symbol_in_module(self->symbol_name, oMODULEo.get(), FALSE));
}

boolean visible_surrogateP(Surrogate* self) {
  // Return `true' if `self' is visible from the current module.
  return (self == lookup_surrogate_in_module(self->symbol_name, oMODULEo.get(), FALSE));
}

char* Generalized_Symbol::visible_name() {
  { Generalized_Symbol* self = this;

    { boolean visibleP = FALSE;

      { Surrogate* test_value_000 = safe_primary_type(self);

        if (subtype_of_symbolP(test_value_000)) {
          { Generalized_Symbol* self_000 = self;
            Symbol* self = ((Symbol*)(self_000));

            visibleP = self == lookup_symbol_in_module(self->symbol_name, oMODULEo.get(), FALSE);
          }
        }
        else if (subtype_of_surrogateP(test_value_000)) {
          { Generalized_Symbol* self_001 = self;
            Surrogate* self = ((Surrogate*)(self_001));

            visibleP = self == lookup_surrogate_in_module(self->symbol_name, oMODULEo.get(), FALSE);
          }
        }
        else {
          { Output_String_Stream* stream_000 = new_output_string_stream();

            *(stream_000->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
            throw *new_stella_exception(stream_000->the_string_reader());
          }
        }
      }
      if (visibleP) {
        return (self->local_print_name());
      }
      else {
        return (compute_full_name(self->local_print_name(), ((Module*)(self->home_context))));
      }
    }
  }
}

Object* symbol_value(Symbol* symbol) {
  // Return the value of `symbol'.  Note, that this value is not
  // visible to code that references a variable with the same name as `symbol'.
  // The `symbol-value' is simply a special property that can always be accessed
  // in constant time.  The `symbol-value' of a symbol can be changed with `setf'.
  return (symbol->symbol_value_and_plist->value);
}

Object* symbol_value_setter(Symbol* symbol, Object* value) {
  { Cons* vplist = symbol->symbol_value_and_plist;

    if (vplist == NIL) {
      vplist = cons(NULL, NIL);
      symbol->symbol_value_and_plist = vplist;
    }
    return (vplist->value = value);
  }
}

Cons* symbol_plist(Symbol* symbol) {
  // Return the property list of `symbol'.  The `symbol-plist'
  // of a symbol can be set with `setf'.  IMPORTANT: Property list are modified 
  // destructively, hence, if you supply it as a whole make sure to always supply
  // a modfiable copy, e.g., by using `bquote'.
  return (symbol->symbol_value_and_plist->rest);
}

Cons* symbol_plist_setter(Symbol* symbol, Cons* plist) {
  { Cons* vplist = symbol->symbol_value_and_plist;

    if (vplist == NIL) {
      vplist = cons(NULL, NIL);
      symbol->symbol_value_and_plist = vplist;
    }
    return (vplist->rest = plist);
  }
}

Object* symbol_property(Symbol* symbol, Standard_Object* key) {
  // Return the property of `symbol' whose key is `eq?' to `key'.
  // Symbol properties can be set with `setf'.
  { Cons* plist = symbol->symbol_value_and_plist->rest;

    while (!(plist == NIL)) {
      if (plist->value == key) {
        return (plist->rest->value);
      }
      plist = plist->rest->rest;
    }
    return (NULL);
  }
}

Object* symbol_property_setter(Symbol* symbol, Object* value, Standard_Object* key) {
  { Cons* vplist = symbol->symbol_value_and_plist;
    Cons* plist = vplist->rest;

    while (!(plist == NIL)) {
      if (plist->value == key) {
        if (((boolean)(value))) {
          plist->rest->value = value;
        }
        else if (!(plist->rest->rest == NIL)) {
          { Cons* restplist = plist->rest;

            plist->value = restplist->rest->value;
            plist->rest = restplist->rest->rest;
          }
        }
        else if (plist == vplist->rest) {
          vplist->rest = NIL;
        }
        else {
          plist->rest->value = value;
        }
        return (value);
      }
      plist = plist->rest->rest;
    }
    if (((boolean)(value))) {
      if (vplist == NIL) {
        vplist = cons(NULL, NIL);
        symbol->symbol_value_and_plist = vplist;
      }
      vplist->rest = cons(key, cons(value, vplist->rest));
    }
    return (value);
  }
}

boolean symbol_constituent_characterP(char character) {
  { Keyword* test_value_000 = oCHARACTER_TYPE_TABLEo[(int)(unsigned char) character];

    if ((test_value_000 == KWD_SYMBOLS_LETTER) ||
        ((test_value_000 == KWD_SYMBOLS_DIGIT) ||
         (test_value_000 == KWD_SYMBOLS_SYMBOL_CONSTITUENT))) {
      return (TRUE);
    }
    else {
      return (FALSE);
    }
  }
}

char* oSYMBOL_ESCAPE_CODE_TABLEo = NULL;

char* oCASE_SENSITIVE_SYMBOL_ESCAPE_CODE_TABLEo = NULL;

char* initialize_symbol_escape_code_table(boolean casesensitiveP) {
  { char* buffer = make_string(256, NULL_CHARACTER);

    { int i = NULL_INTEGER;
      int iter_000 = 0;
      int upper_bound_000 = 255;

      while (iter_000 <= upper_bound_000) {
        i = iter_000;
        iter_000 = iter_000 + 1;
        { Keyword* test_value_000 = oCHARACTER_TYPE_TABLEo[i];

          if (test_value_000 == KWD_SYMBOLS_DIGIT) {
            buffer[i] = ((char) 1);
          }
          else if ((test_value_000 == KWD_SYMBOLS_LETTER) ||
              (test_value_000 == KWD_SYMBOLS_SYMBOL_CONSTITUENT)) {
            if ((!casesensitiveP) &&
                lower_case_characterP((char) i)) {
              buffer[i] = ((char) 2);
            }
            else {
              buffer[i] = ((char) 0);
            }
          }
          else {
            buffer[i] = ((char) 2);
          }
        }
      }
    }
    { char ch = NULL_CHARACTER;
      char* vector_000 = ".+-";
      int index_000 = 0;
      int length_000 = strlen(vector_000);

      while (index_000 < length_000) {
        ch = vector_000[index_000];
        index_000 = index_000 + 1;
        buffer[((int)(unsigned char) ch)] = ((char) 1);
      }
    }
    buffer[((int)(unsigned char) '\\')] = ((char) 3);
    buffer[((int)(unsigned char) '|')] = ((char) 3);
    return (buffer);
  }
}

Keyword* compute_symbol_escape_code(char* name, boolean casesensitiveP) {
  { char* table = (casesensitiveP ? oCASE_SENSITIVE_SYMBOL_ESCAPE_CODE_TABLEo : oSYMBOL_ESCAPE_CODE_TABLEo);
    int escapecode = 0;
    int maxescapecode = 0;
    boolean firstcharP = TRUE;
    boolean couldbenumberP = TRUE;

    { char ch = NULL_CHARACTER;
      char* vector_000 = name;
      int index_000 = 0;
      int length_000 = strlen(vector_000);

      while (index_000 < length_000) {
        ch = vector_000[index_000];
        index_000 = index_000 + 1;
        escapecode = (int)(unsigned char) (table[((int)(unsigned char) ch)]);
        if (escapecode == 0) {
          if (couldbenumberP &&
              ((!(ch == 'E')) &&
               (!(ch == 'e')))) {
            couldbenumberP = FALSE;
          }
        }
        else if (firstcharP) {
          maxescapecode = escapecode;
          couldbenumberP = escapecode == 1;
        }
        else {
          if ((escapecode > 1) &&
              (escapecode > maxescapecode)) {
            maxescapecode = escapecode;
          }
        }
        firstcharP = FALSE;
      }
    }
    if (firstcharP) {
      return (KWD_SYMBOLS_ESCAPED);
    }
    else {
      switch (maxescapecode) {
        case 0: 
          return (KWD_SYMBOLS_UNESCAPED);
        case 1: 
          if (couldbenumberP) {
            { Tokenizer_Table* tok_table_ = oSTELLA_TOKENIZER_TABLEo;
              char* tok_transitions_ = tok_table_->transitions;
              Object** tok_statenames_ = tok_table_->state_names->the_array;
              int tok_tokenstart_ = -1;
              boolean tok_endoftokensP_ = FALSE;
              Tokenizer_Stream_State* tok_streamstate_ = NULL;
              char* tok_buffer_ = name;
              int tok_state_ = 1;
              int tok_nextstate_ = tok_state_;
              int tok_cursor_ = 0;
              int tok_size_ = strlen(name);
              int tok_end_ = tok_size_;

              { Keyword* tok_stellastatename_ = NULL;
                Object** tok_stellalogicalstatenames_ = oSTELLA_LOGICAL_STATE_NAMESo->the_array;
                Keyword* tok_stellalogicalstatename_ = NULL;

                {
                  {
                    tok_tokenstart_ = -1;
                    for (;;) {
                      if (tok_cursor_ == tok_end_) {
                        if (tok_nextstate_ == -1) {
                          tok_endoftokensP_ = TRUE;
                        }
                        else if (coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)((tok_table_->legal_eof_states->the_array)[tok_state_])))) {
                          tok_nextstate_ = -1;
                          if (tok_tokenstart_ == -1) {
                            tok_endoftokensP_ = TRUE;
                          }
                        }
                        else {
                          tok_state_ = 0;
                          tok_nextstate_ = -1;
                        }
                        break;
                      }
                      tok_nextstate_ = (int)(unsigned char) (tok_transitions_[((((tok_state_ << 8)) | ((int)(unsigned char) (tok_buffer_[tok_cursor_]))))]);
                      if ((tok_nextstate_ & 128) == 0) {
                        tok_state_ = tok_nextstate_;
                        tok_cursor_ = tok_cursor_ + 1;
                      }
                      else if (tok_tokenstart_ == -1) {
                        if ((tok_nextstate_ & 64) == 0) {
                          tok_tokenstart_ = tok_cursor_;
                        }
                        tok_state_ = (tok_nextstate_ & 63);
                        tok_cursor_ = tok_cursor_ + 1;
                      }
                      else {
                        break;
                      }
                    }
                    if (((boolean)(tok_streamstate_))) {
                      tok_streamstate_->cursor = tok_cursor_;
                      tok_streamstate_->state = tok_state_;
                    }
                  }
                  tok_stellastatename_ = ((Keyword*)(((Generalized_Symbol*)(tok_statenames_[tok_state_]))));
                  tok_stellalogicalstatename_ = ((Keyword*)(tok_stellalogicalstatenames_[tok_state_]));
                }
                if (tok_stellalogicalstatename_ == KWD_SYMBOLS_SYMBOL) {
                  couldbenumberP = FALSE;
                }
              }
            }
          }
          if (couldbenumberP) {
            return (KWD_SYMBOLS_ESCAPED);
          }
          else {
            return (KWD_SYMBOLS_UNESCAPED);
          }
        break;
        case 2: 
          if ((name[0] == '/') &&
              string_eqlP(name, "/")) {
            return (KWD_SYMBOLS_UNESCAPED);
          }
          else {
            return (KWD_SYMBOLS_ESCAPED);
          }
        break;
        case 3: 
          return (KWD_SYMBOLS_COMPLEX_ESCAPED);
        default:
          { Output_String_Stream* stream_000 = new_output_string_stream();

            *(stream_000->native_stream) << "`" << maxescapecode << "'" << " is not a valid case option";
            throw *new_stella_exception(stream_000->the_string_reader());
          }
        break;
      }
    }
  }
}

void print_symbol_name_readably(char* name, ostream* stream, boolean casesensitiveP) {
  { Keyword* test_value_000 = compute_symbol_escape_code(name, casesensitiveP);

    if (test_value_000 == KWD_SYMBOLS_UNESCAPED) {
      *(stream) << name;
    }
    else if (test_value_000 == KWD_SYMBOLS_ESCAPED) {
      *(stream) << "|" << name << "|";
    }
    else if (test_value_000 == KWD_SYMBOLS_COMPLEX_ESCAPED) {
      *(stream) << "|";
      { char ch = NULL_CHARACTER;
        char* vector_000 = name;
        int index_000 = 0;
        int length_000 = strlen(vector_000);

        while (index_000 < length_000) {
          ch = vector_000[index_000];
          index_000 = index_000 + 1;
          if ((ch == '\\') ||
              (ch == '|')) {
            *(stream) << "\\";
          }
          *(stream) << ch;
        }
      }
      *(stream) << "|";
    }
    else {
      { Output_String_Stream* stream_000 = new_output_string_stream();

        *(stream_000->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
        throw *new_stella_exception(stream_000->the_string_reader());
      }
    }
  }
}

void print_symbol(Symbol* self, ostream* stream) {
  { boolean visibleP = self == lookup_symbol_in_module(self->symbol_name, oMODULEo.get(), FALSE);
    Module* module = ((Module*)(self->home_context));

    if (!visibleP) {
      if (((boolean)(module))) {
        if (oPRINTREADABLYpo.get() &&
            (module == oCOMMON_LISP_MODULEo)) {
          {
            *(stream) << "CL:" << self->symbol_name;
            return;
          }
        }
        else {
          *(stream) << module->module_full_name << "/";
        }
      }
      else {
        if (self->symbol_id == -1) {
          *(stream) << "<<UNINTERNED>>/";
        }
      }
    }
    if (oPRINTREADABLYpo.get()) {
      print_symbol_name_readably(self->symbol_name, stream, oMODULEo.get()->case_sensitiveP);
    }
    else {
      *(stream) << self->symbol_name;
    }
  }
}

void print_surrogate(Surrogate* self, ostream* stream) {
  { boolean visibleP = self == lookup_surrogate_in_module(self->symbol_name, oMODULEo.get(), FALSE);
    Module* module = ((Module*)(self->home_context));

    if (!visibleP) {
      if ((self->symbol_id == -1) ||
          (!((boolean)(module)))) {
        *(stream) << "<<UNINTERNED>>/";
      }
      else {
        *(stream) << module->module_full_name << "/";
      }
    }
    *(stream) << "@";
    if (oPRINTREADABLYpo.get()) {
      print_symbol_name_readably(self->symbol_name, stream, oMODULEo.get()->case_sensitiveP);
    }
    else {
      *(stream) << self->symbol_name;
    }
  }
}

void print_keyword(Keyword* self, ostream* stream) {
  *(stream) << ":";
  if (oPRINTREADABLYpo.get()) {
    print_symbol_name_readably(self->symbol_name, stream, FALSE);
  }
  else {
    *(stream) << self->symbol_name;
  }
}

void help_startup_symbols1() {
  {
    SYM_SYMBOLS_STELLA_MODULE_LISP_PACKAGE = ((Symbol*)(intern_rigid_symbol_wrt_module("MODULE-LISP-PACKAGE", NULL, 0)));
    SYM_SYMBOLS_STELLA_JAVA_PACKAGE = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA-PACKAGE", NULL, 0)));
    SYM_SYMBOLS_STELLA_MODULE_CPP_PACKAGE = ((Symbol*)(intern_rigid_symbol_wrt_module("MODULE-CPP-PACKAGE", NULL, 0)));
    KWD_SYMBOLS_DIGIT = ((Keyword*)(intern_rigid_symbol_wrt_module("DIGIT", NULL, 2)));
    KWD_SYMBOLS_LETTER = ((Keyword*)(intern_rigid_symbol_wrt_module("LETTER", NULL, 2)));
    KWD_SYMBOLS_SYMBOL_CONSTITUENT = ((Keyword*)(intern_rigid_symbol_wrt_module("SYMBOL-CONSTITUENT", NULL, 2)));
    KWD_SYMBOLS_ESCAPED = ((Keyword*)(intern_rigid_symbol_wrt_module("ESCAPED", NULL, 2)));
    KWD_SYMBOLS_UNESCAPED = ((Keyword*)(intern_rigid_symbol_wrt_module("UNESCAPED", NULL, 2)));
    KWD_SYMBOLS_SYMBOL = ((Keyword*)(intern_rigid_symbol_wrt_module("SYMBOL", NULL, 2)));
    KWD_SYMBOLS_COMPLEX_ESCAPED = ((Keyword*)(intern_rigid_symbol_wrt_module("COMPLEX-ESCAPED", NULL, 2)));
    SYM_SYMBOLS_STELLA_STARTUP_SYMBOLS = ((Symbol*)(intern_rigid_symbol_wrt_module("STARTUP-SYMBOLS", NULL, 0)));
    SYM_SYMBOLS_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-STARTUP-CLASSNAME", NULL, 0)));
  }
}

void help_startup_symbols2() {
  {
    define_function_object("NEXT-FREE-OFFSET", "(DEFUN (NEXT-FREE-OFFSET INTEGER) ((SELF EXTENSIBLE-SYMBOL-ARRAY)))", ((cpp_function_code)(&next_free_offset)), NULL);
    define_function_object("FREE-SYMBOL-OFFSET", "(DEFUN FREE-SYMBOL-OFFSET ((SELF EXTENSIBLE-SYMBOL-ARRAY) (OFFSET INTEGER)))", ((cpp_function_code)(&free_symbol_offset)), NULL);
    define_function_object("ADD-TO-SYMBOL-ARRAY-AT", "(DEFUN ADD-TO-SYMBOL-ARRAY-AT ((ARRAY EXTENSIBLE-SYMBOL-ARRAY) (OFFSET INTEGER) (SYMBOL GENERALIZED-SYMBOL)))", ((cpp_function_code)(&add_to_symbol_array_at)), NULL);
    define_function_object("ADD-TO-SYMBOL-ARRAY", "(DEFUN (ADD-TO-SYMBOL-ARRAY INTEGER) ((ARRAY EXTENSIBLE-SYMBOL-ARRAY) (SYMBOL GENERALIZED-SYMBOL)))", ((cpp_function_code)(&add_to_symbol_array)), NULL);
    define_function_object("LEGAL-SYMBOL-ARRAY-OFFSET?", "(DEFUN (LEGAL-SYMBOL-ARRAY-OFFSET? BOOLEAN) ((ARRAY EXTENSIBLE-SYMBOL-ARRAY) (OFFSET INTEGER)))", ((cpp_function_code)(&legal_symbol_array_offsetP)), NULL);
    define_function_object("GET-SYM", "(DEFUN (GET-SYM SYMBOL) ((OFFSET INTEGER)))", ((cpp_function_code)(&get_sym)), NULL);
    define_function_object("GET-SGT", "(DEFUN (GET-SGT SURROGATE) ((OFFSET INTEGER)))", ((cpp_function_code)(&get_sgt)), NULL);
    define_function_object("GET-KWD", "(DEFUN (GET-KWD KEYWORD) ((OFFSET INTEGER)))", ((cpp_function_code)(&get_kwd)), NULL);
    define_function_object("GET-GENERALIZED-SYMBOL-FROM-OFFSET", "(DEFUN (GET-GENERALIZED-SYMBOL-FROM-OFFSET GENERALIZED-SYMBOL) ((SYMBOLARRAY EXTENSIBLE-SYMBOL-ARRAY) (OFFSET INTEGER)))", ((cpp_function_code)(&get_generalized_symbol_from_offset)), NULL);
    define_function_object("GET-SYM-FROM-OFFSET", "(DEFUN (GET-SYM-FROM-OFFSET SYMBOL) ((OFFSET INTEGER)))", ((cpp_function_code)(&get_sym_from_offset)), NULL);
    define_function_object("GET-SGT-FROM-OFFSET", "(DEFUN (GET-SGT-FROM-OFFSET SURROGATE) ((OFFSET INTEGER)))", ((cpp_function_code)(&get_sgt_from_offset)), NULL);
    define_function_object("GET-KWD-FROM-OFFSET", "(DEFUN (GET-KWD-FROM-OFFSET KEYWORD) ((OFFSET INTEGER)))", ((cpp_function_code)(&get_kwd_from_offset)), NULL);
    define_function_object("SELECT-SYMBOL-OFFSET-TABLE", "(DEFUN (SELECT-SYMBOL-OFFSET-TABLE STRING-TO-INTEGER-HASH-TABLE) ((MODULE MODULE) (KINDOFSYM INTEGER)))", ((cpp_function_code)(&select_symbol_offset_table)), NULL);
    define_function_object("SELECT-SYMBOL-ARRAY", "(DEFUN (SELECT-SYMBOL-ARRAY EXTENSIBLE-SYMBOL-ARRAY) ((KINDOFSYM INTEGER)))", ((cpp_function_code)(&select_symbol_array)), NULL);
    define_function_object("LOOKUP-RIGID-SYMBOL-LOCALLY", "(DEFUN (LOOKUP-RIGID-SYMBOL-LOCALLY GENERALIZED-SYMBOL) ((NAME STRING) (MODULE MODULE) (KINDOFSYM INTEGER)))", ((cpp_function_code)(&lookup_rigid_symbol_locally)), NULL);
    define_function_object("YIELD-VISIBLE-RIGID-SYMBOLS-WRT-MODULE", "(DEFUN (YIELD-VISIBLE-RIGID-SYMBOLS-WRT-MODULE (CONS OF GENERALIZED-SYMBOL)) ((NAME STRING) (MODULE MODULE) (KINDOFSYM INTEGER)))", ((cpp_function_code)(&yield_visible_rigid_symbols_wrt_module)), NULL);
    define_function_object("VISIBLE-RIGID-SYMBOLS-WRT-MODULE", "(DEFUN (VISIBLE-RIGID-SYMBOLS-WRT-MODULE (ITERATOR OF GENERALIZED-SYMBOL)) ((NAME STRING) (MODULE MODULE) (KINDOFSYM INTEGER)))", ((cpp_function_code)(&visible_rigid_symbols_wrt_module)), NULL);
    define_function_object("REMOVE-SHADOWED-SYMBOLS", "(DEFUN (REMOVE-SHADOWED-SYMBOLS (CONS OF GENERALIZED-SYMBOL)) ((VISIBLESYMBOLS (CONS OF GENERALIZED-SYMBOL))))", ((cpp_function_code)(&remove_shadowed_symbols)), NULL);
    define_function_object("LOOKUP-RIGID-SYMBOL-OFFSET-WRT-MODULE", "(DEFUN (LOOKUP-RIGID-SYMBOL-OFFSET-WRT-MODULE INTEGER) ((NAME STRING) (MODULE MODULE) (KINDOFSYM INTEGER)))", ((cpp_function_code)(&lookup_rigid_symbol_offset_wrt_module)), NULL);
    define_function_object("LOOKUP-RIGID-SYMBOL-WRT-MODULE", "(DEFUN (LOOKUP-RIGID-SYMBOL-WRT-MODULE GENERALIZED-SYMBOL) ((NAME STRING) (MODULE MODULE) (KINDOFSYM INTEGER)))", ((cpp_function_code)(&lookup_rigid_symbol_wrt_module)), NULL);
    define_function_object("LOOKUP-RIGID-SYMBOL", "(DEFUN (LOOKUP-RIGID-SYMBOL GENERALIZED-SYMBOL) ((NAME STRING) (KINDOFSYM INTEGER)) :DOCUMENTATION \"Return the symbol named `name'.  Case sensitive.\")", ((cpp_function_code)(&lookup_rigid_symbol)), NULL);
    define_function_object("LOOKUP-SYMBOL", "(DEFUN (LOOKUP-SYMBOL SYMBOL) ((NAME STRING)) :DOCUMENTATION \"Return the first symbol with `name' visible from the current module.\" :PUBLIC? TRUE)", ((cpp_function_code)(&lookup_symbol)), NULL);
    define_function_object("LOOKUP-SYMBOL-IN-MODULE", "(DEFUN (LOOKUP-SYMBOL-IN-MODULE SYMBOL) ((NAME STRING) (MODULE MODULE) (LOCAL? BOOLEAN)) :DOCUMENTATION \"Return the first symbol with `name' visible from `module'.\nIf `local?' only consider symbols directly interned in `module'.\nIf `module' is `null', use `*MODULE*' instead.\" :PUBLIC? TRUE)", ((cpp_function_code)(&lookup_symbol_in_module)), NULL);
    define_function_object("LOOKUP-VISIBLE-SYMBOLS-IN-MODULE", "(DEFUN (LOOKUP-VISIBLE-SYMBOLS-IN-MODULE (CONS OF SYMBOL)) ((NAME STRING) (MODULE MODULE) (ENFORCESHADOWING? BOOLEAN)) :DOCUMENTATION \"Return the list of symbols with `name' visible from `module'.\nMore specific symbols (relative to the module precedence order defined by\n`visible-modules') come earlier in the list.  If `module' is `null', start\nfrom `*MODULE*' instead.  If `enforceShadowing?' is true, do not return any\nsymbols that are shadowed due to some :SHADOW declaration.\" :PUBLIC? TRUE)", ((cpp_function_code)(&lookup_visible_symbols_in_module)), NULL);
    define_function_object("LOOKUP-SURROGATE", "(DEFUN (LOOKUP-SURROGATE SURROGATE) ((NAME STRING)) :DOCUMENTATION \"Return the first surrogate with `name' visible from the current module.\" :PUBLIC? TRUE)", ((cpp_function_code)(&lookup_surrogate)), NULL);
    define_function_object("LOOKUP-SURROGATE-IN-MODULE", "(DEFUN (LOOKUP-SURROGATE-IN-MODULE SURROGATE) ((NAME STRING) (MODULE MODULE) (LOCAL? BOOLEAN)) :DOCUMENTATION \"Return the first surrogate with `name' visible from `module'.\nIf `local?' only consider surrogates directly interned in `module'.\nIf `module' is `null', use `*MODULE*' instead.\" :PUBLIC? TRUE)", ((cpp_function_code)(&lookup_surrogate_in_module)), NULL);
    define_function_object("LOOKUP-VISIBLE-SURROGATES-IN-MODULE", "(DEFUN (LOOKUP-VISIBLE-SURROGATES-IN-MODULE (CONS OF SURROGATE)) ((NAME STRING) (MODULE MODULE) (ENFORCESHADOWING? BOOLEAN)) :DOCUMENTATION \"Return the list of surrogates with `name' visible from `module'.\nMore specific surrogates (relative to the module precedence order defined by\n`visible-modules') come earlier in the list.  If `module' is `null', start\nfrom `*MODULE*' instead.  If `enforceShadowing?' is true, do not return any\nsurrogates that are shadowed due to some :SHADOW declaration.\" :PUBLIC? TRUE)", ((cpp_function_code)(&lookup_visible_surrogates_in_module)), NULL);
    define_function_object("LOOKUP-KEYWORD", "(DEFUN (LOOKUP-KEYWORD KEYWORD) ((NAME STRING)) :DOCUMENTATION \"Return the keyword with `name' if it exists.\" :PUBLIC? TRUE)", ((cpp_function_code)(&lookup_keyword)), NULL);
    define_function_object("HELP-INTERN-GENERALIZED-SYMBOL", "(DEFUN (HELP-INTERN-GENERALIZED-SYMBOL GENERALIZED-SYMBOL) ((NAME STRING) (KINDOFSYM INTEGER) (ARRAY EXTENSIBLE-SYMBOL-ARRAY) (OFFSET INTEGER) (MODULE MODULE)))", ((cpp_function_code)(&help_intern_generalized_symbol)), NULL);
    define_function_object("INTERN-RIGID-SYMBOL-WRT-MODULE", "(DEFUN (INTERN-RIGID-SYMBOL-WRT-MODULE GENERALIZED-SYMBOL) ((NAME STRING) (MODULE MODULE) (KINDOFSYM INTEGER)) :PUBLIC? TRUE :DOCUMENTATION \"Return a newly-created or existing rigid symbol with\nname `name'.\")", ((cpp_function_code)(&intern_rigid_symbol_wrt_module)), NULL);
    define_function_object("INTERN-RIGID-SYMBOL-LOCALLY", "(DEFUN (INTERN-RIGID-SYMBOL-LOCALLY GENERALIZED-SYMBOL) ((NAME STRING) (MODULE MODULE) (KINDOFSYM INTEGER)) :PUBLIC? TRUE :DOCUMENTATION \"Return a newly-created or existing rigid symbol\ninterned into the module `module' with name `name'.\")", ((cpp_function_code)(&intern_rigid_symbol_locally)), NULL);
    define_function_object("INTERN-RIGID-SYMBOL-CASE-SENSITIVELY", "(DEFUN (INTERN-RIGID-SYMBOL-CASE-SENSITIVELY GENERALIZED-SYMBOL) ((NAME STRING) (KINDOFSYM INTEGER) (TRYUPCASING? BOOLEAN)) :PUBLIC? TRUE)", ((cpp_function_code)(&intern_rigid_symbol_case_sensitively)), NULL);
    define_function_object("INTERN-PERMANENT-SYMBOL", "(DEFUN (INTERN-PERMANENT-SYMBOL SYMBOL) ((NAME STRING)) :PUBLIC? TRUE :DOCUMENTATION \"Return a newly-created or existing permanent symbol with\nname `name'.\")", ((cpp_function_code)(&intern_permanent_symbol)), NULL);
    define_function_object("INTERN-SYMBOL", "(DEFUN (INTERN-SYMBOL SYMBOL) ((NAME STRING)) :DOCUMENTATION \"Return a newly-created or existing symbol with name `name'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&intern_symbol)), NULL);
    define_function_object("INTERN-SYMBOL-IN-MODULE", "(DEFUN (INTERN-SYMBOL-IN-MODULE SYMBOL) ((NAME STRING) (MODULE MODULE) (LOCAL? BOOLEAN)) :DOCUMENTATION \"Look for a symbol named `name' in `module' (if `local?' do\nnot consider inherited modules).  If none exists, intern it locally in\n`module'.  Return the existing or newly-created symbol.\" :PUBLIC? TRUE)", ((cpp_function_code)(&intern_symbol_in_module)), NULL);
    define_function_object("INTERN-DERIVED-SYMBOL", "(DEFUN (INTERN-DERIVED-SYMBOL SYMBOL) ((BASESYMBOL GENERALIZED-SYMBOL) (NEWNAME STRING)) :PUBLIC? TRUE :DOCUMENTATION \"Return a newly-created or existing symbol with name\n`newName' which is interned in the same module as `baseSymbol'.\")", ((cpp_function_code)(&intern_derived_symbol)), NULL);
    define_function_object("INTERN-SURROGATE", "(DEFUN (INTERN-SURROGATE SURROGATE) ((NAME STRING)) :DOCUMENTATION \"Return a newly-created or existing surrogate with name `name'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&intern_surrogate)), NULL);
    define_function_object("INTERN-SURROGATE-IN-MODULE", "(DEFUN (INTERN-SURROGATE-IN-MODULE SURROGATE) ((NAME STRING) (MODULE MODULE) (LOCAL? BOOLEAN)) :DOCUMENTATION \"Look for a symbol named `name' in `module' (if `local?' do\nnot consider inherited modules).  If none exists, intern it locally in\n`module'.  Return the existing or newly-created symbol.\" :PUBLIC? TRUE)", ((cpp_function_code)(&intern_surrogate_in_module)), NULL);
    define_function_object("INTERN-DERIVED-SURROGATE", "(DEFUN (INTERN-DERIVED-SURROGATE SURROGATE) ((BASESYMBOL GENERALIZED-SYMBOL) (NEWNAME STRING)) :PUBLIC? TRUE :DOCUMENTATION \"Return a newly-created or existing surrogate with name\n`newName' which is interned in the same module as `baseSymbol'.\")", ((cpp_function_code)(&intern_derived_surrogate)), NULL);
    define_function_object("INTERN-KEYWORD", "(DEFUN (INTERN-KEYWORD KEYWORD) ((NAME STRING)) :DOCUMENTATION \"Return a newly-created or existing keyword with name\n`name'.  Storage note: a COPY of `name' is stored in the keyword\" :PUBLIC? TRUE)", ((cpp_function_code)(&intern_keyword)), NULL);
    define_function_object("UNINTERN-SYMBOL", "(DEFUN UNINTERN-SYMBOL ((SELF SYMBOL)) :DOCUMENTATION \"Remove `self' from its home module and the symbol table.\" :PUBLIC? TRUE)", ((cpp_function_code)(&unintern_symbol)), NULL);
    define_function_object("UNINTERN-SURROGATE", "(DEFUN UNINTERN-SURROGATE ((SELF SURROGATE)) :DOCUMENTATION \"Remove `self' from its home module and the surrogate table.\")", ((cpp_function_code)(&unintern_surrogate)), NULL);
    define_method_object("(DEFMETHOD (DELETED? BOOLEAN) ((SELF GENERALIZED-SYMBOL)) :GLOBALLY-INLINE? TRUE (RETURN (EQL? (SYMBOL-ID SELF) -1)))", ((cpp_method_code)(&Generalized_Symbol::deletedP)), ((cpp_method_code)(NULL)));
    define_function_object("INTERN-BOOTSTRAP-SYMBOL-AT", "(DEFUN (INTERN-BOOTSTRAP-SYMBOL-AT GENERALIZED-SYMBOL) ((NAME STRING) (OFFSET INTEGER) (KINDOFSYM INTEGER)) :DOCUMENTATION \"Interns a symbol, or keyword with\n`name' and symbol-id `offset'.\nIf a symbol with that ID already existed, an error is signaled.  This\nfunction is used to recreate the symbol table in a way where symbol IDs\nwhich got hardwired into translated code will refer to the correct objects.\")", ((cpp_function_code)(&intern_bootstrap_symbol_at)), NULL);
    define_function_object("INTERN-SYMBOL-AT", "(DEFUN (INTERN-SYMBOL-AT SYMBOL) ((NAME STRING) (OFFSET INTEGER)))", ((cpp_function_code)(&intern_symbol_at)), NULL);
    define_function_object("INTERN-SURROGATE-AT", "(DEFUN (INTERN-SURROGATE-AT SURROGATE) ((NAME STRING) (OFFSET INTEGER)))", ((cpp_function_code)(&intern_surrogate_at)), NULL);
    define_function_object("INTERN-KEYWORD-AT", "(DEFUN (INTERN-KEYWORD-AT KEYWORD) ((NAME STRING) (OFFSET INTEGER)))", ((cpp_function_code)(&intern_keyword_at)), NULL);
    define_function_object("INITIALIZE-KERNEL-MODULE", "(DEFUN INITIALIZE-KERNEL-MODULE ((MODULE MODULE) (NAME STRING) (FULLNAME STRING) (PARENT MODULE)))", ((cpp_function_code)(&initialize_kernel_module)), NULL);
    define_function_object("INITIALIZE-KERNEL-MODULES", "(DEFUN INITIALIZE-KERNEL-MODULES ())", ((cpp_function_code)(&initialize_kernel_modules)), NULL);
    define_function_object("INITIALIZE-SYMBOLS-AND-KERNEL-MODULES", "(DEFUN INITIALIZE-SYMBOLS-AND-KERNEL-MODULES ())", ((cpp_function_code)(&initialize_symbols_and_kernel_modules)), NULL);
    define_function_object("LOOKUP-TRANSIENT-SYMBOL", "(DEFUN (LOOKUP-TRANSIENT-SYMBOL SYMBOL) ((NAME STRING)))", ((cpp_function_code)(&lookup_transient_symbol)), NULL);
    define_function_object("INTERN-TRANSIENT-SYMBOL", "(DEFUN (INTERN-TRANSIENT-SYMBOL SYMBOL) ((NAME STRING)))", ((cpp_function_code)(&intern_transient_symbol)), NULL);
    define_function_object("FREE-TRANSIENT-SYMBOLS", "(DEFUN FREE-TRANSIENT-SYMBOLS ())", ((cpp_function_code)(&free_transient_symbols)), NULL);
    define_function_object("INTERN-COMMON-LISP-SYMBOL", "(DEFUN (INTERN-COMMON-LISP-SYMBOL SYMBOL) ((NAME STRING)))", ((cpp_function_code)(&intern_common_lisp_symbol)), NULL);
    define_function_object("SYMBOL-COMMON-LISP?", "(DEFUN (SYMBOL-COMMON-LISP? BOOLEAN) ((SELF SYMBOL)))", ((cpp_function_code)(&symbol_common_lispP)), NULL);
    define_function_object("COMMON-LISP-SYMBOL?", "(DEFUN (COMMON-LISP-SYMBOL? BOOLEAN) ((SELF OBJECT)))", ((cpp_function_code)(&common_lisp_symbolP)), NULL);
    define_function_object("INITIALIZE-PARSING-TABLES", "(DEFUN INITIALIZE-PARSING-TABLES () :DOCUMENTATION \"Create indices for recording transient symbols and\nCommon Lisp symbols while parsing.\")", ((cpp_function_code)(&initialize_parsing_tables)), NULL);
    define_function_object("YIELD-GENSYM-NAME", "(DEFUN (YIELD-GENSYM-NAME STRING) ((PREFIX STRING) (COUNTER INTEGER)))", ((cpp_function_code)(&yield_gensym_name)), NULL);
    define_function_object("GENSYM", "(DEFUN (GENSYM SYMBOL) ((PREFIX STRING)) :DOCUMENTATION \"Return a transient symbol with a name beginning with\n`prefix' and ending with a globally gensym'd integer.\" :PUBLIC? TRUE)", ((cpp_function_code)(&gensym)), NULL);
    define_function_object("SURROGATE-TO-SYMBOL", "(DEFUN (SURROGATE-TO-SYMBOL SYMBOL) ((SELF SURROGATE)) :PUBLIC? TRUE)", ((cpp_function_code)(&surrogate_to_symbol)), NULL);
  }
}

void help_startup_symbols3() {
  {
    define_function_object("TYPE-TO-SYMBOL", "(DEFUN (TYPE-TO-SYMBOL SYMBOL) ((TYPE TYPE)) :DOCUMENTATION \"Convert `type' into a symbol with the same name and module.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (INTERN-SYMBOL-IN-MODULE (SYMBOL-NAME TYPE) (INTERNED-IN TYPE) TRUE)))", ((cpp_function_code)(&type_to_symbol)), NULL);
    define_function_object("SYMBOLIZE", "(DEFUN (SYMBOLIZE SYMBOL) ((SURROGATE SURROGATE)) :DOCUMENTATION \"Convert `surrogate' into a symbol with the same name and module.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (TYPE-TO-SYMBOL SURROGATE)))", ((cpp_function_code)(&symbolize)), NULL);
    define_function_object("STRING-TO-SURROGATE", "(DEFUN (STRING-TO-SURROGATE SURROGATE) ((SELF STRING)) :PUBLIC? TRUE)", ((cpp_function_code)(&string_to_surrogate)), NULL);
    define_function_object("SYMBOL-TO-SURROGATE", "(DEFUN (SYMBOL-TO-SURROGATE SURROGATE) ((SELF SYMBOL)) :PUBLIC? TRUE)", ((cpp_function_code)(&symbol_to_surrogate)), NULL);
    define_method_object("(DEFMETHOD (SURROGATIFY SURROGATE) ((SELF SURROGATE)) :PUBLIC? TRUE)", ((cpp_method_code)(&Surrogate::surrogatify)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (SURROGATIFY SURROGATE) ((SELF SYMBOL)) :PUBLIC? TRUE)", ((cpp_method_code)(&Symbol::surrogatify)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (SURROGATIFY SURROGATE) ((SELF STRING)) :PUBLIC? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (SURROGATIFY SURROGATE) ((SELF CONS)) :PUBLIC? TRUE)", ((cpp_method_code)(&Cons::surrogatify)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (SURROGATIFY SURROGATE) ((SELF OBJECT)) :PUBLIC? TRUE)", ((cpp_method_code)(&Object::surrogatify)), ((cpp_method_code)(NULL)));
    define_function_object("TYPIFY", "(DEFUN (TYPIFY TYPE) ((SELF OBJECT)) :PUBLIC? TRUE)", ((cpp_function_code)(&typify)), NULL);
    define_method_object("(DEFMETHOD (KEYWORDIFY KEYWORD) ((SELF KEYWORD)) :PUBLIC? TRUE)", ((cpp_method_code)(&Keyword::keywordify)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (KEYWORDIFY KEYWORD) ((SELF GENERALIZED-SYMBOL)) :PUBLIC? TRUE)", ((cpp_method_code)(&Generalized_Symbol::keywordify)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (KEYWORDIFY KEYWORD) ((SELF STRING-WRAPPER)) :PUBLIC? TRUE)", ((cpp_method_code)(&String_Wrapper::keywordify)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (KEYWORDIFY KEYWORD) ((SELF STRING)) :PUBLIC? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_function_object("IN-PLACE-OBJECTS-TO-SYMBOLS", "(DEFUN IN-PLACE-OBJECTS-TO-SYMBOLS ((CONSLIST CONS)))", ((cpp_function_code)(&in_place_objects_to_symbols)), NULL);
    define_function_object("IN-PLACE-OBJECTS-TO-TYPES", "(DEFUN IN-PLACE-OBJECTS-TO-TYPES ((CONSLIST CONS)))", ((cpp_function_code)(&in_place_objects_to_types)), NULL);
    define_function_object("SURROGATE-NAME?", "(DEFUN (SURROGATE-NAME? BOOLEAN) ((NAME STRING)) :PUBLIC? TRUE)", ((cpp_function_code)(&surrogate_nameP)), NULL);
    define_function_object("QUALIFIED-NAME?", "(DEFUN (QUALIFIED-NAME? BOOLEAN) ((NAME STRING)) :PUBLIC? TRUE)", ((cpp_function_code)(&qualified_nameP)), NULL);
    define_function_object("KEYWORD-NAME?", "(DEFUN (KEYWORD-NAME? BOOLEAN) ((NAME STRING)) :PUBLIC? TRUE)", ((cpp_function_code)(&keyword_nameP)), NULL);
    define_function_object("INTERN-GENERALIZED-SYMBOL", "(DEFUN (INTERN-GENERALIZED-SYMBOL GENERALIZED-SYMBOL) ((NAME STRING)) :PUBLIC? TRUE)", ((cpp_function_code)(&intern_generalized_symbol)), NULL);
    define_function_object("COMPUTE-FULL-NAME", "(DEFUN (COMPUTE-FULL-NAME STRING) ((NAME STRING) (MODULE MODULE)) :PUBLIC? TRUE)", ((cpp_function_code)(&compute_full_name)), NULL);
    define_method_object("(DEFMETHOD (LOCAL-PRINT-NAME STRING) ((SELF GENERALIZED-SYMBOL)))", ((cpp_method_code)(&Generalized_Symbol::local_print_name)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (LOCAL-PRINT-NAME STRING) ((SELF SURROGATE)))", ((cpp_method_code)(&Surrogate::local_print_name)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (RELATIVE-NAME STRING) ((SELF GENERALIZED-SYMBOL)))", ((cpp_method_code)(&Generalized_Symbol::relative_name)), ((cpp_method_code)(NULL)));
    define_function_object("VISIBLE-SYMBOL?", "(DEFUN (VISIBLE-SYMBOL? BOOLEAN) ((SELF SYMBOL)) :DOCUMENTATION \"Return `true' if `self' is visible from the current module.\" :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (EQL? SELF (LOOKUP-SYMBOL-IN-MODULE (SYMBOL-NAME SELF) *MODULE* FALSE))))", ((cpp_function_code)(&visible_symbolP)), NULL);
    define_function_object("VISIBLE-SURROGATE?", "(DEFUN (VISIBLE-SURROGATE? BOOLEAN) ((SELF SURROGATE)) :DOCUMENTATION \"Return `true' if `self' is visible from the current module.\" :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (EQL? SELF (LOOKUP-SURROGATE-IN-MODULE (SYMBOL-NAME SELF) *MODULE* FALSE))))", ((cpp_function_code)(&visible_surrogateP)), NULL);
    define_method_object("(DEFMETHOD (VISIBLE-NAME STRING) ((SELF GENERALIZED-SYMBOL)) :PUBLIC? TRUE)", ((cpp_method_code)(&Generalized_Symbol::visible_name)), ((cpp_method_code)(NULL)));
    define_function_object("SYMBOL-VALUE", "(DEFUN (SYMBOL-VALUE OBJECT) ((SYMBOL SYMBOL)) :DOCUMENTATION \"Return the value of `symbol'.  Note, that this value is not\nvisible to code that references a variable with the same name as `symbol'.\nThe `symbol-value' is simply a special property that can always be accessed\nin constant time.  The `symbol-value' of a symbol can be changed with `setf'.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VALUE (SYMBOL-VALUE-AND-PLIST SYMBOL))))", ((cpp_function_code)(&symbol_value)), NULL);
    define_function_object("SYMBOL-VALUE-SETTER", "(DEFUN (SYMBOL-VALUE-SETTER OBJECT) ((SYMBOL SYMBOL) (VALUE OBJECT)))", ((cpp_function_code)(&symbol_value_setter)), NULL);
    define_function_object("SYMBOL-PLIST", "(DEFUN (SYMBOL-PLIST CONS) ((SYMBOL SYMBOL)) :DOCUMENTATION \"Return the property list of `symbol'.  The `symbol-plist'\nof a symbol can be set with `setf'.  IMPORTANT: Property list are modified \ndestructively, hence, if you supply it as a whole make sure to always supply\na modfiable copy, e.g., by using `bquote'.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (REST (SYMBOL-VALUE-AND-PLIST SYMBOL))))", ((cpp_function_code)(&symbol_plist)), NULL);
    define_function_object("SYMBOL-PLIST-SETTER", "(DEFUN (SYMBOL-PLIST-SETTER CONS) ((SYMBOL SYMBOL) (PLIST CONS)))", ((cpp_function_code)(&symbol_plist_setter)), NULL);
    define_function_object("SYMBOL-PROPERTY", "(DEFUN (SYMBOL-PROPERTY OBJECT) ((SYMBOL SYMBOL) (KEY STANDARD-OBJECT)) :DOCUMENTATION \"Return the property of `symbol' whose key is `eq?' to `key'.\nSymbol properties can be set with `setf'.\")", ((cpp_function_code)(&symbol_property)), NULL);
    define_function_object("SYMBOL-PROPERTY-SETTER", "(DEFUN (SYMBOL-PROPERTY-SETTER OBJECT) ((SYMBOL SYMBOL) (VALUE OBJECT) (KEY STANDARD-OBJECT)))", ((cpp_function_code)(&symbol_property_setter)), NULL);
    define_function_object("SYMBOL-CONSTITUENT-CHARACTER?", "(DEFUN (SYMBOL-CONSTITUENT-CHARACTER? BOOLEAN) ((CHARACTER CHARACTER)))", ((cpp_function_code)(&symbol_constituent_characterP)), NULL);
    define_function_object("INITIALIZE-SYMBOL-ESCAPE-CODE-TABLE", "(DEFUN (INITIALIZE-SYMBOL-ESCAPE-CODE-TABLE STRING) ((CASESENSITIVE? BOOLEAN)))", ((cpp_function_code)(&initialize_symbol_escape_code_table)), NULL);
    define_function_object("COMPUTE-SYMBOL-ESCAPE-CODE", "(DEFUN (COMPUTE-SYMBOL-ESCAPE-CODE KEYWORD) ((NAME STRING) (CASESENSITIVE? BOOLEAN)))", ((cpp_function_code)(&compute_symbol_escape_code)), NULL);
    define_function_object("PRINT-SYMBOL-NAME-READABLY", "(DEFUN PRINT-SYMBOL-NAME-READABLY ((NAME STRING) (STREAM NATIVE-OUTPUT-STREAM) (CASESENSITIVE? BOOLEAN)))", ((cpp_function_code)(&print_symbol_name_readably)), NULL);
    define_function_object("PRINT-SYMBOL", "(DEFUN PRINT-SYMBOL ((SELF SYMBOL) (STREAM NATIVE-OUTPUT-STREAM)))", ((cpp_function_code)(&print_symbol)), NULL);
    define_function_object("PRINT-SURROGATE", "(DEFUN PRINT-SURROGATE ((SELF SURROGATE) (STREAM NATIVE-OUTPUT-STREAM)))", ((cpp_function_code)(&print_surrogate)), NULL);
    define_function_object("PRINT-KEYWORD", "(DEFUN PRINT-KEYWORD ((SELF KEYWORD) (STREAM NATIVE-OUTPUT-STREAM)))", ((cpp_function_code)(&print_keyword)), NULL);
    define_function_object("STARTUP-SYMBOLS", "(DEFUN STARTUP-SYMBOLS () :PUBLIC? TRUE)", ((cpp_function_code)(&startup_symbols)), NULL);
    { Method_Slot* function = lookup_function(SYM_SYMBOLS_STELLA_STARTUP_SYMBOLS);

      set_dynamic_slot_value(function->dynamic_slots, SYM_SYMBOLS_STELLA_METHOD_STARTUP_CLASSNAME, wrap_string("Startup-Symbols"), NULL_STRING_WRAPPER);
    }
  }
}

void startup_symbols() {
  if (current_startup_time_phaseP(0)) {
    if (!((boolean)(oFIXED_SYMBOL_ARRAYo))) {
      initialize_symbols_and_kernel_modules();
    }
    initialize_parsing_tables();
  }
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (current_startup_time_phaseP(2)) {
      help_startup_symbols1();
    }
    if (current_startup_time_phaseP(4)) {
      set_dynamic_slot_value(oSTELLA_MODULEo->dynamic_slots, SYM_SYMBOLS_STELLA_JAVA_PACKAGE, wrap_string("edu.isi.stella"), NULL_STRING_WRAPPER);
      set_dynamic_slot_value(oSTELLA_MODULEo->dynamic_slots, SYM_SYMBOLS_STELLA_MODULE_LISP_PACKAGE, wrap_string("STELLA"), NULL_STRING_WRAPPER);
      set_dynamic_slot_value(oSTELLA_MODULEo->dynamic_slots, SYM_SYMBOLS_STELLA_MODULE_CPP_PACKAGE, wrap_string("stella"), NULL_STRING_WRAPPER);
      oSYMBOL_ESCAPE_CODE_TABLEo = initialize_symbol_escape_code_table(FALSE);
      oCASE_SENSITIVE_SYMBOL_ESCAPE_CODE_TABLEo = initialize_symbol_escape_code_table(TRUE);
    }
    if (current_startup_time_phaseP(6)) {
      finalize_classes();
    }
    if (current_startup_time_phaseP(7)) {
      help_startup_symbols2();
      help_startup_symbols3();
    }
    if (current_startup_time_phaseP(8)) {
      finalize_slots();
      cleanup_unfinalized_classes();
    }
    if (current_startup_time_phaseP(9)) {
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *SYMBOL-ARRAY* (EXTENSIBLE-SYMBOL-ARRAY OF SYMBOL) NULL :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *SURROGATE-ARRAY* (EXTENSIBLE-SYMBOL-ARRAY OF SURROGATE) NULL :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *KEYWORD-ARRAY* (EXTENSIBLE-SYMBOL-ARRAY OF KEYWORD) NULL :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *KEYWORD-OFFSET-TABLE* STRING-TO-INTEGER-HASH-TABLE NULL)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *FIXED-SYMBOL-ARRAY* (EXTENSIBLE-SYMBOL-ARRAY OF SYMBOL) NULL)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *FIXED-SURROGATE-ARRAY* (EXTENSIBLE-SYMBOL-ARRAY OF SURROGATE) NULL)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *FIXED-KEYWORD-ARRAY* (EXTENSIBLE-SYMBOL-ARRAY OF KEYWORD) NULL)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *TRANSIENT-SYMBOL-LOOKUP-TABLE* STRING-HASH-TABLE NULL)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *COMMON-LISP-SYMBOL-LOOKUP-TABLE* STRING-HASH-TABLE NULL)");
      define_stella_global_variable_from_stringified_source("(DEFCONSTANT SYMBOL-SYM INTEGER 0)");
      define_stella_global_variable_from_stringified_source("(DEFCONSTANT SURROGATE-SYM INTEGER 1)");
      define_stella_global_variable_from_stringified_source("(DEFCONSTANT KEYWORD-SYM INTEGER 2)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *ROOT-MODULE* MODULE NULL :PUBLIC? TRUE :DOCUMENTATION \"Pointer to the top-most module ROOT-MODULE.\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *STELLA-MODULE* MODULE NULL :PUBLIC? TRUE :DOCUMENTATION \"Pointer to the module STELLA containing definitions of \nSTELLA classes and methods.\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *COMMON-LISP-MODULE* MODULE NULL :PUBLIC? TRUE :DOCUMENTATION \"Pointer to the module COMMON-LISP containing Common\nLisp symbols (and maybe other things).\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *ALTERNATE-STELLA-MODULE* MODULE NULL :DOCUMENTATION \"Pointer to the module ALTERNATE-STELLA containing\ndefinitions of STELLA classes and methods generated during the\ntranslation of STELLA files.\")");
      define_stella_global_variable_from_stringified_source("(DEFSPECIAL *CONTEXT* CONTEXT NULL :DOCUMENTATION \"Points to the current context (either a module or a\nworld).  Set by calls to `change-world' and `change-module'.\" :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFSPECIAL *MODULE* MODULE NULL :DOCUMENTATION \"Points to the current module.  Set by calls to\n`change-context' and `change-module'.\" :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFCONSTANT SURROGATE-PREFIX-CHARACTER CHARACTER #\\@ :DOCUMENTATION \"Character used to prefix the name of a surrogate.\")");
      define_stella_global_variable_from_stringified_source("(DEFCONSTANT MODULE-SEPARATOR-CHARACTER CHARACTER #\\/ :DOCUMENTATION \"Character used to concatenate a module name\nto a symbol or surrogate name to form a qualified name.\")");
      define_stella_global_variable_from_stringified_source("(DEFCONSTANT MODULE-SEPARATOR-STRING STRING \"/\" :DOCUMENTATION \"String containing character used to concatenate a module name\nto a symbol or surrogate name to form a qualified name.\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *CONTEXT-NUMBER-COUNTER* INTEGER -2 :DOCUMENTATION \"Contains the number of the last-created\ncontext (all 'live' contexts are even-numbered).\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *GENSYM-COUNTER* INTEGER 0 :DOCUMENTATION \"Incremented once for each gensym call.\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *GENSYM-MASK* STRING \"-000\" :DOCUMENTATION \"Defines the separator character for gensyms,\nand the fill digits.\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *SYMBOL-ESCAPE-CODE-TABLE* STRING (INITIALIZE-SYMBOL-ESCAPE-CODE-TABLE FALSE))");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *CASE-SENSITIVE-SYMBOL-ESCAPE-CODE-TABLE* STRING (INITIALIZE-SYMBOL-ESCAPE-CODE-TABLE TRUE))");
    }
  }
}

Symbol* SYM_SYMBOLS_STELLA_MODULE_LISP_PACKAGE = NULL;

Symbol* SYM_SYMBOLS_STELLA_JAVA_PACKAGE = NULL;

Symbol* SYM_SYMBOLS_STELLA_MODULE_CPP_PACKAGE = NULL;

Keyword* KWD_SYMBOLS_DIGIT = NULL;

Keyword* KWD_SYMBOLS_LETTER = NULL;

Keyword* KWD_SYMBOLS_SYMBOL_CONSTITUENT = NULL;

Keyword* KWD_SYMBOLS_ESCAPED = NULL;

Keyword* KWD_SYMBOLS_UNESCAPED = NULL;

Keyword* KWD_SYMBOLS_SYMBOL = NULL;

Keyword* KWD_SYMBOLS_COMPLEX_ESCAPED = NULL;

Symbol* SYM_SYMBOLS_STELLA_STARTUP_SYMBOLS = NULL;

Symbol* SYM_SYMBOLS_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella
